;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Harvest Day: An Amstrad CPC Game 
;;  Copyright (C) 2018 Álvaro Molina García / José Manuel Ladrón de Guevara García, Daniel Saura Martínez
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------


;; Macros

	;; Defines the collectabled entity that adds a value used when it collides with any player
	.macro DefineCollectable _name, _x, _y, _w, _h, _sprite, _mx, _my, _dir, _upd, _vx, _vy, _mov, _del, _block, _op, _dist, _type, _value
	_name:
		DefineEntity _name'_e, _x, _y, _mx, _my, _w, _h, _sprite, _mx, _my, _dir, _upd, _vx, _vy, _mov, _del, _block, _op, _dist, _type
		.db _value
		.db col_locked					;; The quantity of points that a player recieves when he picks it up
		_name'_size = . - _name 	;; Saves the number of bytes that fills the Entity
	.endm

	;; This defines a collectable, with only the essentials parameters
	.macro DefineDefaultCollectable _name, _suf
		DefineCollectable _name'_suf, 0, 0, col_w, col_h, _col1_sp_1, -1, 0, 0x00, #col1_update, 0x00, 0x00, collectable1_mov, collectable1_del, 0x00, 0x00, 0x00, collectable1, 20
	.endm

	.macro DefineDefaultCollectable1 _name
		DefineCollectable _name, 0, 0, col_w, col_h, _col1_sp_1, -1, 0, 0x00, #col1_update, 0x00, 0x00, collectable1_mov, collectable1_del, 0x00, 0x00, 0x00, collectable1, 20
	.endm

	.macro DefineDefaultCollectable2 _name
		DefineCollectable _name, 0, 0, col_w, col_h, _col2_sp_1, -1, 0, 0x00, #col1_update, 0x00, 0x00, collectable2_mov, collectable2_del, 0x00, 0x00, 0x00, collectable2, 40
	.endm

	.macro DefineDefaultCollectable3 _name
		DefineCollectable _name, 0, 0, col_w, col_h, _col3_sp_1, -1, 0, collectable3_dir, #col3_update, collectable3_vx, collectable3_vy, 0x00, 0x00, 0x00, 0x00, 0x00, collectable3, 80
	.endm

	.macro DefineDefaultCollectable4 _name
		DefineCollectable _name, 0, 0, col_w, col_h, _col4_sp_1, -1, 0, 0x00, #col4_update, 0x00, 0x00, collectable4_mov, collectable4_del, 0x00, 0x00, 0x00, collectable4, 100
	.endm

	.macro DefineNCollectables _name, _n
		_c = 0
		.rept _n
			DefineDefaultCollectable _name, \_c
			_c = _c + 1
		.endm
	.endm


;; Macro positions
	c_e = 0
	c_e_size = 21

	c_val = 0 + c_e_size
	c_blocked = 1 + c_e_size

	coll_size = c_e_size + 2

;; Constants

	;; Dimensions of the collectables
	col_w = 0x04
	col_h = 0x10
	col_locked = 0

	;; Collectable 1
		;; Position of collectable 1 in the screen
		collectable1_x = 0
		collectable1_y = 0

		;; Position of collectable 1 on the matrix
		collectable1_mx = 0x04
		collectable1_my = 0x05

		;; Initial velocity of the collectable 1
		collectable1_vx = 0x00
		collectable1_vy = 0x00

		;; Movement delay for collectable 1
		collectable1_mov = 0x00 		;; Won't move

		;; Movement delay for collectable 1
		collectable1_del = 0x00


	;; Collectable 2
		;; Position of collectable 2 in the screen
		collectable2_x = 0
		collectable2_y = 0

		;; Position of collectable 2 on the matrix
		collectable2_mx = 0x05
		collectable2_my = 0x06

		;; Initial velocity of the collectable 2
		collectable2_vx = 0xFF
		collectable2_vy = 0x00

		;; Movement delay for collectable 2
		collectable2_mov = 0x00

		;; Movement delay for collectable 2
		collectable2_del = 0x05


	;; Collectable 3
		;; Position of collectable 3 in the screen
		collectable3_x = 0
		collectable3_y = 0

		;; Position of collectable 3 on the matrix
		collectable3_mx = 0x06
		collectable3_my = 0x07

		;; Initial direction of collectable 3
		collectable3_dir = 0

		;; Initial velocity of the collectable 3
		collectable3_vx = 0x00
		collectable3_vy = 0xFC

		;; Movement delay for collectable 3
		collectable3_mov = 0x03

		;; Movement delay for collectable 3
		collectable3_del = 0x05


	;; Collectable 4
		;; Position of collectable 4 in the screen
		collectable4_x = 0
		collectable4_y = 0

		;; Position of collectable 4 on the matrix
		collectable4_mx = 0x07
		collectable4_my = 0x08

		;; Initial velocity of the collectable 4
		collectable4_vx = 0x00
		collectable4_vy = 0x00

		;; Movement delay for collectable 4
		collectable4_mov = 0x03

		;; Movement delay for collectable 4
		collectable4_del = 0x02



;; Globals
	
	;; Global methods
	.globl col_clear
	.globl col_update
	.globl col_draw
	.globl col_new
	.globl col_remove
	.globl col_doForAllBaseMethods
	.globl col_doForAll
	.globl exchange_collectables
	.globl col_findInCollWithMcoords
	.globl col_initial_drawing
	.globl removed_by_player

	.globl col1_update
	.globl collectable1_mov

	;; Global pointers
	.globl default_coll_1
	.globl default_coll_2
	.globl default_coll_3
	.globl default_coll_4
	

	.globl coll_array
	.globl aux_coll_array1
	.globl aux_coll_array2
	.globl aux_coll_array3
	.globl last_collectable_pointer
	.globl last_collectable_pointer1
	.globl last_collectable_pointer2
	.globl last_collectable_pointer3

	;; Global variables
	.globl num_coll
	.globl aux_num_coll1
	.globl aux_num_coll2
	.globl aux_num_coll3
	.globl current_collectable
	.globl collectables_left
	.globl hud_x
	.globl collectables1_on_level
	.globl collectables2_on_level
	.globl collectables3_on_level
	.globl collectables4_on_level
	

	;; Global constants
	.globl coll_size
	.globl max_num_collectables
	.globl coll_array_complete
	.globl coll_array_size

