;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Harvest Day: An Amstrad CPC Game 
;;  Copyright (C) 2018 Álvaro Molina García / José Manuel Ladrón de Guevara García, Daniel Saura Martínez
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
;; Includes

	.include "entity.h.s"
	.include "collectable.h.s"
	.include "section.h.s"
	.include "player.h.s"
	.include "levelmanager.h.s"
	.include "utils.h.s"
	.include "menu.h.s"
	.include "sprites.h.s"

;; Declarations


	;;DEFAULT ITEMS
	DefineDefaultCollectable1 default_coll_1
	DefineDefaultCollectable2 default_coll_2
	DefineDefaultCollectable3 default_coll_3
	DefineDefaultCollectable4 default_coll_4

	;; We define the first collectable static 

	;; Array of active collectable objects
	coll_array:
	DefineNCollectables coll_array, max_num_collectables
	coll_array_size = . - coll_array

	num_coll: .db #00	
	last_collectable_pointer: .dw coll_array
	coll_array_complete = . - coll_array

	;; Array of inactive collectable objects
	aux_coll_array1:
	DefineNCollectables aux_coll_array1, max_num_collectables
	aux_num_coll1: .db #00	
	last_collectable_pointer1: .dw aux_coll_array1

	aux_coll_array2:
	DefineNCollectables aux_coll_array2, max_num_collectables
	aux_num_coll2: .db #00	
	last_collectable_pointer2: .dw aux_coll_array2

	aux_coll_array3:
	DefineNCollectables aux_coll_array3, max_num_collectables
	aux_num_coll3: .db #00	
	last_collectable_pointer3: .dw aux_coll_array3


	collectables1_on_level: .db 00
	collectables2_on_level: .db 00
	collectables3_on_level: .db 00
	collectables4_on_level: .db 00
	
	max_num_collectables =  20

	current_collectable: .db 00
	collectables_left: .db 00

	hud_x: .db #hud_startx



;; Public methods

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Call clear, uptdate and draw for all items in IX array
;; Destroys: BC, HL, DE
;; Input: 
;;	IX -> Pointer to first item of array
;;	A  -> Size of array
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
col_doForAllBaseMethods:
	;; Checks if array is empty
	ex af, af'
	ld a, (collectables_left)
	cp #0
	jp z, exchange_collectables

	ex af, af'

	cp #0
	ret z
	
	coll_base_loop:

		push af
		
		ld a, c_blocked(ix)
		cp #1
		jr z, skip_do


		call col_clear
		call col_update
		call col_draw

		skip_do:
		pop af


		ld bc, #coll_size
		add ix, bc

		dec a
		jr nz, coll_base_loop


	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Call method transfered through hl
;; Destroys: BC, HL, DE
;; Input: 
;;	IX -> Pointer to first item of array
;;  A  -> Size of array
;;  HL -> Function that executes all the collectables of the array
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
col_doForAll:
	

	;; Checks if array is empty
	cp #0
	ret z
	
	coll_loop:

		push af
		push hl
		
		ld de, #coll_d4_return 
		push de
		jp (hl)

	coll_d4_return:

		pop hl
		pop af


		ld bc, #coll_size
		add ix, bc

		dec a
		jr nz, coll_loop


	ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Creates a new collectable into array
;; Destroys: AF, BC, HL
;; Input: 
;;	BC -> Pointer to num_coll choosed array
;;	HL -> Pointer to last_collectable choosed array
;; Output: 
;;	IX -> Pointer to new collectable
;;	HL -> Pointer to new collectable
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
col_new:
	;; Checks de limit of the array
	ld a, (bc)
	cp #max_num_collectables
	jr z, outbound_coll_new

		;; Increments the collectable counter	
		inc a
		ld (bc), a


		;;Skips if num_coll is 0
		cp #1
		jp z, skip_first_coll

			ld bc, #coll_size
			add hl, bc
			
		
		skip_first_coll:

		;;Stores into ix the pointer of the new collectable
		push hl
		pop ix

	outbound_coll_new:

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Remove actual collectable and copies the last item in that position
;; Destroys: A, BC, DE, HL
;; Input: 
;;	IX -> Position in vector
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
col_remove:
	;;Check if the list of objects is empty 
	ld a, #num_coll
	cp #0
	jr z, skip_col_remove

		push ix		;; Save ix in de
		pop de

		or a
		ld hl, (last_collectable_pointer)	;;Save the last position in HL
		sbc hl, de			;;Check if it is the last component of the list and jump to copy if that is the case
		ld hl, (last_collectable_pointer)	
		jr z, saltar_coll_copy

		ld bc, #coll_size
		ldir				;;Copy the last object of the list
						;;in the position of the object that is going to be erased
		saltar_coll_copy:

			ld bc, #-coll_size
			ld hl, (last_collectable_pointer)
			add hl, bc 
			ld (last_collectable_pointer), hl

			ld a, (num_coll)
			dec a
			ld (num_coll), a

		ret

	skip_col_remove:

	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Clears the collectable if it's not removed
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
col_clear:

	;; Collectable 1 and 2 doesn't need clearing

	ld a, (current_collectable)
	cp #collectable2
	ret m 


	call ent_clearEntity

	ld b, e_ma_x(ix)
	ld c, e_ma_y(ix)
	push bc

	call sec_retValue

	ld a, e
	cp #obstacle_default
	jp m, keep_on_cleaning
	cp #wall
	ret z

		ld b, #obstacle_default
		sub b

		ld b, e_ma_x(ix)
		ld c, e_ma_y(ix)
		ld d, a

		push bc
		call sec_setValue
		pop bc

		call ent_createObstacle


	keep_on_cleaning:
		pop de
		push ix
		call col_findInCollWithMcoords
		push ix
		pop iy
		pop ix

		ret nc

		push ix
		push iy
		pop ix
		call ent_drawEntity

		pop ix



ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Updates the collectable
;; Destroys: AF, BC, DE, HL
;; Input: 
;;	IX -> Pointer to collectable
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
col_update:
	;; Goes to the direction saved on "_upd" of the entity
	ld h, e_upd_h(ix)
	ld l, e_upd_l(ix)

jp (hl)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Draws the collectable
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
col_draw:

	;; Collectable 1 and 2 doesn't need drawing
	ld a, (current_collectable)
	cp #collectable2
	ret m 

	call ren_drawEntity

ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Draws the complete array of collectables for the first time
;; Destroys: 
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
col_initial_drawing:

	ld a, (num_coll)
	ld ix, #coll_array
	coll_base_loop_initial:

		push af
		
		ld a, c_blocked(ix)
		cp #1
		jr z, skip_draw_array_initial

		call ren_drawFrontEntity
		call ren_drawEntity

		skip_draw_array_initial:
		pop af


		ld bc, #coll_size
		add ix, bc

		dec a
		jr nz, coll_base_loop_initial

ret



;; Private methods


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Updates the collectable1
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

col1_update:
	call col_setCollectableOnMatrix

	ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Updates the collectable3
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

col3_update:
	call col_bounce
	call col_setCollectableOnMatrix

	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Updates the collectable4 that avoids the player
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

col4_update:
	call col_avoidPlayer
	call col_setCollectableOnMatrix

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Sets any collectable on the matrix
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


col_setCollectableOnMatrix:

	;; Load its position of the matrix and its value
	ld b, e_mx(ix)
	ld c, e_my(ix)
	ld d, e_type(ix)
	call sec_setValue

ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Avoid the player by the shortests axis 
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;; NOTE: Revise the method of absolute value
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
col_avoidPlayer:

		ld a, e_mov(ix)
		cp #0
		jp nz, still_moving


			ld e_vx(ix), #0
			ld e_vy(ix), #0

			;; This actualizes the anterior positions of the enemy, to realize a clear
			ld a, e_mx(ix)
			ld e_ma_x(ix), a
			ld a, e_my(ix)
			ld e_ma_y(ix), a

			ld e_mov(ix), #collectable4_mov


			;; In this part of the code, we check if the IA is on a border of the map and aligned with the player
			ld iy, #player1_data
			call ent_distanceXY
			ld a, c

			cp #0
			jr nz, skip_borders_y

				ld a, e_my(iy)
				cp #1
				jr nz, skip_border_up

					call checkDown
					cp #1
					jp z, ent_move

					ret

				skip_border_up:
				cp #18
				jr nz, skip_border_down

					call checkUp
					cp #1
					jp z, ent_move

					ret

				skip_border_down:

			skip_borders_y:

			call ent_distanceXY
			ld a, b

			cp #0
			jr nz, skip_borders_x

				ld a, e_mx(iy)
				cp #1
				jr nz, skip_border_left

					call checkRight
					cp #1
					jp z, ent_move

					ret

				skip_border_left:
				cp #11
				jr nz, skip_border_right

					call checkLeft
					cp #1
					jp z, ent_move

					ret

				skip_border_right:

			skip_borders_x:
			;; The checkings of the borders are done respect the player 1


			;; Check the distances on each axis between the player1 and the collectable
			call ent_distanceXY
			ld a, c

			;; A -> Y
			;; B -> X

			sub b
			jp m, checkY 

			checkX:

				call ent_distanceXY
				ld a, d
				cp #0
				jp m, go_right

					call checkLeft
					cp #-1
					jr z, checkY
					jp ent_move

				go_right:

					call checkRight
					cp #-1
					jr z, checkY
					jp ent_move

			checkY:

				call ent_distanceXY
				ld a, e
				cp #0
				jp m, go_down

					call checkUp
					cp #1
					jp z, ent_move
						;; If we can't move on Y we check one last time if we can move on X
						call ent_distanceXY
						ld a, d
						cp #0
						jp m, final_go_right_up

							call checkLeft
							cp #1
							jp z, ent_move
								;; We can't move towards the player on X nor Y
							ret

						final_go_right_up:

							call checkRight
							cp #1
							jp z, ent_move
								;; We can't move towards the player on X nor Y
							ret


				go_down:

					call checkDown
					cp #1
					jp z, ent_move
						;; If we can't move on Y we check one last time if we can move on X
						call ent_distanceXY
						ld a, d
						cp #0
						jp m, final_go_right_down

							call checkLeft
							cp #1
							jp z, ent_move
								;; We can't move towards the player on X nor Y
							ret

						final_go_right_down:

							call checkRight
							cp #1
							jp z, ent_move
								;; We can't move towards the player on X nor Y
							ret


		;; Control of the delay to realize a certain movement
		still_moving:

			ld a, e_del(ix)
			cp #0
			jp nz, stop_it	

				ld e_del(ix), #collectable4_del
				ld a, e_mov(ix)
				dec a

				cp #2 					;; When the enemy is moving, change the sprite
				ld b, #1
				call nz, cambiar_sprite

				ld e_mov(ix), a
				jp ent_move

			;; Control of the delay between movements
			stop_it:
				dec a
				ld e_del(ix), a
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Checkings of IA in all directions
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: A -> -1 If can't move or destroy, 0 If has destroyed && 1 If can move
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

checkUp:

	ld e_dir(ix), #0

	ld b, e_mx(ix)
	ld c, e_my(ix)
	dec c
	call sec_retValue
	ld a, e

	;; Check all the posible types that the collectable can find
	cp #freedom
	jp m, cant_move_up
	jr z, can_move_up
	cp #player1
	jp m, cant_move_up
	cp #obstacle_default
	jp m, can_move_up
		jr cant_move_up

	can_move_up:

		;; We clear the position on the matrix
		ld b, e_mx(ix)
		ld c, e_my(ix)
		ld d, #freedom
		call sec_setValue

		;; We set the future position as the enemy
		ld b, e_mx(ix)
		ld c, e_my(ix)
		dec c
		ld d, e_type(ix)
		call sec_setValue

		;; And then we change our values
		ld c, e_my(ix)
		dec c
		ld e_my(ix), c

		ld e_vy(ix), # -4
		ld a, #1
	ret

	cant_move_up:
		ld a, #-1
ret



checkRight:

	ld e_dir(ix), #1
	
	ld b, e_mx(ix)
	ld c, e_my(ix)
	inc b
	call sec_retValue
	ld a, e

	;; Check all the posible types that the collectable can find
	cp #freedom
	jp m, cant_move_right
	jr z, can_move_right
	cp #player1
	jp m, cant_move_right
	cp #obstacle_default
	jp m, can_move_right
		jr cant_move_right

	can_move_right:

		;; We clear the position on the matrix
		ld b, e_mx(ix)
		ld c, e_my(ix)
		ld d, #freedom
		call sec_setValue

		;; We set the future position as the enemy
		ld b, e_mx(ix)
		ld c, e_my(ix)
		inc b
		ld d, e_type(ix)
		call sec_setValue

		;; And then we change our values
		ld b, e_mx(ix)
		inc b
		ld e_mx(ix), b

		ld e_vx(ix), #01
		ld a, #01
	ret

	cant_move_right:
		ld a, #-1
ret



checkDown:

	ld e_dir(ix), #2

	ld b, e_mx(ix)
	ld c, e_my(ix)
	inc c
	call sec_retValue
	ld a, e

	;; Check all the posible types that the collectable can find
	cp #freedom
	jp m, cant_move_down
	jr z, can_move_down
	cp #player1
	jp m, cant_move_down
	cp #obstacle_default
	jp m, can_move_down
		jr cant_move_down

	can_move_down:

		;; We clear the position on the matrix
		ld b, e_mx(ix)
		ld c, e_my(ix)
		ld d, #freedom
		call sec_setValue

		;; We set the future position as the enemy
		ld b, e_mx(ix)
		ld c, e_my(ix)
		inc c
		ld d, e_type(ix)
		call sec_setValue

		;; And then we change our values
		ld c, e_my(ix)
		inc c
		ld e_my(ix), c

		ld e_vy(ix), #04
		ld a, #01
	ret

	cant_move_down:
		ld a, #-1
ret



checkLeft:

	ld e_dir(ix), #3

	ld b, e_mx(ix)
	ld c, e_my(ix)
	dec b
	call sec_retValue
	ld a, e

	;; Check all the posible types that the collectable can find
	cp #freedom
	jp m, cant_move_left
	jr z, can_move_left
	cp #player1
	jp m, cant_move_left
	cp #obstacle_default
	jp m, can_move_left
		jr cant_move_left

	can_move_left:

		;; We clear the position on the matrix
		ld b, e_mx(ix)
		ld c, e_my(ix)
		ld d, #freedom
		call sec_setValue

		;; We set the future position as the enemy
		ld b, e_mx(ix)
		ld c, e_my(ix)
		dec b
		ld d, e_type(ix)
		call sec_setValue

		;; And then we change our values
		ld b, e_mx(ix)
		dec b
		ld e_mx(ix), b


		ld e_vx(ix), #-1
		ld a, #01
	ret

	cant_move_left:
		ld a, #-1
ret





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Checks if infront of it there's something that can't move towards it and turns right
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

col_bounce:

	ld a, e_mov(ix)
	cp #0
	jp nz, collectable3_moving

		ld a, e_mx(ix)
		ld e_ma_x(ix), a
		ld a, e_my(ix)
		ld e_ma_y(ix), a

		;; We check the value of the obstacle infront of the collectable and save it on a
		ld d, #1
		call ent_setObstacleValues
		call sec_retValue
		ld a, e

		;; React to the value retorned
		;; Removed for optimization purposes
		cp #freedom 				;; A < 0 -> Out of bounds = bounce
		jp m, bounce
		jr z, do_not_bounce
		cp #player1 				;; Collision with the others collectables
		jp m, bounce
		cp #obstacle_default
		jp m, do_not_bounce 		;; Collision with the obstacles
			jr bounce

			bounce:

				ld a, e_dir(ix)
				cp #3
				jr z, set_velocity_up
				cp #0
				jr z, set_velocity_right
				cp #1
				jr z, set_velocity_down
				cp #2
				jr z, set_velocity_left

					ld e_vx(ix), #0
					ld e_vy(ix), #0
				ret

				set_velocity_up:
					ld e_dir(ix), #0
					ld e_vx(ix), #0
					ld e_vy(ix), #-4
				ret

				set_velocity_right:
					ld e_dir(ix), #1
					ld e_vx(ix), #1
					ld e_vy(ix), #0
				ret

				set_velocity_down:
					ld e_dir(ix), #2
					ld e_vx(ix), #0
					ld e_vy(ix), #4
				ret

				set_velocity_left:
					ld e_dir(ix), #3
					ld e_vx(ix), #-1
					ld e_vy(ix), #0
				ret

			ret

			do_not_bounce:


				ld e_mov(ix), #collectable3_mov

				;; Erase collectable of the actual position
				ld b, e_mx(ix)
				ld c, e_my(ix)
				ld d, #0
				call sec_setValue

				;; Set the collectable in the future position
				ld d, #1
				call ent_setObstacleValues
				ld e_mx(ix), b
				ld e_my(ix), c

				ld d, e_type(ix)
				call sec_setValue

			jp ent_move

		
		


	collectable3_moving:

		ld a, e_del(ix)
		cp #0
		jp nz, collectable3_delaying

			ld e_del(ix), #collectable3_del
			ld a, e_mov(ix)
			dec a

			cp #2 					;; When the enemy is moving, change the sprite
			ld b, #1
			call nz, cambiar_sprite

			ld e_mov(ix), a
			jp ent_move

		collectable3_delaying:
			dec a
			ld e_del(ix), a
		ret
ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Check if all collectables in array are blocked 
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
check_obstacled:
	ld a, (num_coll)
	ld ix, #coll_array

	;; Checks if array is empty
	cp #0
	ret z
	
	coll_check_loop:

		push af
		
		ld b, e_mx(ix)
		ld c, e_my(ix)

		call sec_retValue

		ld a, #obstacle_default

		cp (hl)
		jr nz, set_free
			ld a, e_block(ix)
			cp #1
			jr z, set_ocuppied
				jr return_to_check_loop


		set_ocuppied:
			ld c_blocked(ix), #1
			ld b, e_mx(ix)
			ld c, e_my(ix)
			ld d, a
			add a, #-obstacle_default
			call sec_setValue

			ld b, e_mx(ix)
			ld c, e_my(ix)
			call ent_createObstacle
		jr return_to_check_loop


		set_free:
			ld c_blocked(ix), #0
		jr return_to_check_loop



		return_to_check_loop:
		pop af


		ld bc, #coll_size
		add ix, bc

		dec a
		jr nz, coll_check_loop

	ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Exchange the collectable type 
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
exchange_collectables:
	;; Reads the current collectable number
	ld a, (current_collectable)
	inc a

	cp #4
	jr nz, no_end_level
		call next_unlock
		jp menu_level_over
		ret
	no_end_level:



	;; This gets the size of complete array
	;;  including num and pointer
	ld hl, #coll_array_complete
	ld de, #coll_array_complete

	;; Analyzes the current kind of collectables
	cp #1
	jr nz, skip_ex1
		;; Copies the size into bc
		ld b, h
		ld c, l

		;; Puts in hl the pointer of the array
		ld hl, (last_collectable_pointer1)
		sbc hl, bc
		;; Substracts the size of the pointer to get the
		;;  position of the new pointer


		;; Sets the new pointer
		ld (last_collectable_pointer), hl

		;; Finally, puts in hl the pointer to array
		ld hl, #aux_coll_array1

		jr skip_ex3
	skip_ex1:

	;; Adds the size again to calculate the new size to substract
	add hl, de

	cp #2
	jr nz, skip_ex2
		ld b, h
		ld c, l

		ld hl, (last_collectable_pointer2)
		sbc hl, bc
		ld (last_collectable_pointer), hl
		ld hl, #aux_coll_array2


		jr skip_ex3
	skip_ex2:

	add hl, de

	cp #3
	jr nz, skip_ex3
		ld b, h
		ld c, l

		ld hl, (last_collectable_pointer3)
		sbc hl, bc
		ld (last_collectable_pointer), hl
		ld hl, #aux_coll_array3

		jr skip_ex3
	skip_ex3:

	;; Copies into coll_array the choosed array
	ld de, #coll_array
	ld bc, #coll_array_size+1
	ldir

	ld (current_collectable), a


	dec a
	;; In here we've picked up all the current collectables of the level, so we update the HUD
	call col_updateHUD


	ld a, (num_coll)
	cp #0
	jr z, skip_initial_drawing
		;; Draws all the collectables at the init of our level
		call col_initial_drawing
		
		call check_obstacled
	skip_initial_drawing:


	;; Sets the new value of collectables left
	ld a, (num_coll)
	ld (collectables_left), a


	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Destroyes the collectable
;; Destroys: AF, BC, DE, HL
;; Input: 
;;	IX -> Pointer to collectable into array
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
removed_by_player:
	ld a, (collectables_left)
	dec a
	ld (collectables_left), a

	;; We remove it out of the screen
	ld b, e_ma_x(ix)
	ld c, e_ma_y(ix)
	call sec_matrixToScreen
	call ren_clearObstacle

	ld b, e_mx(ix)
	ld c, e_my(ix)
	call sec_matrixToScreen
	call ren_clearObstacle

	;; We set to 0 the value of its position on the matrix, just in case
	ld b, e_mx(ix)
	ld c, e_my(ix)
	ld d, #freedom

	call sec_setValue

	;; Finally we set it to be removed
	jp col_remove




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Returns the entity which is in the coordinates
;; Destroys: AF, BC
;; Input: 
;;	DE -> Matrix coordinates
;; Output:
;;	IX -> Pointer to the object
;;	Cfg-> Set if there's a coincidence
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
col_findInCollWithMcoords:
	ld a, (num_coll)
	ld ix, #coll_array
	
	coll_find_loop:
		push af


		ld a, e_mx(ix)
		cp d
		jr nz, skip_find_y
			ld a, e_my(ix)
			cp e
			jr z, finded


		skip_find_y:
		ld bc, #coll_size
		add ix, bc


		pop af
		dec a
		jr nz, coll_find_loop

	not_finded:
		or a
	ret

	finded:
		pop af
		scf
	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Updates the HUD
;; Destroys: AF, BC, DE, HL
;; Input:
;;	A -> Current collectable
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
col_updateHUD:

	;; Now we have to set the IY on the array of the current collectable
	ld bc, #coll_array_complete
	ld hl, #_collectables_hud_sp_0
	ld de, #collectables1_on_level

	cp #0
	jr z, no_iteration
	hud_iteration:


		push de
		;; Search for the current sprite of the collectable on the HUD
		ld de, #sprite_size
		add hl, de

		pop de
		inc de

		dec a
	jr nz, hud_iteration

	no_iteration:

	;; Now we have the HUD collectable setted on HL.

	;; Now we check if the array had intially a collectable, if it doesn't we don't continue

	ld a, (de)
	cp #0
	ret z


	;; Set and draw the collectable selected on the array

		;; Here we add sprite_size*4 to hl to obtain the gray sprite of the collectable.
		ld de, #sprite_size + #sprite_size + #sprite_size + #sprite_size
		add hl, de

		ld a, (hud_x)
		ld d, a
		ld e, #hud_starty
		ld b, #col_h
		ld c, #col_w
		ld a, #1
		push bc
		push de
		push hl
		call ren_drawBackObstacle

		pop hl
		pop de
		pop bc
		ld a, #1
		call ren_drawFrontObstacle


		;; Now we increment the positioning in X of the HUD
		ld a, (hud_x)
		add #col_w + #sprite_offsetx
		ld (hud_x), a

ret