;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Harvest Day: An Amstrad CPC Game 
;;  Copyright (C) 2018 Álvaro Molina García / José Manuel Ladrón de Guevara García, Daniel Saura Martínez
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
;; Includes
	
	.include "render.h.s"


;; Macros

	.macro DefineEntity _name, _x, _y, _ma_x, _ma_y, _w, _h, _sprite, _mx, _my, _dir, _upd, _vx, _vy, _mov, _del, _block, _op, _dist, _type
	_name:
		DefineDrawableEntity _name'_dw, _x, _y, _w, _h, _sprite
			.db _ma_x, _ma_y 		;; Anterior position of the entity
			.db _mx, _my 			;; Position on the matrix
			.db _dir 				;; Direction where it's looking
			.dw _upd 				;; Method that calls on update
			.db _vx, _vy 			;; Velocity on X and Y
			.db _mov 				;; How many times it moves after choosing where it moves
			.db _del 				;; Delay when it moves
			.db _block 				;; Delay produced with the create or destroy of an obstacle
			.db _op 				;; Boolean that specifies if a player it's creating(O) or destroying(P)
			.db _dist 				;; Counter of obstacles that the entity has created in a row
			.db _type 				;; Expresses which type it's the entity to represent it on the matrix
		_name'_size = . - _name 	;; Saves the number of bytes that fills the Entity
	.endm

	.macro DefineChasing _name
		DefineEntity _name, enemy_initial_x, enemy_initial_y, enemy_initial_mx, enemy_initial_my, enemy_w, enemy_h, _enemy_sp_0, enemy_initial_mx, enemy_initial_my, enemy_dir, #ent_chasingUpdate, enemy_vx, enemy_vy, enemy_mov, enemy_chasing_del, enemy_block, enemy_op, enemy_chase_dist, enemy_chasing
	.endm
	.macro DefineBouncing _name
		DefineEntity _name, enemy_initial_x, enemy_initial_y, enemy_initial_mx, enemy_initial_my, enemy_w, enemy_h, _enemy3_sp_0, enemy_initial_mx, enemy_initial_my, enemy_dir, #ent_bouncingUpdate, enemy_bouncing_vx, enemy_bouncing_vy, 0x00, enemy_bouncing_del, enemy_block, enemy_op, enemy_bouncing_dist, enemy_bouncing
	.endm
	.macro DefineBreaking _name
		DefineEntity _name, enemy_initial_x, enemy_initial_y, enemy_initial_mx, enemy_initial_my, enemy_w, enemy_h, _enemy2_sp_00, enemy_initial_mx, enemy_initial_my, enemy_dir, #ent_breakerUpdate, enemy_vx, enemy_vy, enemy_mov, enemy_breaker_del, enemy_block, enemy_op, enemy_break_dist, enemy_breaker
	.endm

	;; This defines a collectable, with only the essentials parameters
	.macro DefineDefaultEnemy _name, _suf
		DefineEntity _name'_suf, enemy_initial_x, enemy_initial_y, enemy_initial_mx, enemy_initial_my, enemy_w, enemy_h, _enemy_sp_0, enemy_initial_mx, enemy_initial_my, enemy_dir, #ent_chasingUpdate, enemy_vx, enemy_vy, enemy_mov, enemy_chasing_del, enemy_block, enemy_op, enemy_chase_dist, enemy_chasing
	.endm

	.macro DefineNEnemies _name, _n
		_c = 0
		.rept _n
			DefineDefaultEnemy _name, \_c
			_c = _c + 1
		.endm
	.endm


	

;; Macro positions

	e_dw = 0
	e_dw_size = 6
	e_ma_x = 0 + e_dw_size
	e_ma_y = 1 + e_dw_size
	e_mx = 2 + e_dw_size
	e_my = 3 + e_dw_size
	e_dir = 4 + e_dw_size
	e_upd_l = 5 + e_dw_size
	e_upd_h = 6 + e_dw_size
	e_vx = 7 + e_dw_size
	e_vy = 8 + e_dw_size
	e_mov = 9 + e_dw_size
	e_del = 10 + e_dw_size
	e_block = 11 + e_dw_size
	e_op = 12 + e_dw_size
	e_dist = 13 + e_dw_size
	e_type = 14 + e_dw_size

	e_size = 15 + e_dw_size

;; Constants


	;; Enemy constants
		;; Screen initial positions of the enemy
		enemy_initial_x = 0x10
		enemy_initial_y = 0x00

		;; Size of the enemy
		enemy_w = 0x04
		enemy_h = 0x10

		;; Matrix initial positions of the enemy
		enemy_initial_mx = -1
		enemy_initial_my = 0

		;; Initial direction of the enemy
		enemy_dir = 1 	;; Properties: 0 = Up, 1 = Right, 2 = Down, 3 = Left

		;; Initial velocity of the enemy
		enemy_vx = 0
		enemy_vy = 0

		;; Initial velocity of the enemy enemy bouncing
		enemy_bouncing_vx = 1
		enemy_bouncing_vy = 0

		;; Times it truly moves each time press a movable key
		enemy_mov = 3

		;; Delay
			;; Number of iterations between stop and moving again of enemy chasing
			enemy_chasing_del = 5

			;; Number of iterations between stop and moving again of enemy chasing
			enemy_bouncing_del = 5

			;; Number of iterations between stop and moving again of enemy breaker
			enemy_breaker_del = 4


		;; Delay creating or destroying blocks
		enemy_block = 15

		;; Value of creating, destroying or neither of them
		enemy_op = 0

		;; How many obstacles can the enemy bounce break
		enemy_bouncing_dist = 0

		;; How many obstacles can the enemy chase break
		enemy_chase_dist = 1

		;; How many obstacles can break the enemy break
		enemy_break_dist = 20


		;; Time that spends an obstacle to break or create
		obstacle_bc = #1

	;; Types of entity

		;; Free position
			freedom = 0 			;; Properties:	   Movable(Player),     Movable(IA),     Construible, NOT Destroyable


		;; Collectables (the diference resides on the punctuation that adds to the player that picks it up)
			collectable1 = 1 		;; Properties:	   Movable(Player),     Movable(IA),     Construible, NOT Destroyable
			collectable2 = 2 		;; Properties:	   Movable(Player),     Movable(IA),     Construible, NOT Destroyable
			collectable3 = 3 		;; Properties:	   Movable(Player),     Movable(IA),     Construible, NOT Destroyable
			collectable4 = 4 		;; Properties:	   Movable(Player),     Movable(IA),     Construible, NOT Destroyable


		;; Players
			player1 = 5 			;; Properties: NOT Movable(Player),     Movable(IA), NOT Construible, NOT Destroyable
			player2 = 6 			;; Properties: NOT Movable(Player),     Movable(IA), NOT Construible, NOT Destroyable


		;; Enemies (diferent enemies based on how they act)

			;; This enemy bounces when it collides (bounce = turn right)
			enemy_bouncing = 7 		;; Properties:     Movable(Player), NOT Movable(IA), NOT Construible, NOT Destroyable

			;; This enemy chases the player/s with the hability to break 1 obstacle at a time
			enemy_chasing = 8 		;; Properties:     Movable(Player), NOT Movable(IA), NOT Construible, NOT Destroyable

			;; This enemy chases the player/s with the hability to break X obstacles at a time
			enemy_breaker = 9 		;; Properties:     Movable(Player), NOT Movable(IA), NOT Construible, NOT Destroyable


		;; Obstacles (diferent obstacles based on what collectable is inside)
			obstacle_default = 10 	;; Properties: NOT Movable(Player), NOT Movable(IA), NOT Construible,     Destroyable

			;; Contains collectable 1
			obstacle_collectable1=11;; Properties: NOT Movable(Player), NOT Movable(IA), NOT Construible,     Destroyable

			;; Contains collectable 2
			obstacle_collectable2=12;; Properties: NOT Movable(Player), NOT Movable(IA), NOT Construible,     Destroyable

			;; Contains collectable 3
			obstacle_collectable3=13;; Properties: NOT Movable(Player), NOT Movable(IA), NOT Construible,     Destroyable

			;; Contains collectable 4
			obstacle_collectable4=14;; Properties: NOT Movable(Player), NOT Movable(IA), NOT Construible,     Destroyable


		;; Walls
			wall = 15 				;; Properties: NOT Movable(Player), NOT Movable(IA), NOT Construible, NOT Destroyable



;; Globals
	
	;; Global methods
		.globl ent_clearEntity
		.globl ent_clearEntityEnemy
		.globl ent_updateEntity
		.globl ent_drawEntity
		.globl ent_move
		.globl ent_setEntityOnMatrix
		.globl ent_setObstacleValues
		.globl ent_destroyObstacle
		.globl ent_destroyRowObstacle
		.globl ent_doForAllBaseMethods
		.globl ent_doForAll
		.globl ent_new
		.globl ent_distanceXY
		.globl ent_setVelocityFdir
		.globl ent_hitboxed
		.globl ent_findInEntWithMcoords


	;; Global pointers
		.globl enemy_array
		.globl last_enemy_pointer
		.globl default_chasing
		.globl default_bouncing
		.globl default_breaking
		.globl cambiar_sprite
		.globl sprite_dir



	;; Global variables
		.globl e_size
		.globl max_num_enemies
		.globl num_enemies

	;; Global constants
		.globl enemy_array_size
		.globl e_size


