;; Includes

	.include "cpctelera.h.s"
	.include "main.h.s"
	.include "entity.h.s"
	.include "player.h.s"
	.include "section.h.s"
	.include "utils.h.s"
	.include "collectable.h.s"
	.include "sprites.h.s"

;; Declarations

	;enemy_chase1_data:
			;; _name, _x, _y,  _x-1, _y-1,  _x-2, _y-2, _w, _h, _sprite, _mx, _my, _dir, _upd, _vx, _vy, _mov, _del, _block, _op, _dist, _type
	;DefineEntity enemy_chase_1, enemy_initial_x, enemy_initial_y, enemy_initial_mx, enemy_initial_my, enemy_w, enemy_h, _enemy_sp_0, enemy_initial_mx, enemy_initial_my, enemy_dir, #ent_chasingUpdate, enemy_vx, enemy_vy, enemy_mov, enemy_chasing_del, enemy_block, enemy_op, enemy_chase_dist, enemy_chasing

	DefineChasing default_chasing
	DefineBouncing default_bouncing
	DefineBreaking default_breaking


	enemy_array:
	DefineNEnemies enemy_array, max_num_enemies
	enemy_array_size = . - enemy_array

	max_num_enemies = 10
	num_enemies: .db 00
	last_enemy_pointer: .dw enemy_array

;; Public methods

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Call clear, uptdate and draw for all items in IX array
;; Destroys: BC, HL, DE
;; Input: 
;;	IX -> Pointer to first item of array
;;WIP	A  -> Size of array
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_doForAllBaseMethods:
	ld a, (num_enemies)

	;; Checks if array is empty
	cp #0
	ret z
	
	ent_base_loop:

	push af
	call ent_clearEntityEnemy
	call ent_updateEntity
	call ent_drawEntity
	pop af


	ld bc, #e_size
	add ix, bc

	dec a
	jr nz, ent_base_loop


	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Call method transfered through hl
;; Destroys: BC, HL, DE
;; Input: 
;;	IX -> Pointer to first item of array
;;WIP	A  -> Size of array
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_doForAll:
	ld a, (num_enemies)

	;; Checks if array is empty
	cp #0
	ret z
	
	ent_loop:

	push af
	push hl
	
	ld de, #ent_d4_return 
	push de
	jp (hl)

	ent_d4_return:

	pop hl
	pop af


	ld bc, #e_size
	add ix, bc

	dec a
	jr nz, ent_loop


	ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Creates a new entity into array
;; Destroys: AF, BC, HL
;; Input: VOID
;; Output: IX -> Pointer to new collectable
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_new:
	;; Checks de limit of the array
	ld a, (num_enemies)
	cp #max_num_enemies
	jr z, outbound_ent_new

		;; Increments the collectable counter	
		inc a
		ld (num_enemies), a


		;; Puts in hl the last collectable pointer 
		ld hl, (last_enemy_pointer)

		;;Skips if num_coll is 0
		cp #1
		jp z, skip_first_ent

			ld bc, #e_size
			add hl, bc
			ld (last_enemy_pointer), hl
			
		
		skip_first_ent:

		;;Stores into ix the pointer of the new collectable
		push hl
		pop ix

	outbound_ent_new:

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Remove actual entity and copies the last item in that position
;; Destroys: A, BC, DE, HL
;; Input: 
;;	IX -> Position in vector
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_remove:
;;ARREGLAR Traducir
	;;Comprueba si la lista de objetos esta vacía y llama a lo que le corresponda
	ld a, #num_enemies
	cp #0
	jr z, a_lo_que_tenga_que_llamar

		push ix		;;Guarda IX en DE
		pop de

		or a
		ld hl, (last_enemy_pointer)	;;Guarda la última posición en HL
		sbc hl, de			;;Comprueba si es el último de la lista y salta la copia si es así
		ld hl, (last_enemy_pointer)	
		jr z, saltar_ent_copy

		ld bc, #e_size
		ldir				;;Copia el último objeto de la lista
						;;en la posición del que se va a borrar
		saltar_ent_copy:

		ld bc, #-e_size
		ld hl, (last_enemy_pointer)
		add hl, bc ;;ARREGLAR Probar a poner sbc
		ld (last_enemy_pointer), hl

		ld a, (num_enemies)
		dec a
		ld (num_enemies), a

		ret

	a_lo_que_tenga_que_llamar:

	;; ARREGLAR Llamar al gestor de frutas

	ret




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Clears the entity
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output:
;;	B -> ma_x
;;	C -> ma_y
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_clearEntity:


	ld c, e_ma_y(ix)
	ld b, e_ma_x(ix)
	push bc
	call sec_matrixToScreen
	ld c, d 
	ld b, e
	ld a, #1
	call ren_clearAnteriorAnteriorEntity

	ld c, e_my(ix)
	ld b, e_mx(ix)
	call sec_matrixToScreen
	ld c, d 
	ld b, e
	ld a, #1
	call ren_clearAnteriorAnteriorEntity
	pop bc

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Clears an enemy Entity
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_clearEntityEnemy:


	call ent_clearEntity

	;; If behind the enemy is an obstacle, the we draw the obstacle
	push bc
	call sec_retValue
	pop bc
	ld a, e
	cp #obstacle_default
	jp m, check_collectable
	cp #wall
	ret z

		ld b, #obstacle_default
		sub b

		ld b, e_ma_x(ix)
		ld c, e_ma_y(ix)
		ld d, a

		push bc
		call sec_setValue
		pop bc

		call ent_createObstacle
	ret

	check_collectable:
	push ix
	ld d, b
	ld e, c
	call col_findInCollWithMcoords


	jr nc, jump_drawCol
		call ent_drawEntity

	jump_drawCol:

	pop ix


ret




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Updates the entity
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_updateEntity:

	;; Goes to the direction saved on "_upd" of the entity
	ld h, e_upd_h(ix)
	ld l, e_upd_l(ix)

jp (hl)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Draws the entity
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_drawEntity:

	call ren_drawEntity

ret




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Moves the entity
;; Destroys: 
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ent_move:

	;; Set the future to actual
	;; We set the entity screen values with the velocity
	move_on_x:
		ld a, de_x(ix)
		add e_vx(ix)
		ld de_x(ix), a

	move_on_y:
		ld a, de_y(ix)
		add e_vy(ix)
		ld de_y(ix), a

ret



;; MATRIX RELATED FUNCTIONS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Sets the actual position of the entity on the matrix
;; Destroys: A, BC, DE, HL
;; Input: VOID
;; Output: E =-1 -> Error: IndexMatrixOutofBounds 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ent_setEntityOnMatrix:

	;; Save on D the entity's type value for the matrix
	ld a, e_type(ix)
	ld d, a 

	;; Save on B and C the Column and Row of the entity
	ld b, e_mx(ix)
	ld c, e_my(ix)

	call sec_setValue

ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Sets the position 1 block towards the entity's direction 
;; Destroys:
;; Input: D -> How many blocks in that direction
;; Output:
;; B -> Future X matrix
;; C -> Future Y matrix
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ent_setObstacleValues:

	;; We set the entity value
	ld b, e_mx(ix)
	ld c, e_my(ix)

	;; Check where it's looking
	ld a, e_dir(ix)
	cp #0
	jr z, obstacleSetUp
	cp #1
	jr z, obstacleSetRight
	cp #2
	jr z, obstacleSetDown
	cp #3
	jr z, obstacleSetLeft

	;; Set the future position depending the direction(e_dir(ix)) and the distance(D)
	obstacleSetUp:
		ld a, d
		up_loop:
			cp #0
			ret z
			dec c
			dec a
		jr up_loop
	ret

	obstacleSetRight:
		ld a, d
		right_loop:
			cp #0
			ret z
			inc b
			dec a
		jr right_loop
	ret

	obstacleSetDown:
		ld a, d
		down_loop:
			cp #0
			ret z
			inc c
			dec a
		jr down_loop
	ret

	obstacleSetLeft:
		ld a, d
		left_loop:
			cp #0
			ret z
			dec b
			dec a
		jr left_loop
	ret

ret




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Destroys an obstacle on the matrix and clears it
;; Destroys: AF, BC, DE, HL
;; Input: 
;; B -> X column 
;; C -> Y row
;; Output: A = -1 -> Error: There's no obstacle to destroy
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_destroyObstacle:

	;; We save on the stack the BC value to use it later to draw the obstacles
	push bc

	;; Get the matrix value of that position
	call sec_retValue

	pop bc

	;; If it's constructable then proceed
	ld a, e
	cp #obstacle_default
	jp m, cant_be_destroyed
		cp #wall
		jr nz, can_be_destroyed

	cant_be_destroyed:
		ld a, #-1
	ret




	can_be_destroyed:
	;; Set the value of the position to correct obstacle value
	ld a, (hl)
	add a, #-obstacle_default
	ld (hl), a

	;ld a, #obstacle_default

	push ix
	push bc


	;; Now we'll convert the position on the matrix to the position on the screen
	call sec_matrixToScreen

	;; Here we'll clear the obstacle on the 2 buffers

	push de
	call ren_clearObstacle


	pop de
	call ren_clearObstacle


	cp #freedom
	jr z, skip_remove_col_destroying
		pop de
		push de		
		call col_findInCollWithMcoords
		jr nc, skip_remove_col_destroying
		ld c_blocked(ix), #0

		pop bc
		push bc
		ld e_mx(ix), b
		ld e_my(ix), c
		ld e_ma_x(ix), b
		ld e_ma_y(ix), c

		call sec_matrixToScreen

		ld de_x(ix), d
		ld de_y(ix), e

		ld e_mov(ix), #0

		ld a, e_type(ix) 		;; Load the type of collectable in a to check which sprite we should put
		sub #3
		jr nz, no_col1

			ld hl, #_col3_sp_1
			ld de_sp_h(ix), h ;;put in the entity the new sprite
			ld de_sp_l(ix), l
			jr no_col2

		no_col1:
		dec a 
		jr nz, no_col2

			ld hl, #_col4_sp_1
			ld de_sp_h(ix), h ;;put in the entity the new sprite
			ld de_sp_l(ix), l
			jr no_col2

		no_col2:





		call ent_drawEntity
		call ren_drawFrontEntity

	skip_remove_col_destroying:
	pop bc
	pop ix



	;; This assures that we removed the obstacle correctly
	ld a, #0

ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Creates a row of blocks that ends when it can't create more  
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ent_destroyRowObstacle:

	;; First we set the distances to the entities that can break
	
	;; We comprobe if its an enemy and if that's so, then we set his distance

	ld a, e_type(ix)
	cp #enemy_chasing
	jr z, set_enemy_chasing
	cp #enemy_breaker
	jr z, set_enemy_breaker
	cp #enemy_bouncing
	jr z, set_enemy_bouncing

		;; On this zone we arrived at the conclusion that is a player
		jr player_distance

	;; Set the distnace of the especific type of enemy
	set_enemy_bouncing:
		ld b, #enemy_bouncing_dist
	jr set_enemy_distance

	set_enemy_chasing:
		ld b, #enemy_chase_dist
	jr set_enemy_distance

	set_enemy_breaker:
		ld b, #enemy_break_dist


	set_enemy_distance:

	;; If has arrived to the maximum then stop breaking
	ld a, e_dist(ix)
	cp b
	jr z, max_destroyed
		inc a
		ld e_dist(ix), a
		jr make_the_destruction_already


	player_distance:
	;; Increment the distance to destroy an obstacle at the next position (infinite for the player)
	ld a, e_dist(ix)
	inc a
	ld e_dist(ix), a

	;; Proceeds to destroy the respective obstacle
	make_the_destruction_already:
		ld d, a
		call ent_setObstacleValues 					;; Sets the values to destroy an obstacle
		call ent_destroyObstacle 					;; Destroys an obstacle

		cp #0
		ret z 
		;; We assume that the obstacle couldn't be destroyed

		max_destroyed:
			;; In this case we have to stop destroying blocks and reset the distance to 0
			ld e_dist(ix), #0
			ld e_op(ix), #0

ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Calculates the distance between a player and an enemy
;; Destroys: AF, BC, DE
;; Input: 
;; IX -> Enemy
;; IY -> Player
;; Output: 
;; B -> X absolute
;; C -> Y absolute
;; D -> X signed
;; E -> Y signed
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ent_distanceXY:
	;; Calculate the distance on X between the player and the enemy and save it on B
	ld a, e_mx(iy)
	ld b, e_mx(ix)
	sub b
	ld d, a
	call absolute_value
	ld b, a

	;; Calculate the distance on Y between the player and the enemy and save it on C
	ld a, e_my(iy)
	ld c, e_my(ix)
	sub c
	ld e, a
	call absolute_value
	ld c, a
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Returns the entity which is in the coordinates
;; Destroys: AF, BC
;; Input: 
;;	DE -> Matrix coordinates
;; Output:
;;	IX -> Pointer to the object
;;	Cfg-> Set if there's a coincidence
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_findInEntWithMcoords:
	ld a, (num_enemies)
	ld ix, #enemy_array

	cp #0
	jr z, not_finded
	
	ent_find_loop:
		push af


		ld a, e_mx(ix)
		cp d
		jr nz, skip_find_y
			ld a, e_my(ix)
			cp e
			jr z, finded


		skip_find_y:
		ld bc, #e_size
		add ix, bc


		pop af
		dec a
		jr nz, ent_find_loop

	not_finded:
		or a
	ret

	finded:
		pop af
		scf
	ret



;; Private methods

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Updates the enemy chasing  
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ent_chasingUpdate:

	;; Checks if it's creating or destroying, if that's so, then we don't need to do the rest of the update
	call ent_chasePlayer
	call ent_setEntityOnMatrix

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Updates the enemy bouncing
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ent_bouncingUpdate:

	call ent_bounce

ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Checks if infront of him there's something that can't move towards it and turns right
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ent_bounce:

	
	ld a, e_mov(ix)
	cp #0
	jp nz, enemy_moving

		call sprite_dir
		ld a, e_mx(ix)
		ld e_ma_x(ix), a
		ld a, e_my(ix)
		ld e_ma_y(ix), a

		;; We check the value of the obstacle infront of the enemy and save it on a
		ld d, #1
		call ent_setObstacleValues
		call sec_retValue
		ld a, e

		;; React to the value retorned
		cp #freedom
		jp m, bounce		
		cp #enemy_bouncing 			;; 0 <= A < 7 -> Free, Collectable or player = keep_straight 
		jp m, do_not_bounce

			bounce:
				ld a, e_dir(ix)
				inc a
				ld e_dir(ix), a
				cp #4
				jr nz, saltar_set0
					ld e_dir(ix), #0

				saltar_set0:
				jp  ent_setVelocityFdir
				;;Ret inside this function 


			do_not_bounce:


				ld e_mov(ix), #enemy_mov

				;; Borrar enemigo de la posición actual
				ld b, e_mx(ix)
				ld c, e_my(ix)
				ld e_ma_x(ix), b 
				ld e_ma_y(ix), c
				ld d, #0
				call sec_setValue

				;; Setear al enemigo en la posición futura
				ld d, #1
				call ent_setObstacleValues
				ld e_mx(ix), b
				ld e_my(ix), c


				ld d, e_type(ix)
				call sec_setValue
				call ent_clearEntity


			jp ent_move

		
		


	enemy_moving:

		ld a, e_del(ix)
		cp #0
		jr nz, enemy_delaying

			ld e_del(ix), #enemy_bouncing_del
			ld a, e_mov(ix)
			dec a

			cp #2 					;; When the enemy is moving, change the sprite
			ld b, #1
			call nz, cambiar_sprite

			ld e_mov(ix), a
			jp ent_move

		enemy_delaying:
			dec a
			ld e_del(ix), a
		ret
	




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Updates the enemy breaker
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ent_breakerUpdate:
	call ent_chasePlayer
ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Sets entitie's velocity each dir
;; Destroys: A
;; Input: 
;; IX -> Enemy
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ent_setVelocityFdir:
	ld a, e_dir(ix)

	cp #0
	jr z, set_velocity_up
	cp #1
	jr z, set_velocity_right
	cp #2
	jr z, set_velocity_down
	cp #3
	jr z, set_velocity_left

	set_velocity_up:
		ld e_vx(ix), #0
		ld e_vy(ix), #-4
	ret

	set_velocity_right:
		ld e_vx(ix), #1
		ld e_vy(ix), #0
	ret

	set_velocity_down:
		ld e_vx(ix), #0
		ld e_vy(ix), #4
	ret

	set_velocity_left:
		ld e_vx(ix), #-1
		ld e_vy(ix), #0
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Chases the player by the larger axis 
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;; NOTE: Revise the method of absolute value
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_chasePlayer:

		ld a, e_mov(ix)
		cp #0
		jp nz, still_moving

			ld a, e_op(ix)
			cp #0
			jp nz, breaking_bad




					;; This actualizes the anterior positions of the enemy, to realize a clear
					ld a, e_mx(ix)
					ld e_ma_x(ix), a
					ld a, e_my(ix)
					ld e_ma_y(ix), a

					;; Reset the delays of the enemy
					ld e_mov(ix), #enemy_mov


					ld e_vx(ix), #0
					ld e_vy(ix), #0

					;; Calculates the distances
					ld IY, #player1_data
					call ent_distanceXY
					ld a, c

					;; Calculate the major distance and go for it 
					;; A -> Y
					;; B -> X
					sub b
					jp p, checkY



					;; We have the preference on X axis

					;; Check the X axis, if we can move or destroy we do it, if we don't then we pass to checkY 
					checkX:

						call ent_distanceXY
						ld a, d
						cp #0
						jr z, checkY
						jp m, go_left

							;; The enemy decides to go right
							call checkRight
							jr final_check

						go_left:

							;; The enemy decides to go left
							call checkLeft
							jr final_check


					;; Check the Y axis, if we can move or destroy we do it, if we don't then we stop
					checkY:

						call ent_distanceXY
						ld a, e
						cp #0
						jp m, go_up
						ret z

							;; The enemy decides to go down
							call checkDown
							jr final_check

						go_up:

							;; The enemy decides to go up
							call checkUp
							jr final_check


					final_check:
						cp #0
						jr z, final_destroy
						cp #-1
						jr z, final_blocked

							;; We arrive here if we want to only move
							jp ent_move

					;; We arrive here if we decide to destroy the obstacle infront of us
					final_destroy:

						ld e_op(ix), #-1
						ld e_block(ix), #enemy_block

					ret


					;; We arrive here if infront of the enemy is something that can't move towards it or destroy it
					final_blocked:

						;; Here it should be some sort of code that allows the enemy to avoid that "OBSTACLE"

					ret

				

			;; Control of the delay of destroying obstacles
			breaking_bad:
				ld a, e_block(ix)
				cp #0
				jr nz, animate_destruction

					ld e_block(ix), #obstacle_bc
					ret

				animate_destruction:
				cp #animate_destruction
				
				dec a
				ld e_block(ix), a
				cp #0
				ret nz

				;; Here the enemy can finally break the obstacle
				jp z, ent_destroyRowObstacle

			ret

		;; Control of the delay to realize a certain movement
		still_moving:

			ld a, e_del(ix)
			cp #0
			jp nz, stop_it	

				ld e_del(ix), #enemy_chasing_del

				ld a, e_type(ix)
				cp #9
				jr nz, enemy_chasing_delay

				ld e_del(ix), #enemy_breaker_del

				enemy_chasing_delay:
				ld a, e_mov(ix)
				dec a
				cp #2 					;; When the enemy is moving, change the sprite
				ld b, #1
				call nz, cambiar_sprite
				
				ld e_mov(ix), a
				jp ent_move

			;; Control of the delay between movements
			stop_it:
				dec a
				ld e_del(ix), a
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Cheange the sprite's pointer of the entity
;; Destroys: DE, HL, B
;; Input: A -> 1 = ADD, !1 = SUB 
;;        B -> Distancia en sprites
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
cambiar_sprite:

	ld h, de_sp_h(ix)
	ld l, de_sp_l(ix) ;; load in hl the actual sprite
	cp #1
	jr nz, restar_sprite

		ld de, #64  ;; the amount of pixels of a sprite
		jr fin_sprite

	restar_sprite:
		ld de, #-64
	
	fin_sprite:
	
	bucle_sprite:
	add hl, de 		  ;; change in hl the sprite
	dec b 
	jr nz, bucle_sprite

	ld de_sp_h(ix), h ;;put in the entity the new sprite
	ld de_sp_l(ix), l

ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Change the sprite when the enemy change direction
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sprite_dir:
	ld a, e_type(ix)
	sub #6
	ld b, a  ;; 0 bouncing - 1 chasing - 2 breaker


	ld a, e_dir(ix)
	cp #0
	jr z, m_s_arr
	cp #1
	jr z, m_s_der
	cp #2
	jr z, m_s_aba

		;; m_s_izq
		dec b
		jr z, izq_bo

			;;izq_breaking
			dec b 
			jr z, izq_ch
				ld hl, #_enemy2_sp_09
				jr fin_sprite_dir

		izq_ch:
		ld hl, #_enemy_sp_6
		jr fin_sprite_dir

		izq_bo:
		ld hl, #_enemy3_sp_6
		jr fin_sprite_dir

	m_s_arr:
		dec b
		jr z, arr_bo

			dec b 
			jr z, arr_ch
				;;arr_breaking
				ld hl, #_enemy2_sp_03
				jr fin_sprite_dir

		arr_ch:
		ld hl, #_enemy_sp_2
		jr fin_sprite_dir		
		
		arr_bo:
		ld hl, #_enemy3_sp_2
		jr fin_sprite_dir	

	m_s_der:
		dec b
		jr z, der_bo

			dec b
			jr z, der_ch
				;;arr_breaking
				ld hl, #_enemy2_sp_06
				jr fin_sprite_dir

		der_ch:
		ld hl, #_enemy_sp_4
		jr fin_sprite_dir

		der_bo:
		ld hl, #_enemy3_sp_4
		jr fin_sprite_dir

	m_s_aba:
		dec b
		jr z, aba_bo

			dec b
			jr z, aba_ch
			;;arr_breaking
			ld hl, #_enemy2_sp_00
			jr fin_sprite_dir

		aba_ch:
		ld hl, #_enemy_sp_0
		jr fin_sprite_dir

		aba_bo:
		ld hl, #_enemy3_sp_0
		jr fin_sprite_dir


	fin_sprite_dir:

	ld de_sp_h(ix), h
	ld de_sp_l(ix), l


ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Checkings of IA in all directions
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: A -> -1 If can't move or destroy, 0 If has destroyed && 1 If can move
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

checkUp:

	ld e_dir(ix), #0

	ld b, e_mx(ix)
	ld c, e_my(ix)
	dec c
	call sec_retValue
	ld a, e

	;; Check all the posible types that the enemy can find
	cp #freedom
	jp m, not_move_destroy_up
	cp #enemy_bouncing
	jp m, can_move_up
	cp #obstacle_default
	jp m, not_move_destroy_up
	cp #wall
	jp m, can_destroy_up
		jr not_move_destroy_up

	can_move_up:

		;; We clear the position on the matrix
		ld b, e_mx(ix)
		ld c, e_my(ix)
		ld d, #freedom
		call sec_setValue

		;; We set the future position as the enemy
		ld b, e_mx(ix)
		ld c, e_my(ix)
		dec c
		ld d, e_type(ix)
		call sec_setValue

		;; And then we change our values
		ld c, e_my(ix)
		dec c
		ld e_my(ix), c

		ld e_vy(ix), # -4
		
		call sprite_dir

		ld a, #1

	ret

	can_destroy_up:
		call sprite_dir
		ld a, #0
	ret

	not_move_destroy_up:
		ld a, #-1
ret



checkRight:

	ld e_dir(ix), #1
	
	ld b, e_mx(ix)
	ld c, e_my(ix)
	inc b
	call sec_retValue
	ld a, e

	;; Check all the posible types that the enemy can find
	cp #freedom
	jp m, not_move_destroy_right
	cp #enemy_bouncing
	jp m, can_move_right
	cp #obstacle_default
	jp m, not_move_destroy_right
	cp #wall
	jp m, can_destroy_right
		jr not_move_destroy_right

	can_move_right:

		;; We clear the position on the matrix
		ld b, e_mx(ix)
		ld c, e_my(ix)
		ld d, #freedom
		call sec_setValue

		;; We set the future position as the enemy
		ld b, e_mx(ix)
		ld c, e_my(ix)
		inc b
		
		ld d, e_type(ix)
		call sec_setValue

		;; And then we change our values
		ld b, e_mx(ix)
		inc b
		ld e_mx(ix), b

		ld e_vx(ix), #01

		call sprite_dir

		ld a, #01
	ret

	can_destroy_right:
		call sprite_dir
		ld a, #0
	ret

	not_move_destroy_right:
		ld a, #-1
ret



checkDown:

	ld e_dir(ix), #2

	ld b, e_mx(ix)
	ld c, e_my(ix)
	inc c
	call sec_retValue
	ld a, e

	;; Check all the posible types that the enemy can find
	cp #freedom
	jp m, not_move_destroy_down
	cp #enemy_bouncing
	jp m, can_move_down
	cp #obstacle_default
	jp m, not_move_destroy_down
	cp #wall
	jp m, can_destroy_down
		jr not_move_destroy_down

	can_move_down:

		;; We clear the position on the matrix
		ld b, e_mx(ix)
		ld c, e_my(ix)
		ld d, #freedom
		call sec_setValue

		;; We set the future position as the enemy
		ld b, e_mx(ix)
		ld c, e_my(ix)
		inc c
		
		ld d, e_type(ix)
		call sec_setValue

		;; And then we change our values
		ld c, e_my(ix)
		inc c
		ld e_my(ix), c

		ld e_vy(ix), #04
		
		call sprite_dir

		ld a, #01

	ret

	can_destroy_down:
		call sprite_dir
		ld a, #0
	ret

	not_move_destroy_down:
		ld a, #-1
ret



checkLeft:

	ld e_dir(ix), #3

	ld b, e_mx(ix)
	ld c, e_my(ix)
	dec b
	call sec_retValue
	ld a, e

	;; Check all the posible types that the enemy can find
	cp #freedom
	jp m, not_move_destroy_left
	cp #enemy_bouncing
	jp m, can_move_left
	cp #obstacle_default
	jp m, not_move_destroy_left
	cp #wall
	jp m, can_destroy_left
		jr not_move_destroy_left

	can_move_left:

		;; We clear the position on the matrix
		ld b, e_mx(ix)
		ld c, e_my(ix)
		ld d, #freedom
		call sec_setValue

		;; We set the future position as the enemy
		ld b, e_mx(ix)
		ld c, e_my(ix)
		dec b
		
		ld d, e_type(ix)
		call sec_setValue

		;; And then we change our values
		ld b, e_mx(ix)
		dec b
		ld e_mx(ix), b


		ld e_vx(ix), #-1
		
		call sprite_dir
		
		ld a, #01

	ret

	can_destroy_left:
		call sprite_dir
		ld a, #0
	ret

	not_move_destroy_left:
		ld a, #-1
ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Checks the collixion between two entities
;; Destroys: 
;; Input: 
;;	IX -> Entity A
;; 	IY -> Entity B
;; Output: 
;;	Cflag -> Set if they collide
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_hitboxed:
	ld a, de_w(iy)
	ld b, de_x(iy)
	dec a
	add a, b
	cp de_x(ix)
	jp m, no_colisiona
		ld a, de_x(ix)
		ld c, de_w(ix)
		dec c
		add a, c
		cp b
		jp m, no_colisiona
	
	ld a, de_h(iy)
	ld b, de_y(iy)
	dec a
	add a, b
	cp de_y(ix)
	jp m, no_colisiona
		ld a, de_y(ix)
		ld c, de_h(ix)
		dec c
		add a, c
		cp b
		jp m, no_colisiona
	


	colisiona: 
		scf
	ret

	no_colisiona:
		or a

	ret