;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Harvest Day: An Amstrad CPC Game 
;;  Copyright (C) 2018 Álvaro Molina García / José Manuel Ladrón de Guevara García, Daniel Saura Martínez
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.include "main.h.s"
.include "levelmanager.h.s"
.include "section.h.s"
.include "entity.h.s"
.include "collectable.h.s"
.include "player.h.s"
.include "includeLevels.h.s"
.include "menu.h.s"
.include "sprites.h.s"

.globl _level0

;; VARIABLES ZONE
	unlocked_levels: .db 1
	current_level: .db #init_level



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Read the matrix. Starts at the end
;; Destroys: A, BC, DE, HL, BC', DE', HL'
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ld a, #matrix_size
read_matrix:
	ld hl, #matrix_data+matrix_size-1
	ld c, #rows-1

	read_matrix_loop_y:
		ld b, #columns-1
		read_matrix_loop_x:
			ld a, (hl)
				cp #freedom
				call nz, creator_selector

			dec hl
			dec b
			jp p, read_matrix_loop_x
		dec c
	jp p, read_matrix_loop_y


	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Creates each item read from the matrix
;; Destroys: A, BC', DE', HL'
;; Input: 
;;	B  -> Column
;;	C  -> Row
;;	HL -> Pointer to matrix item
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
creator_selector:
	cp #collectable1
	jr nz, saltar_paso_1
		;Checks if array is full
		ld a, (num_coll)
		cp #(max_num_collectables)
		jr z, saltar_paso_1

		exx
			ld bc, #num_coll
			ld hl, (last_collectable_pointer)
			;Creates a new collectable in array
			push af
			ld a, #01
			ld (collectables1_on_level), a
			pop af
			call col_new
			ld (last_collectable_pointer), hl
			push ix
			pop de	;Copies ix into de

			ld hl, #default_coll_1

			ld bc, #coll_size

			;Makes a copy of default collectable into new array item
			ldir
		exx

		;Sets the new coordinates of the item
		ld e_mx(ix), b
		ld e_my(ix), c
		ld e_ma_x(ix), b
		ld e_ma_y(ix), c

		push bc
		call sec_matrixToScreen
		ld de_x(ix), d
		ld de_y(ix), e


		pop bc
		ret

	saltar_paso_1:

	cp #collectable2
	jr nz, saltar_paso_2
		;Checks if array is full
		ld a, (aux_num_coll1)
		cp #(max_num_collectables)
		jr z, saltar_paso_2

		exx
			ld bc, #aux_num_coll1
			ld hl, (last_collectable_pointer1)
			;Creates a new collectable in array
			push af
			ld a, #01
			ld (collectables2_on_level), a
			pop af
			call col_new
			ld (last_collectable_pointer1), hl
			push ix
			pop de	;Copies ix into de

			ld hl, #default_coll_2

			ld bc, #coll_size

			;Makes a copy of default collectable into new array item
			ldir
		exx

		;Sets the new coordinates of the item
		ld e_mx(ix), b
		ld e_my(ix), c
		ld e_ma_x(ix), b
		ld e_ma_y(ix), c

		push bc
		call sec_matrixToScreen
		ld de_x(ix), d
		ld de_y(ix), e

		ld (hl), #0


		pop bc
		ret
	saltar_paso_2:

	cp #collectable3
	jr nz, saltar_paso_3
		;Checks if array is full
		ld a, (aux_num_coll2)
		cp #(max_num_collectables)
		jr z, saltar_paso_3

		exx
			ld bc, #aux_num_coll2
			ld hl, (last_collectable_pointer2)
			;Creates a new collectable in array
			push af
			ld a, #01
			ld (collectables3_on_level), a
			pop af
			call col_new
			ld (last_collectable_pointer2), hl
			push ix
			pop de	;Copies ix into de

			ld hl, #default_coll_3

			ld bc, #coll_size

			;Makes a copy of default collectable into new array item
			ldir
		exx

		;Sets the new coordinates of the item
		ld e_mx(ix), b
		ld e_my(ix), c
		ld e_ma_x(ix), b
		ld e_ma_y(ix), c

		push bc
		call sec_matrixToScreen
		ld de_x(ix), d
		ld de_y(ix), e

		ld (hl), #0

		pop bc
		ret
	saltar_paso_3:

	cp #collectable4
	jr nz, saltar_paso_4
		;Checks if array is full
		ld a, (aux_num_coll3)
		cp #(max_num_collectables)
		jr z, saltar_paso_4

		exx
			ld bc, #aux_num_coll3
			ld hl, (last_collectable_pointer3)
			;Creates a new collectable in array
			push af
			ld a, #01
			ld (collectables4_on_level), a
			pop af
			call col_new
			ld (last_collectable_pointer3), hl
			push ix
			pop de	;Copies ix into de

			ld hl, #default_coll_4

			ld bc, #coll_size

			;Makes a copy of default collectable into new array item
			ldir
		exx

		;Sets the new coordinates of the item
		ld e_mx(ix), b
		ld e_my(ix), c
		ld e_ma_x(ix), b
		ld e_ma_y(ix), c

		push bc
		call sec_matrixToScreen
		ld de_x(ix), d
		ld de_y(ix), e

		ld (hl), #0


		pop bc
		ret
	saltar_paso_4:




	cp #player1
	jr nz, saltar_paso_5
		ld (hl), #0
		
		;Sets the new coordinates of the player
		ld ix, #player1_data
		ld e_mx(ix), b
		ld e_my(ix), c
		ld e_ma_x(ix), b
		ld e_ma_y(ix), c


		push bc
		call sec_matrixToScreen
		ld de_x(ix), d
		ld de_y(ix), e

		pop bc
		ret

	saltar_paso_5:

	;Checks if array is full
	cp #enemy_bouncing
	jr nz, saltar_paso_7
		ld a, (num_enemies)
		cp #(max_num_enemies)
		jr z, saltar_paso_7

		exx
			;Creates a new enemy in array
			call ent_new
			push ix
			pop de	;Copies ix into de
			
			ld hl, #default_bouncing
			
			ld bc, #e_size

			;Makes a copy of default collectable into new array item
			ldir
		exx

		;Sets the new coordinates of the item
		ld e_mx(ix), b
		ld e_my(ix), c
		ld e_ma_x(ix), b
		ld e_ma_y(ix), c


		push bc
		call sec_matrixToScreen
		ld de_x(ix), d
		ld de_y(ix), e


		pop bc
		ret

	saltar_paso_7:

	cp #enemy_chasing
	jr nz, saltar_paso_8
		ld a, (num_enemies)
		cp #(max_num_enemies)
		jr z, saltar_paso_8

		exx
			call ent_new
			push ix
			pop de
			
			ld hl, #default_chasing
			
			ld bc, #e_size

			ldir
		exx
		ld e_mx(ix), b
		ld e_my(ix), c
		ld e_ma_x(ix), b
		ld e_ma_y(ix), c

		push bc
		call sec_matrixToScreen
		ld de_x(ix), d
		ld de_y(ix), e


		pop bc
		ret

	saltar_paso_8:

	cp #enemy_breaker
	jr nz, saltar_paso_9
		ld a, (num_enemies)
		cp #(max_num_enemies)
		jr z, saltar_paso_9

		exx
			call ent_new
			push ix
			pop de
			
			ld hl, #default_breaking
			
			ld bc, #e_size

			ldir
		exx
		ld e_mx(ix), b
		ld e_my(ix), c
		ld e_ma_x(ix), b
		ld e_ma_y(ix), c



		push bc
		call sec_matrixToScreen
		ld de_x(ix), d
		ld de_y(ix), e


		pop bc
		ret

	saltar_paso_9:




	cp #obstacle_default
	jp m, saltar_paso_10

	cp #wall
	jp p, saltar_paso_10
		cp #wall
		jr z, saltar_paso_10

		;Draws the new obstacle
		add a, #-obstacle_default
		ld (hl), a

		push af
		cp #freedom
		jr z, skip_loading_block
			call creator_selector
			ld e_block(ix), #1
		skip_loading_block:

		pop af
		cp #collectable1
		jr z, skip_seting_original_block
			ld (hl), #freedom
		skip_seting_original_block:

		push bc
		push hl
		call ent_createObstacle
		pop hl
		pop bc



		ret

	saltar_paso_10:

	cp #wall+1
	jp m, saltar_paso_11
		cp #wall+1+4
		jp p, saltar_paso_11
		
		;Creates the new enemy
		add a, #-wall-1
		push af

		ld a, #enemy_bouncing
		ld (hl), #enemy_bouncing

		call creator_selector

		pop af

		ld e_dir(ix), a
		
		call ent_setVelocityFdir

		ret

	saltar_paso_11:

	cp #wall+1+4
	jp m, saltar_paso_12
		cp #wall+1+4+4
		jp p, saltar_paso_12
	

		;Creates the new enemy
		add a, #-wall-1-4
		push af

		ld a, #collectable3
		ld (hl), #collectable3

		call creator_selector

		pop af

		ld e_dir(ix), a
		
		call ent_setVelocityFdir



		ret
	saltar_paso_12:

	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Charge into matrix the choosed level
;; Destroys: A, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
load_level:
	call clear_level

	ld a, (current_level)


	cp #0
	ld    hl, #_level0_end
	jr z, descompress_level

	cp #1
	ld    hl, #_level1_end
	jr z, descompress_level

	cp #2
	ld    hl, #_level2_end
	jr z, descompress_level

	cp #3
	ld    hl, #_level3_end
	jr z, descompress_level

	cp #4
	ld    hl, #_level4_end
	jr z, descompress_level

	cp #5
	ld    hl, #_level5_end
	jr z, descompress_level

	cp #6
	ld    hl, #_level6_end
	jr z, descompress_level

	cp #7
	ld    hl, #_level7_end
	jr z, descompress_level

	cp #8
	ld    hl, #_level8_end
	jr z, descompress_level

	cp #9
	ld    hl, #_level9_end
	jr z, descompress_level

	cp #10
	ld    hl, #_level10_end
	jr z, descompress_level

	cp #11
	ld    hl, #_level11_end
	jr z, descompress_level

	cp #12
	ld    hl, #_level12_end
	jr z, descompress_level

	cp #13
	ld    hl, #_level13_end
	jr z, descompress_level

	cp #14
	ld    hl, #_level14_end
	jr z, descompress_level

	cp #15
	ld    hl, #_level15_end
	jr z, descompress_level

	cp #16
	ld    hl, #_level16_end
	jr z, descompress_level




	descompress_level:
	ld    de, #matrix_data+#matrix_size-1
	call cpct_zx7b_decrunch_s_asm

	call read_matrix

	ld a, (num_coll)
	ld (collectables_left), a

	;; Draws all the collectables at the init of our level
	call col_initial_drawing

	;; Now we draw our HUD
	call draw_hud



ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Draws the HUD of the level
;; Destroys: A, BC, DE, HL, IX
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
draw_hud:

	;; Number of collectables
	ld bc, #num_coll

	ld d, #hud_startx
	ld e, #hud_starty

	ld a, #hud_startx
	ld (hud_x), a

	;; Obtain the sprite of the collectable
	ld hl, #_collectables_hud_sp_0

	;; Quantity of arrays of collectables
	ld a, #4

	draw_loop:

		push af
		push hl
		push bc


		ld a, (bc)
		cp #0
		jr z, skip_drawing
			
			
			push de

			;; Set and draw the collectable selected on the array
			ld b, #col_h
			ld c, #col_w
			ld a, #1
			push bc
			push de
			push hl
			call ren_drawBackObstacle

			pop hl
			pop de
			pop bc
			ld a, #1
			call ren_drawFrontObstacle


			pop de
			ld a, d
			add #col_w + #sprite_offsetx
			ld d, a


		skip_drawing:

		pop bc

		push de

		;; Actualize BC -> Number of collectables on the array
		ld de, #coll_array_complete
		ld h, b
		ld l, c
		add hl, de
		ld b, h
		ld c, l
		
		pop de

		pop hl
		push de

		;; Here we actualize the sprite
		ld de, #sprite_size
		add hl, de

		pop de

		;; And now we actualize the number of the rest of collectables
		pop af

		;; If we arrived at the maximum number of collectables, we stop
		dec a
		jp nz, draw_loop


	ret





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Clear vectors of things and screen
;; Destroys: A, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
clear_level:


	ld a, #0
	ld de, #collectables1_on_level
	ld (de), a
	inc de
	ld (de), a
	inc de
	ld (de), a
	inc de
	ld (de), a



	ld hl, #coll_array
	ld de, #coll_array+1	
	ld bc, #coll_array_complete-1
	ld (hl), #0
	ldir
	ld bc, #coll_array_complete-1
	;ld (hl), #0
	ldir
	ld bc, #coll_array_complete-1
	;ld (hl), #0
	ldir
	ld bc, #coll_array_complete-1
	;ld (hl), #0
	ldir

	ld hl, #enemy_array
	ld de, #enemy_array+1	
	ld bc, #enemy_array_size-1
	ld (hl), #0
	ldir
	
	ld a, #0
	ld (num_coll), a
	ld (aux_num_coll1), a
	ld (aux_num_coll2), a
	ld (aux_num_coll3), a
	ld (num_enemies), a

	ld hl, #coll_array
	ld (last_collectable_pointer), hl
	ld hl, #aux_coll_array1
	ld (last_collectable_pointer1), hl
	ld hl, #aux_coll_array2
	ld (last_collectable_pointer2), hl
	ld hl, #aux_coll_array3
	ld (last_collectable_pointer3), hl
	ld hl, #enemy_array
	ld (last_enemy_pointer), hl


	ld a, (current_level)
	
	cp #0 									
	jr nz, no_mapa_0

		ld    hl, #_map_pack_end 			;; Name of map pack
   		ld    de, #decompress_buffer_end 	;; Where we want to decompress end
   		call cpct_zx7b_decrunch_s_asm 		;; decompress


	no_mapa_0:

	cp #1									
	jr nz, no_mapa_1

		ld    hl, #_map_pack_end 			
   		ld    de, #decompress_buffer_end 	
   		call cpct_zx7b_decrunch_s_asm 		


	no_mapa_1:

	cp #4
	jr nz, no_mapa_4 			

		ld    hl, #_map2_pack_end
   		ld    de, #decompress_buffer_end
   		call cpct_zx7b_decrunch_s_asm

	no_mapa_4:


	cp #5
	jr nz, no_mapa_5 			

		ld    hl, #_map6_pack_end
   		ld    de, #decompress_buffer_end
   		call cpct_zx7b_decrunch_s_asm

	no_mapa_5:

	cp #6
	jr nz, no_mapa_6			

		ld    hl, #_map_pack_end
   		ld    de, #decompress_buffer_end
   		call cpct_zx7b_decrunch_s_asm

	no_mapa_6:

	cp #7
	jr nz, no_mapa_7			

		ld    hl, #_map5_pack_end
   		ld    de, #decompress_buffer_end
   		call cpct_zx7b_decrunch_s_asm

	no_mapa_7:
	
	cp #8
	jr nz, no_mapa_8			

		ld    hl, #_map4_pack_end
   		ld    de, #decompress_buffer_end
   		call cpct_zx7b_decrunch_s_asm

	no_mapa_8:

	cp #9
	jr nz, no_mapa_9			

		ld    hl, #_map_pack_end
   		ld    de, #decompress_buffer_end
   		call cpct_zx7b_decrunch_s_asm

	no_mapa_9:

	cp #10
	jr nz, no_mapa_10			

		ld    hl, #_map6_pack_end
   		ld    de, #decompress_buffer_end
   		call cpct_zx7b_decrunch_s_asm

	no_mapa_10:

	cp #11
	jr nz, no_mapa_11			

		ld    hl, #_map_pack_end
   		ld    de, #decompress_buffer_end
   		call cpct_zx7b_decrunch_s_asm

	no_mapa_11:

	cp #12
	jr nz, no_mapa_12			

		ld    hl, #_map_pack_end
   		ld    de, #decompress_buffer_end
   		call cpct_zx7b_decrunch_s_asm

	no_mapa_12:

	cp #13
	jr nz, no_mapa_13			

		ld    hl, #_map3_pack_end
   		ld    de, #decompress_buffer_end
   		call cpct_zx7b_decrunch_s_asm

	no_mapa_13:

	cp #14
	jr nz, no_mapa_14			

		ld    hl, #_map3_pack_end
   		ld    de, #decompress_buffer_end
   		call cpct_zx7b_decrunch_s_asm

	no_mapa_14:


	call configure_map 					;; Draw the map in both buffers


	ld a, #0
	ld (current_collectable), a

	ld ix, #player1_data
	ld e_mov(ix), a
	ld pl_died(ix), a
	ld e_op(ix), a
	ld e_dist(ix), a
	ld e_dir(ix), #2

   	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Increments the current level and calls Load Level
;; Destroys: A, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
next_level:
	ld a, (current_level)
	inc a
	ld (current_level), a

	call load_level

   	jp game_loop


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Increments the number of unlocked levels
;; Destroys: A, E
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
next_unlock:
	ld a, #total_lvls
	ld e, a

	;;Checks if the player reached the limit
	ld a, (current_level)
		cp e
		jp z, menu_win


	ld a, (unlocked_levels)
	inc a
	ld (unlocked_levels), a

	

	ret
