;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Harvest Day: An Amstrad CPC Game 
;;  Copyright (C) 2018 Álvaro Molina García / José Manuel Ladrón de Guevara García, Daniel Saura Martínez
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

;; Include all CPCtelera constant definitions, macros and variables
   .include "cpctelera.h.s"
   .include "main.h.s"
   .include "entity.h.s"
   .include "section.h.s"
   .include "collectable.h.s"
   .include "player.h.s"
   .include "menu.h.s"
   .include "levelmanager.h.s"
   
;;
;;  Start of _DATA area 
;;  SDCC requires at least _DATA and _CODE areas to be declared, but you may use
;;  any one of them for any purpose. Usually, compiler puts _DATA area contents
;;  right after _CODE area contents.
;;
.area _DATA


;; Start of _CODE area
;; 
.area _CODE

;; VARIABLES
musica: .db #music_bytes
active_music: .db #1




;;
;; MAIN function. This is the entry point of the application.
;;    _main:: global symbol is required for correctly compiling and linking
;;
_main::
   
   ld sp, #0x8000  ;; Changes the start direction of the stack to 0x8000 (See stack concept on Amstrad)


   ;; Disable firmware to prevent it from interfering with string drawing
   call cpct_disableFirmware_asm

   call ren_initBuffers    ;; Inits the back buffer to clean all the junk between 0x8000 and 0xC000
   
   ;; Set video mode to 0 (1 byte = 2 pixels on width)
   ld c, #0
   call cpct_setVideoMode_asm

   ;; Set the palette
   ld hl, #_palette
   ld de, #16
   call cpct_setPalette_asm


   ld l, #16
   ld h, #0x16
   call cpct_setPALColour_asm


   ;;Inits music
   ld de, #_song_ingame
   call cpct_akp_musicInit_asm
   ;;Set the function that is call in interruptions
   ld hl, #interruption
   call cpct_setInterruptHandler_asm

   ;;Decompress the tiles
   
   ld    hl, #_tile_pack_end
   ld    de, #decompress_buffer2_end
   call cpct_zx7b_decrunch_s_asm


   inicio_juego:
   jp menu_inicio

   game_start:
   call load_level

   game_loop:

      ld ix, #coll_array
      ld a, (num_coll)
      call col_doForAllBaseMethods

      ld ix, #enemy_array
      call ent_doForAllBaseMethods
      
      ld ix, #player1_data
      call pl_clear
      call pl_update
      call pl_draw

      ;; Check if the Key A is pressed
            ld hl, #Key_0
            call cpct_isKeyPressed_asm                ;; Returns 0 if "HL" not pressed
            jr z, b0_not_pressed
               ld hl, #Key_Shift
               call cpct_isKeyPressed_asm                ;; Returns 0 if "HL" not pressed
               jr z, b0_not_pressed
                  ld a, (current_level)
                  cp #total_lvls
                  call nz, next_level
            b0_not_pressed:


            ld hl, #Key_Esc
            call cpct_isKeyPressed_asm                ;; Returns 0 if "HL" not pressed
            jp nz, menu_pausa

            ld hl, #Joy0_Fire2
            call cpct_isKeyPressed_asm                ;; Returns 0 if "HL" not pressed
            jp nz, menu_pausa


            call check_mute_music

      

      call cpct_waitVSYNC_asm
      call ren_newScene

   jr game_loop


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Checks if player hitted the Mute Button
;; Destroys: AF, HL, DE, BC
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
check_mute_music:
   ld a, (_key_delay_counter)
   or a
   call nz, update_delay
   ret nz
      ;; MUSIC CONTROLS
      ld hl, #Key_M
      call cpct_isKeyPressed_asm                ;; Returns 0 if "HL" not pressed
      call nz, toggle_music

      ld a, #10
      jp set_delay


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Toggle the music
;; Destroys: AF
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
toggle_music:
   ld a, (active_music)
   xor #1
   ld (active_music), a


   ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Execute this function when an interruption happend
;; Destroys: AF, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
interruption:

   ex af, af'
   exx 
   push af
   push bc
   push de 
   push hl
   push iy

   ld a, (musica)    
   dec a
   ld (musica), a    
   jr nz, volver
      ld a, (active_music)
      cp #1
      call z, cpct_akp_musicPlay_asm  ;; play music 
      call nz, cpct_akp_stop_asm ;; stop music


      ld a, #music_bytes
      ld (musica), a    
   
   volver:
      call cpct_scanKeyboard_if_asm  ;; check key pressed
   pop iy
   pop hl
   pop de 
   pop bc
   pop af
   exx
   ex af, af'
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Draw the map in both buffers
;; Destroys: AF, DE, HL, BC, IX,IY
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

configure_map:
   ld bc, #0x1914    ;;Viewport size in tiles (width & height)
   ld de, #_map_W    ;;Tileset width
   ld hl, #ztileset_ptr  ;;pointer to tileset

   call cpct_etm_setDrawTilemap4x8_ag_asm

   ld a, #0 ;; front_buffer
   ld de, #tilemap_ptr
   call drawMap

   ld a, #1 ;; back_buffer
   ld de, #tilemap_ptr
   call drawMap

   ;Configure the draw of a part of the map

   ld bc, #0x0201    ;;Viewport size in tiles (width & height)
   ld de, #_map_W    ;;Tileset width
   ld hl, #ztileset_ptr  ;;pointer to tileset
   call cpct_etm_setDrawTilemap4x8_ag_asm
ret