;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Harvest Day: An Amstrad CPC Game 
;;  Copyright (C) 2018 Álvaro Molina García / José Manuel Ladrón de Guevara García, Daniel Saura Martínez
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.include "menu.h.s"
.include "main.h.s"
.include "cpctelera.h.s"
.include "utils.h.s"
.include "menusinclude.h.s"
.include "sprites.h.s"
.include "levelmanager.h.s"
.include "player.h.s"
.include "entity.h.s"

_submenu_dir: .dw  00, game_start, inicio_juego
DefineDrawableEntity pantalla_inicio, 100, 130, 40, 22, _collectable_sp
DefineDrawableEntity dw_auxiliar, 5, 16, 10, 20, _collectable_sp


;; Variables
	_key_delay_counter: .db 0
	_selector: .db 00
	_variable_auxiliar: .db 00, 00


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Add the menu's selector
;;Input
;;	B -> Control's value
;;	C -> Selector's maximum
;;DESTRUYE: A
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
change_menu_selector:
	call reset_delay

	ld a, (_selector)
	add a, b

	jp m, no_cambiar_selector
		cp c
		jp p, no_cambiar_selector
			ld (_selector), a

	no_cambiar_selector:

	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Decrement the key's delay counter
;;Input:
;;	A -> Counter's value
;;Salida: 
;;	A -> Counter's value -1
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
update_delay:
	dec a
	ld (_key_delay_counter), a
	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Resets the delay counter to keyboard
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
reset_delay:
	ld a, #key_delay
	ld (_key_delay_counter), a
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Sets the delay value
;;INPUT:
;;	A -> Delay
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
set_delay:
	ld (_key_delay_counter), a
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Begining screen state
;;
;;Destroys: A, HL, DE, BC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
menu_inicio:
	ld a, #255
	call set_delay

	ld a, #init_level
	ld (current_level), a

	call reset_stack
	call reiniciar_selector

	ld hl, #_menuiniciopack_end
	call draw_menu
	
	bucle_menu_inicio:
		halt
		ld a, (_key_delay_counter)
		or a
		call nz, update_delay
		jr nz, bucle_menu_inicio
			call cpct_isAnyKeyPressed_asm
			jp nz, game_start

	jr bucle_menu_inicio

restore_game:
	ld a, (back_buffer)
	ld h, a
	ld l, #0

	ld a, (front_buffer)
	ld d, a
	ld e, #0

	ld bc, #0x4000

	ldir

	ld iy, #player1_data
	ld e_del(iy), #pause_del


	ret

menu_muerte:
	call reset_stack
	call reiniciar_selector
	ld a, #1
	jr menu_auxiliar

menu_pausa:
	call reset_stack
	call reiniciar_selector
	ld a, #2
	jr menu_auxiliar

menu_level_over:
	call reset_stack
	call reiniciar_selector
	ld a, #3
	jr menu_auxiliar

reiniciar_selector:
	ld a, #0
	ld (_selector), a
	ret


menu_auxiliar:	
	ld ix, #dw_auxiliar

	dec a
	jr nz, men_aux_1
		ld de, #0x0000
		ld a, #2
		ld hl, #_menudeathpack_end

		jr skip_menu_chooser
	men_aux_1:

	dec a
	jr nz, men_aux_2
		ld de, #resume_game
		ld a, #3
		ld hl, #_menupausepack_end

		jr skip_menu_chooser
	men_aux_2:

	dec a
	jr nz, men_aux_3
		ld de, #next_level
		ld a, #3
		ld hl, #_menunextlevelpack_end

		jr skip_menu_chooser
	men_aux_3:


	skip_menu_chooser:
	ld (_submenu_dir), de

	ld (_variable_auxiliar+1), a

	call draw_menu
	


	call first_aux_time


	push af
	ld a, #255
	call set_delay
	pop af

	menu_auxiliar_loop:
		halt
		ld a, (_key_delay_counter)
		or a
		call nz, update_delay
		call z, control_key_aux

	jr menu_auxiliar_loop

	ret


first_aux_time:
	ld ix, #dw_auxiliar

	ld de_x(ix), #m_f_start_x
	ld de_w(ix), #m_f_w
	ld de_h(ix), #m_f_h

	ld h, #m_aux_space
	ld l, #m_f_start_y
	
	ld a, (_variable_auxiliar+1)
	cp #2

	ld c, a
	call z, calcular_iterator
	ld de_y(ix), l

	jp ren_drawFrontEntity




control_key_aux:
	

	ld a, (_variable_auxiliar+1)
	ld e, a

	ld hl, #Key_W
	call cpct_isKeyPressed_asm
	ld b, #-1
	ld c, e
	jr nz, aux_iterator_check

	ld hl, #Key_S
	call cpct_isKeyPressed_asm
	ld b, #1
	ld c, e
	jr nz, aux_iterator_check
	

	ld hl, #Joy0_Up
	call cpct_isKeyPressed_asm
	ld b, #-1
	ld c, e
	jr nz, aux_iterator_check
	
	ld hl, #Joy0_Down
	call cpct_isKeyPressed_asm
	ld b, #1
	ld c, e
	jr nz, aux_iterator_check


	ld hl, #Key_Return
	call cpct_isKeyPressed_asm
	ld b, #0
	jr nz, enter_aux_menu

	ld hl, #Key_Space
	call cpct_isKeyPressed_asm
	ld b, #0
	jr nz, enter_aux_menu

	ld hl, #Joy0_Fire1
	call cpct_isKeyPressed_asm
	ld b, #0
	jr nz, enter_aux_menu



	ret


aux_iterator_check:
	call change_menu_selector
	jp aux_iterator


enter_aux_menu:
	call nz, aux_iterator
	ld ix, #_submenu_dir
	ld a, (_variable_auxiliar)
	;ld a, #0
	ld e, a
	ld d, #0
	add ix, de
	add ix, de

	ld l, 0(ix)
	ld h, 1(ix)

	
	jp (hl)


resume_game:
	call restore_game


	jp game_loop


aux_iterator:
	;;ARREGLAR call ren_clearEntity

	ld h, de_sp_h(ix)
	ld l, de_sp_l(ix)

	push hl 		;;save the actual sprite 

	push bc

	ld hl, #_borrar_flecha_sp

	ld de_sp_h(ix), h
	ld de_sp_l(ix), l

	call ren_drawFrontEntity

	pop bc

	ld h, #m_aux_space
	ld l, #m_f_start_y
	
	ld a, (_variable_auxiliar+1)
	cp #2
	ld a, (_selector)
	jr nz, no_variable_aux
		inc a
	no_variable_aux:
	ld (_variable_auxiliar), a


	ld a, (_variable_auxiliar+1)
	cp #2

	ld a, (_selector)
	call z, calcular_iterator

	aux_control_x:
		call calcular_iterator
		dec a
		jr nz, aux_control_x

	ld de_y(ix), l

	pop hl
	ld de_sp_h(ix), h 
	ld de_sp_l(ix), l 		;; reload the sprite and draw in the new position

	jp ren_drawFrontEntity


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Menu Win State
;;
;; DESTROYS : A, HL, DE, BC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
menu_win:
	ld a, #255
	call set_delay

	call reset_stack
	call reiniciar_selector

	ld hl, #_menuwinpack_end
	call draw_menu
	
	call cpct_waitVSYNC_asm
	call cpct_waitVSYNC_asm


	ld a, #0
	ld (current_level), a

	win_menu_loop:
		halt
		ld a, (_key_delay_counter)
		or a
		call nz, update_delay
		jr nz, win_menu_loop
			call cpct_isAnyKeyPressed_asm
			jp nz, menu_inicio

	jr win_menu_loop


	ret


calcular_iterator:
	ld e, a

	ld a, l
	add a, h
	ld l, a

	ld a, e


	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Extracts the selected menu into front buffer
;;DESTROYS: AF, HL, DE, BC
;;INPUT
;;	HL -> Pointer to menu image compressed 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
draw_menu:
	push hl
	ld a, (front_buffer)
	ld h, a
	ld l, #0

	ld de, #0x4000-#1
	add hl, de
	ex de, hl

	pop hl
	call cpct_zx7b_decrunch_s_asm


	ret


reset_stack:

	pop hl 
	ld sp, #0x8000  ;; Changes the start direction of the stack to 0x8000 (See stack concept on Amstrad)

	jp (hl)