;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Harvest Day: An Amstrad CPC Game 
;;  Copyright (C) 2018 Álvaro Molina García / José Manuel Ladrón de Guevara García, Daniel Saura Martínez
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
;; Includes
	
	.include "cpctelera.h.s"
	.include "main.h.s"
	.include "entity.h.s"
	.include "player.h.s"
	.include "section.h.s"
	.include "sprites.h.s"
	.include "levelmanager.h.s"
	.include "collectable.h.s"
	.include "menu.h.s"

;; Declarations

	;; Defines the player with the init values of the constants defined on "entity.h.s" for player
	player1_data:
		DefinePlayer player_1, player1_initial_x, player1_initial_y, player1_w, player1_h, _player_sp_0, player1_initial_mx, player1_initial_my, player1_dir, #ent_player1Actions, player1_vx, player1_vy, player1_mov, player1_del, player1_block, player1_op, player1_dist, player1, 0x00, 0x00, 0x00, 0x00

;; Public methods

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Clears the collectable if it's not removed
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pl_clear:

	ld a, pl_died(ix)
	cp #0
	jp nz, pl_die

	call ent_clearEntity

ret

pl_die:
	ld hl, #_player_sp_1
	ld de_sp_h(ix), h
	ld de_sp_l(ix), l
	jp menu_muerte


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Updates the collectable
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pl_update:

	;; Checks if the collectable has been picked up
	ld a, pl_died(ix)
	cp #0
	ret nz

	push ix
		call check_collision_collectable
		call check_collision_enemy
	pop ix
	;; Goes to the direction saved on "_upd" of the entity
	ld h, e_upd_h(ix)
	ld l, e_upd_l(ix)

jp (hl)


check_collision_enemy:
	ld a, (num_enemies)
	ld iy, #enemy_array

	cp #0
	ret z


	check_collision_enemy_loop:
		push af

		call ent_hitboxed
		jr nc, skip_kill_player
			ld pl_died(ix), #1
		skip_kill_player:

		pop af


		ld bc, #e_size
		add iy, bc

		dec a
	jr nz, check_collision_enemy_loop

	ret


check_collision_collectable:
	ld a, (num_coll)
	ld iy, #coll_array

	cp #0
	ret z
	
	check_collision_collectable_loop:
		push af

		call ent_hitboxed
		jr nc, skip_get_collectable
			push ix
			push iy
			pop ix
			call removed_by_player

			jp popping

			pop ix
		skip_get_collectable:

		pop af


		ld bc, #coll_size
		add iy, bc

		dec a
	jr nz, check_collision_collectable_loop

	ret

popping:
	pop af
	pop af

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Draws the collectable
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pl_draw:

	;; Checks if the collectable has been picked up
	ld a, pl_died(ix)
	cp #0
	ret nz

	call ren_drawEntity

ret


;; Private methods


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Updates the player 1
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_player1Actions:

	;; Checks the inputs and move the player1
	call ent_movKeyboard1
	call ent_setEntityOnMatrix

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Checks if there's collision "up"
;; Destroys: A
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_upDir:

	;; We prepare our parameters X and Y on B and C to check the position of the matrix "up" to the entity
	ld b, e_mx(ix)
	ld c, e_my(ix)
	dec c

	;; Get the value of that position
	call sec_retValue

	;; If it's a free space or it's a collectable, we can move towards it
	ld a, e
	cp #freedom
	jp m, obstacled_up
		cp #player1
		jp m, not_obstacled_up

			obstacled_up:
				ld e_vy(ix), #0 	;; If there's an obstacle we won't move towards it
				ld a, #-1
			ret

	not_obstacled_up:

	;; We change the previous position of the entity on the matrix to free
	ld b, e_mx(ix)
	ld c, e_my(ix)
	ld d, #freedom
	call sec_setValue

	;; Now that we'll move, we will change the value of the entity on the next position of the matrix
	ld a, e_my(ix)
	ld e_ma_y(ix),a
	dec a
	ld e_my(ix), a
	ld a, #0

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Checks if there's collision "right"
;; Destroys: A
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_rightDir:

	;; We prepare our parameters X and Y on B and C to check the position of the matrix "right" to the entity
	ld b, e_mx(ix)
	ld c, e_my(ix)
	inc b

	;; Get the value of that position
	call sec_retValue

	;; If it's a free space or it's a collectable, we can move towards it
	ld a, e
	cp #freedom
	jp m, obstacled_right
		cp #player1
		jp m, not_obstacled_right

			obstacled_right:
				ld e_vx(ix), #0 	;; If there's an obstacle we won't move towards it
				ld a, #-1
			ret

	not_obstacled_right:

	;; We change the previous position of the entity on the matrix to free
	ld b, e_mx(ix)
	ld c, e_my(ix)
	ld d, #freedom
	call sec_setValue

	;; Now that we'll move, we will change the value of the entity on the next position of the matrix
	ld a, e_mx(ix)
	ld e_ma_x(ix), a
	inc a
	ld e_mx(ix), a
	ld a, #0

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Checks if there's collision "down"
;; Destroys: A
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_downDir:

	;; We prepare our parameters X and Y on B and C to check the position of the matrix "right" to the entity
	ld b, e_mx(ix)
	ld c, e_my(ix)
	inc c

	;; Get the value of that position
	call sec_retValue

	;; If it's a free space or it's a collectable, we can move towards it
	ld a, e
	cp #freedom
	jp m, obstacled_down
		cp #player1
		jp m, not_obstacled_down

			obstacled_down:
				ld e_vy(ix), #0 	;; If there's an obstacle we won't move towards it
				ld a, #-1
			ret

	not_obstacled_down:

	;; We change the previous position of the entity on the matrix to free
	ld b, e_mx(ix)
	ld c, e_my(ix)
	ld d, #freedom
	call sec_setValue

	;; Now that we'll move, we will change the value of the entity on the next position of the matrix
	ld a, e_my(ix)
	ld e_ma_y(ix),a
	inc a
	ld e_my(ix), a
	ld a, #0

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Checks if there's collision "left"
;; Destroys: A
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_leftDir:

	;; We prepare our parameters X and Y on B and C to check the position of the matrix "right" to the entity
	ld b, e_mx(ix)
	ld c, e_my(ix)
	dec b

	;; Get the value of that position
	call sec_retValue

	;; If it's a free space or it's a collectable, we can move towards it
	ld a, e
	cp #freedom
	jp m, obstacled_left
		cp #player1
		jp m, not_obstacled_left

			obstacled_left:
				ld e_vx(ix), #0 	;; If there's an obstacle we won't move towards it
				ld a, #-1
			ret

	not_obstacled_left:

	;; We change the previous position of the entity on the matrix to free
	ld b, e_mx(ix)
	ld c, e_my(ix)
	ld d, #freedom
	call sec_setValue

	;; Now that we'll move, we will change the value of the entity on the next position of the matrix
	ld a, e_mx(ix)
	ld e_ma_x(ix),a
	dec a
	ld e_mx(ix), a
	ld a, #0

ret



;; MATRIX RELATED FUNCTIONS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Creates an obstacle on the matrix and draws it
;; Destroys: AF, BC, DE, HL
;; Input: 
;; B -> X column
;; C -> Y row
;; Output: A = -1 -> Error: Position not constructable
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ent_createObstacle:

	;; We save on the stack the BC value to use it later to draw the obstacles
	push bc

	;; Get the matrix value of that position
	call sec_retValue

	pop bc

	push bc
	push hl
	push ix
	push de

	call ent_findInEntWithMcoords

	pop de
	pop ix
	pop hl
	pop bc

	ld a, #-1
	ret c


	;; If it's constructable then proceed
	ld a, e
	cp #player1
	jp m, can_be_placed
		ld a, #-1
	ret



	;; Set the value of the position to correct obstacle value
	can_be_placed:
	ld a, (hl)
	add a, #obstacle_default
	ld (hl), a

	;ld a, #obstacle_default
	push af	

	push ix
	push bc
	cp #obstacle_default
	jr z, skip_remove_col_creating
		pop de
		push bc
		call col_findInCollWithMcoords
		jr nc, skip_remove_col_creating
		ld c_blocked(ix), #1

		;; We remove it out of the screen
		ld b, e_ma_x(ix)
		ld c, e_ma_y(ix)
		call sec_matrixToScreen
		call ren_clearObstacle

		ld b, e_mx(ix)
		ld c, e_my(ix)
		call sec_matrixToScreen
		call ren_clearObstacle


		ld a, e_type(ix) 		;; Load the type of collectable in a to check which sprite we should put
		dec a
		jr nz, no_col1

			ld hl, #_col1_sp_0
			jr skip_default_sprite

		no_col1:
		dec a 
		jr nz, no_col2

			ld hl, #_col2_sp_0
			jr skip_default_sprite

		no_col2:
		dec a 
		jr nz, no_col3

			ld hl, #_col3_sp_0
			jr skip_default_sprite

		no_col3:
		ld hl, #_col4_sp_0
		jr skip_default_sprite



	
	skip_remove_col_creating:

	ld hl, #_obstacle_sp_1

	skip_default_sprite:
	

	pop bc
	pop ix
	pop af	
	;; Now we'll convert the position on the matrix to the position on the screen
	call sec_matrixToScreen


	;; Here we will draw the obstacle on the 2 buffers
	ld b, #section_h
	ld c, #section_w

	push de
	push hl
	ld a, #1
	call ren_drawBackObstacle

	pop hl
	pop de
	ld b, #section_h
	ld c, #section_w
	ld a, #1
	call ren_drawFrontObstacle

	;; This assures that we created the obstacle correctly
	ld a, #0

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Creates a row of blocks that ends when it can't create more  
;; Destroys:
;; Input: 
;; 	B -> X column 
;; 	C -> Y row
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ent_createRowObstacle:


	;; Increment the distance to create an obstacle at the next position
	ld a, e_dist(ix)
	inc a
	ld e_dist(ix), a

	ld d, a
	call ent_setObstacleValues 					;; Sets the values to create an obstacle
	call ent_createObstacle 					;; Creates an obstacle

	cp #0
	ret z
		;; We assume that the obstacle couldn't be created

		;; In this case we have to stop creating blocks and reset the distance to 0
		ld e_dist(ix), #0
		ld e_op(ix), #0

ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Adds a puntuation to the player
;; Destroys: IY
;; Input: A -> Value died 
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pl_change_died:

	ld iy, #player1_data
	ld pl_died(iy), a
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Sets the direction of the player
;; Destroys: A, B 
;; Input:
;;	B -> Direction to set 
;; Output:
;;	A -> 0 Move
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pl_check_dir:

	;; If the player is already looking, then we return 0 on A
	ld a, e_dir(ix)
	sub b
	cp #0
	ret z

	;; Here we change to the direction
	ld a, b
	ld e_dir(ix), a

	;; And the we change the sprite
	;; Looking up
	cp #0
	jr nz, not_look_up 
	ld hl, #_player_sp_3
	ld de_sp_h(ix), h
	ld de_sp_l(ix), l
	jr look_setted
	not_look_up:

	;; Looking right
	cp #1
	jr nz, not_look_right
	ld hl, #_player_sp_5
	ld de_sp_h(ix), h
	ld de_sp_l(ix), l
	jr look_setted
	not_look_right:

	;; Looking down
	cp #2
	jr nz, not_look_down 
	ld hl, #_player_sp_1
	ld de_sp_h(ix), h
	ld de_sp_l(ix), l
	jr look_setted
	not_look_down:

	;; Looking left
	cp #3
	jr nz, not_look_left
	ld hl, #_player_sp_7
	ld de_sp_h(ix), h
	ld de_sp_l(ix), l
	jr look_setted
	not_look_left:

	look_setted:

	ld e_del(ix), #player1_dir_del

	ld a, #-1

ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Checks if the keyboard is being pressed and sets the movement to player 1
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ent_movKeyboard1:
	

	;; This delay makes that the player can move fluid between matrix positions 
	ld a, e_mov(ix)
	cp #0
	jp nz, moving

			ld a, e_mx(ix)
			ld b, e_my(ix) 
			ld e_ma_x(ix) , a
			ld e_ma_y(ix) , b

			ld a, e_op(ix)
			cp #0
			jp nz, break_or_destroy


				;; Each time the player stops pressing a direction key the entity will stop moving
				ld e_vx(ix), #0
				ld e_vy(ix), #0

				ld a, e_del(ix)
				cp #0
				jr z, not_low_delay

					dec a
					ld e_del(ix), a
					ret

				not_low_delay:

				;; Check if the Key Space is pressed
				ld hl, #Key_Space
				call cpct_isKeyPressed_asm  					;; Returns 0 if "HL" not pressed
				jr z, space_not_pressed
					ld a, e_del(ix)
					cp #0
					jr nz, skip_fire
					call player_fire

					skip_fire:
					ret

				space_not_pressed:




				;; Check if the Key W is pressed
				ld hl, #Key_W
				call cpct_isKeyPressed_asm  					;; Returns 0 if "HL" not pressed
				jr z, w_not_pressed

					ld b, #0									;; Sets the player looking to the right
					call pl_check_dir
					cp #0
					ret nz

					call pl_movePlayerFdir
					jp somthing_pressed

				w_not_pressed:


				;; Check if the Key D is pressed
				ld hl, #Key_D
				call cpct_isKeyPressed_asm  					;; Returns 0 if "HL" not pressed
				jr z, d_not_pressed

					ld b, #1									;; Sets the player looking to the right
					call pl_check_dir
					cp #0
					ret nz

					call pl_movePlayerFdir
					jp somthing_pressed

				d_not_pressed:

				;; Check if the Key S is pressed
				ld hl, #Key_S
				call cpct_isKeyPressed_asm  					;; Returns 0 if "HL" not pressed
				jr z, s_not_pressed

					ld b, #2									;; Sets the player looking to the right
					call pl_check_dir
					cp #0
					ret nz

					call pl_movePlayerFdir
					jr somthing_pressed

				s_not_pressed:


				;; Check if the Key A is pressed
				ld hl, #Key_A
				call cpct_isKeyPressed_asm  					;; Returns 0 if "HL" not pressed
				jr z, a_not_pressed

					ld b, #3									;; Sets the player looking to the right
					call pl_check_dir
					cp #0
					ret nz

					call pl_movePlayerFdir
					jr somthing_pressed

				a_not_pressed:



				;; Control the code from the joysticks

				;; Check if the Key P is pressed
				ld hl, #Joy0_Fire1
				call cpct_isKeyPressed_asm  					;; Returns 0 if "HL" not pressed
				jr z, joyfire_not_pressed
					ld a, e_del(ix)
					cp #0
					jr nz, skip_fire1
					call player_fire

					skip_fire1:
					ret


				joyfire_not_pressed:


				;; Check if the Key W is pressed
				ld hl, #Joy0_Up
				call cpct_isKeyPressed_asm  					;; Returns 0 if "HL" not pressed
				jr z, joyw_not_pressed

					ld b, #0									;; Sets the player looking to the right
					call pl_check_dir
					cp #0
					ret nz

					call pl_movePlayerFdir
					jr somthing_pressed

				joyw_not_pressed:


				;; Check if the Key D is pressed
				ld hl, #Joy0_Right
				call cpct_isKeyPressed_asm  					;; Returns 0 if "HL" not pressed
				jr z, joyd_not_pressed

					ld b, #1									;; Sets the player looking to the right
					call pl_check_dir
					cp #0
					ret nz

					call pl_movePlayerFdir
					jr somthing_pressed

				joyd_not_pressed:

				;; Check if the Key S is pressed
				ld hl, #Joy0_Down
				call cpct_isKeyPressed_asm  					;; Returns 0 if "HL" not pressed
				jr z, joys_not_pressed

					ld b, #2									;; Sets the player looking to the right
					call pl_check_dir
					cp #0
					ret nz

					call pl_movePlayerFdir
					jr somthing_pressed

				joys_not_pressed:


				;; Check if the Key A is pressed
				ld hl, #Joy0_Left
				call cpct_isKeyPressed_asm  					;; Returns 0 if "HL" not pressed
				jr z, joya_not_pressed

					ld b, #3									;; Sets the player looking to the right
					call pl_check_dir
					cp #0
					ret nz

					call pl_movePlayerFdir
					jr somthing_pressed

				joya_not_pressed:
				nothing_pressed:
				ld a, #-1
				
				;; This makes that the player will be able to execute one or less key function on each iteration
				somthing_pressed:

					cp #0
					jr z, will_move

						ld e_mov(ix), #0
						ld e_del(ix), #0
						ret

					will_move:
				jp ent_move

			break_or_destroy:

				ld a, e_block(ix)
				cp #0
				jr nz, animating

					ld e_block(ix), #obstacle_bc
					ret

				animating:
					dec a
					ld e_block(ix), a
					cp #0
					ret nz

					;; Here is the final of the animation, so we break the obstacle

					;; Checks if it's creating or destroying, if that's so, then we don't need to do the rest of the update
					ld a, e_op(ix)
					cp #1
					jp z, ent_createRowObstacle
					cp #-1
					jp z, ent_destroyRowObstacle


			ret

	moving:
	ld a, e_del(ix)
	cp #0
	jr nz, player_delaying

		ld e_del(ix), #player1_del
		;; This allows the delay to have a duration expressed on iterations of this method
		ld a, e_mov(ix)
		cp #2
		jr nz, continue_mov

			ex af, af'
			ld a, #0
			ld b, #1
			call cambiar_sprite

			ex af, af'

		continue_mov:
		dec a
		ld e_mov(ix), a

		jp ent_move

	player_delaying:
	dec a
	ld e_del(ix), a


ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Actions to do when a player press Fire
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
player_fire:
	ld b, e_mx(ix)
	ld c, e_my(ix)
	ld d, #1
	call ent_setObstacleValues
	call sec_retValue
	

	ld a, e
	cp #player1
	jp p, #set_player_breaking
		ld e_op(ix), #1
		ld e_block(ix), #player1_block 							;; We declare that we are destroying obstacles
		ret
	

	set_player_breaking:
		ld e_op(ix), #-1 
		ld e_block(ix), #player1_block							;; We declare that we are creating obstacles
		ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Actions to do when a player press Fire
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pl_movePlayerFdir:
	;; We set the delay again to move between matrix sections
	ld e_mov(ix), #player1_mov
	ld e_del(ix), #player1_del

	ld a, e_dir(ix)


	cp #0
	jr nz, not_looking_up
		ld e_vy(ix), #-04 	;; Sets the velocity to move one pos of the matrix to the right
		ld hl, #_player_sp_3
		ld de_sp_h(ix), h
		ld de_sp_l(ix), l
		jp ent_upDir
	not_looking_up:

	cp #1
	jr nz, not_looking_right
		ld e_vx(ix), #1 	;; Sets the velocity to move one pos of the matrix to the right
		ld hl, #_player_sp_5
		ld de_sp_h(ix), h
		ld de_sp_l(ix), l
		jp ent_rightDir
	not_looking_right:

	cp #2
	jr nz, not_looking_down
		ld e_vy(ix), #4 	;; Sets the velocity to move one pos of the matrix to the right	
		ld hl, #_player_sp_1
		ld de_sp_h(ix), h
		ld de_sp_l(ix), l
		jp ent_downDir
	not_looking_down:

	cp #3
	jr nz, not_looking_left
		ld e_vx(ix), #-1 	;; Sets the velocity to move one pos of the matrix to the right
		ld hl, #_player_sp_7
		ld de_sp_h(ix), h
		ld de_sp_l(ix), l
		jp ent_leftDir
	not_looking_left: