;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Harvest Day: An Amstrad CPC Game 
;;  Copyright (C) 2018 Álvaro Molina García / José Manuel Ladrón de Guevara García, Daniel Saura Martínez
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
;; Includes

	.include "render.h.s"
	.include "main.h.s"

;; Declarations

	back_buffer: .db 0x80 	;; This is the video memory
	front_buffer: .db 0xC0 	;; This is an extra video memory to avoid flickering (twice buffer concept)

;; Public Methods

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Clears the drawable entity with the anterior anterior position
;; Destroys: AF, BC, DE, HL
;; Input: 
;; A -> Load back buffer->1 / Load front buffer other number
;; B -> Y anterior anterior value
;; C -> X anterior anterior value
;; Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_clearAnteriorAnteriorEntity:


	cp #1
	jr nz, cargar_front

		ld a, (back_buffer) 						;; We'll draw the entity on the backbuffer (can be on 8000 or C000)
		jr continuar_clearAA

	cargar_front:
	ld a, (front_buffer)	
	continuar_clearAA:

	ld d, a
	ld e, #0x00 								;; DE = Pointer to the start of the screen
	push bc
	call cpct_getScreenPtr_asm  				;; Gets on HL the pointer to x,y position on Hx(Hexadecimal)

	pop bc
	push hl
	call ren_calculateMapPtr 					;; Gets on HL the pointer to the map position
	ex de, hl 									;; DE = Pointer to the start part of the map that we want to draw
	pop hl 										;; HL = Pointer to the part of the window where we want to draw

	push ix
 	call cpct_etm_drawTilemap4x8_ag_asm  		;; Draw a part of the map 
 	pop ix
ret




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Draws the drawable entity
;; Destroys: AF, BC, DE, HL
;; Input: 
;; Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_draw:

	ld d, a
	ld e, #0x00 								;; DE = Pointer to the start of the screen
	ld c, de_x(ix) 								;; B = Entity X
	ld b, de_y(ix) 								;; C = Entity Y 
	call cpct_getScreenPtr_asm  				;; Gets on HL the pointer to x,y position on Hx(Hexadecimal)

	ld b, de_h(ix) 								;; Height of the Drawable Entity
	ld c, de_w(ix)								;; Width of the Drawable Entity
	ex de, hl 									;; DE = Pointer of the screen to the up left corner of our entity
	ld h, de_sp_h(ix)
	ld l, de_sp_l(ix)
	jp cpct_drawSprite_asm



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Draws the drawable entity in the back buffer
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_drawEntity:
	ld a, (back_buffer) 						;; We'll draw the entity on the backbuffer (can be on 8000 or C000)
	jp ren_draw



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Draws the drawable entity in the front buffer
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_drawFrontEntity:
	ld a, (front_buffer)
	jp ren_draw



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Draws an obstacle on the back buffer
;; Destroys: AF, BC, DE, HL
;; Input: 
;; B -> Height
;; C -> Width
;; D -> X position
;; E -> Y position
;; HL -> Sprite
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_drawBackObstacle:

	cp #0
	jp z, ren_clearObstacle
	push hl
	push bc
	ld b, e 	;; B = Entity Y -> E
	ld c, d 	;; C = Entity X -> D

	ld a, (back_buffer) 						;; We'll draw the entity on the backbuffer (can be on 8000 or C000)
	ld d, a
	ld e, #0x00 								;; DE = Pointer to the start of the screen
	call cpct_getScreenPtr_asm  				;; Gets on HL the pointer to x,y position on Hx(Hexadecimal)

	pop bc 										;; Height and width
	ex de, hl 									;; DE = Pointer of the screen to the up left corner of our entity
	pop hl 										;; HL = sprite
	jp cpct_drawSprite_asm



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Draws an obstacle on the front buffer
;; Destroys: AF, BC, DE, HL
;; Input: 
;; B -> Height
;; C -> Width
;; D -> X position
;; E -> Y position
;; HL -> Sprite
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_drawFrontObstacle:

	cp #0
	jp z, ren_clearObstacle
	push hl
	push bc
	ld b, e 	;; B = Entity Y -> E
	ld c, d 	;; C = Entity X -> D

	ld a, (front_buffer) 						;; We'll draw the entity on the backbuffer (can be on 8000 or C000)
	ld d, a
	ld e, #0x00 								;; DE = Pointer to the start of the screen
	call cpct_getScreenPtr_asm  				;; Gets on HL the pointer to x,y position on Hx(Hexadecimal)

	pop bc 										;; Height and width
	ex de, hl 									;; DE = Pointer of the screen to the up left corner of our entity
	pop hl 										;; HL = sprite
	jp cpct_drawSprite_asm


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Switches the 2 buffers
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ren_newScene:

	jp ren_switchBuffers


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Inits the buffer 0x8000
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ren_initBuffers:

	ld hl, #0x8000
	ld (hl), #0
	ld de, #0x8000 + 1
	ld bc, #0x4000 - 1
	ldir

ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Draw the map in both buffers
;; Destroys: AF, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

drawMap:

   cp #1
   jr z, drawBackMap

   		ld a, (front_buffer)
   		jr fin_drawMap

   drawBackMap:
   ld a, (back_buffer)

   fin_drawMap:
   ld h, a 
   ld l, #0

   jp cpct_etm_drawTilemap4x8_ag_asm  

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Clear the position of the obstacle
;; Destroys: AF, BC, DE, HL
;; Input: 
;; E -> Y screen position
;; D -> X screen position
;; Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


ren_clearObstacle:

	ld b, e 	;; B = Entity Y -> E
	ld c, d 	;; C = Entity X -> D
	push bc 
	ld a, #1
	call ren_clearAnteriorAnteriorEntity
	ld a, #2
	pop bc 
	jp ren_clearAnteriorAnteriorEntity

;; Private Methods


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Calculate the pointer to the map position
;; Destroys: AF, BC, DE, HL
;; Input: 
;; B -> Y screen position
;; C -> X screen position
;; Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_calculateMapPtr:

	ld hl, #tilemap_ptr
	ld de, #230
	add hl, de 

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Clears the back buffer
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ren_clearBackBuffer:

	ld a, (back_buffer)
	jr ren_clearBuffer

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Clears the front buffer
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ren_clearFrontBuffer:

	ld a, (front_buffer)
	jr ren_clearBuffer

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Clears a buffer
;; Destroys: AF, BC, DE, HL
;; Input: 
;; A -> Buffer
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ren_clearBuffer:

	ld h, a 
	ld de, #0
	ld l, e 
	ld bc, #0x4000
	call cpct_memset_f64_asm

ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Switches the direction between the two buffers
;; Destroys: AF, BC, DE, HL
;; Input: VOID
;; Output: VOID
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ren_switchBuffers:

	ld a, (back_buffer)
	ld b, a
	ld a, (front_buffer)
	ld (back_buffer), a 
	ld a, b 
	ld (front_buffer), a

	srl b 
	srl b 
	ld l, b

jp cpct_setVideoMemoryPage_asm 
