;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Harvest Day: An Amstrad CPC Game 
;;  Copyright (C) 2018 Álvaro Molina García / José Manuel Ladrón de Guevara García, Daniel Saura Martínez
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
;; Includes

	.include "section.h.s"


;; Declarations

	matrix_data:
	DefineMatrix section, rows, columns


;; Public methods


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Sets a value on a section of the matrix 
;; Destroys: A, BC, DE, HL
;; Input:
;; B -> X column 
;; C -> Y row
;; D -> Value
;; Output: 
;; E -> -1 	If the section is out of bounds
;; E ->  0  If the section is NOT out of bounds
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sec_setValue:

	;; The method sec_getPtr destroys DE so we'll save that value on the stack to use it later
	push de

	;; We need the pointer to the section on HL so...
	call sec_getPtr
	
	

	ld a, e
	pop de
	cp #0
	jr z, correctly_setted
		ld e, #-1
	ret

	correctly_setted:
		;; Now we only change the value and done, easy as that
		ld (hl), d

	ret




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Rets the value of a section of the matrix 
;; Destroys: A, BC, DE, HL
;; Input:
;; B -> X column 
;; C -> Y row
;; Output: 
;; E -> -1 	If the section is out of bounds
;; E ->  Value  If the section is NOT out of bounds
;; HL -> Pointer to the position of the matrix changed
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sec_retValue:
	
	;; We need the pointer to the section in HL to obtain the value of that section
	call sec_getPtr

	;; Put on E(Output) the value of that section if we achived the right pointer
	ld a, e
	cp #0
	jr nz, getPtr_fail

		ld e, (hl)
	ret

	getPtr_fail:

		ld e, #-1

ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Converts the X and Y of the matrix to the X and Y of the screen
;; Destroys: A, BC, DE
;; Input:
;; B -> X column 
;; C -> Y row
;; Output: 
;; D -> X Screen value
;; E -> Y Screen value
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sec_matrixToScreen:

	;; We have to init the values of the registers that will be used
	ld de, #0x0000

	convert_X:

		;; Now check if it's on the first column
		ld a, b
		cp #0
		jr z, convert_Y

		ld a, #0
		ld d, #section_w
		loop_convert_X:
			add d
			dec b
		jr nz, loop_convert_X
		ld d, a


	convert_Y:
		;; Now check if it's on the first column
		ld a, c
		cp #0
		ret z

		ld a, #0
		ld e, #section_h
		loop_convert_Y:
			add e
			dec c
		jr nz, loop_convert_Y
		ld e, a

ret

;; Private methods


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition: Gets the pointer of the section "X", "Y"
;; Destroys: A, BC, DE, HL
;; Input:
;; B -> X column 
;; C -> Y row
;; Output: 
;; E -> -1 	If the section is out of bounds
;; HL -> #0x4000  If the section is out of bounds
;;
;; E -> 0 	If the section is NOT out of bounds
;; HL -> Direction  If the section is NOT out of bounds
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sec_getPtr:

	;; HL will be the offset, so we set it to 0 to calculate the direction relatively(only adding values, not setting) 
	ld hl, #0x0000

	;; If X < 0 -> Out of bounds
	ld a, b
	cp #0
	jp m, out_of_bounds

	;; If X > (max X -1) -> Out of bounds
	cp #columns
	jp p, out_of_bounds


	;; If Y > (max Y -1) -> Out of bounds
	ld a, c
	cp #rows
	jp p, out_of_bounds


	;; If Y < 0 -> Out of bounds
	cp #0
	jr z, add_columns 	;; If we are on the first row we pass to calculate only the columns
	jp m, out_of_bounds


	;; At this point the positions are correct

	
	
	;; Calculate the total direction to the section on the matrix
	;; Adds "columns" * "Y" to HL
	add_rows:
		ld d, #0x00
		ld e, #columns
		row_loop:
			add hl, de
			dec c
		jr nz, row_loop

	;; Adds "X" to HL
	add_columns:
		ld d, #0x00
		ld e, b
		add hl, de

	ex de, hl
	;; We got on DE the offset of matrix_data now we add it to HL
	ld hl, #matrix_data
	add hl, de

	;; Now we got HL to work with it later
	ld e, #0

ret

	;; This means the position X and Y it's in the matrix
	out_of_bounds:

	ld e, #-1
	ld hl, #0x4000

ret