;;-----------------------------LICENSE NOTICE------------------------------------
;;  Deepers Warrens
;;  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.module assembly
;; Adapted from cpct_drawSprite (CPCtelera)

.macro dec__ixh
   .dw #0x25DD  ;; Opcode for dec ixh
.endm

.macro ld__ixh    Value 
   .db #0xDD, #0x26, Value  ;; Opcode for ld ixh, Value
.endm

;; void asm_drawMapTile(u8* sprite, u8* memory, u8 height) __z88dk_callee;
_asm_drawMapTile::
   ld (drawTile_restore_ix + 2), ix  ;; [6] Save IX to restore it before returning

   ;; Get Parameters from the Stack
   pop  af          ;; [3] AF = Return Address
   pop  de          ;; [3] DE = Sprite Address
   pop  hl          ;; [3] HL = Video Memory Destination Address
   dec  sp          ;; [1] SP-- argument Height is one Byte
   pop  ix          ;; [4] IXH = Tile Height / IXH = Useless
   push af          ;; [4] AF = Returning back address in the stack because function uses __z88dk_callee convention
   
drawTile_height_loop:
   ; HL = SRC ; DE = DEST
   ex   de, hl      ;; [1] HL and DE are exchanged every line to do 16-bits maths with DE                   
   ld   bc, #0x0800 ;; [3] 0x800 bytes is the distance in memory from one pixel line to the next within every 8 pixel lines
								
   ldi              ;; [5] Copy Tile Width = 4-Bytes 
   ldi              ;; [5] | (DE++) = (HL++)
   ldi              ;; [5] | BC--
   ldi              ;; [5] |
  
   dec__ixh          ;; [2] IXL holds sprite height. We decrease it to count another pixel line finished
   jp   z, drawTile_restore_ix ;; [2/3] If 0, we have finished the last sprite line.
   
   ex   de, hl      ;; [1] DE has destination, but we have to exchange it with HL to be able to do 16-bits maths
   add  hl, bc      ;; [3] We add 0x800 minus the width of the sprite (BC) to destination pointer 
   
   ld   a, h        ;; [1] We check if we have crossed video memory boundaries (which will happen every 8 lines). 
   and  #0x38       ;; [2] leave out only bits 13,12 and 11 from new memory address (00xxx000 00000000)
   jp   nz, drawTile_height_loop ;; [2/3]  by checking the 4 bits that identify present memory line. 

   ld   bc, #0xC050           ;; [3] BC = 0xC050 offset next line
   add  hl, bc                ;; [3] HL += BC (0xC050)
   jp  drawTile_height_loop   ;; [3] Jump to continue with next pixel line
   
	
;; void asm_drawFullMapTile(u8* sprite, u8* memory) __z88dk_fastcall;
_asm_drawFullMapTile::
   ld (drawTile_restore_ix + 2), ix  ;; [6] Save IX to restore it before returning

   pop  af          ;; [3] AF = Return Address
   pop  de          ;; [3] DE = Sprite Address
   pop  hl          ;; [3] HL = Video Memory Destination Address
   push af          ;; [4] AF = Returning back address in the stack because function uses __z88dk_callee convention
   
   ld__ixh  #16     ;; [3] IXH = Tile Height (16)
   
drawFullTile_height_loop:
   ; HL = SRC ; DE = DEST
   ex   de, hl      ;; [1] HL and DE are exchanged every line to do 16-bits maths with DE                   
   ld   bc, #0x0800 ;; [3] 0x800 bytes is the distance in memory from one pixel line to the next within every 8 pixel lines
								
   ldi              ;; [5] Copy Tile Width = 4-Bytes 
   ldi              ;; [5] | (DE++) = (HL++)
   ldi              ;; [5] | BC--
   ldi              ;; [5] |
  
   dec__ixh          ;; [2] IXL holds sprite height. We decrease it to count another pixel line finished
   jp   z, drawTile_restore_ix ;; [2/3] If 0, we have finished the last sprite line.
   
   ex   de, hl      ;; [1] DE has destination, but we have to exchange it with HL to be able to do 16-bits maths
   add  hl, bc      ;; [3] We add 0x800 minus the width of the sprite (BC) to destination pointer 
   
   ld   a, h        ;; [1] We check if we have crossed video memory boundaries (which will happen every 8 lines). 
   and  #0x38       ;; [2] leave out only bits 13,12 and 11 from new memory address (00xxx000 00000000)
   jp   nz, drawFullTile_height_loop ;; [2/3]  by checking the 4 bits that identify present memory line. 

   ld   bc, #0xC050           ;; [3] BC = 0xC050 offset next line
   add  hl, bc                ;; [3] HL += BC (0xC050)
   jp  drawFullTile_height_loop   ;; [3] Jump to continue with next pixel line	
   
   
 drawTile_restore_ix:
	ld   ix, #0000    		  ;; [4] Placeholder for restoring IX value before returning   
	ret                       ;; [3] Return