;;-----------------------------LICENSE NOTICE------------------------------------
;;  Deepers Warrens
;;  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.module assembly

;; Adapted from cpct_memset_f64 (CPCtelera)
;; void asm_fillView(u8* backBuffer) __z88dk_fastcall;
_asm_fillView::	
	;; Parameter *memory* is directly given in HL register, using __z88dk_fastcall convention		  			

	;; Not fill Border Right 4-bytes 
	ld   bc, #0x0008               ;; [3] BC = 4-bytes Border Width
	sbc  hl, bc                    ;; [4] HL = HL - BC
   
	ld   de, #0x01C0               ;; [3] DE = 0x640 + Next PixelLine End (0x01C0) = 0x800
	ld   c,  #0x50                 ;; [2] BC = 80-bytes ScanLine (0x0050)
	ld   a,  #08                   ;; [2] A = Number of Character per line (8)
   
	exx                            ;; [1] Switch to Alternate Registers
	ld   de, #0x0000               ;; [3] DE' = Color Background 
	ld   c,  #5                    ;; [2] C' = Character Lines to fill (5)		
	ld   b,  c                     ;; [1] B' = C'

fillLoop:
	;; Save SP to restore it later, as this function makes use of it
	di                             ;; [1] Disable interrupts first
	ld   (fv_restore_sp + 1), sp   ;; [5] Save SP to recover it later on

	ex   af', af'     ;; [1] AF' <-> AF
	ld   a,   #4      ;; [2] 4 scanlines to fill	
	
fillScanline:
	exx               ;; [1] Switch to Standard Registers

	;; Move SP to the end of the array       
	add  hl, bc       ;; [3] HL += BC (0x50) Start CharacterLine (HL points to the end of the array)
	ld   sp, hl       ;; [2] SP = HL (SP points to the end of the array)     
	
	exx               ;; [1] Switch to Alternate Registers
	
	;; Fill Center View 72-bytes 
	push de           ;; [4] Push 8-bytes
	push de           ;; [4]  
	push de           ;; [4]
	push de           ;; [4]
	
	push de           ;; [4] Push 8-bytes
	push de           ;; [4]  
	push de           ;; [4]
	push de           ;; [4]  

	push de           ;; [4] Push 8-bytes
	push de           ;; [4]  
	push de           ;; [4]
	push de           ;; [4]

	push de           ;; [4] Push 8-bytes
	push de           ;; [4]  
	push de           ;; [4]
	push de           ;; [4]

	push de           ;; [4] Push 8-bytes
	push de           ;; [4]  
	push de           ;; [4]
	push de           ;; [4]
	
	push de           ;; [4] Push 8-bytes
	push de           ;; [4]  
	push de           ;; [4]
	push de           ;; [4]  

	push de           ;; [4] Push 8-bytes
	push de           ;; [4]  
	push de           ;; [4]
	push de           ;; [4]
	
	push de           ;; [4] Push 8-bytes
	push de           ;; [4]  
	push de           ;; [4]
	push de           ;; [4]
	
	;; Not fill Border Left 4-bytes 
	
	;; Counter fill 4 scanlines
	dec a                ;; [1] A'--
	jr  nz, fillScanline ;; [2/3] A !=0 

fv_restore_sp:
	ld   sp, #0000    ;; [3] Placeholder for restoring SP value before returning
	ei                ;; [1] Reenable interrupts	
	
	ex   af', af'     ;; [1] AF <-> AF'
	
	djnz fillLoop     ;; [3/4] B'-- less 4 ScanLines to fill		
	ld   b,  c        ;; [1] B' = C' Character Lines to fill (5)
	
	exx               ;; [1] Switch to Default Registers
	add  hl, de       ;; [3] HL += DE (Next PixelLine End)
	exx               ;; [1] Switch to Alternate Registers
	
	dec  a            ;; [1] A-- Number of Character per line (8)
	jr   nz, fillLoop ;; [2/3]
	
	exx               ;; [1] Switch to Default Registers
	ret               ;; [3] Return  