;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Deeper Warrens
;;  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
;;
;;  This file is part of CPCtelera: An Amstrad CPC Game Engine 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.module assembly

;; cpct_hflipSpriteM0 from CPCTelera 1.5.0

.macro cpctm_reverse_mode_0_pixels_of_A  TReg
   rlca            ;; [1] | Rotate A twice to the left to get bits ordered...
   rlca            ;; [1] | ... in the way we need for mixing, A = [23456701]
  
   ;; Mix TReg with A to get pixels reversed by reordering bits
   xor TReg        ;; [1] | TReg = [01234567]
   and #0b01010101 ;; [2] |    A = [23456701]
   xor TReg        ;; [1] |   A2 = [03254761]
   rrca            ;; [1] Rotate right to get pixels reversed A = [10325476]
.endm

_cpct_hflipSpriteM0::
   ;; Parameter retrieval from stack
   pop  hl     ;; [3] HL = return address
   pop  bc     ;; [3] BC = height / width of the sprite
   ex (sp), hl ;; [6] HL = Sprite start address pointer, while leaving return address 
   ;; ... in the stack, as this function uses __z88dk_callee convention

   ld_b_hl     = 0x46
   jr_one      = 0x18 + 0x100 * (          one  - (varjump + 3))
   jr_byteloop = 0x18 + 0x100 * ((nextbyte + 1) - (varjump + 2))
   
   dec hl             ;; [2] HL points to the byte previous to the start of the sprite array

   ;; Divide sprite width by 2 and check if there is remainder to know if the 
   ;; sprite is odd-wide (if remainder, then Carry = 1)
   srl c              ;; [2]   C = C / 2 (Sprite width / 2)
   jr  c, odd_width   ;; [2/3] Carry = 1, means Odd sprite width

even_width:
   ld  de, #jr_byteloop ;; [3] DE = Opcode for "JR nextbyte+1" operation, to jump to the byte loop
   ld  (varjump), de    ;; [6] Insert "JR nextbyte+1" at varjump location
   jr nextrow           ;; [3] Start processing the loop for the first 2 bytes
   
odd_width: 
   ld   a, #ld_b_hl   ;; [2] A = Opcode for "LD B, (HL)" operation
   ld (varjump), a    ;; [4] Insert "LD B, (HL)" at varjump location
   ld  de, #jr_one    ;; [3] DE = Opcode for "JR one" operation
   ld (varjump+1), de ;; [6] Insert "JR one" at varjump+1 location (right after LD B, (HL))
   inc c              ;; [1] C++ (C = Width + 1, as reversing the central byte will require 1 more looping...
                      ;;     ... and also the central byte will be at [HL + Width/2 + 1] )
nextrow:
   push bc      ;; [4] Save Height/Width into the stack for later recovery
   ld   b, #0   ;; [2] BC  = C = Width/2 (In order to easily add the width to HL)
   add  hl, bc  ;; [3] HL += BC (Add Width/2 to start-of-row pointer, HL, to make it point to the central byte)
   ld   d, h    ;; [1] | DE = HL, so that DE also points to the central byte of the row
   ld   e, l    ;; [1] | 

varjump:
   ;; Odd-width varjump code (3-bytes, longer one)
   ld   b, (hl) ;; [2] B = central byte of the row, ready to be reversed
   jr   one     ;; [3] Jump to the second part of the internal loop, to reverse only the central byte

nextbyte:
   dec de          ;; [2] DE-- (Move DE from the central byte to the first of the row)
   inc hl          ;; [2] HL++ (Move HL from the central byte to the last of the row)

   ;; Flip byte pointed by DE
   ld a, (de)      ;; [2] A = Byte pointed by DE (2 pixels)
   ld b, a         ;; [1] B = A, Copy of A, required by reverse macro
   
   ;; Reverse (flip) both pixels contained in A (using B as temporary storage)
   cpctm_reverse_mode_0_pixels_of_A b ;; [7] 

   ;; Switch DE flipped byte and HL byte to be flipped
   ld b, (hl)      ;; [2] B = Byte pointed by HL (2 pixels)
   ld (hl), a      ;; [2] Store flipped byte from (DE) at (HL)

   ;; Flip byte pointed by HL and store it at (DE)
one:
   ld a, b         ;; [1] A = B = Copy of byte pointed by HL (2 pixels)

   ;; Reverse (flip) both pixels contained in A (using B as temporary storage)
   cpctm_reverse_mode_0_pixels_of_A b ;; [7]

   ld (de), a      ;; [2] Store flipped byte from (HL) at (DE)

   ;; C is the counter of pairs of bytes to be flipped and switched. 
   dec c           ;; [1]   C-- (one less pair of bytes to be flipped and switched)
   jr nz, nextbyte ;; [2/3] If C != 0, there are still bytes to be flipped and switched, so continue


   ;; All bytes in the present row have been flipped and switched. 
   ;; Recover BC (Height/Width) form the stack, decrement height and check if there are
   ;; still more rows to be flipped
   pop  bc      ;; [3]   BC contains (H: height, L: width of the sprite)
   djnz nextrow ;; [3/4] B-- (Height--). If B!=0, there are still more rows pending, so continue

   ret          ;; [3] All sprite rows flipped. Return.