;;-----------------------------LICENSE NOTICE------------------------------------
;;  Deepers Warrens
;;  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;; Credits:
;;    Original routine optimized by @Docent and discussed in CPCWiki :
;; http://www.cpcwiki.eu/forum/programming/cpctelera-colorize-sprite/
;;
;; Thanks to all of them for their help and support.
;;-------------------------------------------------------------------------------
.module assembly

dc_mode0_colorize:: .db 0x00, 0x40, 0x04, 0x44, 0x10, 0x50, 0x14, 0x54, 0x01, 0x41, 0x05, 0x45, 0x11, 0x51, 0x15, 0x55

;; Macro to convert Pixel to xAxC xBxD format
.macro convertPixel              
	;; From cpct_px2byteM0
	ld   bc, #dc_mode0_colorize  ;; [3] BC points to conversion table (dc_mode0_ct)
	
	;; Compute BC += A
	add  c                 ;; [1] | C += A
	ld   c, a              ;; [1] |
	ld   a, #0             ;; [2] A = 0 (preserving Carry Flag)
	adc  b                 ;; [1] | B += Carry
	ld   b, a              ;; [1] |

    ;; A = *(BC + A)
    ld   a, (bc)           ;; [2] A = Value stored at the table pointed by BC
.endm

;; Macro: ld__ixl_c from CPCTelera
;;    Opcode for "LD IXL, C" instruction
;; 
.macro ld__ixl_c
   .DW  #0x69DD  ;; ld ixl, c 
.endm

;; Macro: ld__b_ixl from CPCTelera
;;    Opcode for "LD B, IXL" instruction
;; 
.macro ld__b_ixl
   .DW  #0x45DD    ;; ld b, ixl
.endm

_asm_spriteColorize::

	;; GET Parameters from the stack 
	ld (sprite_colorize_restore_ix + 2), ix  ;; [6] Save IX to restore it before returning
	pop   hl                     ;; [3] HL = Return Address

	exx 
	pop   hl                     ;; [3] HL' = Source address (Sprite)
	pop   de                     ;; [3] DE' = Destination Sprite color
	pop   bc                     ;; [5] BC' = (B = Sprite Height, C = Width)
	exx

	ex   (sp), hl                ;; [6] HL = (H = newColor, L = oldColor)
								;; ... and leave Return Address at (SP) as we don't need to restore
								;; ... stack status because callin convention is __z88dk_callee

	;; Convert newColor to pixel format (E)
	ld a, h                ;; [1]  A = H new color index
	convertPixel	       ;; [10] | Convert into A
	ld e, a                ;; [1]  | E = A new color      : xAxC xBxD

	;; Convert oldColor to pixel format (D)
	ld a, l                ;; [1]  A = L old color index
	convertPixel	       ;; [10] | Convert into A
	ld d, a                ;; [1]  | D = A old color      : xAxC xBxD
	
	ld c, #0x55            ;; [2] C = Mask to get pixel A : xAxC xBxD
	exx                    ;; [1] Switch to Alternate registers
	
	ld__ixl_c              ;; [1] IXL = C (Width)
	ld c, b                ;; [1] C = B (Height)
	
convertLoop:	
	ld__b_ixl              ;; [2] B = IXL (Sprite Width)
	
lineLoop:
	ld	a, (hl)            ;; [2] A = (HL) current Byte of sprite
    exx                    ;; [1] Switch to Default registers
	
	ld  l, a               ;; [1] L = A current Byte of sprite : ABAB ABAB
	and c                  ;; [2] A |= C (C = 0x55)            : xBxB xBxB
	
	cp  d                  ;; [1] Test if pixel (A) is the old colour to be replaced (D)
	jr 	nz, readPixelA     ;; [2/3] If not equal go to next pixel 
        ld  a, e           ;; [1] else A = new colour to set (E)

;; Pixel Mode 0 = ABAB ABAB
readPixelA:
	ld  h, a               ;; [1] H = A (current colorized sprite) : xBxB xBxB
	ld  a, l               ;; [1] L = A current Byte of sprite     : ABAB ABAB
	rrca                   ;; [1] A (current byte of sprite) >> 1  : ABAB ABAB -> xABA BABA
	and c                  ;; [2] A |= Mask (0x55)                 : xAxA xAxA
	
	cp  d                  ;; [1] Test if pixel (A) is the old colour to be replaced (D)
	jr 	nz, readPixelB     ;; [2/3] If not equal go to next pixel 
        ld  a, e           ;; [1] else A = new colour to set (E)
	
readPixelB:
	rlca                   ;; [1] A = xAxA xAxA << 1  : AxAx AxAx
	or  h                  ;; [1] A |= H (xBxB xBxB)  : ABAB ABAB
  
setByte:  
	exx                    ;; [1] Switch to Alternate registers

	ld  (de), a            ;; [2] Update current sprite byte with pixels
	inc  hl                ;; [2] Next byte sprite source
	inc  de                ;; [2] Next byte sprite colorized
	djnz lineLoop          ;; [3] Decrement B (Width) if != 0 goto lineLoop
	
nextLine:	
	dec	c                  ;; [1] Decrement C (Height) 
	jr 	nz, convertLoop    ;; [2/3] If != O goto convertLoop
	
sprite_colorize_restore_ix:
   ld   ix, #0000          ;; [4] Restore IX before returning
   ret                     ;; [3] Return to caller