//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Deeper Warrens
//  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#define BONUS_C

/** CPCTelera includes */
#include <cpctelera.h>
#include <stdio.h>

/** Structures includes */
#include "..\structures.h"
#include "..\game_const.h"

/** Game includes */
#include "..\player\player.h"
#include "..\view\view.h"

/** Tools includes */
#include "..\video\video.h"
#include "..\asm\drawMapTile.h"

/** Sprites includes */
#include "..\sprites\buffs.h"

/** Bonus include */
#include "bonus.h"

#undef BONUS_C

/** Local defines */
#define MAX_BONUS	5
#define BONUS_X		(SCREEN_CX - BONUS_CX)
#define BONUS_Y		0
#define OFFSET_BAR	2
#define SEPARATOR	2

#define OFFS_BONUS	(BONUS_CY + OFFSET_BAR + SEPARATOR)
#define BONUS_CX	GBUFFS_BUFFS_0_W
#define BONUS_CY	GBUFFS_BUFFS_0_H
#define BONUS_SIZE	(BONUS_CX*BONUS_CY)

#define BONUS_VALUE	8
#define BONUS_DEL	0

/** Local function */
void DelBonus(SBonus* bonusToDel);

/** Local data */
static SBonus _bonus[MAX_BONUS];
static u8 _nbBonus;

/*****************************************************/
/*													 */
/*	Init Bonus										 */
/*												     */
/*****************************************************/
void InitBonus()
{
	_nbBonus = 0;
}

/*****************************************************/
/*													 */
/*	Is Bonus activate								 */
/*												     */
/*****************************************************/
BOOL HandleBonus(u8 type, u8 action)
{
	SBonus* bonus = _bonus;

	for (u8 i = 0; i < _nbBonus; i++)
	{
		if (bonus->type == type)
		{		
			if (action == BONUS_EXISTING && type > PERM_BONUS)		
				bonus->value = BONUS_VALUE;					
			else
				bonus->value = BONUS_DEL;
				
			return TRUE;
		}
		bonus++;
	}
	return FALSE;
}

/*****************************************************/
/*													 */
/*	Remove Bonus									 */
/*												     */
/*****************************************************/
void RemoveBonus(u8 type)
{
	HandleBonus(type, BONUS_REMOVE);
	UpdateBonus();
}

/*****************************************************/
/*													 */
/*	Add Bonus										 */
/*												     */
/*****************************************************/
void AddBonus(u8 type)
{
	if (!HandleBonus(type, BONUS_EXISTING))
	{
		if (_nbBonus < MAX_BONUS)
		{
			SBonus* newBonus = &_bonus[_nbBonus++];
			newBonus->type = type;
			newBonus->value = BONUS_VALUE;

			gPlayerStats.status |= type;
		}
	}
}

/*****************************************************/
/*													 */
/*	Delete Bonus									 */
/*												     */
/*****************************************************/
void DelBonus(SBonus* bonusToDel)
{
	SBonus* lastBonus = &_bonus[--_nbBonus];
	gPlayerStats.status &= ~bonusToDel->type;

	if (bonusToDel != lastBonus)
		cpct_memcpy(bonusToDel, lastBonus, sizeof(SBonus));
}

/*****************************************************/
/*													 */
/*	Update Bonus									 */
/*												     */
/*****************************************************/
void UpdateBonus()
{
	if (_nbBonus > 0)
	{
		SBonus* bonus = _bonus;		
		
		for (u8 i = 0; i < _nbBonus; i++)
		{
			if (bonus->type > PERM_BONUS || bonus->value == BONUS_DEL)
			{
				if (bonus->value > 0)
					bonus->value--;
				else
				{						
					DelBonus(bonus);
					DrawSolidBoxBothBuffers(BORDER_COLOR, BONUS_X, BONUS_Y + OFFS_BONUS * _nbBonus, BONUS_CX, BONUS_CY + OFFS_BONUS);
				}
			}
			
			bonus++;
		}
	}
}

/*****************************************************/
/*													 */
/*	Get Bonus Sprite								 */
/*												     */
/*****************************************************/
u8* GetBonusSprite(u8 type)
{
	static const u8 sBonusSprite[] = 
	{ 	
		BONUS_STRENGTH, 
		BONUS_INVISIBILITY, 
		MALUS_POISON, 
		PERM_PROTECTION,
		PERM_RESURRECTION,
		PERM_SWORD_MASTER, 
		PERM_THROW_MASTER 
	};

	for (u8 i = 0; i < sizeof(sBonusSprite); i++)
	{
		if (sBonusSprite[i] == type)
			return gBuffs_tileset[i];
	}

	return NULL;
}

/*****************************************************/
/*													 */
/*	Draw Bonus										 */
/*												     */
/*****************************************************/
void DrawBonus()
{
	if (_nbBonus > 0)
	{	
		SBonus* bonus = _bonus;

		for (u8 i = 0; i < _nbBonus; i++)
		{
			u8 yPos = BONUS_Y + OFFS_BONUS * i;

			u8* bonusSprite = GetBonusSprite(bonus->type);
			u8* vmem = GetBackScreenPtr(BONUS_X, yPos + OFFS_BONUS - SEPARATOR);

			asm_drawMapTile(bonusSprite, GetBackScreenPtr(BONUS_X, yPos), BONUS_CY);
						
			if (bonus->type > PERM_BONUS)
				DrawLine(vmem, bonus->value, BONUS_VALUE, COLOR_BRIGHT_YELLOW, COLOR_BRIGHT_RED);
			else
				cpct_memset(vmem, cpctm_px2byteM0(COLOR_BLACK, COLOR_BLACK), BONUS_VALUE/2);

			bonus++;
		}
	}
}

/*****************************************************/
/*													 */
/*	Get Bonus of object								 */
/*												     */
/*****************************************************/
u8 GetBonusOfObject(u8 objectId)
{
	static const u8 sBonusMagic[] = { PERM_PROTECTION, PERM_RESURRECTION, PERM_SWORD_MASTER, PERM_THROW_MASTER };
	static const u8 sBonusPotions[] = { BONUS_STRENGTH, BONUS_INVISIBILITY };
	
	const u8* bonus = sBonusPotions;
	u8 bonusId = objectId - OBJ_POTION_BOMB;
	
	if (objectId >= OBJ_MAGIC_PROTECTION)
	{
		bonus = sBonusMagic;
		bonusId = objectId - OBJ_MAGIC_PROTECTION;		
	}
	
	return bonus[bonusId];
}
