//-----------------------------LICENSE NOTICE------------------------------------
//  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/** System includes */
#include <cpctelera.h>
#include <stdio.h>

/** Game includes */
#include "..\structures.h"

/** Tools includes */
#include "..\video\video.h"

/** FontHelper includes */
#include "fontsHelper.h"

/** Fonts includes */
#include "number.h"

/** Static variables*/
static BOOL _isNumberFont;
static u8 _buffer;
static u8* _letters;

/** Defines */
#define NUMBER_FONTS_INDEX	12

#define FONT_DOT			10
#define FONT_MINUS			11
#define FONT_SPACE			38

/*****************************************************/
/*													 */
/*	Set font type							 		 */
/*												     */
/*****************************************************/
void SetNumberFont(BOOL isNumberFont)
{
	_isNumberFont = isNumberFont;
}

/*****************************************************/
/*													 */
/*	Set font letter sprites					 		 */
/*												     */
/*****************************************************/
void SetFontLetterSprites(u8* letters)
{
	_letters = letters;
}

/*****************************************************/
/*													 */
/*	Set destination buffers					 		 */
/*												     */
/*****************************************************/
static void SetFontDestBuffers(u8 bufferDest)
{
	_buffer = bufferDest;
}

/*****************************************************/
/*													 */
/*	Convert int to string							 */
/*												     */
/*****************************************************/
static u8 ConvertIntToChar(u16 num, char* str)
{
	u16 n = num;
	u8 i, len = 0;

	while (n != 0)
	{
		len++;
		n /= 10;
	}

	for (i = 0; i < len; i++)
	{
		str[len - (i + 1)] = num % 10 + '0';
		num = num / 10;
	}

	return len;
}

/*****************************************************/
/*													 */
/*	Display character							 	 */
/*												     */
/*****************************************************/
static void DisplayChar(u8 x, u8 y, char c)
{
	/** Is letter or number character */
	u8 charaIndex = (c > 64) ? c - 65 : c - 48;
	u8* spriteChara = (_isNumberFont ? (u8*)g_number : (u8*)_letters) + charaIndex*FONT_SIZE;

	if (_buffer & (BACK_BUFFER | BOTH_BUFFER))
		cpct_drawSprite(spriteChara, GetBackScreenPtr(x, y), FONTS_CX, FONTS_CY);

	if (_buffer & (SCREEN_BUFFER | BOTH_BUFFER))
		cpct_drawSprite(spriteChara, GetScreenPtr(x, y), FONTS_CX, FONTS_CY);
}

/*****************************************************/
/*													 */
/*	Display String							 		 */
/*												     */
/*****************************************************/
void PrintStr(u8 x, u8 y, const char* text, u8 bufferDest)
{
	SetFontDestBuffers(bufferDest);

    while(*text != '\0')
    {
		if (*text != ' ')
			DisplayChar(x, y, *text);
			
		x += FONTS_CX;
        text++;
    }
}

/*****************************************************/
/*													 */
/*	Display Int							 			 */
/*												     */
/*****************************************************/
void PrintInt(u8 x, u8 y, u16 val, u8 nbDigits, u8 bufferDest)
{		
	/** [0, 65535] */
	static const char conv[6];
	u8 length;
	
	cpct_memset(conv, 0, 5);
	length = ConvertIntToChar(val, conv);

	if (nbDigits > length)
	{
		static char digits[6];
		
		u8 i;
		for (i = 0; i < nbDigits; i++)
			digits[i] = '0';

		digits[i] = '\0';

		for (i = 0; i < length; i++)
			digits[nbDigits - length + i] = conv[i];

		PrintStr(x, y, digits, bufferDest);
	}
	else
		PrintStr(x, y, conv, bufferDest);
}