//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Deeper Warrens
//  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>

/** Structure includes */
#include "..\structures.h"
#include "..\game_const.h"

/** Game includes */
#include "..\ui\ui.h"
#include "..\player\player.h"
#include "..\missiles\missiles.h"
#include "..\view\view.h"

/** Tools includes */
#include "..\video\video.h"
#include "..\tools\tools.h"
#include "..\sound\sound.h"
#include "..\asm\fillView.h"
#include "..\asm\ZX7Decrunch.h"

/** External variable */
extern u8 gTimer;
extern u8* DeeperWarrens_SongStart;
extern u8* DeeperWarrensIntro_SongStart;

/*****************************************************/
/*													 */
/*	Interruption handler							 */
/*												     */
/*****************************************************/
void sInterruptHandler(void)
{
	static u8 sInterrupt = 0;

	if (++sInterrupt == 6)
	{				
		PlaySound();
		
		cpct_scanKeyboard_if();

		sInterrupt = 0;		
		gTimer++;
	}
}

CPCT_ABSOLUTE_LOCATION_AREA(0x8000);

/** Local temporary variable */
const u8 _temp[BOARD_CX/4 * BOARD_CY];

/** Sprites includes */
#include "..\sprites\title_0.h"
#include "..\sprites\title_1.h"
#include "..\sprites\title_2.h"
#include "..\sprites\title_3.h"

/** Intro includes */
#include "..\intro\intro.h"
#include "..\sound\song\musicIntro.h"

/*****************************************************/
/*													 */
/*	Init Video										 */
/*												     */
/*****************************************************/
void InitVideo()
{
	/** Set Mode 0 */
	cpct_setVideoMode(0);
	
	/** Set Palette */
	SetGamePalette();
}

/*****************************************************/
/*													 */
/*	Init Board										 */
/*												     */
/*****************************************************/
void InitBoard()
{
	#define NB_PARTS	4
	#define BOARD_PART	BOARD_CX/NB_PARTS

	const u8* sTitle[] = { G_board_0, G_board_1, G_board_2, G_board_3 };
	u8* to = GetScreenPtr(0, SCREEN_CY - BOARD_CY);	
	u8 i;

	/** Set All black */
	SetAllPalette(HW_BLACK);
	
	for (i = 0; i < NB_PARTS; i++)
	{
		ZX7Decrunch(sTitle[i], _temp);				
		cpct_drawSpriteMaskedAlignedTable(_temp, to, TITLE_PART, TITLE_CY, gMaskTable);
		to += BOARD_PART;
	}
}

/*****************************************************/
/*													 */
/*	Start game										 */
/*												     */
/*****************************************************/
void StartGame()
{
	gTimer = 0;
	
	SetNumberFont(TRUE);
	InitBoard();
	SetGamePalette();
#ifndef WINCPCTELERA
	SelectMusic(&DeeperWarrens_SongStart);
#endif
	InitHero();
}

/*****************************************************/
/*													 */
/*	Initialize										 */
/*												     */
/*****************************************************/
void Initialize()
{
	cpct_disableFirmware();

	InitSFX();
#ifndef WINCPCTELERA	
	SelectMusic(&DeeperWarrensSongIntro);
#endif
	SetInterruptHandler(sInterruptHandler);
	InitVideo();
	
	DrawIntro();
	StartGame();
}

CPCT_RELOCATABLE_AREA();

/*****************************************************/
/*													 */
/*	Initialize										 */
/*												     */
/*****************************************************/
void StartInitialize()
{
	Initialize();
	CopyScreenToBackBuffer();
}