//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Deeper Warrens
//  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>

/** Structures includes */
#include "..\structures.h"

/** Input */
#include "input.h"

/*************** Global Keys *************/
const cpct_keyID gKeys[NB_KEYS]		 = { Key_CursorUp, Key_CursorDown, Key_CursorLeft, Key_CursorRight, Key_Space, Key_Copy, Key_1, Key_2, Key_3, Key_4 };
const cpct_keyID gJoy[NB_JOY_KEYS]	 = { Joy0_Up, Joy0_Down, Joy0_Left, Joy0_Right, Joy0_Fire1, Joy0_Fire2 };

/*****************************************************/
/*													 */
/*	Check input keys								 */
/*												     */
/*****************************************************/
u8 CheckUserInput() 
{
	#define EVENT_SELECT_RELEASE	2

	static u8 sSelectKeyRelease = 0;
	u8 move = NO_MOVE;

	for (u8 i = 0; i < NB_KEYS; i++)
	{
		u8 sel = i;
		BOOL isKeyPress = cpct_isKeyPressed(gKeys[sel]);

		if (!isKeyPress && sel < NB_JOY_KEYS)
			isKeyPress = cpct_isKeyPressed(gJoy[sel]);

		if (isKeyPress)
		{
			if (sel >= KEY_SEL1)
				return sel - KEY_SEL1 + SEL1;

			if (sel == KEY_UP)
				move = DOWN;
			else
			if (sel == KEY_DOWN)
				move = TOP;

			if (sel == KEY_LEFT)
				move |= LEFT;
			else
			if (sel == KEY_RIGHT)
				move |= RIGHT;

			if (sel == KEY_FIRE)
   				move |= FIRE;
			else
			if (sel == KEY_CHANGE_SEL && sSelectKeyRelease > EVENT_SELECT_RELEASE)
			{
				move |= SELECT;
				sSelectKeyRelease = 0;
			}
		}
	}

	sSelectKeyRelease++;
	return move;
}

/*****************************************************/
/*													 */
/*	Wait for fire									 */
/*												     */
/*****************************************************/
void WaitForFire()
{
	u8 input = CheckUserInput();
	while (CheckUserInput() == input) {};
	while (CheckUserInput() != FIRE) {};
}