//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Deeper Warrens
//  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** Local defines */
#define OFFSET_X		(TILE_CX + 1)
#define OFFSET_Y		(TILE_CY + 2)
#define OFFSET_MENU_Y	(OFFSET_Y + 4)
#define TITLE_OFFSET_Y	(FONTS_CY + TILE_CY/2)
#define TITLE_CX		80
#define TITLE_CY		40
#define START_INFO_Y	(TITLE_CY + 16)
#define SPACE_X			2
#define SPACE_Y			4

#define FLAMES_LEFT_X	2
#define FLAMES_LEFT_Y	(SCREEN_CY - FLAMES_CY)
#define FLAMES_RIGHT_X	(SCREEN_CX - FLAMES_CX - 2)
#define FLAMES_RIGHT_Y	(SCREEN_CY - FLAMES_CY)
#define FLAMES_CX		6
#define FLAMES_CY		24
#define MAX_FLAMES		16
#define UPDATE_FLAMES	25

#define FLAMES_LEFT		1
#define FLAMES_RIGHT	2
#define FLAMES_BOTH		3

#define POS_TEXT_X		30
#define POS_TEXT_Y		(START_INFO_Y + 4)

#define POS_SEL_X		18
#define POS_SEL_Y		(START_INFO_Y + 20)
#define SWORD_CX		8		
#define SWORD_CY		9

#define INTRO_BORDER_CX	5	
#define INTRO_BORDER_CY	20
#define BORDER_END_CX	2	
#define BORDER_END_CY	11

#define CREDIT_UPDATE 	15
#define CRED_OFFSET_Y	(TILE_CY - 2)
#define POS_CREDIT_X	(INTRO_BORDER_CX + 4)
#define POS_CREDIT_Y	(POS_SEL_Y + CRED_OFFSET_Y * 7 + 8)
#define POS_MAX_COLOR	40

#define COLOR_UPDATE 	8
#define FONT_MENU_CX	4
#define FONT_MENU_CY	8
#define FONT_MENU_SZ	(FONT_MENU_CX*FONT_MENU_CY)

#define LOGO_CPCTELERA	3
#define LOGO_ARKOS		4

/** Tools includes */
#include "..\asm\drawMapTile.h"
#include "..\input\input.h"
#include "..\fonts\fontsHelper.h"
#include "..\view\view.h"

/** Sprite includes */
#include "..\sprites\letter.h"
#include "..\sprites\sword.h"
#include "..\sprites\flames.h"
#include "..\sprites\board_0.h"
#include "..\sprites\board_1.h"
#include "..\sprites\board_2.h"
#include "..\sprites\board_3.h"
#include "..\sprites\intro_border.h"
#include "..\sprites\intro_border_end.h"
#include "..\sprites\menu.h"

/** Game includes */
#include "..\monsters\monsters.h"
#include "redefine.h"

/** Static variables */
static u8 _updateTime;
static u8 _currentCredit;
static const u8 _tempFlames[FLAMES_CX * FLAMES_CY];
static u16 _seed;
static u8 _posColorBar;

/*****************************************************/
/*													 */
/*	Init Hero equipment								 */
/*												     */
/*****************************************************/
void InitHero()
{
	//gPlayerInv.equipment[MISSILE_SLOT] = DAGGER; //FIREBALL; //DAGGER
	DrawObjectInSlot(MISSILE_SLOT, OBJ_AMMO);
	UpdateLife();
}

/*****************************************************/
/*													 */
/*	Draw borders									 */
/*												     */
/*****************************************************/
void DrawBorders(u8 start)
{
	u8 i;
	ZX7Decrunch(G_intro_border, _temp);		
	
	for (i = start; i < SCREEN_CY/INTRO_BORDER_CY; i++)
	{
		u8 y = i*INTRO_BORDER_CY;
	
		cpct_drawSprite(_temp, GetScreenPtr(0, y), INTRO_BORDER_CX, INTRO_BORDER_CY);
		cpct_drawSprite(_temp + INTRO_BORDER_CX*INTRO_BORDER_CY, GetScreenPtr(SCREEN_CX - INTRO_BORDER_CX, y), INTRO_BORDER_CX, INTRO_BORDER_CY);
		
		if ((i % 2) == 0)
		{
			y += INTRO_BORDER_CY/2;
			cpct_drawSprite(g_intro_border_end_0, GetScreenPtr(INTRO_BORDER_CX, y), BORDER_END_CX, BORDER_END_CY);
			cpct_drawSprite(g_intro_border_end_1, GetScreenPtr(SCREEN_CX - INTRO_BORDER_CX - BORDER_END_CX, y), BORDER_END_CX, BORDER_END_CY);
		}
	}
}

/*****************************************************/
/*													 */
/*	Draw Title										 */
/*												     */
/*****************************************************/
void DrawTitleIntro()
{
	#define NB_PARTS	4
	#define TITLE_PART	TITLE_CX/NB_PARTS
	
	const u8* sTitle[] = { G_title_0, G_title_1, G_title_2, G_title_3 };
	u8* to = GetScreenPtr(0, 0);

	for (u8 i = 0; i < NB_PARTS; i++)
	{
		ZX7Decrunch(sTitle[i], _temp);				

		cpct_drawSpriteMaskedAlignedTable(_temp, to, TITLE_PART, TITLE_CY, gMaskTable);
		to += TITLE_PART;
	}
}

/*****************************************************/
/*													 */
/*	Draw Flame										 */
/*												     */
/*****************************************************/
void DrawFlame(u8 indexFlame, u8* vmem)
{
	ZX7Decrunch(gFlames[indexFlame % MAX_FLAMES], _tempFlames);	
	cpct_drawSprite(_tempFlames, vmem, FLAMES_CX, FLAMES_CY);
}

/*****************************************************/
/*													 */
/*	Draw Flames										 */
/*												     */
/*****************************************************/
void DrawFlames(u8 flames)
{
	static u8 indexFlame = 0;
	
	if ((_updateTime++ % UPDATE_FLAMES) == 0)
	{		
		indexFlame++;
		
		if (flames & FLAMES_LEFT)
			DrawFlame(indexFlame, GetScreenPtr(FLAMES_LEFT_X, FLAMES_LEFT_Y));
		
		if (flames & FLAMES_RIGHT)
			DrawFlame(indexFlame + 1, GetScreenPtr(FLAMES_RIGHT_X, FLAMES_RIGHT_Y));
	}
}

/*****************************************************/
/*													 */
/*	Hide and clear screen							 */
/*												     */
/*****************************************************/
void ClearScreen(u8 color)
{
	SetAllPalette(HW_BLACK);
	cpct_memset(GetScreenPtr(0, 0), cpctm_px2byteM0(color, color), VMEM_SIZE);
}

/*****************************************************/
/*													 */
/*	Clear infos										 */
/*												     */
/*****************************************************/
void ClearInfos()
{
	cpct_drawSolidBox(GetScreenPtr(0, START_INFO_Y), cpctm_px2byteM0(COLOR_YELLOW, COLOR_YELLOW), SCREEN_CX/2, SCREEN_CY - START_INFO_Y);
	cpct_drawSolidBox(GetScreenPtr(SCREEN_CX/2, START_INFO_Y), cpctm_px2byteM0(COLOR_YELLOW, COLOR_YELLOW), SCREEN_CX/2, SCREEN_CY - START_INFO_Y);
}

/*****************************************************/
/*													 */
/*	Set fonts										 */
/*												     */
/*****************************************************/
void SetFonts()
{
	ZX7Decrunch(G_letter, _temp);
	SetFontLetterSprites(_temp);	
	
	SetNumberFont(FALSE);
}

/*****************************************************/
/*													 */
/*	Print Text in Intro								 */
/*												     */
/*****************************************************/
void PrintStrIntro(u8 x, u8 y, const char* pText)
{
	PrintStr(x, y, pText, SCREEN_BUFFER);
}

/*****************************************************/
/*													 */
/*	Print Text in Intro								 */
/*												     */
/*****************************************************/
void PrintStrIntroName(u8 x, u8 y, u8 index, const char* name)
{
	PrintStrIntro(x + TILE_CX + SPACE_X, y + SPACE_Y + OFFSET_Y*index + TITLE_OFFSET_Y, name);
}

/*****************************************************/
/*													 */
/*	Print Credit									 */
/*												     */
/*****************************************************/
void PrintCredit(u8 x, const char* credit)
{
	cpct_drawSolidBox(GetScreenPtr(POS_CREDIT_X, POS_CREDIT_Y-2), cpctm_px2byteM0(COLOR_YELLOW, COLOR_YELLOW), 62, 10);
	PrintStrIntro(x, POS_CREDIT_Y, credit);
}

/*****************************************************/
/*													 */
/*	Draw Color Bar 									 */
/*												     */
/*****************************************************/
void DrawColorBar(u8 posColorBar, u8 posy)
{
	u8* mem = GetScreenPtr(POS_CREDIT_X + 10, posy);

	cpct_memset(mem + 2, cpctm_px2byteM0(COLOR_BRIGHT_RED, COLOR_BRIGHT_RED), 38);
	cpct_memset(GetScreenPtr(POS_CREDIT_X + 10, posy + 1), cpctm_px2byteM0(COLOR_BLACK, COLOR_BLACK), 42);
	
	*mem = cpctm_px2byteM0(COLOR_RED, COLOR_RED);
	*(mem + 1) = cpctm_px2byteM0(COLOR_BRIGHT_RED, COLOR_RED);

	*(mem + 40) = cpctm_px2byteM0(COLOR_RED, COLOR_BRIGHT_RED);
	*(mem + 41) = cpctm_px2byteM0(COLOR_RED, COLOR_RED);

	*(mem + posColorBar) = cpctm_px2byteM0(COLOR_PINK, COLOR_BRIGHT_WHITE);
	*(mem + posColorBar + 1) = cpctm_px2byteM0(COLOR_BRIGHT_WHITE, COLOR_PINK);
}

/*****************************************************/
/*													 */
/*	Draw Credits 									 */
/*												     */
/*****************************************************/
void DrawCredits()
{
	if ((_updateTime % CREDIT_UPDATE) == 0)
	{
		if (_currentCredit == 0)
			PrintCredit(POS_TEXT_X - 13, "A GAME BY ARNAUD BOUCHE");
		
		if (_currentCredit == 85)
			PrintCredit(POS_TEXT_X - 16, "INTRO BY CPC POWER AND CED");
		
		if (_currentCredit == 170)
		{
			PrintCredit(POS_TEXT_X - 19, "USING CPCTELERA   AND ARKOS");
			ZX7Decrunch(G_menu, _temp);

			cpct_drawSprite(_temp + LOGO_CPCTELERA*FONT_MENU_SZ, GetScreenPtr(POS_TEXT_X + 12, POS_CREDIT_Y-2), FONT_MENU_CX, FONT_MENU_CY);
			cpct_drawSprite(_temp + LOGO_ARKOS*FONT_MENU_SZ, GetScreenPtr(POS_TEXT_X + 36, POS_CREDIT_Y-2), FONT_MENU_CX, FONT_MENU_CY);

			ZX7Decrunch(G_letter, _temp);
		}
		
		_currentCredit++;
	}

	if ((_updateTime % COLOR_UPDATE) == 0)
	{
		if (_posColorBar++ == POS_MAX_COLOR)
			_posColorBar = 0;

		DrawColorBar(_posColorBar, POS_CREDIT_Y - 10);
		DrawColorBar(POS_MAX_COLOR - _posColorBar , POS_CREDIT_Y + 13);
	}
}

/*****************************************************/
/*													 */
/*	Wait any key									 */
/*												     */
/*****************************************************/
void WaitNext(u8 flames)
{
	while (CheckUserInput() == NO_MOVE)
	{
		DrawFlames(flames);
	}
}

/*****************************************************/
/*													 */
/*	Wait Key										 */
/*												     */
/*****************************************************/
u8 WaitKeyMenu()
{
	_seed++;
	DrawCredits();
	DrawFlames(FLAMES_BOTH);
	return CheckUserInput();
}

/*****************************************************/
/*													 */
/*	Draw Monster									 */
/*												     */
/*****************************************************/
void DrawMonstersIntro(u8 x, u8 y)
{
	static const char sMonsterName[][8] = { "RATS", "GOBLIN", "MUMMY", "WARLOCK", "IMP", "WORM", "CYCLOP" };
	
	u8 index;
	PrintStrIntro(x, y, "MONSTERS");
	
	for (index = 0; index < MONST_NB; index++)
		asm_drawMapTile(DeflateSpriteMonster(MONST_START + index), GetScreenPtr(x, y + OFFSET_Y*index + TITLE_OFFSET_Y), TILE_CY);
		
	for (index = 0; index < MONST_NB; index++)
		PrintStrIntroName(x, y, index, sMonsterName[index]);
}

/*****************************************************/
/*													 */
/*	Draw Tiles										 */
/*												     */
/*****************************************************/
void DrawTilesIntro(u8 x, u8 y)
{
	static const u8 tileId[] = { OBJ_HOLE_PATH, DGN_OPEN_PRESSURE_PLATE, DGN_FIRE_PATH, DGN_WATER_PATH, DGN_TELEPORTER, DGN_TORCH, DGN_SPIKE_TRAP };
	static const char sTileName[][10] = { "HOLE", "MECHANISM", "FIRE", "WATER", "TELEPORT", "TORCH", "SPIKE" };
	
	u8 index;
	PrintStrIntro(x, y, "TILES");

	for (index = 0; index < sizeof(tileId); index++)
		DrawTileTransparent(DeflateSpriteTile(tileId[index]), GetScreenPtr(x, y + OFFSET_Y*index + TITLE_OFFSET_Y));
		
	for (index = 0; index < sizeof(tileId); index++)
		PrintStrIntroName(x, y, index, sTileName[index]);
}

/*****************************************************/
/*													 */
/*	Draw breakable Tiles							 */
/*												     */
/*****************************************************/
void DrawBreakableTilesIntro(u8 x, u8 y)
{
	static const u8 tileId[] = { OBJ_SKULL, OBJ_CRATE, OBJ_ROCK, OBJ_BOMB };
	static const char sTileName[][6] = { "SKULL", "CRATE", "ROCK", "BOMB" };
	
	u8 index;
	PrintStrIntro(x, y, "BREAKABLE");

	for (index = 0; index < sizeof(tileId); index++)
		DrawTileTransparent(DeflateSpriteTile(tileId[index]), GetScreenPtr(x, y + OFFSET_Y*index + TITLE_OFFSET_Y));

	for (index = 0; index < sizeof(tileId); index++)
		PrintStrIntroName(x, y, index, sTileName[index]);
}

/*****************************************************/
/*													 */
/*	Get position of item in intro					 */
/*												     */
/*****************************************************/
u8* GetPosItem(u8 index, u8 length, u8 x, u8 y)
{
	return GetScreenPtr((index % length) * OFFSET_X + x, (index / length) * OFFSET_Y + y + TITLE_OFFSET_Y);
}

/*****************************************************/
/*													 */
/*	Draw Food										 */
/*												     */
/*****************************************************/
void DrawFoodIntro(u8 x, u8 y)
{
	#define NB_FOOD		4
	u8 index;
	
	PrintStrIntro(x, y, "FOOD");
	
	for (index = 0; index < NB_FOOD; index++)
		DrawTileTransparent(DeflateSpriteTile(OBJ_FOOD_CORN + index), GetPosItem(index, 2, x, y));
}

/*****************************************************/
/*													 */
/*	Draw Treasures									 */
/*												     */
/*****************************************************/
void DrawTreasuresIntro(u8 x, u8 y)
{
	#define NB_TREASURES	5
	u8 index;

	PrintStrIntro(x, y, "TREASURES");
	
	for (index = 0; index < NB_TREASURES; index++)
		DrawTileTransparent(DeflateSpriteTile((index < 4) ? OBJ_TREASURE_GOLD + index : OBJ_CHEST_CLOSED), GetPosItem(index, 3, x, y));
}

/*****************************************************/
/*													 */
/*	Draw Weapons									 */
/*												     */
/*****************************************************/
void DrawWeapons(u8 x, u8 y)
{
	#define NB_WEAPONS	6
	u8 index;
		
	PrintStrIntro(x, y, "WEAPONS");
	
	for (index = 0; index < NB_WEAPONS; index++)
		DrawTileTransparent(DeflateSpriteTile((index < 5) ? OBJ_WEAPON_DAGGER + index : OBJ_AMMO), GetPosItem(index, 3, x, y));
}

/*****************************************************/
/*													 */
/*	Draw Armors										 */
/*												     */
/*****************************************************/
void DrawArmors(u8 x, u8 y)
{
	#define NB_ARMOR	6
	u8 index;

	PrintStrIntro(x, y, "ARMORS");

	for (index = 0; index < NB_ARMOR; index++)
		DrawTileTransparent(DeflateSpriteTile(OBJ_ARMOR_TUNIC + index), GetPosItem(index, 3, x, y));
}

/*****************************************************/
/*													 */
/*	Draw Item for intro								 */
/*												     */
/*****************************************************/
void DrawItemsIntro(u8 itemId, u8 index, u8* name, u8 x, u8 y)
{
	DrawTileTransparent(DeflateSpriteTile(itemId + index), GetScreenPtr(x, y + OFFSET_Y*index + TITLE_OFFSET_Y));
	PrintStrIntroName(x, y, index, name);
}

/*****************************************************/
/*													 */
/*	Draw Potions									 */
/*												     */
/*****************************************************/
void DrawPotionsIntro(u8 x, u8 y)
{
	#define NB_POTIONS 4

	static const char sPotionName[][10] = { "BOMB", "INVISIBLE", "LIFE", "STRENGTH" };
	u8 index;

	PrintStrIntro(x, y, "POTIONS");

	for (index = 0; index < NB_POTIONS; index++)
		DrawItemsIntro(OBJ_POTION_BOMB, index, sPotionName[index], x, y);
}

/*****************************************************/
/*													 */
/*	Draw Artifacts									 */
/*												     */
/*****************************************************/
void DrawArtifactsIntro(u8 x, u8 y)
{
	#define NB_ARTIFACTS 4

	static const char sArtifactName[][14] = { "PROTECTION", "RESURRECTION", "MASTER SWORD", "MASTER THROW" };
	u8 index;

	PrintStrIntro(x, y, "ARTIFACTS");

	for (index = 0; index < NB_ARTIFACTS; index++)
		DrawItemsIntro(OBJ_MAGIC_PROTECTION, index, sArtifactName[index], x, y);
}

/*****************************************************/
/*													 */
/*	Draw Treasures									 */
/*												     */
/*****************************************************/
void DrawKeysIntro(u8 x, u8 y)
{
	PrintStrIntro(x, y, "KEYS");

	DrawTileTransparent(DeflateSpriteTile(OBJ_KEY_GOLD), GetScreenPtr(x, y + TITLE_OFFSET_Y));
	DrawTileTransparent(DeflateSpriteTile(OBJ_KEY_SILVER), GetScreenPtr(x, y + TITLE_OFFSET_Y + OFFSET_Y));
}

/*****************************************************/
/*													 */
/*	Set Intro palette								 */
/*												     */
/*****************************************************/
void SetIntroPalette()
{
	SetGamePalette();
	cpct_setPALColour(0, 0x44);
	cpct_setBorder(HW_BLUE);
}

/*****************************************************/
/*													 */
/*	Draw informations on objects					 */
/*												     */
/*****************************************************/
void DrawInfosObjects()
{
	DrawFoodIntro(0, POS_TEXT_Y);
	DrawTreasuresIntro(15, POS_TEXT_Y);
	DrawWeapons(42, POS_TEXT_Y);
	DrawArmors(66, POS_TEXT_Y);

	DrawKeysIntro(0, POS_TEXT_Y + 56);
	DrawPotionsIntro(15, POS_TEXT_Y + 56);
	DrawArtifactsIntro(42, POS_TEXT_Y + 56);

	WaitNext(FLAMES_BOTH);
	ClearInfos();
}

/*****************************************************/
/*													 */
/*	Draw informations on monster and tiles			 */
/*												     */
/*****************************************************/
void DrawInfosMonstersTiles()
{
	DrawMonstersIntro(0, POS_TEXT_Y);
	DrawTilesIntro(30, POS_TEXT_Y);
	DrawBreakableTilesIntro(62, POS_TEXT_Y);
	
	WaitNext(FLAMES_RIGHT);
	ClearInfos();
}

/*****************************************************/
/*													 */
/*	Draw First letter of menu						 */
/*												     */
/*****************************************************/
void DrawFirstLetter()
{
	u8 i;
	ZX7Decrunch(G_menu, _temp);

	for (i = 0; i < 3; i++)
	{
		u8* vmem = GetScreenPtr(POS_TEXT_X, POS_SEL_Y + i*OFFSET_MENU_Y - 2);
		cpct_drawSprite(_temp + i*FONT_MENU_SZ, vmem, FONT_MENU_CX, FONT_MENU_CY);
	}
}

/*****************************************************/
/*													 */
/*	Draw Menu Text									 */
/*												     */
/*****************************************************/
void DrawMenuText()
{
	DrawBorders(2);
	DrawFirstLetter();
	SetFonts();
	
	PrintStrIntro(POS_TEXT_X + 4, POS_SEL_Y, "TART");
	PrintStrIntro(POS_TEXT_X + 4, POS_SEL_Y + OFFSET_MENU_Y, "NSTRUCTIONS");
	PrintStrIntro(POS_TEXT_X + 4, POS_SEL_Y + OFFSET_MENU_Y*2, "EDEFINE KEYS");

	_currentCredit = 0;
}

/*****************************************************/
/*													 */
/*	Clear Menu Selection							 */
/*												     */
/*****************************************************/
void ClearSelection(u8 selection)
{
	u8 posY = POS_SEL_Y + selection*OFFSET_MENU_Y - 2;
	cpct_drawSolidBox(GetScreenPtr(POS_SEL_X, posY), cpctm_px2byteM0(COLOR_YELLOW, COLOR_YELLOW), SWORD_CX, SWORD_CY);
	
	cpct_memset(GetScreenPtr(POS_TEXT_X - 2, posY + 15), cpctm_px2byteM0(COLOR_YELLOW, COLOR_YELLOW), 34);
}

/*****************************************************/
/*													 */
/*	Draw Menu Selection								 */
/*												     */
/*****************************************************/
void DrawSelection(u8 selection)
{
	static const u8 sLineLength[] = { 14, 28, 30 };

	u8 posY = POS_SEL_Y + selection*OFFSET_MENU_Y - 2;
	u8* vmem = GetScreenPtr(POS_TEXT_X - 2, posY + 15);
	u8 size = sLineLength[selection];
	u8 middle = size / 2;
	
	cpct_drawSprite(g_sword, GetScreenPtr(POS_SEL_X, posY), SWORD_CX, SWORD_CY);
	
	*vmem = cpctm_px2byteM0(COLOR_GREEN, COLOR_GREEN);
	*(vmem + 1) = cpctm_px2byteM0(COLOR_BRIGHT_GREEN, COLOR_GREEN);
	cpct_memset(vmem + 2, cpctm_px2byteM0(COLOR_BRIGHT_GREEN, COLOR_BRIGHT_GREEN), size);

	vmem += middle;
	*(vmem + 2) = cpctm_px2byteM0(COLOR_BRIGHT_YELLOW, COLOR_BRIGHT_YELLOW);
	*(vmem + 3) = cpctm_px2byteM0(COLOR_BRIGHT_YELLOW, COLOR_BRIGHT_GREEN);

	vmem += middle;
	*(vmem + 1) = cpctm_px2byteM0(COLOR_GREEN, COLOR_BRIGHT_GREEN);
	*(vmem + 2) = cpctm_px2byteM0(COLOR_GREEN, COLOR_GREEN);
}

/*****************************************************/
/*													 */
/*	Draw instructions								 */
/*												     */
/*****************************************************/
void DrawInstructions()
{
	ClearInfos();
	DrawInfosObjects();
	DrawInfosMonstersTiles();
	DrawMenuText();
}

/*****************************************************/
/*													 */
/*	Draw redefine keys								 */
/*												     */
/*****************************************************/
void DrawRedefineKeys()
{
	ClearInfos();
	DrawBorders(2);
	SetFonts();
	RedefineKeys();
	ClearInfos();
	DrawMenuText();
}

/*****************************************************/
/*													 */
/*	Draw Menu										 */
/*												     */
/*****************************************************/
void DrawMenu()
{
	#define START		0
	#define INSTRUCT	1
	#define REDEFIN		2

	u8 selection = 0;
	u8 prevKey = NO_MOVE;

	SetFonts();
	DrawMenuText();
	DrawSelection(selection);
	
	while (TRUE)
	{
		u8 key = WaitKeyMenu();
		if (key != NO_MOVE && prevKey != key)
		{
			if (key == DOWN && selection > START)
			{
				ClearSelection(selection--);
			}
			else 
			if (key == TOP && selection < REDEFIN)
			{
				ClearSelection(selection++);
			}
			else
			if (key == FIRE)
			{
				if (selection == INSTRUCT)
				{
					DrawInstructions();
				}
				else if (selection == REDEFIN)
				{		
					DrawRedefineKeys();
				}
				else
				{
					cpct_srand8(_seed == 0 ? 1 : _seed);
					break;
				}
			}
			
			DrawSelection(selection);
		}
		
		prevKey = key;		
	}
}

/*****************************************************/
/*													 */
/*	Draw introduction								 */
/*												     */
/*****************************************************/
void DrawIntro()
{
	ClearScreen(COLOR_YELLOW);
	DrawBorders(0);
	DrawTitleIntro();	
	
	SetIntroPalette();
	DrawMenu();
	
	ClearScreen(COLOR_BLACK);
}
