//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Deeper Warrens
//  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** Sprite includes */
#include "..\sprites\redif_keys.h"

/** External variables */
extern cpct_keyID gKeys[NB_KEYS];

/** External Functions **********/
extern void DrawFlames(u8 flames);

/** Local defines */
#define POS_KEY_X		POS_TEXT_X - 4
#define POS_REDEF_X		POS_TEXT_X + 6

#define LOGO_KEY_CX		7
#define LOGO_KEY_CY		12
#define LOGO_KEY_SZ		(LOGO_KEY_CX*LOGO_KEY_CY)

/*****************************************************/
/*													 */
/*	Get current key presses							 */
/*												     */
/*****************************************************/
static cpct_keyID GetKeyPressed() 
{	
	/** From Ronaldo in cpcwiki forum */
	#define KEY_STATUS_SIZE		10
	
	u8* keyStatus = cpct_keyboardStatusBuffer;
	u8 i;

	while (!cpct_isAnyKeyPressed())
	{		
		DrawFlames(FLAMES_BOTH);
	};

	for (i = 0; i < KEY_STATUS_SIZE; i++)
	{
		cpct_keyID keypressed = *keyStatus++ ^ 0xFF;
		if (keypressed)
			return (keypressed << 8) + i; 
	}

	return 0;
}

/*****************************************************/
/*													 */
/*	Redefine a action key							 */
/*												     */
/*****************************************************/
static void RedefineKey(u8 idKey) 
{
	cpct_keyID key = GetKeyPressed();
	cpct_keyID* defKey = (cpct_keyID*)&gKeys[idKey];
	
	while (cpct_isKeyPressed(key));

	*defKey = key;
}

/*****************************************************/
/*													 */
/*	Draw key to redefine							 */
/*												     */
/*****************************************************/
static void DrawRedefineKey(u8 y, u8 key)
{
	u8* vmem = GetScreenPtr(POS_REDEF_X, y);
	
	ZX7Decrunch(G_redif_keys, _temp);
	cpct_drawSprite(_temp + key*LOGO_KEY_SZ, vmem, LOGO_KEY_CX, LOGO_KEY_CY);

	RedefineKey(key);
}

/*****************************************************/
/*													 */
/*	RedifinesKeys									 */
/*												     */
/*****************************************************/
void RedefineKeys()
{
	#define NB_KEYS		6
	u8 key;

	PrintStr(POS_KEY_X + 2, POS_TEXT_Y, "REDEFINITION", SCREEN_BUFFER);
									
	while (cpct_isAnyKeyPressed());
									
	/** Draw keys to define */
	for (key = 0; key < NB_KEYS; key++)
		DrawRedefineKey(key * OFFSET_Y + (POS_TEXT_Y + OFFSET_Y + 5), key);

}
