//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Deeper Warrens
//  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>

/** Structures includes */
#include "structures.h"

/** Game includes */
#include ".\view\view.h"
#include ".\player\player.h"
#include ".\initialize\initialize.h"
#include ".\ui\ui.h"
#include ".\level\level.h"
#include ".\missiles\missiles.h"
#include ".\monsters\monsters.h"

/** Tools includes */
#include ".\video\video.h"

/** Global */
u8 gTimer;

/*****************************************************/
/*													 */
/*	Initializations									 */
/*												     */
/*****************************************************/
void InitGame()
{
	InitView();
	InitPlayer();
	
	DrawFullUI();
	FlipBuffer();
	DrawFullUI();
	
	InitMissiles();
	
	EnterLevel();
}

/*****************************************************/
/*													 */
/*	Draw main routine								 */
/*												     */
/*****************************************************/
void main(void) 
{
	/** Change stack location in order to use HW double buffer */
	cpct_setStackLocation((u8*)NEW_STACK_LOC);
	StartInitialize();
	InitGame();
		
	while (TRUE)
	{
		u8 move = MovePlayer();
						
		DrawView();
			
		UpdatePlayer(move);			
		UpdateMonsters();
			
		DrawMonsters();
		
		UpdateMissiles();	
		RefreshUI();

		FlipBuffer();
	}
}
