//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Deeper Warrens
//  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#define PLAYER_ID			0

// Position character
#define HERO_CX				TILE_CX
#define HERO_CY				TILE_CY

#define POS_HERO_X 			(SCREEN_CX - HERO_CX)/2
#define POS_HERO_Y 			(SCREEN_CY - HERO_CY)/2

// Animations character index in tileset 
#define NB_MOVE				2
#define MOVE_TOP			0
#define MOVE_DOWN			(3*NB_MOVE)
#define MOVE_LEFT			(1*NB_MOVE)
#define MOVE_RIGHT			(2*NB_MOVE)

#define SPEED_H				2
#define SPEED_V				4

#define MAX_SLOTS			4
#define MAX_INVENTORY		4
#define ALL_INVENTORY		(MAX_SLOTS+MAX_INVENTORY)
#define INV_NOTHING			0

// CAUTION : Max Values <= 32767
#define START_LIFE			1000
#define START_XP			0
#define MAX_XP				1000
#define START_FOOD			1
#define MAX_FOOD			90
#define START_GOLD			0
#define MAX_GOLD			10000
#define MAX_AMMO			32
#define XP_STEP				1000
#define LIFE_LVL_UP			50
#define START_LEVEL			1
#define LEVEL_MAX			5
#define STATUS_OK			0

enum
{
	SEL_INV1,
	SEL_INV2,
	SEL_INV3,
	SEL_INV4,
	SEL_MISSILE
};

enum
{
	ARMOR_SLOT,
	WEAPON_SLOT,
	MAGIC_SLOT,
	MISSILE_SLOT,
	INVENTORY_SLOT,
	INV_SLOT_1 = INVENTORY_SLOT,
	INV_SLOT_2,
	INV_SLOT_3,
	INV_SLOT_4
};

typedef struct
{
	u8 nbObject, selection;
	u8 nbKeysGold, nbKeysSilver;
	u16 nbAmmo;
	u8 equipment[MAX_SLOTS];
	u8 inventory[MAX_INVENTORY];	
} SInventory;

typedef struct
{
	u8 damage, protection;	
	u8 status, level;
	u16 life, maxLife;
	u16 food;
	u16 xp, gold;
} SStats;

typedef struct
{
	u8 impactHit;
	u8 missile;
	u8 food;
	u8 effect;
	u8 wait;
} SCounters;

#ifndef PLAYER_C
	extern u8 gPlayerMapx;
	extern u8 gPlayerMapy;
	extern u8 gPlayerX;
	extern u8 gPlayerY;
	extern u8 gPrevPlayerX;
	extern u8 gPrevPlayerY;	
	extern u8 gPlayerDir;
	extern u8 gPlayerTileOn;

	extern SInventory gPlayerInv;
	extern SStats gPlayerStats;
	extern SCounters gPlayerCounters;
#endif

BOOL IsPlayerCollide();
BOOL IsPlayerBonus(u8 bonus);
void PlayerIsHit(u8 damage, BOOL ignoreArmor);
void InitPlayer();
void UpdatePlayer(u8 dir);
void EnterPlayer();
void GainXP(u8 xp);
u16 GetNextLevelXP();
u8 GetPlayerAttack();
void ActivateObjectFromInv(u8 slot);
u8 MovePlayer();
void AnimationExit();
u8* InteractPlayer(u8* object);

#define IsPlayerBonus(bonus) (BOOL)(gPlayerStats.status & bonus)