//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Deeper Warrens
//  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <cpctelera.h>

/*************** Sound includes ****************/
#include "sound.h"

/*************** Sound external ****************/
#ifndef WINCPCTELERA
	extern u8* sfx_SoundEffects;
#endif

#define NO_FX	0

static void* _currentMusic;
static void* _selMusic;
static u8 _currentFx;

/*****************************************************/
/*													 */
/*	Init SFX							 			 */
/*												     */
/*****************************************************/
void InitSFX()
{
#ifndef WINCPCTELERA
	PLY_LW_InitSoundEffects(&sfx_SoundEffects);
#endif
}

/*****************************************************/
/*													 */
/*	Set SFX							 				 */
/*												     */
/*****************************************************/
void PlaySFX(u8 fx)
{
	_currentFx = fx;
}

/*****************************************************/
/*													 */
/*	Init SFX							 			 */
/*												     */
/*****************************************************/
void ActivateSFX()
{
	static const u8 sChannel[] = 
	{
		CHANNEL_B,	// FX_IMPACT_CHANNEL
		CHANNEL_C,	// FX_EXPLOSION_CHANNEL
		CHANNEL_A,	// FX_MAGIC_CHANNEL
		CHANNEL_A,	// FX_ACTION_CHANNEL
		CHANNEL_A,	// FX_MISSILE_CHANNEL
		CHANNEL_C	// FX_TREASURE_CHANNEL
	};

	if (_currentFx != NO_FX)
	{
#ifndef WINCPCTELERA
		PLY_LW_PlaySoundEffect(_currentFx, sChannel[_currentFx - 1], 0);
#endif
		_currentFx = NO_FX;
	}
}

/*****************************************************/
/*													 */
/*	Stop All SFX						 			 */
/*												     */
/*****************************************************/
/*void StopSFX()
{
#ifndef WINCPCTELERA
	PLY_LW_StopSoundEffectFromChannel(CHANNEL_A);
	PLY_LW_StopSoundEffectFromChannel(CHANNEL_B);
	PLY_LW_StopSoundEffectFromChannel(CHANNEL_C);
#endif
}*/

/*****************************************************/
/*													 */
/*	Select new music					 			 */
/*												     */
/*****************************************************/
void SelectMusic(void* music)
{
	_selMusic = music;
}

/*****************************************************/
/*													 */
/*	Activate music					 				 */
/*												     */
/*****************************************************/
void ActivateMusic()
{
#ifndef WINCPCTELERA
	if (_currentMusic != _selMusic)
	{
		_currentMusic = _selMusic;
		PLY_LW_Init(_currentMusic, 0);
	}
#endif
}

/*****************************************************/
/*													 */
/*	Play sound						 				 */
/*												     */
/*****************************************************/
void PlaySound()
{
	ActivateMusic();
	ActivateSFX();

#ifndef WINCPCTELERA
	PLY_LW_Play();
#endif
}