//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Deeper Warrens
//  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------	

/** CPCTelera includes */
#include <cpctelera.h>

/** Tools includes */
#include ".\tools.h"
#include "..\video\video.h"
#include "..\asm\setInterruptHandler.h"

/*****************************************************/
/*													 */
/*	Find Sprite Index From Direction				 */
/*												     */
/*****************************************************/
u8 FindSpriteIndex(u8 dir)
{
	static const u8 sDirSprite[] = { 0x02, 0x0A, 0x08, 0x09, 0x01, 0x05, 0x04, 0x06 };

	for (u8 i = 0; i < sizeof(sDirSprite); i++)
	{
		if (sDirSprite[i] == dir)
			return i;
	}
	return 0;
}

/*****************************************************/
/*													 */
/*	Get positive value								 */
/*												     */
/*****************************************************/
u8 GetAbsValue(i8 value)
{
	return value < 0 ? -value : value;
}

/*****************************************************/
/*													 */
/*	Set interrupt handler				 			 */
/*												     */
/*****************************************************/
void SetInterruptHandler(TInterrupt interruptFunction)
{
	cpct_waitVSYNC();
#ifndef WINCPCTELERA
	__asm
	halt
	halt
	__endasm;
#endif
	cpct_waitVSYNC();

#ifndef WINCPCTELERA
	asm_setInterruptHandler(interruptFunction);
#else
	cpct_setInterruptHandler(interruptFunction);	
#endif
}

/*****************************************************/
/*													 */
/*	Get random number modulo			 			 */
/*												     */
/*****************************************************/
u8 GetRandModulo(u8 modulo)
{
	return cpct_rand() % modulo;
}

/*****************************************************/
/*													 */
/*	Wait 2s								 			 */
/*												     */
/*****************************************************/
void Wait()
{
	for (u8 i = 0; i < 100; i++)
		cpct_waitVSYNC();
}