//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Deeper Warrens
//  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#define UI_C

/** CPCTelera includes */
#include <cpctelera.h>

/** Structures includes */
#include "..\structures.h"

/** Game includes */
#include "..\video\video.h"
#include "..\view\view.h"
#include "..\fonts\fontsHelper.h"
#include "..\ui\ui.h"
#include "..\player\player.h"
#include "..\bonus\bonus.h"

/** Sprites includes */
#include "..\sprites\life.h"

#undef UI_C

#define	BOARD_TOP		VIEW_HEIGHT

#define NB_DIGIT_GOLD	5
#define NB_DIGIT_LIFE	4
#define NB_DIGIT_LEVEL	1

#define COLOR_BACK		COLOR_BLACK
#define COLOR_LIFE		COLOR_BRIGHT_RED

#define POS_EQU_X		(6/2)
#define POS_EQU_Y		(BOARD_TOP + 22)
#define EQU_OFFSET		(6/2)

#define POS_INV_X		(92/2)
#define POS_INV_Y		POS_EQU_Y
#define POS_INV_SEL_X	(92/2)
#define POS_INV_SEL_Y	(POS_INV_Y + 1)
#define INV_SEL_CY		(TILE_CY - 2)
#define INV_SPACE		(10/2)
#define INV_OFFSET		(92/2)

#define POS_LIFE_X		(76/2)
#define POS_LIFE_Y		SCREEN_CY
#define LIFE_CX			G_LIFE_0_W
#define LIFE_CY			G_LIFE_0_H
#define LIFE_BAR_CY		33
#define LIFE_INC		((START_LIFE + (LIFE_LVL_UP*LEVEL_MAX))/LIFE_BAR_CY)

#define POS_LEVEL_X		(8/2)
#define POS_LEVEL_Y		(BOARD_TOP + 7)

#define POS_XP_BAR_X	(20/2)
#define POS_XP_BAR_Y	(BOARD_TOP + 9)
#define XP_BAR_CX		32

#define POS_FOOD_BAR_X	(102/2)
#define POS_FOOD_BAR_Y	(BOARD_TOP + 8)
#define LIFE_FOOD_CX	(18/2)
#define LIFE_FOOD_CY	5

#define POS_GOLD_X		(134/2)
#define POS_GOLD_Y		(BOARD_TOP + 7)

#define POS_AMMO_X		(60/2)
#define POS_AMMO_Y		(BOARD_TOP + 18)
#define AMMO_BAR_CX		16
#define AMMO_INC		(MAX_AMMO/AMMO_BAR_CX)

#define POS_KEYS_X		0
#define POS_KEYS_Y		0
#define KEY_GOLD		0
#define KEY_SILVER		1

#define UI_UPDATE		2

/** Internal variables */
static u8 _updateLife;

/*****************************************************/
/*													 */
/*	Get screen position of slot						 */
/*												     */
/*****************************************************/
u8 GetSlotPos(u8 index)
{
	u8 offset = (index < INVENTORY_SLOT) ? EQU_OFFSET : INV_OFFSET;

	if (index >= INVENTORY_SLOT)
		index -= INVENTORY_SLOT;
	
	return index * (TILE_CX + INV_SPACE) + offset;
}

/*****************************************************/
/*													 */
/*	Draw inventory Selection						 */
/*												     */
/*****************************************************/
void DrawInvSelection()
{
	for (u8 i = 0; i < MAX_INVENTORY; i++)
	{
		u8 x = GetSlotPos(i + INVENTORY_SLOT) - 1;
		u8 color = (i == gPlayerInv.selection) ? cpctm_px2byteM0(COLOR_BRIGHT_RED, COLOR_BRIGHT_CYAN) : cpctm_px2byteM0(COLOR_BRIGHT_RED, COLOR_BLUE);
		
		DrawSolidBoxBothBuffers(color, x, POS_INV_SEL_Y, 1, INV_SEL_CY);
	}
}

/*****************************************************/
/*													 */
/*	Clean slot										 */
/*												     */
/*****************************************************/
void CleanSlot(u8 index)
{
	u8 x = GetSlotPos(index);
	DrawSolidBoxBothBuffers(cpctm_px2byteM0(COLOR_BLUE, COLOR_BLUE), x, POS_EQU_Y, TILE_CX, TILE_CY);
}

/*****************************************************/
/*													 */
/*	Set object in slot								 */
/*												     */
/*****************************************************/
void DrawObjectInSlot(u8 index, u8 inventoryObject)
{
	u8 x = GetSlotPos(index);
	
	CleanSlot(index);
	DrawTileTransparentBothBuffers(DeflateSpriteTile(inventoryObject), x, POS_EQU_Y);
}

/*****************************************************/
/*													 */
/*	Draw Inventory									 */
/*												     */
/*****************************************************/
void DrawInventory()
{
	u8 i;
	for (i = 0; i < MAX_INVENTORY; i++)
	{
		u8 objectInv = gPlayerInv.inventory[i];
		u8 slot = INVENTORY_SLOT + i;
		
		if (objectInv != INV_NOTHING)
			DrawObjectInSlot(slot, objectInv);
		else
			CleanSlot(slot);
	}
	
	DrawInvSelection();
}

/*****************************************************/
/*													 */
/*	Draw Life										 */
/*												     */
/*****************************************************/
void DrawLife()
{
	u8 lifePos = gPlayerStats.life/LIFE_INC;
	u8 maxLifePos = gPlayerStats.maxLife/LIFE_INC;
	
	for (u8 i = 0; i < maxLifePos; i++)
	{
		u8* backMem = GetBackScreenPtr(POS_LIFE_X, POS_LIFE_Y - i);
		cpct_drawSprite(i > lifePos ? g_life_0 : g_life_1, backMem, LIFE_CX, LIFE_CY);
	}
}

/*****************************************************/
/*													 */
/*	Draw XP											 */
/*												     */
/*****************************************************/
void DrawXP()
{
	u8 xpBar = gPlayerStats.xp / (GetNextLevelXP() / XP_BAR_CX);
	DrawLineBothBuffer(POS_XP_BAR_X, POS_XP_BAR_Y, xpBar, XP_BAR_CX, COLOR_BRIGHT_YELLOW, COLOR_BRIGHT_RED);	
	DrawLineBothBuffer(POS_XP_BAR_X, POS_XP_BAR_Y + 1, xpBar, XP_BAR_CX, COLOR_ORANGE, COLOR_RED);	
}

/*****************************************************/
/*													 */
/*	Draw PLayer Level								 */
/*												     */
/*****************************************************/
void DrawPlayerLevel()
{
	PrintInt(POS_LEVEL_X, POS_LEVEL_Y, gPlayerStats.level, NB_DIGIT_LEVEL, BOTH_BUFFER);
}

/*****************************************************/
/*													 */
/*	Draw Food										 */
/*												     */
/*****************************************************/
void DrawFood()
{
	static const u8 colors[] = { COLOR_BRIGHT_RED, COLOR_ORANGE, COLOR_BRIGHT_YELLOW, COLOR_BRIGHT_GREEN };
	
	u8 foodBar = (u8)gPlayerStats.food;
	u8 foodBarPart = foodBar / 5;
	u16 colorIndex = ((foodBar * 100 - 1) / MAX_FOOD) * (sizeof(colors));
	
	u8 color = colors[colorIndex/100];

	for (u8 i = 0; i < LIFE_FOOD_CY; i++)
		DrawLineBothBuffer(POS_FOOD_BAR_X, POS_FOOD_BAR_Y + i, foodBarPart, LIFE_FOOD_CX*2, (i != 4 ? color : COLOR_BACK), COLOR_BLUE);
}

/*****************************************************/
/*													 */
/*	Draw Food										 */
/*												     */
/*****************************************************/
void DrawGold()
{
	PrintInt(POS_GOLD_X, POS_GOLD_Y, gPlayerStats.gold, NB_DIGIT_GOLD, BOTH_BUFFER);
}

/*****************************************************/
/*													 */
/*	Draw Ammo										 */
/*												     */
/*****************************************************/
void DrawAmmo()
{
	u8 nbAmmo = gPlayerInv.nbAmmo/4;
	DrawLineBothBuffer(POS_AMMO_X, POS_AMMO_Y, nbAmmo, AMMO_BAR_CX/2, COLOR_BRIGHT_YELLOW, COLOR_BRIGHT_RED);	
	DrawLineBothBuffer(POS_AMMO_X, POS_AMMO_Y + 1, nbAmmo, AMMO_BAR_CX/2, COLOR_ORANGE, COLOR_RED);	
}

/*****************************************************/
/*													 */
/*	Draw Key Type									 */
/*												     */
/*****************************************************/
void DrawKeyType(u8 pos, u8 nbKey, u8* spriteKey)
{
	for (u8 i = 0; i < nbKey; i++)
		DrawSpriteBothBuffers(spriteKey, POS_KEYS_X, pos + POS_KEYS_Y + i * TILE_CY, TILE_CX, TILE_CY);
}

/*****************************************************/
/*													 */
/*	Remove Key										 */
/*												     */
/*****************************************************/
void RemoveKey()
{
	DrawSolidBoxBothBuffers(cpctm_px2byteM0(COLOR_BLACK, COLOR_BLACK), POS_KEYS_X, (gPlayerInv.nbKeysGold + gPlayerInv.nbKeysSilver)*TILE_CY, TILE_CX, TILE_CY);
}

/*****************************************************/
/*													 */
/*	Draw Keys										 */
/*												     */
/*****************************************************/
void DrawKeys()
{
	DrawKeyType(POS_KEYS_Y, gPlayerInv.nbKeysGold, DeflateSpriteTile(OBJ_KEY_GOLD));
	DrawKeyType(POS_KEYS_Y + gPlayerInv.nbKeysGold * TILE_CY, gPlayerInv.nbKeysSilver, DeflateSpriteTile(OBJ_KEY_SILVER));
}

/*****************************************************/
/*													 */
/*	Draw Full UI									 */
/*												     */
/*****************************************************/
void DrawFullUI()
{
	DrawLife();
	DrawXP();
	DrawPlayerLevel();
	DrawFood();
	DrawGold();
	DrawAmmo();
	DrawKeys();
}

/*****************************************************/
/*													 */
/*	Refresh Life									 */
/*												     */
/*****************************************************/
void RefreshUI()
{
	if (_updateLife > 0)
	{
		_updateLife--;
		DrawLife();
	}

	DrawKeys();
	DrawBonus();
}

/*****************************************************/
/*													 */
/*	Clean life										 */
/*												     */
/*****************************************************/
void CleanLife()
{
	DrawSolidBoxBothBuffers(cpctm_px2byteM0(COLOR_BLACK, COLOR_BLACK), 
							POS_LIFE_X, 
							POS_LIFE_Y - LIFE_BAR_CY, 
							LIFE_CX, LIFE_BAR_CY);
}

/*****************************************************/
/*													 */
/*	Set life Update									 */
/*												     */
/*****************************************************/
void UpdateLife()
{
	_updateLife = UI_UPDATE;
}
