//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Deeper Warrens
//  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

// Double buffer location
#define SCREEN_BUFF_LOC		0x8000
#define VMEM_SIZE			0x4000

enum
{
	SCREEN_BUFFER	= 0x01,
	BACK_BUFFER		= 0x02,
	BOTH_BUFFER		= 0x04
};

enum
{
	COLOR_YELLOW,	// 00
	COLOR_BACKGROUND = COLOR_YELLOW,
	COLOR_BLACK,			// 01
	COLOR_BLUE,				// 02
	COLOR_BRIGHT_BLUE,		// 03
	COLOR_RED,				// 04
	COLOR_BRIGHT_RED,		// 05
	COLOR_PURPLE,			// 06
	COLOR_GREEN,			// 07
	COLOR_SKY_BLUE,			// 08
	COLOR_WHITE,			// 09
	COLOR_ORANGE,			// 10
	COLOR_PINK,				// 11
	COLOR_BRIGHT_GREEN,		// 12
	COLOR_BRIGHT_CYAN,		// 13
	COLOR_BRIGHT_YELLOW,	// 14
	COLOR_BRIGHT_WHITE		// 15
};

// Mask table location
#define MASK_TABLE_LOC	(SCREEN_BUFF_LOC - 0x100)	// 0x7F00 (0x100 = Size of Mask Table)

// New stack location
#define NEW_STACK_LOC	(MASK_TABLE_LOC - 0x100)	// 0x7E00 (x100 = Size of Stack)

cpctm_declareMaskTable(gMaskTable);

void InitVideo();
void FlipBuffer();

void SetAllPalette(u8 color);
void SetGamePalette();

u8* GetScreenPtr(u8 xPos, u8 yPos);
u8* GetBackScreenPtr(u8 xPos, u8 yPos);
u8* GetBackBufferPtr();
void CopyScreenToBackBuffer();
u8* DecompToBackBuffer(u8* compress);

void DrawTileTransparent(const u8* sprite, const u8* vmem);
void DrawTileTransparentBothBuffers(const u8* sprite, u8 x, u8 y);
void DrawSolidBoxBothBuffers(u8 color, u8 x, u8 y, u8 cx, u8 cy);
void DrawSpriteBothBuffers(const u8* sprite, u8 x, u8 y, u8 cx, u8 cy);
void DrawLine(u8* vmem, u8 sizeInPix, u8 maxInPix, u8 color, u8 colorBack);
void DrawLineBothBuffer(u8 x, u8 y, u8 sizeInPix, u8 maxInPix, u8 color, u8 colorBack);

#define cpctm_px2byteM0(X, Y) (u8)(	((X & 0x01) << 6 | (X & 0x02) << 1 | (X & 0x04) << 2 | (X & 0x08) >> 3) << 1 | \
									((Y & 0x01) << 6 | (Y & 0x02) << 1 | (Y & 0x04) << 2 | (Y & 0x08) >> 3) )
									