//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Deeper Warrens
//  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#define VIEW_C

/** CPCTelera includes */
#include <cpctelera.h>
#include <stdio.h>

/** Helper include*/
#include "..\wincpctelera\wincpctelera_helper.h"

/** Structures includes */
#include "..\structures.h"

/** Game includes */
#include "..\view\view.h"
#include "..\player\player.h"
#include "..\level\level.h"
#include "..\monsters\monsters.h"

/** Tools includes */
#include "..\input\input.h"
#include "..\video\video.h"
#include "..\tools\tools.h"
#include "..\fonts\fontsHelper.h"
#include "..\asm\fillView.h"
#include "..\asm\drawMapTile.h"
#include "..\asm\ZX7Decrunch.h"

/** Sprites includes */
#include "..\sprites\tiles_items.h"
#include "..\sprites\anim_tiles_items.h"

/** View include */
#include "view.h"

#undef VIEW_C

/** Local defines */
#define ALT_SPRITE_TIMER	8
#define STEP_ANIM			2
#define START_POS			2

#define LEVEL_INFO_X		0
#define LEVEL_INFO_Y		(VIEW_HEIGHT - 8)
#define LEVEL_INFO_DIGITS	1

#define START_TILES_X		2
#define END_TILE_X			(NB_TILES_CX + START_TILES_X - 1)
#define NB_TILES_COLL		4

/** Local Structure */
typedef struct
{
	u8 type;
	u8 sprite[TILE_SIZE];
}
STileItemCache;

/** Internal variables */
static u8 _modViewx, _modViewy;		// Inter tile position
static u8 _viewx;					// View position
static u16 _viewy;			
static u8 _animSprite, _animSpriteSlow;
static const u8* _sprite;

/** Global variables */
const u8* gTilemap;

/** External variable */
extern u8 gTimer;
extern SDungeon gDungeon;

/*****************************************************/
/*													 */
/*	Get tile sprite									 */
/*												     */
/*****************************************************/
u8* DeflateSpriteTile(u8 tileType)
{
	#define NB_TILES_BUFFER		8

	static STileItemCache sSpritesItems[NB_TILES_BUFFER];
	static u8 sFreeSlot = 0;
	
	u8* sprite;
	STileItemCache* spriteTile = sSpritesItems;

	u8 tileTypeId = tileType - DGN_ITEMS + 1;

	for (u8 i = 0; i < NB_TILES_BUFFER; i++)
	{
		if (spriteTile->type == tileTypeId)
			return spriteTile->sprite; 
			
		spriteTile++;			
	}

	/** Decrunch used sprite */
	spriteTile = &sSpritesItems[sFreeSlot];
	spriteTile->type = tileTypeId;
	sprite = spriteTile->sprite;

	ZX7Decrunch(gTilesetItems[tileType - DGN_ITEMS], sprite);

	sFreeSlot = (++sFreeSlot == NB_TILES_BUFFER) ? 0 : sFreeSlot;
	return sprite;
	
}

/*****************************************************/
/*													 */
/*	Init View										 */
/*												     */
/*****************************************************/
void InitView()
{
	_viewx = 0;
	_viewy = 0;
	
	_modViewx = 0;
	_modViewy = 0;
}

/*****************************************************/
/*													 */
/*	Center Player at position						 */
/*												     */
/*****************************************************/
void MovePlayerAt(u8 mapx, u8 mapy)
{
	mapx += START_TILES_X;

	/** Test if in horizontal scroll view */
	if (mapx > 9  && mapx < 23)
	{
		_modViewx = 2;
		mapx = mapx - 9;
		_viewx = mapx*TILE_CX + _modViewx;
		gPlayerX = POS_HERO_X;
		gPlayerMapx = mapx;
	}
	else
	{
		/** Left Static part of view */
		_modViewx = 0;		
		if (mapx < 23)
		{
			_viewx = 0;
			gPlayerMapx = 0;			
		}
		/** Right Static part of view */
		else
		{
			_viewx = 16* TILE_CX;
			gPlayerMapx = 16;
		}

		gPlayerX = (mapx - gPlayerMapx + 1)*TILE_CX;
	}
	
	/** Test if in vertical scroll view */
	if (mapy >= 6 && mapy < 28)
	{
		_modViewy = 4;
		mapy = mapy - 6;
		_viewy = mapy*TILE_CY + _modViewy;
		gPlayerY = POS_HERO_Y;
		gPlayerMapy = mapy;
	}
	else
	{		
		/** Upper Static part of view */
		_modViewy = 0;
		if (mapy < 28)
		{
			_viewy = 0;
			gPlayerMapy = 0;
		}
		/** Lower Static part of view */
		else
		{
			_viewy = 22 * TILE_CY;
			gPlayerMapy = 22;
		}

		gPlayerY = (mapy - gPlayerMapy)*TILE_CY;
	}
}

/*****************************************************/
/*													 */
/*	Get tiles collided								 */
/*												     */
/*****************************************************/
u8** GetCollidedTiles(SRect* tiles)
{
	static const u8* sTileElem[NB_TILES_COLL];

	u8* tileMap = gLevel + tiles->yt*MAP_CX;
	sTileElem[0] = &tileMap[tiles->xl];
	sTileElem[1] = &tileMap[tiles->xr];
		
	tileMap = gLevel + tiles->yd*MAP_CX;
	sTileElem[2] = &tileMap[tiles->xl];
	sTileElem[3] = &tileMap[tiles->xr];

	return sTileElem;
}

/*****************************************************/
/*													 */
/*	Test if collision								 */
/*												     */
/*****************************************************/
u8* GetCollidedItem(u8 type, SRect* tiles)
{	
	u8* tileReturn = NULL;
	const u8** tilesCollided = GetCollidedTiles(tiles);
	
	if (type == PLAYER)
		gPlayerTileOn = DGN_GROUND;

	/** Test tiles */
	for (u8 i = 0; i < NB_TILES_COLL; i++)
	{
		const u8* curTile = *tilesCollided;
		if (curTile != tileReturn)
		{
			BOOL isStaticTile = FALSE;

			u8 tileId = *curTile;
			if (tileId != DGN_GROUND)
			{
				isStaticTile = TRUE;

				if (type == PLAYER)
					isStaticTile = (tileId != DGN_ILLUSION_WALL_V && tileId != DGN_ILLUSION_WALL_H);
			}

			if (isStaticTile)
			{
				if (type == PLAYER)
				{
					const u8* interactTile = InteractPlayer((u8*)curTile);
					if (interactTile != NULL)
					{
						if (tileId == DGN_EXIT)
							return NULL;

						tileReturn = (u8*)interactTile;
						if (tileId == DGN_SHOP)
							break;
					}
				}
				else
					return (u8*)curTile;
			}
		}

		tilesCollided++;
	}

	return tileReturn;
}

/*****************************************************/
/*													 */
/*	Get tiles										 */
/*												     */
/*****************************************************/
SRect* GetTiles(u8 mapx, u8 mapy, u8 x, u8 y)
{
	static SRect tiles;

	tiles.xr = tiles.xl = mapx + x / TILE_CX;
	if (x % TILE_CX != 0)
		tiles.xr++;
	
	tiles.yd = tiles.yt = mapy + y / TILE_CY;
	if (y % TILE_CY != 0)
		tiles.yd++;
	
	return &tiles;
}

/*****************************************************/
/*													 */
/*	Get player tiles								 */
/*												     */
/*****************************************************/
SRect* GetPlayerCoveredTiles()
{
	u8 x = gPlayerX + _modViewx - BORDER_CX;
	u8 y = gPlayerY + _modViewy;
	
	return GetTiles(gPlayerMapx, gPlayerMapy, x, y);
}

/*****************************************************/
/*													 */
/*	Get player position								 */
/*												     */
/*****************************************************/
SPos* GetPlayerPos()
{
	static SPos pos;

	pos.mapx = (gPlayerMapx + gPlayerX / TILE_CX) - BORDER_CX / TILE_CX;
	pos.mapy = gPlayerMapy + gPlayerY / TILE_CY;

	pos.x = gPlayerX % TILE_CX + _viewx % TILE_CX;
	pos.y = gPlayerY % TILE_CY + _viewy % TILE_CY;

	pos.dir = gPlayerDir;

	return &pos;
}

/*****************************************************/
/*													 */
/*	Get item tiles									 */
/*												     */
/*****************************************************/
SRect* GetItemCoveredTiles(SPos* pos)
{
	return GetTiles(pos->mapx, pos->mapy, pos->x, pos->y);
}

/*****************************************************/
/*													 */
/* Test items collision								 */
/*												     */
/*****************************************************/
BOOL AreItemsCollide(u8 x1, u8 y1, u8 x2, u8 y2)
{
#define ValueInRange(value, min, max)	((value >= min) && (value < max))
			
	return	(ValueInRange(x1, x2, x2 + TILE_CX) ||
			ValueInRange(x2, x1, x1 + TILE_CX)) 
			&& 
			(ValueInRange(y1, y2, y2 + TILE_CY) ||
			ValueInRange(y2, y1, y1 + TILE_CY));
}

/*****************************************************/
/*													 */
/*	Scroll sprite X									 */
/*												     */
/*****************************************************/
void ScrollGroundX(u8 move)
{
	u8 scroll = FALSE;
	BOOL isRight = move & RIGHT;
	
	u8 oldCharax	= gPlayerX;
	u8 oldMapx		= gPlayerMapx;
	u8 oldModViewx	= _modViewx;
	u8 oldViewx		= _viewx;
	
	u8 playerX		= gPlayerX;
	u8 viewX		= oldViewx;
	
	if (playerX == POS_HERO_X)
	{
		/** Compute character in world position */
		if (isRight)
		{
			if (viewX < VIEW_CX)
			{
				_viewx += SPEED_H;
				scroll = TRUE;
			}
		}
		else
		{		
			if ( viewX > 0)
			{
				_viewx -= SPEED_H;
				scroll = TRUE;
			}
		}
	}
	
	if (scroll == TRUE)
	{
		viewX = _viewx;
		
		gPlayerMapx = viewX / TILE_CX;
		_modViewx = viewX % TILE_CX;
	}
	else
	{
		if (isRight)
		{
			if (playerX < LIMIT_RIGHT)	
				gPlayerX++;
		}
		else
		{
			if (playerX > LIMIT_LEFT) 
				gPlayerX--;
		}
	}
	
	if (IsPlayerCollide())
	{
		gPlayerX	= oldCharax;
		gPlayerMapx	= oldMapx;		
		_modViewx	= oldModViewx;
		_viewx		= oldViewx;
	}
}

/*****************************************************/
/*													 */
/*	Scroll sprite Y									 */
/*												     */
/*****************************************************/
void ScrollGroundY(u8 move)
{	
	u8 scroll = FALSE;
	BOOL isTop = move & TOP;

	u8 oldCharay	= gPlayerY;
	u8 oldMapy		= gPlayerMapy;	
	u8 oldModViewy	= _modViewy;
	u16 oldViewy	= _viewy;
	
	u8 playerY		= gPlayerY;
	u16 viewY		= oldViewy;
	
	if (playerY == POS_HERO_Y)
	{
		/** Compute character in world position */
		if (isTop)
		{
			if (viewY < VIEW_CY)
			{
				_viewy += SPEED_V;
				scroll = TRUE;
			}
		}
		else
		{		
			if (viewY > 0)
			{
				_viewy -= SPEED_V;
				scroll = TRUE;
			}	
		}		
		
	}
	
	if (scroll == TRUE)
	{
		viewY = _viewy;
		
		gPlayerMapy = viewY / TILE_CY;
		_modViewy = viewY % TILE_CY;
	}
	else
	{
		if (isTop)
		{
			if (playerY < LIMIT_TOP)
				gPlayerY += SPEED_V;
		}
		else
		{
			if (playerY > LIMIT_DOWN)
				gPlayerY -= SPEED_V;
		}
	}

	if (IsPlayerCollide())
	{
		gPlayerMapy	= oldMapy;	
		gPlayerY	= oldCharay;		
		_modViewy	= oldModViewy;		
		_viewy		= oldViewy;
	}
}

/*****************************************************/
/*													 */
/*	Scroll View										 */
/*												     */
/*****************************************************/
void ScrollView(u8 dir)
{
	/** Move vertically */
	if (dir & (TOP | DOWN))
		ScrollGroundY(dir);

	/** Move horizontally */
	if (dir & (LEFT | RIGHT))
		ScrollGroundX(dir);
}

/*****************************************************/
/*													 */
/*	Convert SPos in SScreen							 */
/*												     */
/*****************************************************/
BOOL ConvertToScreen(SPos* pos, SScreen* screen)
{
	u8 screenCY = 0;
	u8 playerMapy = gPlayerMapy;
	u8 viewY = _modViewy;

	i16 y = (pos->mapy - playerMapy)*TILE_CY + pos->y - viewY;
	
	if (y < VIEW_HEIGHT)
	{	
		if (y < 0)
		{
			if (y > (-TILE_CY))
				screenCY = (u8)(TILE_CY + y);
				
			screen->y = 0;
		}
		else
		{
			screenCY = ((u8)(y) > VIEW_HEIGHT - TILE_CY) ? VIEW_HEIGHT - (u8)(y) : TILE_CY;
			screen->y = (u8)y;
		}
	}
	
	if (screenCY != 0)
		screen->x = (pos->mapx - gPlayerMapx)*TILE_CX + pos->x - _modViewx + BORDER_CX;
	
	screen->cy = screenCY;
	
	return (screenCY > 0 && screen->x > (VIEW_LEFT - TILE_CX) && screen->x < VIEW_RIGHT);
}

/*****************************************************/
/*													 */
/*	Clear item on border							 */
/*												     */
/*****************************************************/
void ClearTileBorder(u8* video, u8 x, u8 cy)
{
	u8* fillVideo = (video - x) + ((x < VIEW_LEFT) ? TILE_CX + 1 : VIEW_RIGHT);
	cpct_drawSolidBox(fillVideo, BORDER_COLOR, TILE_CX-1, cy);
}

/*****************************************************/
/*													 */
/*	Draw sprite to view								 */
/*												     */
/*****************************************************/
void DrawItem(SScreen* screen, u8* sprite, BOOL transparent)
{
	u8 x = screen->x;
	u8 cy = screen->cy;	

	u8* video = GetBackScreenPtr(x, (u8)screen->y);
	
	u8 spriteOffY = (screen->y == 0) ? (TILE_CY - cy) * TILE_CX : 0;
	u8* spriteOffset = sprite + spriteOffY;
		
	if (transparent)
		cpct_drawSpriteMaskedAlignedTable(spriteOffset, video, TILE_CX, cy, gMaskTable);
	else
	{
		if (cy != TILE_CY)
			asm_drawMapTile(spriteOffset, video, cy);
		else
			asm_drawFullMapTile(sprite, video);
	}

	if (x < VIEW_LEFT || x > VIEW_RIGHT - TILE_CX)
		ClearTileBorder(video, x, cy);
}

/*****************************************************/
/*													 */
/*	Update alternate sprite							 */
/*												     */
/*****************************************************/
void UpdateSprites()
{
	if (gTimer > ALT_SPRITE_TIMER)
	{	
		gTimer = 0;
		if (++_animSprite > STEP_ANIM)
		{
			_animSprite = 0;

			if (++_animSpriteSlow > STEP_ANIM)
				_animSpriteSlow = 0;
		}
	}
}

/*****************************************************/
/*													 */
/*	Get tile animation								 */
/*												     */
/*****************************************************/
u8 GetAnimateTile(u8 tile)
{
	return (tile == DGN_TORCH || tile == DGN_TELEPORTER) ? _animSprite : _animSpriteSlow;
}

/*****************************************************/
/*													 */
/*	Draw Map										 */
/*												     */
/*****************************************************/
void DrawMap()
{		
	#define H_WALL			5*TILE_SIZE
	#define V_WALL			6*TILE_SIZE

	u8* pvmem = GetBackBufferPtr();
	u8* tileMap = gLevel + gPlayerMapx + gPlayerMapy*MAP_CX;
	u8 curTile = DGN_GROUND;
		
	u8 cy = TILE_CY - _modViewy;
	u8 spriteOffY = _modViewy * TILE_CX;
		
	u8 y = 0;
	while (y <= NB_TILES_CY)
	{			
		if (cy != 0)
		{						
			u8 x;
			for (x = START_TILES_X; x < END_TILE_X; x++)
			{
				u8 tile = *tileMap;				
				if (tile != DGN_GROUND && tile != OBJ_INVISIBLE_WALL && ((x + 1) != END_TILE_X  || _modViewx != 0))
				{				
					if (tile < MONSTERS)
					{													
						u8* video = pvmem - _modViewx + x*TILE_CX;							
						
						if (curTile != tile)
						{
							u8 curAnim;
							
							curTile = tile;					
							if (curTile > DGN_WALLS || curTile <= WALLS_END)
							{
								_sprite = (u8*)gLevelTileset + ((tile > DGN_WALLS) ? (tile == DGN_ILLUSION_WALL_H || tile == DGN_TRIGGER_WALL_H) ? H_WALL : V_WALL :
									(tile - 1)*TILE_SIZE);
							}
							else
							if (tile >= DGN_ALTERN_SPRITE && tile <= DGN_ALTERN_END && (curAnim = GetAnimateTile(tile)) > 0)
							{
								u8 tileAnim = (tile - DGN_ALTERN_SPRITE)*STEP_ANIM + curAnim - 1;
								_sprite = (u8*)(g_anim_tiles_items + (u16)(tileAnim*TILE_SIZE));
							}
							else
								_sprite = DeflateSpriteTile(curTile);
						}

						if (cy != TILE_CY)
							asm_drawMapTile((u8*)(_sprite + spriteOffY), video, cy);
						else
							asm_drawFullMapTile((u8*)_sprite, video);
						
						if (_modViewx != 0 && (x == START_TILES_X || x == END_TILE_X - 1))
							ClearTileBorder(video, x*TILE_CX - _modViewx, cy);					
					}
					else
					{
						u8 xMonst = x + gPlayerMapx - START_TILES_X;
						u8 yMonst = y + gPlayerMapy;
						CreateMonster((u8*)tileMap, xMonst, yMonst);
					}
				}			
				
				tileMap++;
			}
		}
		else 
			tileMap += NB_TILES_CX - 1;
		
		tileMap += (MAP_CX - NB_TILES_CX + 1);
		
		if (++y == NB_TILES_CY)
		{
			cy = _modViewy;	
			pvmem = GetBackScreenPtr(0, VIEW_HEIGHT - cy);
		}
		else
		{			
			spriteOffY = 0;
			cy = TILE_CY;
			pvmem = GetBackScreenPtr(0, y*TILE_CY - _modViewy);
		}
	}
 }

/*****************************************************/
/*													 */
/*	Move X											 */
/*												     */
/*****************************************************/
void MoveX(SPos* pos, BOOL moveLeft)
{
	#define SPEED_X			1
	#define LIMIT_TILE_X	(TILE_CX - SPEED_X) // 8 - 1 = 14
	
	if (moveLeft)
	{
		if (pos->x == 0)
		{
			pos->x = LIMIT_TILE_X;
			pos->mapx--;
		}
		else
			pos->x -= SPEED_X;
	}
	else
	{
		if (pos->x == LIMIT_TILE_X)
		{
			pos->x = 0;
			pos->mapx++;
		}		
		else
			pos->x += SPEED_X;
	}
}

/*****************************************************/
/*													 */
/*	Move Y											 */
/*												     */
/*****************************************************/
void MoveY(SPos* pos, BOOL moveUp)
{
	#define SPEED_Y			2
	#define LIMIT_TILE_Y	(TILE_CY - SPEED_Y) // 16 - 2 = 14

	if (moveUp)
	{
		if (pos->y == 0)
		{
			pos->y = LIMIT_TILE_Y;
			pos->mapy--;
		}
		else
			pos->y -= SPEED_Y;
	}
	else
	{
		if (pos->y == LIMIT_TILE_Y)
		{
			pos->y = 0;
			pos->mapy++;
		}
		else
			pos->y += SPEED_Y;
	}
}

/*****************************************************/
/*													 */
/*	Fill screen										 */
/*												     */
/*****************************************************/
void FillScreen()
{
	u8* vmem = GetBackBufferPtr();
	
#ifdef WINCPCTELERA
	cpct_memset(vmem, 0, 80*160);
#else
	asm_fillView(vmem);
		
#endif // WINCPCTELERA
}

/*****************************************************/
/*												     */
/*	Draw Level number								 */
/*												     */
/*****************************************************/
void DrawLevelNumber()
{
	PrintInt(LEVEL_INFO_X, LEVEL_INFO_Y, gDungeon.level, LEVEL_INFO_DIGITS, BOTH_BUFFER);
}

/*****************************************************/
/*												     */
/*	Draw View										 */
/*												     */
/*****************************************************/
void DrawView()
{
	FillScreen();	
	UpdateSprites();
	DrawMap();
}
