//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Deeper Warrens
//  Copyright (C) 2018 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

// Tile size
#define TILE_CX				4
#define TILE_CY				16
#define TILE_SIZE			(TILE_CX*TILE_CY)

// Level tile size
#define TILESET_LEVEL_CX 	4
#define TILESET_LEVEL_CY 	256
#define TILESET_SIZE		(TILESET_LEVEL_CX*TILESET_LEVEL_CY)

// Border size
#define BORDER_CX			(TILE_CX*2)
#define BORDER_CY			TILE_CY

// Map size
#define MAP_CX				32
#define MAP_CY				32
#define MAP_SIZE			(MAP_CX*MAP_CY)

// Map view
#define MAP_VIEW_CX			(u8)(SCREEN_CX / MAP_CX)
#define MAP_VIEW_CY			(u8)(SCREEN_CY / MAP_CY)

// Nb tiles on screen
#define NB_TILES_CX			18
#define NB_TILES_CY			10
#define OUTSIDE_TILES		2

// View size
#define VIEW_LEFT			(TILE_CX*2)
#define VIEW_RIGHT			(SCREEN_CX - TILE_CX*2)

#define VIEW_CY				((MAP_CY - NB_TILES_CY)*TILE_CY)
#define VIEW_CX				((MAP_CX - NB_TILES_CX)*TILE_CX + BORDER_CX)
#define VIEW_HEIGHT			(NB_TILES_CY*TILE_CY)
#define VIEW_WIDTH			(NB_TILES_CX*TILE_CX)

// View limit
#define LIMIT_TOP			(VIEW_HEIGHT - HERO_CY - TILE_CY)
#define LIMIT_DOWN			BORDER_CY

#define LIMIT_RIGHT			(SCREEN_CX - BORDER_CX - TILE_CX - HERO_CX)
#define LIMIT_LEFT			(BORDER_CX + TILE_CX)

#define BORDER_COLOR		cpctm_px2byteM0(1,1)

#define IS_DOOR(tileId)		(tileId >= DGN_DOOR_V_RED && tileId <= DGN_DOOR_H_BLUE)

void MoveY(SPos* pos, BOOL moveUp);
void MoveX(SPos* pos, BOOL moveLeft);

void MovePlayerAt(u8 mapx, u8 mapy);

void ScrollView(u8 dir);
void InitView();
void DrawView();
void DrawLevelNumber();
void DrawItem(SScreen* screen, u8* sprite, BOOL transparent);
BOOL ConvertToScreen(SPos* pos, SScreen* screen);
BOOL AreItemsCollide(u8 x1, u8 y1, u8 x2, u8 y2);

u8* DeflateSpriteTile(u8 tileType);
SRect* GetItemCoveredTiles(SPos* pos);
SRect* GetPlayerCoveredTiles();
u8 GetAnimateTile(u8 tile);
u8* GetCollidedItem(u8 type, SRect* tiles);
SPos* GetPlayerPos();
