;***********************************************************************
;MAIN SCRIPT
;***********************************************************************
.assem_run:					;winape run
	di
	im 1
	ld hl,#c9fb				;deshabilita firmware
	ld (#38),hl				;ret en &38
	ld sp,STACK				;mover la pila
	ei
	;------------
	if M4_ENABLED
	call find_m4_rom			;busca la expansion M4 Board 
	endif					;(usada ms adelante si se detecta)
	;------------
	call scr_fill				;rellena - limpia #C000 y #8000
	;------------
	;cambiar mode y paleta
	ld bc,#7F8C				;gate array (8C mode 0)
	out (c),c				;set mode
	ld hl,pal1				;pal addr
	ld a,(hl)				;contenido de pal
	ld c,#10				;border (#7f10)
	out (c),c				;border
	out (c),a				;set border
	xor a					;ini pen (0)
	ld d,15					;end pen +1
	call setcolors				;HL,A,D	(pal addr, ini pen, end pen)
	;------------
	call dump_sprites			;volcar sprites rotados en otra zona de memoria
	ifnot ROM
	call mod_sprites			;modifica sprites con el fondo de pantalla
	endif
	;------------
	call init_table				;crea una tabla de direcciones de pantalla
;***********************************************************************
.open:					;creditos al inicio
	if SHWOCREDITS
	call clear_scr
	call credits
	call credits_pause
	endif
;***********************************************************************
.menu:
	ld sp,STACK			;reinicia la pila
	if M4_ENABLED
	call get_time			;M4 ntp time
	endif
	call PLY_Stop			;arkos music stop
	call crt8000			;pantalla en #8000
	if ROM
	call clear_scr			;limpiar pantalla #C000
	else
	call copy_buffer		;copiar pantalla #8000 en #C000
	endif
	call setup_menu			;pintar menu
	call crtC000			;pantalla en #C000
.menu_loop:
	call wait_vsync			;esperar vsync
	call menu_scan			;control de teclas del menu
	call menu_counter		;contador demo
	jr menu_loop			;bucle del menu
;***********************************************************************
.prep_start:
	call clear_results		;borrar resultados
	if REG
	call regia_clear		;borrar datos de registros
	endif
.prep_start_wresults:
	call PLY_Stop			;music stop
	call crt8000
	if ROM
	call clear_scr			;limpiar pantalla
	else
	call copy_buffer
	endif

	call init1			;inicializa vars
	
	ld a,(modo)
	cp 6:
	jr z,ready			;modo 6 (demo) se salta waiting

	call setup_balls		;pinta balls
	call setup_players		;pinta texto players,score
	;----------------------
	call crtC000
	call points_bonus		;rutina de bonus
	ld de,#0D08			;shape psg reg/val
	call psg_reg			;apao bug song1 canal b
	ld de,SONG1			;music
	call PLY_Init			;arkos player init
;***********************************************************************
.waiting:
	call wait_vsync							;espera vsync
	ld a,(options):bit BIT_SOUND,a:call nz,PLY_Play			;arkos music
	call keyb_scan							;buscar tecla
	ld a,(keyb_buffer+LIN_EXIT):bit BIT_EXIT,a:jp z,menu		;ESC, menu
	ld a,(keyb_buffer+LIN_READY):bit BIT_READY,a:jr z,ready		;tecla enter (return), ready
	;ld a,(keyb_buffer+LIN_KEY0):;bit BIT_KEY0,a:;jr z,ready	;tecla 0, ready
	jr waiting
;***********************************************************************
.ready:
	call PLY_Stop			;arkos music stop
	call crt8000			;pantalla en #8000
	ifnot ROM
	call disspr_text_mod1		;modificar rutina de texto
	endif
	call clear_scr			;limpiar pantalla
	call setup_board		;pinta tablero
	call setup_score		;pinta score
	call setup_balls		;pinta balls
	call init2			;inicializa vars
	call ball_init			;inicializa bola
	call msg_start			;pinta mensaje
	call crtC000			;pantalla en #C000
.ready_countdown:
	call wait_vsync			;espera vsync
	call draw_moves			;pinta figuras
	call keyb_scan			;escanea teclado
	call controlUD1			;j1 se puede mover
	call controlUD2			;j2 se puede mover
	call controlIA2_prep		;ia2 coloca figuras
	call controlIA1_prep		;ia1 coloca figuras
	call dummy_modaddr		;act. posiciones
	ld de,COUNTDOWN_STR_POS		;string pos
	call countdown			;texto cuenta atrs (retorna flag zero)
	jr nz,ready_countdown		;bucle
;***********************************************************************
.start:
	call posession_reset		;init posession
	call points_restore		;init score (points)
	call snd_whistle		;sonido comenzar partido
	call check_balls2		;decrementa 1a bola al sacar
	call show_ball			;mostrar bola
	ld sp,STACK
;***********************************************************************
.mainloop:
	call wait_vsync2	;espera vsync
	call restore_scr	;pinta back buffer
	ld hl,(bxy)		;ball XY
	call goals		;comprobar goles, limites ball X (req. HL)
	call bposy       	;comprobar limites ball Y	(req. HL)
	call bmovex         	;mueve ball X	(requiere HL)
	call bmovey         	;mueve ball Y	(requiere HL)
	call draw_moves		;pinta figuras
	call show_ball		;guarda buffer y pinta bola
	call check_ball_area	;resuelve area
	call shot_count		;shot timeout
	if FAKEBY
	call ia_fakey		;imprecisin IA
	endif
	call keyb_scan		;escanear
	call controlUD1		;control j1
	call controlUD2		;control j2
	call controlIA1		;control ia1	
	call controlIA2		;control ia2	
	call dummy_modaddr	;modifica posiciones de figuras	y back buffer		
	call touch_ball		;lee buffer de la bola y resuelve rebotes disparo etc
	call posession
	call ball_count		;decrementar contadores ball xy
;------------------------------------------------------------------------
	ld a,(keyb_buffer+LIN_EXIT):bit BIT_EXIT,a:jp nz,mainloop	;no ESC, mainloop
	call msg_exit							;mostrar msg salir
	.loop_exit:
		call wait_vsync
		call keyb_scan						;buscar tecla S o N
		ld a,(keyb_buffer+LIN_YES):bit BIT_YES,a:jp z,menu		;salir si, ir a menu
		ld a,(keyb_buffer+LIN_NO):bit BIT_NO,a:jr z,return_main_loop	;salir no, volver al mainloop
		jr loop_exit
	.return_main_loop:
		call points_restore					;pintar puntos
		jp mainloop						;vuelve al juego
;***********************************************************************
.wait_vsync:				;antes de hacer el vsync cada 8 frames cambiara la pluma 15				
	ld a,1				;variable
	rrca				;gira el byte
	ld (wait_vsync+1),a		;automodificar variable
	bit 1,a				;si el bit es 0
	jr z,wait_vsync2		;salta cambio de color
	ld a,(flickcolor)		;color parpadeante
	xor 2				;invertir el bit
	ld (flickcolor),a		;guarda color
	ld bc,#7f0f			;llama al gate array
	out (c),c
	out (c),a			;cambia color
.wait_vsync2:				;y ahora si, vsync
	ld b,#f5
	.vsync:
	in a,(c)
	rra
	jr nc,vsync
	ret