;********************************************************************************
;Inicializar Tabla scr addr
;********************************************************************************
;Routine based on the Kevin Thacker's techniques / initialise by AMSDOS
;cpctech.cpc-live.com/source/rotospr.html
;www.cpcwiki.eu/forum/programming/asm-source-code/100/
 .init_table:
 	ld hl,#c000			;direccin de comienzo
	ld de,tableC000			;apuntar a donde guardar
	ld b,25				;num filas
 .init_table_lp1:
	 push bc			;preservar registros
	 push hl
		ld a,8			;num lineas por fila
		.init_table_lp2:
		ex af,af'		;preservar lineas
		ex de,hl		;intercambiar regs
		ld (hl),e:inc hl	;anotar direccin e inc puntero
		ld (hl),d:inc hl
		ex de,hl		;intercambiar regs
		ld bc,#800		;linea
		add hl,bc		;aadir linea
		ex af,af'		;recuperar num lineas
		dec a			;dec lineas
		jr nz,init_table_lp2	;bucle si nozero
	 pop hl				;recuperar direccin
	 ld bc,#50			;bloque de lineas
	 add hl,bc			;aadir bloque de lineas
	 pop bc				;recuperar filas
	 djnz init_table_lp1		;dec filas y bucle mientras nozero
	 ret

;********************************************************************************
;CRTC
;********************************************************************************
;cambiar memoria de video
.crt8000:ld a,#20:jr crt
.crtC000:ld a,#30:;jr crt
.crt:	;A (valor reg 12)
	ld bc,#bc0c			;crt reg 12
	out (c),c			;enviar reg
	inc b		;#BD		;crt valor
	out (c),a			;enviar valor
	dec b		;#BC
	inc c		;#0D		;crt reg 13
	out (c),c			;enviar reg
	inc b		;#BD		;crt valor
	defb #ed,#71	;out (c),0	;enviar valor
	ret

;********************************************************************************
;Setup match - options, var init, etc
;********************************************************************************

.init1:
	ld a,(menu_num_balls)			;copia bolas del menu
	ld (balls),a				;a bolas de juego
	ret
.init2:
	;----------------------------------------------------
	;options setup
	;----------------------------------------------------
	ld a,(options)				;bits de opciones
	ld d,a					;pasar a d
	
	;autobar1
	ld hl,c1_autobar			;apuntar a rutina
	bit BIT_AUTOBAR1,d			;mirar bit de opciones
	call z,ret_setup			;deshabilitar rutina
	call nz,nop_setup			;habilitar rutina
	
	;autozone1
	ld hl,c1_autozone
	bit BIT_AUTOZONE1,d
	call z,ret_setup
	call nz,nop_setup
	
	;autobar2
	ld hl,c2_autobar
	bit BIT_AUTOBAR2,d		
	call z,ret_setup
	call nz,nop_setup
	
	;autozone2
	ld hl,c2_autozone
	bit BIT_AUTOZONE2,d
	call z,ret_setup
	call nz,nop_setup


	;posession
	bit BIT_POSESSION,d			;mirar bit de opciones	
	ld a,#c9				;ret
	call z,posession_setup			;deshabilitar rutinas
	ld a,#00				;nop
	call nz,posession_setup			;habilitar rutinas

	;----------------------------------------------------
	;hands
	;----------------------------------------------------
	;modifica rutina de sprite
	ld a,#10			
	ld (disspr_height+1),a		;alto
	ld a,#02
	ld (disspr_jr+1),a		;salto
	ld a,#FB
	ld (disspr_line+1),a		;sig.linea
	
	;control init
	ld a,(c1_zone):call c1_hands		;determinar posicin de manos al inicio
	ld a,(c2_zone):call c2_hands
	
	;ia init
	ld a,(ia_zone)				;cargar bits IA
	bit BIT_IA1,a				;zona habilitada?	
	call nz,c1_hands			;pintar pareja de manos IA1
	ld a,(ia_zone)				;cargar bits IA
	bit BIT_IA2,a				;zona habilitada?	
	call nz,c2_hands			;pintar pareja de manos IA2

	;----------------------------------------------------
	;otros
	;----------------------------------------------------
	ld a,COUNTDOWN_TIME			;constante
	ld (countdown_num+1),a			;resetear cuenta-atrs
	
	if REG
	jp event_cancel				;cancelar ltimo evento (si lo hay)
	else
	ret
	endif
	
;modificar rutinas
;------------------
.posession_setup:				
	ld (posession),a	
	ld (posession_reset),a
	ret

.nop_setup:ld (hl),0:ret
.ret_setup:ld (hl),#c9:ret

;********************************************************************************
; DUMPS & FLIPS
;********************************************************************************
.dump_sprites:
	;blocks & equip
	ld hl,blockL_spr+2		;origen
	ld de,BLKREVSPR_LOC		;destino
	ld a,104			;lineas (alto)
	ld c,3				;bytes (ancho)
	call dump_horizontal
	
	;hand1L -> hand1R
	ld hl,hand1LA_spr+4		;origen
	ld de,hand1RA_spr		;destino
	ld a,32				;lineas (alto)
	ld c,5				;ancho
	call dump_horizontal

	ld a,#80			;bar pixel fix
	ld (hand1RA_spr+2),a
	ld (hand1RB_spr+2),a

	;hand1R  -> hand2L
	ld hl,HAND2L_LOC-5		;origen
	ld de,HAND2L_LOC		;destino
	ld a,32				;lineas (alto)
	ld c,5				;ancho
	call dump_vertical

	;hand1L + handle -> hand2R + handle
	ld hl,hand1LB_end-5		;origen
	ld de,hand2RB_spr		;destino
	ld a,48				;lineas (alto)
	ld c,5				;ancho
	;call dump_vertical

;vuelca sprites vertical 
;lee de abajo a arriba y copia de arriba a abajo
.dump_vertical: ;HL, DE, A, C (origen, destino, height, width)
	ld b,0
	.dump_vertical_lp:
	push hl				;preservar regs
	push bc
	ldir				;volcar ancho
	pop bc				;recuperar regs
	pop hl
	sbc hl,bc			;decrementar puntero
	dec a				;decrementar num lineas
	jr nz,dump_vertical_lp		;bucle
	ret 
	
;vuelca sprites horizontal 
;lee de dcha a izq y copia de izq a dcha con el color del byte invertido
.dump_horizontal: ;HL, DE, A, C (origen, destino, height, width)
	ex af,af'
		ld b,c			;bytes por linea
		.dump_horizontal_ln_lp
		push bc
		ld a,(hl)		;byte de origen
		call rev_pairs_a	;invertir byte
		ld (de),a		;byte a destino
		dec hl			;decrementar origen
		inc de			;incrementar destino
		pop bc
		djnz dump_horizontal_ln_lp	;bucle
		;ld b,0			;no es necesario, djnz deja B a 0
		add hl,bc		;aadir offset de C al puntero
		add hl,bc		;(baja 2 lineas, la copiada y la sig.)
	ex af,af'
	dec a				;dec height
	jr nz,dump_horizontal		;bucle mientras nozero
	ret

;invierte parejas de bits en el acumulador
.rev_pairs_a:	;in A -- out A
	ld b,a				;preservar byte
	rla				;girar a la izquierda
	and %10101010			;operar bits izquierdos
	ld c,a				;guardar bits
	ld a,b				;recuperar byte
	rrca				;girar dcha
	and %01010101			;operar bits derechos
	or c				;combinar resultados
	ret

;********************************************************************************
; MOD TRANSPARENT SPRITES
;********************************************************************************
;modifica pixels de fondo por los de pantalla en sprites contenidos en las siguientes tablas.
ifnot ROM
.mod_sprites:
	ld hl,equip_3x12_spr_table:call modspr_table
	ld hl,equip_6x2_spr_table:call modspr_table
	ld hl,equip_2x4_spr_table:;jr modspr_table
	;continua
.modspr_table:
	ld a,(hl):inc hl        ;width
	ld (modspr_width+1),a	;modificar rutina
	ld a,(hl):inc hl       	;height
	ld (modspr_height+1),a	;modificar rutina
	ld a,(hl):inc hl	;indice
	.modspr_table_lp:	;HL, A (tabla, ndice)
	ex af,af'		;preservar indice
	ld e,(hl):inc hl        ;scr addr
	ld d,(hl):inc hl 
	ld a,(hl):inc hl      	;sprite addr
	push hl			;preservar puntero
	ld h,(hl)		;sprite addr -> HL
	ld l,a
	call modspr		;rutina sprite (HL,DE)
	pop hl			;recuperar puntero
	inc hl
	ex af,af'		;recuperar indice
	dec a			;decrementar indice
	jr nz,modspr_table_lp 	;bucle
	ret	

.modspr:	;HL, DE (spr addr, scr addr)
	.modspr_height:
	ld b,12				;variable
	.modspr_lp:
	push de
		.modspr_width:
		ld c,3			;variable
		.modspr_lp2:
		call modspr_px		;modificar pixel
		inc hl			;inc punteros
		inc de
		dec c			;dec width
		jr nz,modspr_lp2	;bucle	
	pop de
	ld a,d
	add #08				;add line
	ld d,a
	call c,scr_next_line_de		;si hay acarreo calcular siguiente bloque de lineas
	djnz modspr_lp			;dec height, bucle mientras nz
	ret

;scr_next_line By Kevin Thacker
;www.cpctech.org.uk/source/cpl_spr.asm

.scr_next_line_de:	;DE (scr addr)
	ld a,e
	add #50
	ld e,a
	ld a,d
	adc #c0
	ld d,a
	ret

.modspr_px: ;HL, DE (spr addr, scr addr)
	ld a,(hl)		;contenido del sprite
	or a:jr z,modspr_px0	;buscar color de fondo (ink0)
	cp #04:jr z,modspr_px1	;buscar color de fondo (ink0/ink2)
	cp #08:jr z,modspr_px2	;buscar color de fondo (ink2/ink0)
	ret			;si no hay fondo, volver e incrementar punteros
.modspr_px0:
	ld a,(de)		;cargar pixels de pantalla
	ld (hl),a		;guardar resultado en el sprite
	ret
.modspr_px1:			;pixeles derechos
	ld a,(de)		;cargar pixels de pantalla
	and #aa			;anular pixels izquierdos de pantalla (si los hay)
	or (hl)			;combinar pixeles
	ld (hl),a		;guardar resultado en el sprite
	ret
.modspr_px2:			;pixeles izquierdos
	ld a,(de)		;lo mismo
	and #55			;anular los derechos
	or (hl)
	ld (hl),a
	ret
endif

