;***********************************************************************
; HANDS / HANDLES
;***********************************************************************
;funciones comunes manos y mangos
.get_table:	;in B, HL (hand bit value, table) - out HL (table w/offset)
	ld de,8				;num bytes offset entre tablas
	ld a,128			;iniciar indice (bit 7)
	.get_table_lp:
	cp b				;comparar indice y value
	ret z				;retorna tabla sprite (hl)
	add hl,de			;sino, incrementar offset puntero
	rra				;girar byte a la dcha
	jr get_table_lp			;bucle

.get_hand_flag:	;in B (bit value)  - out L (bit ins)
	ld hl,INS_BIT0			;L instruccion bit 7 (#47)
	ld de,8				;E restar instruccion bit
	ld a,1				;A indice
	.get_hand_flag_lp:
	cp b				;comparar indice y value
	jr z,hand_flag			;si son igual ir a subrutina
	add hl,de			;incrementar instruccion bit
	rla				;girar byte a la izq		
	jr get_hand_flag_lp		;bucle

.hand_flag:	;in L (bit ins) - out AF (hands byte, zero flag)
	ld a,l
	ld (hand_common_ins_bit+1),a	;modificar instruccion
	ld a,(hands)			;cargar hands byte
	.hand_common_ins_bit:	
	bit 0,a				;comparar bit en instruccion modificada
	ret				;retornar F (zero flag) / A (hands byte)
;-------------------------------------------------
;manos desde autozona
.autohand:	;B	(bit value)
	call get_hand_flag		;comprobar flag
	jr z,autohand_back
	;jr nz,autohand_forw		;continua (no es necesario)
.autohand_forw:	;B	(bit value)
	ld hl,hand_pt1_1_spr_table	;establecer puntero (mano delante sin offset)
	jr autohands_tables
.autohand_back:	;B	(bit value)
	ld hl,hand_pt1_2_spr_table	;establecer puntero (mano detras sin offset)
	;jr autohands_tables		;continua (no es necesario)
.autohands_tables:;B,HL	(bit value, table)
	call get_table			;buscar offset de la tabla
	ld a,1				;numero de tablas (sprites)
	jr table_spr_lp			;tabla sprite sin indice
;-------------------------------------------------
;mangos
.handle_tables:	;B	(bit value)
	ld hl,handle_pt1_spr_table	;establecer puntero (mangos)
	call get_table			;buscar offset de la tabla
	ld a,2				;numero de tablas (sprites)
	jr table_spr_lp			;tabla sprite sin indice
;-------------------------------------------------
;manos desde control
.hand_back:	;B	(bit value)
	call get_hand_flag:ret z	;salir si el flag es zero
	ld hl,hand_pt1_2_spr_table	;establecer puntero
	jr hand_xor
.hand_forw:	;B	(bit value)
	call get_hand_flag:ret nz	;salir si el flag no es zero
	ld hl,hand_pt1_1_spr_table	;establecer puntero
	;jr hand_xor			;continua (no es necesario)
.hand_xor:	;A, B, HL (hands byte, hand bit, sprite table)
	ex af,af'			;preservar hands byte
	call get_table			;buscar offset de la tabla
	ex af,af'			;recuperar hands byte
	xor b				;alternar bit seleccionado
	ld (hands),a			;guardar hands byte
	ld a,1				;numero de tablas (sprites)
	jr table_spr_lp			;tabla sprite sin indice

;***********************************************************************
; SPRITE TABLE 
;***********************************************************************
;routine based on techniques of Prodatron
;www.amstrad.es/forum/viewtopic.php?t=161&start=45

.table_spr:	;HL (tabla)	;con medidas e indices preajustados en la tabla
	ld b,(hl):inc hl
	ld a,6				;MAX BYTES
	sub b
	add a
	ld (disspr_jr+1),a		;width	
	xor a
	sub b
	ld (disspr_line+1),a		;sig. linea
	ld a,(hl):inc hl
	ld (disspr_height+1),a		;height
	ld a,(hl):inc hl		;indice
	.table_spr_lp:	;HL, A (tabla, ndices)	;sin medidas e indices preajustados en la tabla
	ex af,af'		;preservar indice
	ld c,(hl):inc hl        ;scr addr
	ld b,(hl):inc hl
	ld e,(hl):inc hl      	;sprite addr
	ld d,(hl):inc hl
	push hl			;preservar puntero
	ld h,b			;scr addr -> HL
	ld l,c
	.disspr_height:
	ld a,8				;repetir bucle (altura)
	.disspr_lp: 
	ex de,hl			;intercambiar spr/scr addr
	.disspr_jr:jr disspr_jr		;salto relativo ldi (width)
	ldi:ldi:ldi:ldi:ldi:ldi
	dec a				;decrementar alto
	jr z,zzzzzz			;si zero, terminar
	.disspr_line:
	ld bc,#700			;offset relativo sig.linea
	ex de,hl			;intercambiar spr/scr addr
	add hl,bc			;aadir linea y comprobar acarreo
	jr nc,disspr_lp			;si no hay carry, bucle
	ld bc,#c050			;offset sig. bloque de lineas
	add hl,bc			;aadir bloque
	jr disspr_lp			;bucle
	zzzzzz:
	pop hl			;recuperar puntero
	ex af,af'		;recuperar indice
	dec a			;decrementar indice
	jr nz,table_spr_lp 	;bucle mientras nozero
	ret

;***********************************************************************
; BUFFER / TEXT
;***********************************************************************
;routine based on techniques of Prodatron
;www.amstrad.es/forum/viewtopic.php?t=161&start=45

.restore_scr:
	ld hl,#0000		;variable
	ld de,back_buffer	;back de la bola
	ld a,5			;alto
.disspr2:		;A, HL, DE (height, scr addr, sprite addr)
	ex de,hl		;intercambiar spr/scr addr
	ldi:ldi			;pintar 2 pixels de ancho
	dec a			;decrementar alto
	ret z			;salir si fin alto
	ld bc,#7fe		;sig.linea
	ex de,hl		;intercambiar spr/scr addr
	add hl,bc		;aadir linea y comprobar acarreo
	jr nc,disspr2		;si no hay carry, bucle
	ld bc,#c050		;offset sig. bloque de lineas
	add hl,bc		;aadir bloque
	jr disspr2		;bucle

;***********************************************************************
; TABLEBOARD 
;***********************************************************************
.setup_board: ;pintar mesa
ifnot ROM:call fill_box_mod1:endif
	call nl_de2_line_two				;mod salta lineas
	ld hl,board_fills_sp:call table_fill		;lineas, fills
	call nl_de2_line_normal				;mod normal
	ld hl,board_fills:call table_fill		;lineas, fills
	ld hl,board_3x8_spr_table:call table_spr	;sprites mesa
	ifnot ROM
	ld hl,lata_6x14_spr_table:call table_spr
	endif
	ld hl,all_handle_spr_table:call table_spr	;mangos
	;figuras
	ld a,#ff:ld (show_move),a			;ff pintar todo
	;continuar draw_moves
;***********************************************************************
; DRAW MOVES
;***********************************************************************
.draw_moves:			;8492
	ld a,(show_move)		;control de bits de las figuras
	bit BIT_PT1,a:call nz,draw_pt1
	bit BIT_DL2,a:call nz,draw_dl2	;si el bit es 1 pinta el correspondiente
	bit BIT_DL1,a:call nz,draw_dl1
	bit BIT_DF2,a:call nz,draw_df2
	bit BIT_DF1,a:call nz,draw_df1
	bit BIT_MD2,a:call nz,draw_md2
	bit BIT_MD1,a:call nz,draw_md1
	bit BIT_PT2,a:call nz,draw_pt2
	xor a:ld (show_move),a		;una vez pintadas se resetea hasta el sig frame
	ret
.draw_pt1:				;dibuja portero
	ld a,PT_NUM_FIG			;num figuras
	.column_pt1:ld hl,#c3C9		;scr addr
	.choose_pt1:ld de,p1c_spr	;sprite		
	ld bc,PT_OFFSET_SCR		;offset entre figuras
	jr dummy_spr			;rutina sprite
.draw_df1:
	ld a,DF_NUM_FIG						
	.column_df1:ld hl,#c2e1
	.choose_df1:ld de,p1c_spr
	ld bc,DF_OFFSET_SCR		
	jr dummy_spr			
.draw_dl2:
	ld a,DL_NUM_FIG
	.column_dl2:ld hl,#c249
	.choose_dl2:ld de,p2c_spr
	ld bc,DL_OFFSET_SCR
	jr dummy_spr	
.draw_md1:
	ld a,MD_NUM_FIG
	.column_md1:ld hl,#c201
	.choose_md1:ld de,p1c_spr
	ld bc,MD_OFFSET_SCR
	jr dummy_spr	
.draw_md2:
	ld a,MD_NUM_FIG
	.column_md2:ld hl,#c209
	.choose_md2:ld de,p2c_spr
	ld bc,MD_OFFSET_SCR
	jr dummy_spr	
.draw_dl1:
	ld a,DL_NUM_FIG
	.column_dl1:ld hl,#c261
	.choose_dl1:ld de,p1c_spr
	ld bc,DL_OFFSET_SCR
	jr dummy_spr
.draw_df2:
	ld a,DF_NUM_FIG
	.column_df2:ld hl,#c309
	.choose_df2:ld de,p2c_spr
	ld bc,DF_OFFSET_SCR
	jr dummy_spr	
.draw_pt2:
	ld a,PT_NUM_FIG
	.column_pt2:ld hl,#c401
	.choose_pt2:ld de,p2c_spr
	ld bc,PT_OFFSET_SCR
	jr dummy_spr
;----------------------------------------------------------------
;routine based on techniques of Prodatron
;www.amstrad.es/forum/viewtopic.php?t=161&start=45

.dummy_spr:	;A,BC,HL,DE (num figuras, offset figuras, scr addr figura1, sprite addr)
	ex af,af'		;preservar num figuras
	push bc			;preservar regs
	push hl
	push de
	ld a,8			;loop alto 
	.dummy_lp2:
	ex de,hl		;intercambia regs
	ldi:ldi:ldi:ldi:ldi:ldi	;pintar 6 pixels de ancho
	dec a			;dec alto
	jr z,dummy_end		;si zero fin alto
	ld bc,#7fa		;sig line
	ex de,hl		;intercambia regs
	add hl,bc		;aadir linea y comprobar acarreo
	jr nc,dummy_lp2		;si no hay carry, bucle
	ld bc,#c050		;offset sig. bloque de lineas
	add hl,bc		;aadir bloque
	jr dummy_lp2		;bucle
	.dummy_end:
	pop de			;recuperar regs
	pop hl
	pop bc
	add hl,bc		;aadir offset
	ex af,af'		;recuperar num figuras
	dec a			;dec num figuras
	jr nz,dummy_spr		;bucle si no zero
	ld a,(show_move)	;retornar show_move para el sig. call
	ret
;----------------------------------------------------------------
.dummy_modaddr:	;actualiza scr addr de figuras
	ld hl,column_pt1+1	;apuntar a addr. a modificar
	ld de,pt1_pos		;apuntar a posiciones
	ld bc,12		;bytes offset entre addrs. a modificar
	ld a,(show_move)	;bits
	.dummy_modaddr_lp:
	bit 7,a			;comprobar bit 7
	jr nz,dummy_find	;si no es zero, modificar addr
 	inc de			;si es zero incrementar punteros
 	inc hl
	jr dummy_next		;saltar modificar
	;--------------------------------------------------------
	.dummy_find:
	ex af,af'		;preservar bits
	push bc			;preservar offset
	ld a,(de)		;contenido pos
	inc de			;inc puntero pos
	push de			;preservar puntero pos
	ex de,hl		;intercambia regs
	ld h,(hl)		;contenido de pos a HL
	ld l,a			;contenido de pos a HL
	call find_addr		;convertir pos en addr
	ex de,hl		;intercambia regs
	ld (hl),e		;modificar addr en column byte1
	inc hl			;inc puntero column
	ld (hl),d		;modificar addr en column byte2
	pop de			;recuperar puntero pos
	pop bc			;recuperar offset
	ex af,af'		;recuperar bits
	;--------------------------------------------------------
	.dummy_next:
	inc de			;inc puntero pos
	add hl,bc		;aadir offset al puntero column
	rla			;girar el byte a la izq dejando el bit 0 a 0
	or a			;modificar flag
	jr nz,dummy_modaddr_lp	;bucle mientras no zero
	;sigue modificando el addr del back buffer ahorrando un call
.back_buffer_addr:
	call find_ball		;convertir pos xy en scr addr
	ld (restore_scr+1),hl	;modificar addr del back buffer
	ret
;----------------------------------------------------------------
.find_ball:	;ball XY directo
 	ld hl,(bxy)		;pos xy 16bits
 	;continua find_addr
;----------------------------------------------------------------
;get_scr_addr by Kevin Thacker
;cpctech.cpc-live.com/source/rotospr.html
.find_addr: ;in HL (scr pos) - out HL (scr addr)
	ld bc,tableC000		;puntero a la tabla de direcciones
     	ld a,h			;guardar (x) en A
     	ld h,0			;resetear H
     	add hl,hl		;doblar L (y)
	add hl,bc		;aadir la tabla
	ld c,a			;recuperar (x)
    	ld a,(hl)		;get values
     	inc hl
     	ld h,(hl)
     	ld l,a
     	ld b,0			;resetear B
     	add hl,bc		;aadir C (x)
	ret