
;;  Action: Opens an input buffer and reads  the first block of the
;;          file
;;  Entry:  B contains the length of  the filename, HL contains the
;;          filename's address, and DE contains  the address of the
;;          2K buffer to use for reading the file
;;  Exit:   If the file  was  opened  successfully,  then  Carry is
;;          true, Zero is false, HL  holds  the address of a buffer
;;          contauling the file header  data,  DE holds the address
;;          of the destination  for  the  file,  BC  holds the file
;;          length, and A holds the  file  type; if the read stream
;;          is already  open  then  Carry  and  Zero  are  false, A
;;          contains an error nurnber  (664/6128  only)  and BC, DE
;;          and HL are corrupt;  if  ESC  was  pressed by the user,
;;          then Carry is false,  Zero  is  true,  A holds an error
;;          number (664/6128 only) and BC,  DE  and HL are corrupt;
;;          in all cases, IX and  the  other flags are corrupt, and
;;          the others are preserved
;;  Notes:A filename of zero length  means  'read  the neXt file on
;;          the tape'; the stream remains  open  until it is closed
;;          by either CAS IN CLOSE or CAS IN ABANDON
;;  Disc:   Similar to tape except that  if  there  is no header on
;;          the file, then a fake header is put into memory by this
;;          routine
.equ cas_in_open, 0xBC77


;;  Action: Closes an input file
;;  Entry:  No entry conditions
;;  Exit:   If the file was closed successfully, then Carry is true
;;          and A is corrupt; if the read stream was not open, then
;;          Carry is false, and  A  holds  an  error code (664/6128
;;          only); in both cases, BC,  DE,  HL  and the other flags
;;          are all corrupt
;;  Disc:   All the above applies, but  also  if the file failed to
;;          close for any other reason,  then  Carry is false, Zero
;;          is true and A contains  an  error  number; in all cases
;;          the drive motor is turned off immediately
.equ cas_in_close, 0xBC7A


;;  Action: Reads in a single byte from a file
;;  Entry:  No entry conditions
;;  Exit:   If a byte was read, then  Carry is true, Zero is false,
;;          and A contains the byte read  from the file; if the end
;;          of file was reached, then  Carry  and Zero are false, A
;;          contains an error number (664/6128  only) or is corrupt
;;          (for the 464); if ESC was pressed, then Carry is false,
;;          Zero is true, and  A  holds  an  error number (664/6128
;;          only) or is corrupt (for the 464); in all cases, IX and
;;          the  other  flags  are  corrupt,  and  all  others  are
;;          preserved
;;  Disc:   All the above applies for the disc routine
.equ cas_in_char, 0xBC80


.equ cas_in_direct, 0xBC83



;; =================================================
;; Read a file byte-by-byte
;; 
;; Input:
;; HL = address to load file data
;; BC = zero terminated string containing the filename
;; =================================================
cpct_loadFromDisc_asm::
    ld (ph_loadAddress+1), hl
    ld (ph_filename+1), bc

    ld d, #-1                   ;; |
    charCountLoop:              ;; |
        ld a, (bc)              ;; |
        inc bc                  ;; |
        inc d                   ;; |
        cp #0                   ;; |
        jp nz, charCountLoop    ;; |
    ld b, d                     ;; b = filename length


    ph_filename:
    ld hl,#0x0000               ;; |
    call cas_in_open            ;; Open file for reading


    ph_loadAddress:
    ld hl, #0x0000
    call cas_in_direct
    
    call cas_in_close
    ret


