;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################

;.area _CODE

;.include "cpctSymbols.h.s"
;.include "dbuffer.h.s";

;.include "menuSymbols.h.s" 

;.globl _g_avatar_0
;.globl _g_avatar_1
;.globl _g_avatar_2
;.globl _g_cpct_0

;; ========================================================
;; ========================================================
;; PRIVATE DATA
;; ========================================================
;; ========================================================
;; Constants
;.equ videoStart, #highBuffer

.equ avatarPos, #80*10+31
.equ avatarSize, #0x250D

.equ qualPos, #80*4+4
qual0Text: .asciz "PROGRAMMING, GRAPHICS, MUSIC & SOUND"
qual1Text: .asciz "            PROGRAMMING"
qual2Text: .asciz "      PROGRAMMING & EXTRA ART"

.equ namePos, #80*6+10
name0Text: .asciz "   daniel ponsoda montiel"
name1Text: .asciz "  luis fernando perez perez"
name2Text: .asciz "juan miguel castillo zaragoza"

.equ cpctTextPos, #80*6+0
cpctText: .asciz "     acknowledgements to all CPCTELERA                 contributors"

.equ cpctLogoPos, #80*11+28
.equ cpctLogoSize, #0x1C1A

.equ endPos, #80*20+18
endText: .asciz "press SPACE to go back"
endEraseText: .asciz "                      "

waitTime: .dw #1000 ; Time between gurús

;; ========================================================
;; Show the credits on the screen. 
;; ========================================================
credits_start::
    call credits_eraseCenter

    ld hl, #endPos                              ;; Draws the text for resume
    ld bc, #endText
    call credits_drawString

    ;; Draws first developer
    ld hl, #qualPos
    ld bc, #qual0Text
    call credits_drawString

    ld hl, #namePos
    ld bc, #name0Text
    call credits_drawString

    ld hl, #videoStart
    ld bc, #avatarPos
    add hl, bc
    ld de, #_g_avatar_0
    ex de, hl
    ld bc, #avatarSize
    call cpct_drawSprite_asm

        call credits_waitTime

        call credits_eraseCenter

        ;; Draws second developer
    ld hl, #qualPos
    ld bc, #qual1Text
    call credits_drawString

    ld hl, #namePos
    ld bc, #name1Text
    call credits_drawString

    ld hl, #videoStart
    ld bc, #avatarPos
    add hl, bc
    ld de, #_g_avatar_1
    ex de, hl
    ld bc, #avatarSize
    call cpct_drawSprite_asm

        call credits_waitTime

        call credits_eraseCenter

        ;; Draws third developer
    ld hl, #qualPos
    ld bc, #qual2Text
    call credits_drawString

    ld hl, #namePos
    ld bc, #name2Text
    call credits_drawString

    ld hl, #videoStart
    ld bc, #avatarPos
    add hl, bc
    ld de, #_g_avatar_2
    ex de, hl
    ld bc, #avatarSize
    call cpct_drawSprite_asm

        call credits_waitTime

        call credits_eraseCenter

        ;; Draws cpctelera credits
    ld hl, #cpctTextPos
    ld bc, #cpctText
    call credits_drawString

    ld hl, #videoStart
    ld bc, #cpctLogoPos
    add hl, bc
    ld de, #_g_cpct_0
    ex de, hl
    ld bc, #cpctLogoSize
    call cpct_drawSprite_asm

        call credits_waitTime
    
    endCredits:                         ;; Cames here when SPACE is pressed

        call credits_eraseCenter

        ld hl, #endPos                          ;; Deletes the text for resume
    ld bc, #endEraseText
    call credits_drawString

    ret

;; ========================================================
;; Erases the center of the screen preserving the bottom logos. 
;; ========================================================
credits_eraseCenter::
    ld de, #videoStart                  ;; Draws a square on the left
    ld a, #0xFF
    ld b, #0x9D
    ld c, #0x30 
    call cpct_drawSolidBox_asm

    ld bc,#0x30                                 ;; Draws a square on the right
    ld hl, #videoStart
    add hl, bc
    ex de, hl
    ld a, #0xFF
    ld b, #0x9D
    ld c, #0x20
    call cpct_drawSolidBox_asm
    call cpct_waitVSYNC_asm     ;; Waits VSYNC signal to begin draw the new screen
    ret

;; ========================================================
;; Waits waitTime.
;; ========================================================
credits_waitTime:
        ld de, #-1
        ld hl, (waitTime)
        waitTime_loop:
                halt
                push hl
                push de
                call cpct_scanKeyboard_asm          ;; Update keyboard state table
        ld    hl, #Key_Space                ;; hl = key code to check
        call  cpct_isKeyPressed_asm         ;; l = a = key state
        cp    #0                            ;; | 
        jr    z, space_not_pressed          ;; if (a!=0) {
            pop de                                                      ;; Pops loop data
                pop hl                                                  ;; | 
                pop hl                                                  ;; Pops the PC of the call
            jp endCredits                                       ;; Exits the credits
        space_not_pressed:                  ;; Else continues with the time countdown
        pop de
        pop hl
                add hl, de
        jr c, waitTime_loop
    ret

;; ========================================================
;; Draws string letters on the given position
;; Input:
;; HL = pos : Text position from the screen init
;; BC = text : Pointer to the text string
;; ========================================================
credits_drawString::
    push bc
    call cpct_waitVSYNC_asm
    pop bc
    ld de, #videoStart
    add hl, de
    ex de, hl
    call cpctex_drawTileStringM0
    ret
  