############################ BEGIN GPL LICENSE BLOCK #############################
##    This file is part of legend of steel.
##
##    Legend of steel is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    Legend of steel is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
##
##    Developed for the CPCRetrodev contest 2018 edition.
##
##     Developers                           Contact
##    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
##     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
##     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
##     Ponsoda Montiel, Daniel              dponsoda@gmail.com
##
############################# END GPL LICENSE BLOCK #############################

##################################################
#                                                #
# COMPILACIÓN PARA EL BLOQUE DEL MENU            #
#                                                #
##################################################

notice=";; Generated by release.sh * DO NOT EDIT *"
# Hace que se llame al menu al comienzo
echo "$notice" > menu/incl/startMenu.incl
echo call menu_start >> menu/incl/startMenu.incl

# Establecer el símbolo global de la función que inicia el menu
echo "$notice" > menu/incl/menu.incl
echo .globl menu_start >> menu/incl/menu.incl

# Incluir el código del menu en la compilación tras los assets del menu
echo "$notice" > menu/incl/menuSymbols.incl
echo _g_flame:: .dw _g_flame_0, _g_flame_1, _g_flame_2, _g_flame_3 >> menu/incl/menuSymbols.incl
echo .include \"../menu/src/common.s\" >> menu/incl/menuSymbols.incl
echo .include \"../menu/src/menu.s\" >> menu/incl/menuSymbols.incl
echo .include \"../menu/src/credits.s\" >> menu/incl/menuSymbols.incl
echo .include \"../menu/src/tutorial.s\" >> menu/incl/menuSymbols.incl
echo .include \"../menu/src/cpct_akp1.s\" >> menu/incl/menuSymbols.incl

# Nos aseguramos de que la proxima llamada a make volverá 
# a recompilar los ficheros que han cambiado
touch src/main.s
touch src/menu.h.s
touch src/menuSymbols.s

# Compilar todo el proyecto
make

if [ $? -ne 0 ]; then
  echo "ERROR: Compiling for MENU block"
  exit 1
fi



##################################################
#                                                #
# EXTRACCIÓN EL CÓDIGO DEL MENU DEL DSK GENERADO #
#                                                #
##################################################

# Extraer temporalmente el binario del juego que está en el dsk
idsk=${CPCT_PATH}/tools/iDSK-0.13/bin/iDSK
${idsk} legend.dsk -g legend.bin

# Extraer el código del menu de ese binario
fileStart=$((16#80)) # 0x80 de la cabecera amsdos
bhex=`grep 'menu_code_begin' obj/legend.map | cut -d$' ' -f6`
ehex=`grep 'menu_code_end' obj/legend.map | cut -d$' ' -f6`
b=$((16#${bhex}))
e=$((16#${ehex}))
size=$((e-b))
start=$((b-fileStart+1))
# Guardar en code.bin el código del menu
tail -c +${start} legend.bin | head -c ${size} > menu/obj/code.bin

#Eliminar el binario temporal extraido (conservamos el código del menu en code.bin)
rm legend.bin




##################################################
#                                                #
# INTEGRACIÓN DEL CODIGO DEL MENU EN EL BINARIO  #
#                                                #
##################################################

# Crear assets.bin con todos los assets del menu
cat menu/assetsObj/intro.bin > dsk/assets.bin
cat menu/assetsObj/avatars.bin >> dsk/assets.bin
cat menu/assetsObj/menu-title.bin >> dsk/assets.bin
cat menu/assetsObj/panel.bin >> dsk/assets.bin
cat menu/assetsObj/todStudios.bin >> dsk/assets.bin
cat menu/assetsObj/cpctelera.bin >> dsk/assets.bin
cat menu/assetsObj/flame.bin >> dsk/assets.bin
cat menu/assetsObj/keySpace.bin >> dsk/assets.bin
cat menu/assetsObj/keyArrow.bin >> dsk/assets.bin
cat menu/assetsObj/arrowH.bin >> dsk/assets.bin
cat menu/assetsObj/arrowV.bin >> dsk/assets.bin
cat menu/assetsObj/joyButton.bin >> dsk/assets.bin
cat menu/assetsObj/joyBall.bin >> dsk/assets.bin
cat menu/assetsObj/joyStick.bin >> dsk/assets.bin
echo -n -e '\x00' >> dsk/assets.bin
cat menu/obj/code.bin >> dsk/assets.bin

# Incluir los assets en el dsk final
#${idsk} legend.dsk -i dsk/assets.bin -f
#${idsk} legend.dsk -i dsk/legend.bas -f
#${idsk} legend.dsk -i dsk/legend.scr -f

# Asegurarnos de que se incluyen todos los ficheros en el dsk
touch dsk/legend.bas
touch dsk/legend.scr



##################################################
#                                                #
# COMPILACIÓN PARA EL BLOQUE PRINCIPAL           #
#                                                #
##################################################

# Obtener la dirección de ejecución del menu
execAddr=`grep 'menu_start' obj/legend.map | cut -d$' ' -f6`
execAddr=`echo ${execAddr} | tail -c +5`

# Establecer la dirección de ejecución en el simbolo menu_start
echo "$notice" > menu/incl/menu.incl
echo .equ menu_start, 0x${execAddr} >> menu/incl/menu.incl

# NO incluir el código del menu en la compilación
echo "$notice" > menu/incl/menuSymbols.incl
echo ";; (no code)" >> menu/incl/menuSymbols.incl

# Establecer el tamaño del bloque del menu
fileSize=`stat --printf="%s" dsk/assets.bin`
echo "$notice" > menu/incl/menuSize.incl
echo "ld de,#$fileSize" >> menu/incl/menuSize.incl

# Nos aseguramos de que la proxima llamada a make volverá 
# a recompilar los ficheros que han cambiado
touch src/main.s
touch src/menu.h.s
touch src/menuSymbols.s



# Compilar solo el juego
rm legend.dsk
make

if [ $? -ne 0 ]; then
  echo "ERROR: Compiling for MAIN block"
  exit 2
fi

# Incluir el archivo del menu con cabecera Amsdos 
# (cargará en memoria de video temporalmente)
${idsk} legend.dsk -i dsk/assets.bin -t 1 -c C000 -f