;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################

.include "characters/frames.h.s"
.include "animation.h.s"

;;====================================================
;; Updates an animation (i.e. for an entity)
;; 
;; Input:
;; IX = animation: Pointer to the animation to update
;; D = pixX: Virtual x position of entity
;;====================================================
animation_update::
    ;ld      a, (currentAni)             ;; |
    ;ld      b, a                        ;; |
    ld      b, animation_currentAni (ix)
    ld      a, #_g_aniWalkRight        ;; |
    cp      b                           ;; |
    jr      z, updateAniHoriz1          ;; |
    ld      a, #_g_aniWalkRightSlimer
    cp      b
    jr      z, updateAniHoriz1          ;; |
    ld      a, #_g_aniWalkLeft         ;; |
    cp      b                           ;; Si la animación actual es 
    jr      z, updateAniHoriz1
    ld      a, #_g_aniWalkLeftSlimer
    cp      b
    jr      nz, updateAniNoHoriz        ;; de caminar en horizontal
        updateAniHoriz1:
        ;ld  a, (pixX)                   ;;     - El frame actual depende del
        ld a, d
        and #15                         ;;       modulo de 16 con la posición en pantalla
        ld hl, #ph_isHoriz
        ld (hl), #0xC9                  ;; Pone RET en el ph
        jr updateAniHoriz               ;;       

    updateAniNoHoriz:                   ;; Si la animación actual no es caminar en horizontal
        ;ld      a, (currentFrame)           ;; // Si el frame actual ordena detener la animacion
        ld      a, animation_currentFrame (ix)
        cp      #-2                         ;; if (currentFrame==-2)
        ret     z                           ;;   return
        ld hl, #ph_isHoriz
        ld (hl), #0x00                  ;; Pone NOP en el ph
    updateAniHoriz:
    

    ld      e, a                        ;; |
    ld      d, #0x00                    ;; DE = currentFrame

    ;ld      a, (currentAni)             ;; |
    ld      a, animation_currentAni (ix)
    ld      l, a                        ;; |
    ld      h, #0x00                    ;; HL = currentAni

    ld      bc, #_g_animations             ;; |
    add     hl, bc                      ;; HL = animations[currentAni]

    ld      a, e                        ;; |
    cp      #-1                         ;; | // Si el frame actual no ordena hacer LOOP:
    jr      z, noLoop                   ;; if (currentFrame!=-1) {
                                        ;;    // Posicionar HL en el frame actual
        add     hl, de                  ;;    HL = HL + currentFrame
    
    noLoop:                             ;; }

    ld      a, (hl)                     ;; A = numTile

    ex      de, hl

    ;ld      hl, (heroTileSet)           ;; |
    ld      h, animation_tileset + 1 (ix)
    ld      l, animation_tileset (ix)
    add     a, a                        ;; |
    ld      c, a                        ;; |
    ld      b, #0x00                    ;; |
    add     hl, bc                      ;; HL = heroTileSet + numTile*2

    ld      c, (hl)                     ;; |
    inc     hl                          ;; |
    ld      b, (hl)                     ;; BC = *(heroTileSet + numTile*2)
    ;ld      (currentSprite), bc         ;; Establece el sprite actual
    ld      animation_currentSprite + 1 (ix), b
    ld      animation_currentSprite (ix), c



    ph_isHoriz:
    ret
    ;ld      a, (currentAni)
    ;ld      b, a
    ;ld      a, (aniWalkHoriz)
    ;cp      b
    ;ret     z

    ex      de, hl                      ;;
    inc     hl                          ;;
    ld      a, (hl)                     ;; A = siguiente frame

    cp      #0xFF                       ;; |
    jr      z, specialFrame             ;; |
    cp      #0xFE                       ;; |
    jr      z, specialFrame             ;; if (A>=0) {
        ;ld      a, (currentFrame)       ;;
        ld      a, animation_currentFrame (ix)
        inc     a                       ;;     A = currentFrame + 1
    
    specialFrame:                       ;; }

        ;ld      (currentFrame), a       ;; currentFrame = A
        ld  animation_currentFrame (ix), a

    ret
    


;;====================================================
;; Sets current frame sequence of an animation
;;
;; Input:
;; IX = animation : Pointer to the animation
;; A = sequence : Reference to the frame sequence
;;====================================================
animation_set::
    ;ld      b, a
    ;ld      a, (currentAni)             ;; |
    ld      b, animation_currentAni (ix)
    cp      b                           ;; |
    ret     z                           ;; if (ani == currentAni) return

    different:
    ;ld      a, b                        ;; |
    ;ld      (currentAni), a             ;; currentAni = ani
    ld animation_currentAni (ix), a

    ;ld      a, #0x00                    ;; |
    ;ld      (currentFrame), a           ;; currentFrame = 0
    ld animation_currentFrame (ix), #0

    ret