;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################

.include "cpctSymbols.h.s"


cpctex_setDrawTileFunction::
   ld (ph_drawFunction+1), bc
   ret


;; ==========================================================
;; Sets size value for cpct_etm_drawTilemap4x8
;;
;; Input registers:
;; B    = h         : Height in tiles of the window to be drawn
;; C    = w         : Width in tiles of the window to be drawn
;;
;; Destroys:
;; A
;; ==========================================================
cpct_etm_setDrawTilemapSize4x8_asm::

    ld        a, c                         ;; |
    ld (ph_drawTileMap_w+1), a             ;; |
    ld (ph_drawTileMap_w_+1), a            ;; | Guardar parámetro width

    ld        a, b                         ;; |
    ld (ph_drawTileMap_h+1), a             ;; | Guardar parámetro height

    ret

;; ==========================================================
;; Sets internal configuration values for cpct_etm_drawTilemap4x8
;;
;; Input registers:
;; B    = h         : Height in tiles of the window to be drawn
;; C    = w         : Width in tiles of the window to be drawn
;; DE   = mapW      : Width of the tilemap
;; HL   = tileset   : Pointer to the start of the tileset definition
;;
;; Destroys:
;; A
;; ==========================================================
cpct_etm_setDrawTilemap4x8_asm::
    ;ld        e, a                        ;; |
    ;ld        d, #0x00                    ;; |
    ld (ph_drawTileMap_mapW+1), de         ;; | Guardar parámetro mapW

    ld        a, c                         ;; |
    ld (ph_drawTileMap_w+1), a             ;; |
    ld (ph_drawTileMap_w_+1), a            ;; | Guardar parámetro width

    ld        a, b                         ;; |
    ld (ph_drawTileMap_h+1), a             ;; | Guardar parámetro height

    ld (ph_drawTileMap_tileSet + 1), hl
    ret


;; ==========================================================
;; Gets a pointer to the tile corresponding to the x and y
;; coordinates for a given tilemap
;;
;; Input registers:
;; B     = posY    : Y coordinate of the tile
;; C     = posX    : X coordinate of the tile
;; E     = mapW    : width of the tilemap
;; HL    = tilemap : pointer to the tilemap to draw
;;
;; Returns:
;; HL = pointer to the tile as requested
;;
;; Destroys:
;; HL, BC
;; ==========================================================
cpct_etm_getTilemapPtr_asm::
    push hl
    ld        h, b                          ;; | H = posY
    ld        l, #0x00                      ;; | 

                                            ;; Multiplication algorithm
                                            ;; Gets HL = H times D
    ld        b, #0x08                      ;; | counter for 8 iterations
    repeatSum:                              ;; |
        add      hl, hl                     ;; |
        jr       NC, noCarry                ;; | 
        add      hl, de                     ;; |
        noCarry:                            ;; | 
        djnz   repeatSum                    ;; | HL = posY * mapW

    ld b, #0x00                             ;; | BC = posX
    add   hl,bc                             ;; | HL = HL + posX

    pop bc                                  ;; BC = pointer to tilemap

    add      hl, bc                         ;; HL = HL + tilemap

    ;; We finally return HL = tilemap + posY * mapW + posX
   
    ret


;; ==========================================================
;; Draws a tilemap
;;
;; Required state:
;;    Requires a tileSet previously set by a call to cpctex_setTileSet
;; 
;; Input registers:
;; HL = dst         : Destination pointer to video memory
;; DE = tilemap     : Tile map to draw
;; ==========================================================
cpct_etm_drawTilemap4x8_asm::                                 ;;

    ld (ph_dst+1), hl                       ;; Puntero a la memoria de video                 
    ;ld (ph_tileMap+1), de                   ;; Puntero al primer tile del tilemap

    ld b, d
    ld c, e

ph_dst:
    ld     de, #0000                       ;; Placeholder para puntero a memoria de video
   
    ;; ###########################
    ;; Comienzo Bucle externo (y)
    ;; ###########################
    exx                                    ;; |
    ph_drawTileMap_h:                      ;; (Placeholder para height)
    ld       e,#00                         ;; |
    exx                                    ;; |
beginOuterLoop:                           ;; \
    exx                                    ;; |
    dec      e                             ;; |
    jp       m, endOuterLoop               ;; for (y=0; y!=h; ++y) {


    ;; ###########################
    ;; Comienzo Bucle interno (x)
    ;; ###########################
    ph_drawTileMap_w:                              ;; (Placeholder para width)
    ld       b, #00                        ;; |
    exx                                    ;; |
beginInnerLoop:                           ;; |
    exx                                    ;; |
    dec      b                             ;; |
    exx                                    ;; |
    jp       m, endInnerLoop               ;; for (x=0; x!=w; ++x) {


    ld       a, (bc)                       ;; | Nota: BC es el puntero al tileMap
    add      a                             ;; | Nota: Multiplicamos x2 porque lo que obtenemos es la posición
    ld       l, a                          ;; |       en un un array de punteros (el tileSet)
    ld       h, #0x00                      ;; HL = *tileMap       
    ;; OJO: limita a 256 tiles por tileSet. 
    ;; Para que no tenga este limite, quitar "add a" y añadir al final:
    ;add     hl, hl                         ;; HL = *tileMap

    inc     bc                             ;; |
    push    bc                             ;; tileMap++

    ph_drawTileMap_tileSet:
    ld      bc, #0000                      ;; |
    add     hl, bc                         ;; HL = tileSet + HL

    ld       a, (hl)                       ;; |
    inc     hl                             ;; | Nota: En resumen, las últimas ordenes equivalen a:
    ld       h, (hl)                       ;; |       HL = tileSet[*tileMap++]
    ld       l, a                          ;; HL = *(HL)

    push    de
    ;; Dibuja un tile indicado por HL en la posicion DE
    ;call   cpct_drawTileAligned4x8_f_asm   ;; | cpct_drawTileAligned4x8_f(HL, DE)
    ph_drawFunction:
    call   cpct_drawTileAligned4x8_f_asm   ;; | cpct_drawTileAligned4x8_f(HL, DE)
    ;call cpctex_drawHalf2TileAligned4x8_asm
    pop     de                             ;; Recupera puntero a memoria de video
    pop     bc                             ;; Recupera tileMap

    inc de                                   ;; |
    inc de                                   ;; |
    inc de                                   ;; |
    inc de                                   ;; dst += 4


    ;; Al final del bucle:
    ;; DE <- dst
    ;; BC <- tilemap

    ;; ###########################
    ;; Fin Bucle interno (x)
    ;; ###########################        
    jr   beginInnerLoop                    ;; Fin del bucle interno (x)
endInnerLoop:                             ;; |

    push    de

    ph_drawTileMap_mapW:                               ;;
    ld      hl, #0000                      ;; Placeholder para mapW

    ph_drawTileMap_w_:                     ;;
    ld       e, #00                        ;; Placeholder para width

    ld       d, #0x00                      ;; |
    sbc     hl, de                         ;; |
    add     hl, bc                         ;; | tileMap += mapW-w

    ld      b, h                           ;; |
    ld      c, l                           ;; | BC = tileMap

    ld       a, e                          ;; |
    add      a                             ;; |
    add      a                             ;; |
    ld       l, a                          ;; |
    ld       a, #0x50                      ;; | Para bajar una fila sumamos 80 bytes (160 pixeles / 2)
    sub      l                             ;; | Restamos el ancho (w) para situarnos al comienzo de la fila de tiles
    ld       h, #0x00                      ;; | Se multiplica el ancho x4 porque son tiles de 4 bytes (8 pixeles)
    ld       l, a                          ;; HL = 80-(w*4)

    pop     de                             ;; |
    add     hl, de                         ;; |
    ex      de, hl                         ;; dst = dst + HL

    ;; Al final del bucle:
    ;; DE <- dst
    ;; BC <- tilemap

    ;; ###########################
    ;; Fin Bucle externo (y)
    ;; ###########################
    jp   beginOuterLoop                    ;; Fin del bucle enterno (y)
endOuterLoop:                             ;; |
    exx

    ret
