;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################

.area _CODE

;; ========================================================
;; ========================================================
;; INCLUDED GLOBAL SYMBOLS
;; ========================================================
;; ========================================================
.include "cpctSymbols.h.s"
.include "dbuffer.h.s"

;; ========================================================
;; ========================================================
;; CONSTANTS
;; ========================================================
;; ========================================================
;.equ lowBuffer, 0x8030
;.equ highBuffer, 0xC030;

;.equ lowCRTC, 0x2018
;.equ highCRTC, 0x3018



;; ========================================================
;; ========================================================
;; PRIVATE DATA
;; ========================================================
;; ========================================================
CRTCValue: .dw highBuffer


;; ========================================================
;; ========================================================
;; PUBLIC DATA
;; ========================================================
;; ========================================================
_g_backBuffer::  .dw lowBuffer    ;0x8000
_g_frontBuffer:: .dw highBuffer

_g_backCRTC:: .dw lowCRTC
_g_frontCRTC:: .dw highCRTC


;; ========================================================
;; ========================================================
;; PUBLIC FUNCTIONS
;; ========================================================
;; ========================================================

dbuffer_getCRTCValue:
    ld hl, (CRTCValue)
    ret

;; ========================================================
;; Swap back buffer and front buffer pointers
;;
;; ========================================================
dbuffer_swap::
    ld hl, (_g_backBuffer)
    ld de, (_g_frontBuffer)
    ld (_g_backBuffer), de
    ld (_g_frontBuffer), hl

    ld hl, (_g_backCRTC)
    ld de, (_g_frontCRTC)
    ld (_g_backCRTC), de
    ld (_g_frontCRTC), hl

    call dbuffer_setCRTCValue

    ret


;; ========================================================
;; Sets higher buffer as front buffer
;;
;; ========================================================
dbuffer_setHighFront::
    ld hl, #lowBuffer
    ld (_g_backBuffer), hl
    ld hl, #highBuffer
    ld (_g_frontBuffer), hl

    ld hl, #lowCRTC
    ld (_g_backCRTC), hl
    ld hl, #highCRTC
    ld (_g_frontCRTC), hl
    
    call dbuffer_setCRTCValue

    ret


;; ========================================================
;; Sets lower buffer as front buffer
;;
;; ========================================================
dbuffer_setLowFront::
    ld hl, #highBuffer
    ld (_g_backBuffer), hl
    ld hl, #lowBuffer
    ld (_g_frontBuffer), hl
    
    ld hl, #highCRTC
    ld (_g_backCRTC), hl
    ld hl, #lowCRTC
    ld (_g_frontCRTC), hl

    call dbuffer_setCRTCValue

    ret

;; ========================================================
;; Fills back buffer with zeros
;;
;; ========================================================
dbuffer_clearBack::
    ld de, (#_g_backBuffer)             ;; |
    ld  a, #0xFF                           ;; |
    ld bc, #0x4000-0x30                      ;; |
    call cpct_memset_asm                ;; Clear the back buffer

    ret


;; ========================================================
;; Fills front buffer with zeros
;;
;; ========================================================
dbuffer_clearFront::
    ld de, (#_g_frontBuffer)            ;; |
    ld  a, #0xFF                           ;; |
    ld bc, #0x4000-0x30                      ;; |
    call cpct_memset_asm                ;; Clear the back buffer

    ret



;dbuffer_init::
;    ld hl, (_g_frontBuffer)
;    call dbuffer_init
;    ld (_g_frontCRTC), hl;

;    ld hl, (_g_backBuffer)
;    call dbuffer_init
;    ld (_g_backCRTC), hl
;    ret;

;dbuffer_initBuffer:;

;    ld a, l
;    srl a
;    ld l, a
;    
;    ld a, h
;    srl a
;    and #3
;    
;    srl h
;    srl h
;    or h;

;    ret


;; ================================================================
;; Establece los registros 12 y 13 del CRTC para cambiar
;; el puntero a la memoria de video
;; 
;; Input:
;;   HL = Info para los registros 12 y 13
;;   bits [13..10] : Página (se multiplicará por 4) (Ej. 3h = Ch) 
;;                   1 página = 1024 bytes
;;   bits [9..0] : Offset (1 unidad = 2 bytes de video)
;; ================================================================
dbuffer_setCRTCValue::
    ld (CRTCValue), hl

    push hl
    call cpct_setVideoMemoryOffset_asm  ;; Siguiente posición del offset de video
    pop hl
    ld l,h
    call cpct_setVideoMemoryPage_asm

    ret