;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################


.include "animation.h.s"
.include "collision.h.s"

;;===========================================
;;===========================================
;; ENTITY PUBLIC FUNCTIONS
;;===========================================
;;===========================================

.globl entity_init

.globl entity_draw
.globl entity_drawIfItem
.globl entity_drawIfAlive
.globl entity_drawIfDead
.globl entity_drawBack
.globl entity_drawFront

.globl entity_eraseBlack
.globl entity_eraseBlackBack
.globl entity_eraseBlackFront
.globl entity_erase
.globl entity_eraseEnd

.globl entity_update
.globl entity_moveLeft
.globl entity_moveRight
.globl entity_moveUp
.globl entity_moveDown
.globl entity_stand

.globl entity_moveLeftNoAni
.globl entity_moveRightNoAni
.globl entity_moveUpNoAni
.globl entity_moveDownNoAni

.globl entity_displace

.globl entity_setPosX
.globl entity_setPosY
.globl entity_getTilePos

.globl entity_setTileset
.globl entity_setAnimation
.globl entity_lookRight
.globl entity_lookLeft

.globl entity_new
.globl entity_forEach

.globl entity_setWorldPtr
.globl entity_deleteFromWorld

.globl entity_adjustItem
.globl entity_checkDropItem
;;===========================================
;;===========================================
;; PUBLIC MACROS
;;===========================================
;;===========================================

.macro entity_defineBase _name, _x, _y, _w, _h, _currSeq, _numFrame, _tileSet, _currSprite, _cbLogic, _typeid
    _name::
    collision_define _name'_boundingBox, _x, _y, _w, _h
    animation_define _name'_animation, _currSeq, _numFrame, _tileSet, _currSprite
    .db _x      ;; Erase position x
    .db _y      ;; Erase position y
    .db _w      ;; Erase width
    .db _h      ;; Erase height
    .db _x*2    ;; Virtual position x
    .db _y/2    ;; Virtual position y
    .db 0       ;; Hozizontal flip flag
    .dw _cbLogic  ;; Pointer to callback for entity logic
    .db _typeid
    .dw 0x0000  ;; Pointer to entity in world definition
.endm

.macro entity_define _name, _x, _y, _w, _h, _currSeq, _numFrame, _tileSet, _currSprite, _cbLogic, _typeid
    entity_defineBase _name, _x, _y, _w, _h, _currSeq, _numFrame, _tileSet, _currSprite, _cbLogic, _typeid
    .db 0 ;; entity_state
    .db 0,0,0,0,0,0 ;; entity_customData
.endm

.macro entity_defineCharacter _name, _x, _y, _w, _h, _currSeq, _numFrame, _tileSet, _currSprite, _cbLogic, _typeid, _state, _d0, _d1, _d2, _d3, _d4, _d5
    entity_defineBase _name, _x, _y, _w, _h, _currSeq, _numFrame, _tileSet, _currSprite, _cbLogic, _typeid
    .db _state ;; entity_state
    .db _d0,_d1,_d2,_d3,_d4,_d5 ;; entity_customData
.endm

.macro entity_defineDefaultEntity _name, _suf
    entity_define _name'_suf, 0xAA,0x00,0x00,0x00, 0x00,0x00,0x0000,0x0000, 0xFFFF, 0x00
.endm

.macro entity_defineStatic _name, _x, _y, _w, _h, _tileSet, _currSprite, _cbLogic, _typeid
    entity_define _name, _x, _y, _w, _h, _g_aniStand, 0, _tileSet, _currSprite, _cbLogic, _typeid
.endm

.equ entity_tAnimation, 0
.equ entity_tHero, 0
.equ entity_tItem, 1
.equ entity_tEnemy, 2


;;===========================================
;;===========================================
;; PUBLIC CONSTANTS
;;===========================================
;;===========================================
.equ entity_boundingBox, 0
.equ entity_pos, entity_boundingBox + collision_posX
.equ entity_posX, entity_boundingBox + collision_posX
.equ entity_posY, entity_boundingBox + collision_posY
.equ entity_width, entity_boundingBox + collision_width
.equ entity_height, entity_boundingBox + collision_height

.equ entity_animation, SIZEOF_collision + 0

.equ entity_sprite, SIZEOF_collision + animation_currentSprite

.equ entity_erasePos, SIZEOF_collision + SIZEOF_animation + 0
.equ entity_eraseX, SIZEOF_collision + SIZEOF_animation + 0
.equ entity_eraseY, SIZEOF_collision + SIZEOF_animation + 1
.equ entity_eraseW, SIZEOF_collision + SIZEOF_animation + 2
.equ entity_eraseH, SIZEOF_collision + SIZEOF_animation + 3
.equ entity_pixX, SIZEOF_collision + SIZEOF_animation + 4
.equ entity_pixY, SIZEOF_collision + SIZEOF_animation + 5
.equ entity_flipH, SIZEOF_collision + SIZEOF_animation + 6
.equ entity_logic, SIZEOF_collision + SIZEOF_animation + 7
.equ entity_typeid, SIZEOF_collision + SIZEOF_animation + 9
.equ entity_worldPtr, SIZEOF_collision + SIZEOF_animation + 10
.equ entity_state, SIZEOF_collision + SIZEOF_animation + 12
.equ entity_customData, SIZEOF_collision + SIZEOF_animation + 13

.equ SIZEOF_entity, SIZEOF_collision + SIZEOF_animation + 19


.equ entity_noDir, -1
.equ entity_dirLeft, 0
.equ entity_dirRight, 1
.equ entity_dirUp, 2
.equ entity_dirDown, 3