;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################

;;.area _DATA

.area _CODE

;; ========================================================
;; ========================================================
;; INCLUDED GLOBAL SYMBOLS
;; ========================================================
;; ========================================================
;.include "cpctelera.h.s"

;.include "entity.h.s"
.include "map.h.s"
.include "dbuffer.h.s"
.include "cpctSymbols.h.s"
.include "scroll.h.s"
.include "fps.h.s"
.include "entityOrc.h.s"

.include "characters/frames.h.s"

.globl _g_map2
.globl flagDropItem



;; ========================================================
;; ========================================================
;; CONSTANTS
;; ========================================================
;; ========================================================
.equ minX, map_minX               ;; Limite a la izquierda (en pixels)
.equ maxX, map_maxX              ;; Limite a la derecha   (en pixels)

.equ minY, map_minY               ;; Limite minY en unidades virtuales de pantalla
.equ maxY, map_maxY              ;; Limite maxY en unidades virtuales de pantalla
  
;; ========================================================
;; ========================================================
;; PRIVATE DATA
;; ========================================================
;; ========================================================


;; ========================================================
;; ========================================================
;; PUBLIC FUNCTIONS
;; ========================================================
;; ========================================================

entity_init::

    ;ld hl, (_g_warrior)
    ;ld (currentSprite), hl
    ;ld hl, #_g_warrior
    ;ld (heroTileSet), hl

    ;ld a, (_g_aniStand)
    ;ld b, a
    ;call animation_set
    ret




;;===========================================
;; Updates the entity
;;
;; Input:
;; IX = entity : Pointer to entity to update
;;===========================================
entity_update::
                                    



    ;; Call function that defines the logic for this entity
    ;;
    ld hl, #logicCallbackReturn     ;; |
    push hl                         ;; Save return address on the stack
    
    ld h, entity_logic + 1 (ix)     ;; |
    ld l, entity_logic (ix)         ;; Call address is IX + entity_logic
    jp (hl)                         ;; Jump to entity logic
    logicCallbackReturn:            ;; ... and return here when finish
    

    ;; If this entity is a static item, we don't need to animate it
    ld a, entity_typeid (ix)
    cp #entity_tItem
    ret z

    ;; Update animation for this entity
    ;;
    ld d, entity_pixX (ix)          
    push ix                         ;; We need to pass the right pointer
    ld bc, #entity_animation        ;; to the animation's structure that
    add ix, bc                      ;; actually is nested in the entity's structure
    call animation_update           ;; (animationPtr = IX + entity_animation)
    pop ix                          ;; |

    ret


entity_eraseBlack::
entity_eraseBlackBack::
    ld de, (_g_backBuffer)
    ld b, entity_posY (ix)
    ld c, entity_posX (ix)
    call cpct_getScreenPtr_asm        ;; |
    ex de, hl
    jp _entity_eraseBlack

entity_eraseBlackFront::
    ld de, (_g_frontBuffer)
    ld b, entity_posY (ix)
    ld c, entity_posX (ix)
    call cpct_getScreenPtr_asm        ;; |
    ex de, hl

_entity_eraseBlack:  
    ld a, #0xFF ;color
    ld b, entity_height (ix)
    ld c, entity_width (ix)
    call cpct_drawSolidBox_asm

    ret

;;============================
;; Erase the hero
;; DESTROYS: AF, BC, DE, HL
;;============================
entity_erase::
    call entity_erasePrepareSize
    ld d, entity_eraseY (ix) 
    ld e, entity_eraseX (ix) 
    call map_eraseSprite
    call entity_updateEraseValues
    ret

entity_eraseEnd::
    ;call entity_updateEraseValues
    ;call entity_erasePrepare
    ;call map_eraseSpriteEnd

    call entity_erasePrepareSize
    ld d, entity_posY (ix) 
    ld e, entity_posX (ix) 
    call map_eraseSprite
    call entity_erasePrepareSize
    ld d, entity_eraseY (ix) 
    ld e, entity_eraseX (ix) 
    call map_eraseSpriteFront
    ret


entity_erasePrepareSize:
    ld b, entity_eraseH (ix)
    ld a, b
    srl b
    srl b
    srl b
    and #7
    jr z, noIncH
    inc b
    noIncH:

    ld c, entity_eraseW (ix)
    ld a, c
    srl c
    srl c
    and #3
    jr z, noIncW
    inc c
    noIncW:


    ;ld bc, #0x0201                  ;; tamaño del sprite en tiles

    ;ld d, entity_eraseY (ix)        ;; posición del sprite en pantalla
    ;ld e, entity_eraseX (ix)        ;; |

    ret


entity_updateEraseValues:
    ;; The new erase position is the current
    ;; entity's position before updating
    ld a, entity_posX (ix)          ;; |
    ld entity_eraseX (ix), a        ;; Update erase position X

    ld a, entity_posY (ix)          ;; |
    ld entity_eraseY (ix), a        ;; Update erase position Y

    ld a, entity_width (ix)
    ld entity_eraseW (ix), a

    ld a, entity_height (ix)
    ld entity_eraseH (ix), a
    ret


entity_drawIfDead::
    ld a, entity_typeid (ix)
    cp #entity_tEnemy
    ret nz
    ld a, entity_state (ix)
    cp #orc_stDead
    ret nz
    jp entity_draw

entity_drawIfAlive::
    ld a, entity_typeid (ix)
    cp #entity_tEnemy
    ret nz
    ld a, entity_state (ix)
    cp #orc_stDead
    ret z
    jp entity_draw

entity_drawIfItem::
    ld a, entity_typeid (ix)
    cp #entity_tItem
    ret nz
    jp entity_draw

;;============================
;; Draw the hero
;; DESTROYS: AF, BC, DE, HL
;;============================
entity_draw::
entity_drawBack::
    ld de, (#_g_backBuffer)          
    jp _entity_draw

entity_drawFront::
    ld de, (#_g_frontBuffer)

_entity_draw:
    ;push af                         ;; reserve "af" for hero's color
    ;; Calculate screen position
    
    ld b, entity_posY (ix)
    ld c, entity_posX (ix)
    ;ld  bc, (pos)                     ;; |
    call cpct_getScreenPtr_asm        ;; |
    ex de, hl                         ;; DE = posición en memoria de video

    ;ld bc, #0x1004                              ;; IX = alto x ancho
    ;ld hl, (currentSprite)                      ;; BC = Sprite a dibujar
    ;call cpct_drawSpriteMasked_asm


    ;; Draw sprite
    
    ;ld ix, #hero_animation

    ; BC = Sprite a dibujar
    ;ld b, animation_currentSprite + 1 (ix)
    ;ld c, animation_currentSprite (ix)
    ld b, entity_sprite + 1 (ix)
    ld c, entity_sprite (ix)


    ;; If this entity is an item, we draw it as a tile (much faster) 
    ld a, entity_typeid (ix)
    cp #entity_tItem
    jp nz, drawAnimatedEntity
        ld h, b
        ld l, c
        call cpct_drawTileAligned4x8_f_asm
        ret

    ;; Draw entity as an animation
    ;; (it supports transparent pixels and hFlip state)
    drawAnimatedEntity:
    
    ld hl, #cpctex_maskTable0M0                             ;; HL = Tabla de máscara

    ;ld a, (flipH)                                           ;; 
    ld a, entity_flipH (ix)

    push ix
    exx
    ld e, entity_width (ix)
    ld d, entity_height (ix)
    ld__ixl_e
    ld__ixh_d
    exx

    ;ld ix, #0x1004                                          ;; IX = alto x ancho
    cp #0                                                   ;; |
    jr z, drawNoFlip                                        ;; if (flipH!=0) {
        exx                                                 ;;     
        ld hl, #cpctex_hFlipTable0M0                        ;;     HL = Tabla de inversión horizontal
        exx                                                 ;;     
        call cpctex_drawSpriteMaskedAlignedTableHFlip_asm   ;;     Dibujar el sprite invertido
        pop ix
        ret                                                 ;;     return
    drawNoFlip:                                             ;; }

    call cpct_drawSpriteMaskedAlignedTable_asm              ;; Dibujar el sprite normal
    pop ix

    ret


;;============================
;; Moves hero to right
;; DESTROYS: 
;;============================
entity_moveRight::
    ;ld hl, #_g_map2
    ;call scroll_left

    call entity_lookRight         ;; Orienta el sprite a la derecha

    ld a, #_g_aniWalkRight     ;; |

    push ix
    ld de, #entity_animation
    add ix, de
    call animation_set          ;; Establecer animación
    pop ix
 

entity_moveRightNoAni::
    ; Comprobar si no hay colisión a la derecha

    ;ld hl, #posX
    ;inc (hl)
    inc entity_posX (ix)

    ;; Comprobar colision en la nueva posición
    ld__d_ixh
    ld__e_ixl
    ld hl, #entity_boundingBox
    add hl, de
    ;ld hl, #boundingBox
    call map_checkCollision

    ;ld hl, #posX
    ;dec (hl)
    dec entity_posX (ix)

    cp #1
    jr    z, moveRightEnd


    ;ld     a, (pixX)            ;; Obtener la posición horizontal el pixels
    ld  a, entity_pixX (ix)
    cp     #maxX-8              ;; El limite por la derecha es el limite menos el tamaño del sprite
    
    jr    z, moveRightEnd       ;; if (pisX!=(maxX-8)) {
        inc   a                 ;;     |
        ;ld   (pixX), a          ;;     |
        ld  entity_pixX (ix), a
        srl   a                 ;;     |
        ;ld   (posX), a          ;;     | posX = (++pixX)/2
        ld entity_posX (ix), a
        ld a, #1
        ret
    moveRightEnd:               ;; }
    ld a, #0
    ret


;;============================
;; Moves hero to left
;; DESTROYS: 
;;============================
entity_moveLeft::
    ;ld hl, #_g_map2
    ;call scroll_right

    call entity_lookLeft          ;; Orienta el sprite a la izquierda

    ld a, #_g_aniWalkLeft      ;; |

    push ix
    ld de, #entity_animation
    add ix, de
    call animation_set          ;; Establecer animación
    pop ix


entity_moveLeftNoAni::
    ; Comprobar si no hay colisión a la izquierda
    ;ld hl, #posX                ;; Mover provisionalmente un paso 
    ;dec (hl)                    ;;
    dec entity_posX (ix)

    ;; Comprobar colision en la nueva posición
    ld__d_ixh
    ld__e_ixl
    ld hl, #entity_boundingBox
    add hl, de
    ;ld hl, #boundingBox
    call map_checkCollision

    ;ld hl, #posX                ;; Restaurar posición previa
    ;inc (hl)                    ;;
    inc entity_posX (ix)

    cp #1                       ;; 
    jr    z, moveLeftEnd        ;; Si no hay colisión {

    ;ld    a, (pixX)             ;;   Obtener la posición horizontal el pixels
    ld  a, entity_pixX (ix)
    cp    #minX                 ;;   Comprobar el limite por la izquierda

    jr    z, moveLeftScroll     ;;   if (posX!=minX) {
        dec   a                 ;;       |
        ;ld   (pixX), a          ;;       |
        ld entity_pixX (ix), a
        srl   a                 ;;       |
        ;ld   (posX), a          ;;       | posX = (--pixX)/2
        ld entity_posX (ix), a
        ld a, #1          ;;   } else {
        ret
    moveLeftScroll:             ;;      Si sale de la pantalla, hacer scroll
        
        ld a, #1
        ret
                                ;;   }
    moveLeftEnd:                ;;   
                                ;; }
    ld a, #0
    ret

;;============================
;; Moves hero up
;; DESTROYS: 
;;============================
entity_moveUp::
    ;ld hl, #_g_map2
    ;call scroll_down
    
    ld a, #_g_aniWalkUp         ;; |
    
    push ix
    ld de, #entity_animation
    add ix, de
    call animation_set          ;; Establecer animación
    pop ix  


entity_moveUpNoAni::
    ; Comprobar si no hay colisión arriba

    dec entity_posY (ix)
    dec entity_posY (ix)

    ;; Comprobar colision en la nueva posición
    ld__d_ixh
    ld__e_ixl
    ld hl, #entity_boundingBox
    add hl, de
    ;ld hl, #boundingBox
    call map_checkCollision

    inc entity_posY (ix)
    inc entity_posY (ix)

    cp #1
    jr    z, moveUpEnd


    ;ld    a, (pixY)             ;; Obtener la posición horizontal el pixels
    ld a, entity_pixY (ix)
    cp    #minY                 ;; Comprobar el limite por la izquierda

    jr    z, moveUpEnd          ;; if (posX!=minX) {
        dec   a                 ;;     |
        ld entity_pixY (ix), a  ;;     |
        add  a,a                ;;     |
        ld entity_posY (ix), a  ;;     | posY = (++pixY)*2
        ld a, #1
        ret
    moveUpEnd:                  ;; }
    ld a, #0
    ret

;;============================
;; Moves hero down
;; DESTROYS: 
;;============================
entity_moveDown::
    ;ld hl, #_g_map2
    ;call scroll_up

    ld a, #_g_aniWalkDown      ;; |
    push ix                     ;;
    ld de, #entity_animation    ;;
    add ix, de                  ;;
    call animation_set          ;; Establecer animación
    pop ix                      ;;

entity_moveDownNoAni::
    ; Comprobar si no hay colisión arriba
    inc entity_posY (ix)
    inc entity_posY (ix)

    ;; Comprobar colision en la nueva posición
    ld__d_ixh
    ld__e_ixl
    ld hl, #entity_boundingBox
    add hl, de
    call map_checkCollision
    
    dec entity_posY (ix)
    dec entity_posY (ix)

    cp #1
    jr    z, moveDownEnd

    ld a, entity_pixY (ix)      ;; Obtener la posición vertical el pixels
    cp    #maxY-8               ;; Comprobar el limite por la abajo

    jr    z, moveDownEnd        ;; if (posX!=minX) {
    jp    p, moveDownEnd
        inc   a                 ;;     |
        ld entity_pixY (ix), a  ;;     |
        add  a,a                ;;     |
        ld entity_posY (ix), a  ;;     | posY = (--pixY)*2
        ld a, #1
        ret
    moveDownEnd:
    ld a, #0  
    ret


;;==================================
;; Pone la animación de parado
;; DESTROYS:
;;==================================
entity_stand::
    ld a, #_g_aniStand            ;;
    push ix
    ld de, #entity_animation
    add ix, de
    call animation_set          ;; Establecer animación
    pop ix

    ret


entity_setPosX::
    ld entity_pixX (ix), a  ;;     | pixX = A
    srl  a                  ;;     |
    ld entity_posX (ix), a  ;;     | posX = pixX/2
    ret

entity_setPosY::
    ld entity_pixY (ix), a  ;;     | pixY = A
    add  a,a                ;;     |
    ld entity_posY (ix), a  ;;     | posY = pixY*2
    ret

entity_getTilePos::
    ld a, entity_posX (ix)
    srl a
    srl a
    ld c, a

    ld a, entity_posY (ix)
    srl a
    srl a
    srl a
    ld b, a

    ld a, entity_posY (ix)
    and #0b111
    jp z, tileYOk
        inc b
    tileYOk:
    
    ret


;; ========================================================
;; ========================================================
;; PRIVATE FUNCTIONS
;; ========================================================
;; ========================================================


;;==================================
;; Gira sprite a la izquierda
;; DESTROYS:
;;==================================
entity_lookLeft::
    ld a, entity_flipH (ix)
    cp #1
    jr z, alreadyLookLeft
        xor #1
        ld entity_flipH (ix), a
    alreadyLookLeft:
    ret

;;==================================
;; Gira sprite a la derecha
;; DESTROYS:
;;==================================
entity_lookRight::
    ld a, entity_flipH (ix)
    cp #0
    jr z, alreadyLookRight
        xor #1
        ld entity_flipH (ix), a
    alreadyLookRight:
    ret


;;=======================================
;; Displaces entity to a given direction
;; DESTROYS:
;;=======================================
entity_displace:
    cp #entity_dirLeft
    jp z, entity_moveLeftNoAni
    cp #entity_dirRight
    jp z, entity_moveRightNoAni
    cp #entity_dirUp
    jp z, entity_moveUpNoAni
    jp entity_moveDownNoAni
    ret

;;==========================================
;; Configures this entity for a new tileset
;;
;; Input:
;; HL = tileset = new tileset for the entity
;; B = h = height for each tile
;; C = w = width for each tile (in bytes)
;; 
;; Destroys:
;; HL, DE
;;==========================================
entity_setTileset::
    ld entity_animation + animation_tileset + 1 (ix), h
    ld entity_animation + animation_tileset (ix), l
    ld entity_width (ix), c
    ld entity_height (ix), b
    
    ld e, (hl)
    inc hl
    ld d, (hl)

    ld entity_animation + animation_currentSprite + 1 (ix), d
    ld entity_animation + animation_currentSprite (ix), e

    ret

;;==========================================
;; Set entity's animation sequence
;;
;; Input:
;; A = frame sequence
;; 
;;==========================================
entity_setAnimation::
    push ix
    ld de, #entity_animation
    add ix, de
    call animation_set          
    pop ix

    ret



entity_setWorldPtr::
    ld entity_worldPtr + 1 (ix), h
    ld entity_worldPtr (ix), l
    ret

entity_deleteFromWorld::
    ld h, entity_worldPtr + 1 (ix)
    ld l, entity_worldPtr (ix)
    ;ld (hl), #0x01                  ;; |
;    inc hl                          ;; Set world pointer to 1 for this entity
;    ld (hl), #0x00                  ;; (means "skip" when loading entities)
    
    ld (hl), #0x02                  ;; Set world pointer to 1 for this entity
                                    ;; (means "skip" when loading entities)
    ret


;;==========================================
;; Adjust item pos
;;
;; Input:
;; IX = Pointer item  A = Pos x     B = Pos Y
;;==========================================
entity_adjustItem::
    and #0b11111100     ;; Calculate posX tile exact
    add a                   ;; pixX = x*2
    call entity_setPosX         ;; Set pos X money
    ld a, b             ;; Load pos y
    and #0b11111000     ;; Calculate posY tile exact
    srl a               ;;  pixY
    add #4              ;; Posicionar en pies de enemigo
    call entity_setPosY        ;; Set pos Y money
    
    ;ld__d_ixh
;    ld__e_ixl
;    ex de, hl
;    call map_checkCollision
;    cp #1
;    ret nz

    ;;Comprobar un tile antes horizontal
    ld a, entity_pixX(ix)
    sub #8
    call entity_setPosX
    ld__d_ixh
    ld__e_ixl
    ex de, hl
    call map_checkCollision
    cp #1
    ret nz

    ;;Comprobar un tile despues horizontal
    ld a, entity_pixX(ix)
    add #16
    call entity_setPosX
    ld__d_ixh
    ld__e_ixl
    ex de, hl
    call map_checkCollision
    cp #1
    ret nz

    ;;Comprobar un tile antes vertical
    ld a, entity_pixX(ix)
    sub #8
    call entity_setPosX
    ld a, entity_pixY(ix)
    sub #4
    call entity_setPosY
    ld__d_ixh
    ld__e_ixl
    ex de, hl
    call map_checkCollision
    cp #1
    ret nz

    ;;Comprobar un tile despues vertical
    ld a, entity_pixY(ix)
    add #8
    call entity_setPosY
    ret 

;;==========================================
;; Check if item is drop in this updarte
;;
;;==========================================
entity_checkDropItem::
    ld a, (flagDropItem)
    cp #0
    ld a, #1
    ret nz
    ld (flagDropItem), a
    ld a, #0
    ret