;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################


;; - El orco tiene 3 estados: Think, Seek, Attack (Pensar, buscar, atacar)
;; - Estado think:
;;   - Decide entre los posibles caminos una dirección y distancia para caminar y pasa al estado seek
;;   - La dirección será válida. Si detecta un muro con el sensor, escoge otra
;; - Estado seek:
;;   - Se mueve en la dirección del camino escogido
;;   - Si detecta un muro cerca con un sensor, vuelve a estado think para cambiar de camino antes de chocar con él
;;   - Si detecta al jugador a menos de 8 casillas de distancia (distancia de Manhattan), cambia a estado attack
;;   - Si el jugador está a distancia visible, pero hay un muro delante, el orco detecta antes el muro e ignora al jugador
;;   - Si ha terminado de recorrer la distancia establecida vuelve a think para escoger otra
;; - Estado Attack:
;;   - Sigue al jugador por camino de Manhattan
;;   - Si en algún momento lo pierde de vista, pasa al estado Think


.include "cpctSymbols.h.s"
;.include "entity.h.s"
.include "map.h.s"
.include "scroll.h.s"
.include "world.h.s"
.include "math.h.s"
.include "characters/frames.h.s"
.include "entityWarrior.h.s"
.include "entityOrc.h.s"
.include "sfx/sfx.h.s"

.globl hero

.globl _g_orc
.globl _g_orc_00
.globl coins_prototype
.globl heart_prototype
entity_defineCharacter orc_prototype, 40, 136, 4, 16, 0, 0, _g_orc, _g_orc_00, orc_logic, entity_tEnemy, 0, 0, 0, 3, 0, 0, 1

orc_sequences:: .db _g_aniStand, _g_aniWalkLeft, _g_aniWalkRight, _g_aniWalkUp, _g_aniWalkDown, _g_aniDead
orc_currentSequences:: .dw orc_sequences

.equ orc_aniStand, 0
.equ orc_aniWalkLeft, 1
.equ orc_aniWalkRight, 2
.equ orc_aniWalkUp, 3
.equ orc_aniWalkDown, 4
.equ orc_aniDead, 5
.equ noItem, 64                   ;;Prob no drop item (64/255 = 25%)
.equ itemHeart, 77               ;; Prob item heart (13/255 = 5%)

.equ orc_visibleDistance, 8

;orc_updateFlag: .db #3
orc_dirSpiral: .db #0

;;============================
;;============================
;; LOCAL VARIABLES
;;============================
;;============================
heroDifX: .db 0     ;; pos X difference to hero (can be negative)
heroDifY: .db 0     ;; pos Y difference to hero (can be negative)
heroDist: .db 0     ;; distance to hero (manhattan distance)
startX:   .db 0     ;; start X coordinate to check sensor line
startY:   .db 0     ;; start Y coordinate to check sensor line
moneyItem: .dw coins_prototype
heartItem: .dw heart_prototype


;;============================
;; Manage orc's state machine
;; DESTROYS: 
;;============================
orc_logic::
;    ld a, (orc_updateFlag)
;    dec a
;    ld (orc_updateFlag), a
;    cp #0
;    ret nz;

;    ld a, #3
;    ld (orc_updateFlag), a

    call orc_checkHeroSword

    ld a, entity_state (ix)
    cp #orc_stThink
    jp z, orc_updateStateThink

    cp #orc_stSeek
    jp z, orc_updateStateSeek

    cp #orc_stAttack
    jp z, orc_updateStateAttack

    cp #orc_stHit
    jp z, orc_updateStateHit

    cp #orc_stDead
    jp z, orc_updateStateDead

    ret


enemy_setStateThink::
orc_setStateThink::
    ld orc_wait (ix), #50
    ld entity_state (ix), #orc_stThink
    call entity_stand
    ret

;;============================
;; Update state think
;; DESTROYS: 
;;============================
orc_updateStateThink:
    
    ;call orc_checkHeroSword

    ld a, orc_wait (ix)
    dec a
    jp p, noNegative
        ld orc_wait (ix), #0
        call orc_setStateSeek
        ret
    noNegative:
    ld orc_wait (ix), a
    
    cp #0
    ret nz

    call orc_setStateSeek

    ret


orc_setStateSeek:    

    ld entity_state (ix), #orc_stSeek           ;; Set new state
    ld a, #10                                   ;; Sets seek spiral counter to 10 iterations
    ld (orc_dirSpiral), a                       ;; |

    chooseDirLoop:
        ld a, (orc_dirSpiral)                       ;; Check seek spiral counter
        dec a                                       ;; |
        ret z                                       ;; |
        ld (orc_dirSpiral), a                       ;; |
        call cpct_getRandom_mxor_u8_asm             ;; Random (0,3) for direction
        ld a, l                                     ;; |
        and #0b00000011                             ;; |
        ld orc_dir (ix), a      ;; |
        
        ld d, a
        ld e, #2
        call entity_getTilePos
        ld a, d
        call orc_sensorCheck
        cp #0
        jp nz, chooseDirLoop

    ;ld orc_dir (ix), #orc_dirRight

    call cpct_getRandom_mxor_u8_asm             ;; |
    ld a, l                                     ;; |
    and #0b00111111                             ;; Random (15,64) for distance
    or  #0b00001111                             ;; |
    ld orc_dist (ix), a                         ;; |
    ret


;;============================
;; Update state seek
;; DESTROYS: 
;;============================
orc_updateStateSeek::
    
    ;call orc_checkHeroSword
    ;; Check sensors
    call entity_getTilePos              ;; Get current position in tiles
    ld a, orc_dir (ix)                  ;; Get current direction
    call orc_sensorCheck                ;; Check sensors 
    cp #0
    jp z, boundsOk                      ;; If obstacle detected to that direction
        call orc_setStateThink          ;;     "think" another direction
        ret
    boundsOk:                           ;; else:

    ld a, (hero_state)
    cp #hero_stDead
    jp z, orc_seekNoHeroFound
    cp #hero_stWin
    jp z, orc_seekNoHeroFound
    
    call orc_getHeroDirection
    cp #entity_noDir
    jp z, orc_seekNoHeroFound
        call orc_setStateAttack
        ret
    orc_seekNoHeroFound:

;    call orc_getDistanceToHero
;    cp #7
;    jp p, noHeroFound
;        call orc_getObstaclesToHero
;    noHeroFound:

    call orc_move                       ;; Move orc as normal

    ld a, orc_dist (ix)                 ;;
    dec a                               ;;
    ld orc_dist (ix), a                 ;; If defined distance hasn't been reached
    ret nz                              ;;     end update
    call orc_setStateThink              ;; "think" another direction when finish
    ret


orc_setStateAttack::
    ld entity_state (ix), #orc_stAttack           ;; Set new state
    ret

;;============================
;; Update state attack
;; DESTROYS: 
;;============================
orc_updateStateAttack::

    call orc_checkHero
    ;call orc_checkHeroSword
    ld a, entity_state (ix)
    cp #orc_stThink
    ret z
    call orc_getHeroDirection
    cp #entity_noDir
    jp z, orc_attackNoHeroFound
        
        ld orc_dir (ix), a
        ld orc_dist (ix), #8
        call orc_move                       ;; Move orc to hero
        ret
    orc_attackNoHeroFound:
    call orc_setStateThink
    ret

orc_setStateHit::
    ld a, #_g_sfx_kickToId          ;; Sound effect ID
    ld c, #0                        ;; Channel
    ld b, #0                        ;; Volume (inverted 0=max)
    call cpct_akp2_SFXPlay_asm

    ld entity_state (ix), #orc_stHit

    ld orc_dist (ix), #12

    ld a, (hero_direction)
    ld orc_dir (ix), a
    ret

orc_updateStateHit:

    ld a, orc_dir (ix)              ;; |
    call entity_displace            ;; Displacement caused by hit

    ld a, orc_dist (ix)             ;; |
    dec a                           ;; decrement orc_dist
    ld orc_dist (ix), a             ;; if orc_dist != 0
    ret nz                          ;;     not finished displacing orc

    ld a, orc_life (ix)             ;; |
    dec a                           ;; decrement orc_life
    ld orc_life (ix), a             ;; if orc_life != 0
    jp nz, orc_setStateThink        ;;     set state think

    call orc_setStateDead           ;; else, set state dead

    ret




;;==========================================
;; Decide what item drop the entity
;;
;; Input:
;; IX = Pointer to entity
;; 
;;==========================================
orc_dropItem::
    call entity_isVectorFull
    cp #1
    ret z
    call cpct_getRandom_mxor_u8_asm             ;; |
    ld a, l                                     ;; |
    cp #noItem
    ret m       ;; If is less than noItem finish
    cp #itemHeart
    jp m, loadHeart     ;; If is heart load heart

    
    ;;Else create money
    ld hl, (moneyItem)
    jp createItem
    
    loadHeart:
    ld hl, (heartItem)
    
    createItem:
    call entity_new     ;; vector add new entity ball with prototype dates DE = money entity
    ld a, entity_posX (ix)
    ld b, entity_posY (ix)
    push ix
    ld__ixh_d                   ;; |
    ld__ixl_e                   ;; IX = money entity
    call entity_adjustItem
    pop ix                      ;; Obtain current entity
    
    ret

orc_setStateDead::
    ld a, #_g_sfx_killId            ;; Sound effect ID
    ld c, #0                        ;; Channel
    ld b, #0                        ;; Volume (inverted 0=max)
    call cpct_akp2_SFXPlay_asm

    ld entity_state (ix), #orc_stDead
    ld c, #orc_aniDead
    call orc_setAnimation
    ;ld a, #_g_aniDead
    ;call entity_setAnimation
    ;; Comentado porque da bugs de dibujados
    call orc_dropItem 
    ret

orc_updateStateDead:
    ld a, entity_animation + animation_currentFrame (ix)
    cp #-2
    ret nz
    ;; ...

    ret




;; ===================================================
;; Returns distance to hero
;; 
;; Input:
;; IX = orc = this orc entity
;; 
;; Returns:
;; BC = (Y,X) difference in tiles
;; A = absolute Manhattan distance in tiles
;; ===================================================
orc_getHeroDistance:
    push ix                             ;; |
    ld ix, #hero                        ;; |
    call entity_getTilePos              ;; |
    inc b                               ;; |
    pop ix                              ;; Get hero position

    ld h, b                             ;; |
    ld l, c                             ;; HL = hero position (Y, X)

    call entity_getTilePos              ;; |
    inc b

    ld a, c                             ;; |  
    ld (startX), a                      ;; startX = entity_posX
    ld a, b                             ;; |
    ld (startY), a                      ;; startY = entity_posY

    ld d, b                             ;; |
    ld e, c                             ;; HL = orc position (Y, X)

    ld a, h                             ;; |
    sub d                               ;; |
    ld b, a                             ;; B = Y distance

    ld a, l                             ;; |
    sub e                               ;; |
    ld c, a                             ;; C = Y distance

    call absA                           ;; |
    ld h, a                             ;; H = abs(Y distance)

    ld a, b                             ;; |
    call absA                           ;; A = abs(X distance)

    add h                               ;; A = abs(Y distance) + abs(X distance)

    ret


orc_getHeroDirection::

    call orc_getHeroDistance            ;; Get distance to hero

    cp #orc_visibleDistance             ;; |
    ld (heroDist), a                    ;; |
    ld a, #entity_noDir                 ;; If orc is out of visible distance
    ret p                               ;;    return A = orc_noDir

    ld a, c                             ;; |
    ld (heroDifX), a                    ;; |
    ld a, b                             ;; Save pos difference values
    ld (heroDifY), a                    ;; for next use


    cp #0
    jp nz, noAlignedH
        call orc_checkDirH
        jp justHorizontal
    noAlignedH:
    ld a, c
    cp #0
    jp nz, noAlignedV
        call orc_checkDirV
        jp justVertical
    noAlignedV:
                                        ;; Check collisions to hero across
    call orc_checkDirH                  ;; the horizontal axis

    cp #0                               ;;
    jp nz, heroVertFirst                ;; if there is no horizontal collision 
        ld a, (startX)                  ;;     |
        ld b, a                         ;;     |
        ld a, (heroDifX)                ;;     |
        add b                           ;;     Move startX to make an "L" check
        ld (startX), a                  ;;     startX = startX + heroDifX

                                        ;; Check collisions to hero across
    call orc_checkDirV                  ;; the vertical axis
    
    justHorizontal:
    cp #0                               ;; Here, if detect collision going vertical,
    ld a, #entity_noDir                 ;; return A = noDir, as we know that
    ret nz                              ;; we have collision going horizontal too

    ld a, (heroDifX)                    ;; 
    cp #0                               ;; Going horizontal we can meet hero so,
    ld a, #entity_dirRight              ;; if heroDifX is positive, 
    ret p                               ;;     return A = rigth
    ld a, #entity_dirLeft               ;; else,     
    ret                                 ;;     return A = left

    ;; At this point, we know that if we go horizontal first we meet a wall
    ;; so let's try vertical movement first
    heroVertFirst:
        call orc_checkDirV              ;; Check collisions going vertical
        cp #0                           ;;     
        ld a, #entity_noDir             ;; if collision is found
        ret nz                          ;;     return A = orc_noDir

        ld a, (startY)                  ;; |
        ld b, a                         ;; |
        ld a, (heroDifY)                ;; |
        add b                           ;; Move startY to make an "L" check
        ld (startY), a                  ;;     startY = startY + heroDifY

        
                                        ;; Check collisions to hero across
        call orc_checkDirH              ;; the horizontal axis
        
        justVertical:
        cp #0                           ;; Here, if detect collision going horizontal,
        ld a, #entity_noDir             ;; return A = noDir, as we know that
        ret nz                          ;; we have collision going horizontal too

        ld a, (heroDifY)                ;; 
        cp #0                           ;; Going vertical we can meet hero so,
        ld a, #entity_dirDown           ;; if heroDifY is positive, 
        ret p                           ;;     return A = down
        ld a, #entity_dirUp             ;; else,     
        ret                             ;;     return A = up



orc_checkDirH:
    ld a, (heroDifX)                    ;; |
    cp #0                               ;; |
    jp p, checkDirHDistRight            ;; If (heroDifX is negative)
        ld hl, #-1                      ;;     increment = HL = -1
        jp checkDirHDistLeft            ;; |
    checkDirHDistRight:                 ;; else
    ld hl, #1                           ;;     increment = HL = 1 
    checkDirHDistLeft:                  ;; 

    call orc_checkDir
    ret

orc_checkDirV:
    ld a, (heroDifY)                    ;; |
    cp #0                               ;; |
    jp p, checkDirVDistRight            ;; If (heroDifX is negative)
        ld hl, #-map_width              ;;     increment = HL = -1
        jp checkDirVDistLeft            ;; |
    checkDirVDistRight:                 ;; else
    ld hl, #map_width                   ;;     increment = HL = 1 
    checkDirVDistLeft:                  ;; 

    call orc_checkDir

    ret

orc_checkDir:
    call absA                           ;; |
    ld e, a                             ;; E = abs(A)

    ld a, (startX)                      ;; |
    ld c, a                             ;; C = startX
    ld a, (startY)                      ;; |
    ld b, a                             ;; B = startY
    
    call orc_sensorCheckLine            ;; return orc_sensorCheckLine
    ret

orc_setAnimation:
    ld hl, (orc_currentSequences)
    ld b, #0
    add hl, bc
    ld a, (hl)
    call entity_setAnimation
    ret


;;============================
;; Move orc over the scene
;; DESTROYS: 
;;============================
orc_move::
    
    ld a, orc_dist (ix)
    cp #0
    jp nz, notStand
        call entity_stand
        ret

    notStand:
    ld a, orc_dir (ix)
    cp #entity_dirLeft
    jp nz, notLeft
        ld c, #orc_aniWalkLeft
        call orc_setAnimation
        call entity_lookLeft
        call entity_moveLeftNoAni
        jp movA

    notLeft:
    cp #entity_dirRight
    jp nz, notRight
        ld c, #orc_aniWalkRight
        call orc_setAnimation
        call entity_lookRight
        call entity_moveRightNoAni

        movA:
        cp #0
        ret nz
        ld c, #orc_aniWalkDown
        call orc_setAnimation
        call entity_moveDownNoAni
        ret

    notRight:
    cp #entity_dirUp
    jp nz, notUp
        ld c, #orc_aniWalkUp
        call orc_setAnimation
        call entity_moveUpNoAni
        jp movB
    
    notUp:                                      ;; Should be orc_dir = orc_dirDown
        ld c, #orc_aniWalkDown
        call orc_setAnimation
        call entity_moveDownNoAni

        movB:
        cp #0
        ret nz
        ld c, #orc_aniWalkLeft
        call orc_setAnimation
        call entity_moveLeftNoAni    
        ret



orc_sensorCheck:

;    push af                     ;; Deletes one inc b on the 
;    ld a, entity_height (ix)    ;; selected labels when is a babie
;    cp #7                       ;; slimer (height = 7)
;    jp nz, notBabieSlimer       ;; for solve the bug of stick inside walls
;    ld a, #0                    ;; |
;    jp endBabieSlimerCheck      ;; |
;    notBabieSlimer:             ;; |
;    ld a, #4                    ;; |
;    endBabieSlimerCheck:        ;; |
;    ld (ph_noincLeftA), a       ;; |
;    ld (ph_noincLeftB), a       ;; |
;    ld (ph_noincLeftC), a       ;; |
;    pop af                      ;; |

    cp #entity_dirLeft
    jp nz, boundsNoLeft
        ld a, c
        cp #3
        ld a, #1
        ret m
        ld hl, #-1
        ld e, #1
        dec c                               ;; Start with the next tile
        ph_noincLeftA:
        inc b                               ;; Check tile at the bottom of the character
        call orc_sensorCheckLine
        ret

    boundsNoLeft:
    cp #entity_dirRight
    jp nz, boundsNoRight
        ld a, c
        cp #map_width-2
        ld a, #1
        ret p
        ld hl, #1
        ld e, #2
        inc c                               ;; Start with the next tile
        ph_noincLeftB:
        inc b                               ;; Check tile at the bottom of the character
        call orc_sensorCheckLine
        ret

    boundsNoRight:
    cp #entity_dirUp
    jp nz, boundsNoUp
        ld a, b
        cp #5
        ld a, #1
        ret m
        ld hl, #map_width
        ld e, #2
        call orc_sensorCheckDoubleVertical
        ret

    boundsNoUp:
        ld a, b
        cp #map_height-4
        ld a, #1
        ret p
        ld hl, #map_width
        ld e, #2
        inc b                               ;; Start with the next tile
        ph_noincLeftC:
        inc b                               ;; Check tile at the bottom of the character
        call orc_sensorCheckDoubleVertical
        ret


;; ===================================================
;; Check a vertical line and another line to the right
;;
;; E = dist = Distance to check
;; HL = inc = tile increment for each step
;; C = posX = X position of the entity
;; B = posY = Y position of the entity
;; ===================================================
orc_sensorCheckDoubleVertical:
    push hl
    push de
    push bc
    call orc_sensorCheckLine
    pop bc
    pop de
    pop hl
    cp #0
    ret nz
    ;ret

    ld a, entity_posX (ix)
    and #0b11
    ret z

    inc c
    call orc_sensorCheckLine

    ret

;; ===================================================
;; Check collisions in tiles to a given direction
;; 
;; Input:
;; E = dist = Distance to check
;; HL = inc = tile increment for each step
;; C = posX = Orc position X in tiles
;; B = posY = Orc position Y in tiles
;; ===================================================
orc_sensorCheckLine:
    
    push hl
    push de
    call map_getTile                    ;; HL = tilePtr
    pop de
    pop bc

    checkLeftLoop:
        ld a, (map_solidThreshold)
        ld d,a
        ld a, (hl)                      ;; A = *tilePtr
        cp d
        ;cp #solidThreshold              ;;
        jp p, noCollisionLeft           ;; if (A<solidThreshold)
            ld a, #1
            ret                         ;;     return D = dist
        noCollisionLeft:
    
        add hl, bc

        dec e                           ;; distToCheck--
        jp nz, checkLeftLoop            ;; if distanceToCheck!=0, loop

    ld a, #0
    ret


orc_checkHeroSword:

    ld a, entity_state (ix)
    cp #orc_stHit
    ret z
    cp #orc_stDead
    ret z

    ld a, (hero_attacking)
    cp #0
    ret z

    ld__d_ixh
    ld__e_ixl
    ld hl, #hero_sword
    call collision_check
    cp #0
    ret z

    call orc_setStateHit
    ret

;; ===================================================
;; Check collisions with hero and hit hero
;; 
;; Output:
;; A = 0 no collision, 1 collision and hit, 2 no attack
;; ===================================================
orc_checkHero_ns::
    ld a, (hero_state)
    cp #hero_stHit
    jp z, no_attack
    cp #hero_stDead
    jp z, no_attack
    cp #hero_stWin
    jp z, no_attack

    ld__d_ixh
    ld__e_ixl
    ld hl, #hero
    call collision_check
    cp #0
    jp z, no_collision

    ld a, orc_dir (ix)
    push ix
    ld de, #hero
    ;ld b, orc_attackPts (ix)
    ld b, orc_attackPts (ix)
    ld__ixh_d
    ld__ixl_e
    call hero_setStateHit
    pop ix
    ld a, #1
    ret

    no_attack:
    ld a, #2
    ret

    no_collision:
    ld a, #0
    ret


orc_checkHero::
    call orc_checkHero_ns    
    cp #0
    call nz, orc_setStateThink

    ret
