;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################


.include "cpctSymbols.h.s"
;.include "entity.h.s"
;.include "map.h.s"
;.include "scroll.h.s"
.include "scene.h.s"
;.include "world.h.s"
;.include "math.h.s"
.include "characters/frames.h.s"
.include "entityWarrior.h.s"
.include "entityOrc.h.s"
.include "characters/frames.h.s"
.include "sfx/sfx.h.s"

.globl hero
.globl _g_babiSlimer

.globl _g_slimer
.globl _g_slimer_0

entity_defineCharacter slimer_prototype, 40, 136, 4, 16, 0, 0, _g_slimer, _g_slimer_0, slimer_logic, entity_tEnemy, 0, 0, 0, 3, 0, 0, 1

;; Animation sequencies for slimer
slimer_sequences:: .db _g_aniStandSlimer, _g_aniWalkLeftSlimer, _g_aniWalkRightSlimer, _g_aniWalkUpSlimer, _g_aniWalkDownSlimer, _g_aniDeadSlimer

.equ slim_stSticked, 5
.equ slim_stBabed, 6
.equ slim_babieQuant, 3
slim_babieCounter: .db 0x00

.equ slim_initAttackTime, 50
.equ slim_isBabie, entity_customData + 4 ; default 0
;.equ slimer_damageAmount, entity_customData + 8 ; default 1

;;============================
;;============================
;; LOCAL VARIABLES
;;============================
;;============================

;;============================
;; Manage slimer's state machine
;; DESTROYS: 
;;============================
slimer_logic::
    ld a, (scene_motionDivisor) ;; Check if is needed to do some action this time
    cp #1                       ;; |
    ret z                       ;; |

    ld a, (hero_sticked)        ;; Check if hero is sticked by slimer
    cp #0                       ;; and if is sticked he cant hit the slimer with his sword
    jp nz, slimer_heroSticked   ;; |
    call orc_checkHeroSword     ;; Checks slimer collision with hero sword

    slimer_heroSticked:
    
    ld hl, #slimer_sequences
    ld (orc_currentSequences), hl

    call case_state
        ld hl, #orc_sequences
        ld (orc_currentSequences), hl
        ret

    case_state:
        ld a, entity_state (ix)
        cp #orc_stThink
        jp z, slimer_updateStateThink;
        cp #orc_stSeek
        jp z, orc_updateStateSeek;
        cp #orc_stAttack
        jp z, slimer_updateStateAttack;
        cp #orc_stHit
        jp z, orc_updateStateHit;
        cp #orc_stDead
        jp z, slimer_updateStateDead
        cp #slim_stSticked
        jp z, slimer_updateStateSticked;
        ret


;;============================
;; Update state think
;; DESTROYS: 
;;============================
slimer_updateStateThink:
    
    ;call orc_checkHeroSword

    ld a, orc_wait (ix)
    dec a
    jp p, noNegative
        ld orc_wait (ix), #0
        call orc_setStateSeek
        ret
    noNegative:
    ld orc_wait (ix), a
    
    cp #0
    ret nz

    call orc_setStateSeek

    ret


;;============================
;; If slimer is big turns it into babes, if it's a babe turns it into dead
;; 
;;============================
slimer_updateStateDead::

    ld a, slim_isBabie (ix)             ;; Check if it is a babe
    cp #0                               ;; |
    jp z, notBabe
    ;call slimer_setStateDead               ;; if its a babe sets state dead
    ret

    notBabe:
    push ix                             ;; Save IX to restore it at the end


;    ;; Check for tile collision 
;    ;;
;    ld a, entity_posY (ix)              ;; Decrease y position temporally
;    sub a, #4                           ;; in order to check for collision
;    ld entity_posY (ix), a              ;; |;

;    ld__d_ixh                           ;; |
;    ld__e_ixl                           ;; |
;    ex de,hl                            ;; |
;    call map_checkCollision             ;; Check for collision against a wall
;    
;    push af                             ;; |
;    ld a, entity_posY (ix)              ;; Restore original Y position
;    add a, #4                           ;; |
;    ld entity_posY (ix), a              ;; |
;    pop af                              ;; |;

;    cp #0                               ;;
;    jr z, noCollisionOnCreate           ;; if (collision==true)
;        ld a, entity_pixY (ix)          ;;     Move the entity 4 pixels down
;        add a, #4                       ;;     as it's 4 pixels smaller.
;        call entity_setPosY             ;;     This way it will be aligned
;    noCollisionOnCreate:                ;;     to the bottom of the original sprite (bigger)

    call slimer_convertToBabie

    ld a, #slim_babieQuant
    ld (slim_babieCounter), a

    new_babie:                          ;; Checks if needs to create a new babie
    ld a, (slim_babieCounter)           ;; | 
    dec a                               ;; | 
    jp z, endCreation                   ;; | 
    ld (slim_babieCounter), a           ;; |

    call entity_isVectorFull            ;; Check if vector is full before add new babie
    cp #1                               ;; | 
    jp z, endCreation                   ;; |
    ld hl, #slimer_prototype            ;; Add new slimer to vector
    call entity_new                     ;; returns new entity pointer in DE
    ld a, entity_pixX (ix)              ;; Reads the actual entity coords and set them to the new babie
    ld b, entity_pixY (ix)              ;; | (puts new adjacent to the last one)
    ld__ixh_d                           ;; |
    ld__ixl_e                           ;; loads new entity from DE to IX
    ;add a, #5                           ;; |
    call entity_setPosX                 ;; Set new entity pos x saved in a
    ld a, b                             ;; |
    ;add a, #5                           ;; |
    call entity_setPosY                 ;; Set new entity pos x saved in b

    ld a, (slim_babieCounter)
    bit 0, a
    jp z, dirUp 
    ld orc_dir(ix), #entity_dirUp
    jp dirEnd
    dirUp:
    ld orc_dir(ix), #entity_dirDown
    dirEnd:

    call slimer_convertToBabie
    jp new_babie

    endCreation:
    pop ix
    ret


slimer_convertToBabie:
    ld a, #1                            ;; Sets babie flag to true
    ld slim_isBabie (ix), a             ;; | 
    ld a, #2                            ;; Sets new life level              
    ld orc_life (ix), a                 ;; |         
    ;ld bc, #0x0703                      ;; Sets babie tileset
    ld bc, #0x1003                      ;; Sets babie tileset
    ld hl, #_g_babiSlimer               ;; |
    call entity_setTileset              ;; |     
    call slimer_setStateHit             ;; Changes state to think
    ret


;;============================
;; Sticks to the hero when is close
;; 
;;============================
slimer_updateStateAttack:
    call orc_getHeroDirection
    cp #entity_noDir
    jp z, orc_attackNoHeroFound
        ld orc_dir (ix), a
        ld orc_dist (ix), #8
        call orc_move                   ;; Move slimer to hero
        call orc_checkHero_ns           ;; Check collision with hero
        cp #1                           ;; and changes to stick state if hits hero
        jp z, slimer_setStateSticked    ;; |
        ret
    orc_attackNoHeroFound:
    call orc_setStateThink
    ret

;;============================
;; Does the hero movement, the countdown between attacks once sticked and hurts
;; 
;;============================
slimer_updateStateSticked:
    ld a, (hero_sticked)                    ;; Check if hero is sticked by slimer
    cp #0                                   ;; |
    jp z, orc_setStateThink                 ;; If not, changes the state to think

    ld a, (#hero + entity_pixX)             ;; Reads the hero's coords and set them to the slimer
    call entity_setPosX                     ;; |
    ld a, (#hero + entity_pixY)             ;; |
    call entity_setPosY                     ;; |

    ld a, (warrior_stickedHurtCountdown)    ;; Decrements time to attack and if it's ended...
    dec a                                   ;; |
    ld (warrior_stickedHurtCountdown), a    ;; |
    cp #0                                   ;; |
    ret nz                                  ;; |
    ld a, (#slim_initAttackTime)            ;; ...restores the value and attacks
    ld (warrior_stickedHurtCountdown), a    ;; |
    call orc_checkHero_ns                   ;; |

slimer_setStateSticked:
    ld a, (#slim_initAttackTime)            ;; Inits the attack counter
    ld (warrior_stickedHurtCountdown), a    ;; |
    call hero_setStickedState
    ld entity_state (ix), #slim_stSticked
    ret

slimer_setStateHit:
    ld entity_state (ix), #orc_stHit
    ld orc_dist (ix), #12
    ret


slimer_setStateDead::
    ld a, #_g_sfx_killId            ;; Sound effect ID
    ld c, #0                        ;; Channel
    ld b, #0                        ;; Volume (inverted 0=max)
    call cpct_akp2_SFXPlay_asm

    ld entity_state (ix), #orc_stDead
    ;ld a, #_g_aniStandSlimer
    ;call orc_setAnimation
    ld a, #_g_aniDeadSlimer
    call entity_setAnimation
    ret