;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################

.include "cpctSymbols.h.s"
.include "entityVector.h.s"

.equ entity_maxEntities, 20


;; Reserved vector for entities
entity_vector::
entity_defineNEntities entity_vector, entity_maxEntities
;; Number of instantiated entities
entity_count: .db 0x00
;; Pointer to the next entity to instantiate
entity_nextPtr: .dw #entity_vector

entity_currentPtr:: .dw #0x0000

;;=====================================================
;; Creates a new entity
;;
;; Input:
;; HL = pointer to the entity prototype to instantiate
;;  
;; Returns:
;; DE = pointer to the new entity
;;=====================================================
entity_new::
    push hl                     ;; Save the prototype pointer to the stack

    ld a, (entity_count)        ;; |
    inc a                       ;; |
    ld (entity_count), a        ;; entity_count ++

    ld hl, (entity_nextPtr)     ;; 
    ld d, h                     ;;
    ld e, l                     ;; DE = entity_nextPtr
    ld (entity_currentPtr), de
    ld bc, #SIZEOF_entity       ;; 
    add hl, bc                  ;; 
    ld (entity_nextPtr), hl     ;; entity_nextPtr += SIZEOF_entity
    
    pop hl                      ;; HL = entity prototype
    
    ldir                        ;; Copy BC bytes from HL to DE
    
    ld de, (entity_currentPtr)

    ret


;;=====================================================
;; Deletes a given entity
;;
;; Input:
;; IX = pointer to the entity to delete
;;  
;;=====================================================
entity_delete::
    call entity_eraseEnd

    ld__d_ixh
    ld__e_ixl

    ld a, (entity_count)        ;; A = number of entities

    cp #0                       ;;
    ret z                       ;; if entity_count == 0, return

    dec a                       ;; | 
    ld (entity_count), a        ;; entity_count --
    
    cp #0                       ;; If the entity is the last one
    jp z, noCopy                ;; there's no need to perform a copy

    ld hl, (entity_currentPtr)  ;;
    ld a, d                     ;;
    cp h                        ;;
    jr nz, doCopy               ;;
    ld a, e                     ;; If the entity to delete is the last entity
    cp l                        ;; on the vector, there's no need to perform a copy
    jr nz, doCopy               ;;
    
    jr noCopy    
    
    doCopy:
    ld hl, (entity_currentPtr)  ;; |
    ld bc, #SIZEOF_entity       ;; Copy all bytes from the top entity to the
    ldir                        ;; entity to delete (therefore this entity will be overwritten)

    noCopy:

    ld hl, (entity_currentPtr)  ;; |
    ld (entity_nextPtr), hl     ;; nextPtr = currentPtr

    ld bc, #SIZEOF_entity       ;;
    or a                        ;;
    sbc hl, bc                  ;;
    ld (entity_currentPtr), hl  ;; currentPtr = currentPtr - SIZEOF_entity

    ret

;;========================================================
;; Executes a function for each entity in entities vector
;;
;; Input:
;; HL = pointer to the function to execute
;;========================================================
entity_forEach::
    ld a, (entity_count)        ;; A = number of entities
    cp #0
    ret z
    ld ix, #entity_vector       ;; IX = first entity of the vector
    ld (ph_forEachFn+1), hl     ;; Set function to execute for each entity

    forEach_loop:               ;; For each entity
        
        push af                 ;; |
        push ix
        ph_forEachFn:           ;; |
        call #0                 ;; Call to function
        pop ix
        pop af                  ;; 

        ld bc, #SIZEOF_entity   ;; |
        add ix, bc              ;; Move to the next entity

        dec a                   ;; |
        jp nz, forEach_loop     ;; loop if we have entities left

    ret


;;========================================================
;; Clears the entity vector.
;; Sets entity_count to 0, and entity_nextPtr 
;; to the first entity
;;
;;========================================================
entity_clearAll::

    ld a, #0
    ld (entity_count), a        ;; entity_count = 0
    
    ld hl, #entity_vector
    ld (entity_nextPtr), hl     ;; entity_nextPtr += SIZEOF_entity

    ret

;;========================================================
;; Check if the vector is full
;; OUTPUT -> A = 0 is not full, 1 is full
;;========================================================
entity_isVectorFull::
    ld a, (entity_count)
    ld b, #entity_maxEntities
    cp b
    jp z, isFull
        ld a, #0
        ret
    isFull:
    ld a, #1

    ret