;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################

;.include "cpctelera.h.s"
.include "cpctSymbols.h.s"
.include "entity.h.s"
.include "map.h.s"
.include "scroll.h.s"
.include "world.h.s"
.include "scene.h.s"
.include "characters/frames.h.s"
.include "dbuffer.h.s"
.include "panel.h.s"
.include "entityWarrior.h.s"
.include "gameState.h.s"
.include "gameOver.h.s"

.include "sfx/sfx.h.s"

.include "input.h.s"

.globl _g_map2


.globl _g_warriorAttackH_0
.globl _g_warriorAttackH_1
.globl _g_warriorAttakH
.globl _g_warriorAttackV_0
.globl _g_warriorAttackV_1
.globl _g_warriorAttakV
.globl _g_warrior
.globl _g_warrior_00

;entity_define hero, 40, 136, 4, 16, 0, 0, _g_warrior, _g_warrior_00, logic_warrior, entity_tHero
entity_define hero, 36, 144, 4, 16, 0, 0, _g_warrior, _g_warrior_00, logic_warrior, entity_tHero

collision_define hero_sword, 0, 0, 4, 16

hero_attacking:: .db 0x00  ;0 = No attack, 1 = Attack
hero_sticked::   .db 0x00  ;0 = No sticked, 1 = Sticked

attackDir:: .db 0 ; 0 = left, 1 = right, 2 = up, 3 = down
;dimAttack:: .dw 0x0000 ;;W, H

;;================================
;;================================
;; LOCAL VARIABLES
;;================================
;;================================
;; Attack pressed flag
flagAttackPressed: .db 0x00
flagDropItem:: .db 0x00
warrior_keyDanceState:       .db 0x00    ; Next key expected left/right
warrior_keyDanceCount:       .db 0x00    ; How many correct left/right dances need to leave
warrior_keyDanceResetCount:  .db 0x00    ; Countdown to reset dance counter
warrior_stickedHurtCountdown::  .db 0x00    ; Countdown between sticked hurts

;; Warrior state:
hero_state::
warrior_state: .db warrior_stNormal

.equ warrior_stNormal, hero_stNormal
.equ warrior_stAttack, hero_stAttack
.equ warrior_stHit, hero_stHit
.equ warrior_stDead, hero_stDead
.equ warrior_stWin, hero_stWin
.equ warrior_expecLeft, 0
.equ warrior_expecRight, 1
.equ warrior_danceTimes, 6 ; Amount of danceTimes to leave the stick state
.equ warrior_countDownStick, 37 ;Time limit to do the amount of dancetimes 

preAttackX: .db 0
preAttackY: .db 0
hero_direction::
preDir: .db 0
waitFrame: .db 0

.equ warrior_left, 0
.equ warrior_right, 1
.equ warrior_up, 2
.equ warrior_down, 3

displaceDir: .db 0
displaceDist: .db 0

;;================================
;; Manage warrior's state machine
;; DESTROYS: 
;;================================
logic_warrior::
    ;;Desactivate flag drop item
    ld a, #0
    ld (flagDropItem), a 

    ld a, (hero_sticked)        ;; Check if hero is sticked by slimer
    cp #0                       ;; |
    jp z, notSticked            ;; |

    call warrior_updateSticked

    ld a, (scene_motionDivisor) ;; Check if is needed to do some action this time
    cp #1                       ;; |
    ret z                       ;; |

    
    notSticked:
    ;call gameOver_show

    ld a, (warrior_state)

    cp #warrior_stAttack
    jp z, warrior_updateStateAttack

    cp #warrior_stNormal
    jp z, warrior_updateStateNormal

    cp #warrior_stDead
    jp z, warrior_updateStateDead

    cp #warrior_stWin
    jp z, warrior_updateStateWin

    cp #warrior_stHit
    jp z, warrior_updateStateHit


    ret

warrior_updateSticked:
    ;; To leave the stick mode has a count of 6 correct left/right dances in a countdown of 99.
    ;; If you have not done all 6 dances in 99, the dances counter is reset.

    ld a, (warrior_keyDanceState)           ;; Loads keydance state
    cp #warrior_expecRight                  ;; if 0 left is expected
    jp z, rightExpected                     ;; if 1 right is expected
    ;; left expected
    ld    hl, (keyLeft)                     ;; Check key
    call  cpct_isKeyPressed_asm             ;; |
    cp    #0                                ;; |
    jp z, danceError                        ;; Not expected, go to error
                                            ;; Left expected and readed
    ld a, #warrior_expecRight               ;; Changes next expected to right
    ld (warrior_keyDanceState),a
    jp successExpected

    rightExpected:
    ld    hl, (keyRight)                    ;; Check key
    call  cpct_isKeyPressed_asm             ;; |
    cp    #0                                ;; |
    jp z, danceError                        ;; Not expected, go to error
                                            ;; Right expected and readed
    ld a, #warrior_expecLeft                ;; Changes next expected to left
    ld (warrior_keyDanceState), a
    
    successExpected:                        ;; Increments the success dances counter
    ld a, (warrior_keyDanceCount)           ;; |
    dec a                                   ;; |
    jp z, warrior_unsetStickedState         ;; Changes state if count is finished (Reset counters)
    ld (warrior_keyDanceCount), a           ;; If need more dances, updates keyDanceCount counter and...
    
    danceError:                             ;; If the received key is not the expected or its ok
    ld a, (warrior_keyDanceResetCount)      ;; but needs more dances.
    dec a                                   ;; |
    jp z, resetDance                        ;; Reset the dance count if count is finished
    ld (warrior_keyDanceResetCount), a      ;; If not, updates keyDanceResetCount counter and exits
    ret

    resetDance:
    ld a, #warrior_danceTimes               ;; Reset the key dance count              
    ld (warrior_keyDanceCount), a           ;; |
    ld a, #warrior_countDownStick           ;; Reset the key dance countdown              
    ld (warrior_keyDanceResetCount), a      ;; |
    ret

warrior_unsetStickedState:
    ld a, #0                                ;; Changes the hero state to unsticked         
    ld (hero_sticked), a                    ;; |
    ret

hero_setStickedState::
warrior_setStickedState:
    ld a, #warrior_expecLeft                ;; Sets first expected to left
    ld (warrior_keyDanceState), a           ;; |
    ld a, #1                                ;; Changes the hero state to sticked
    ld (hero_sticked), a                    ;; |
    ld a, #warrior_danceTimes               ;; Reset the key dance count              
    ld (warrior_keyDanceCount), a           ;; |
    ld a, #warrior_countDownStick           ;; Reset the key dance countdown              
    ld (warrior_keyDanceResetCount), a      ;; |
    ret

hero_setStateNormal::
warrior_setStateNormal:
    ld a, #warrior_stNormal
    ld (warrior_state), a
    ret


warrior_updateStateNormal:
;    ld a, (waitFrame)               ;; Discards the first warrior update intentionally
;    cp #0                           ;; This solves an erase issue on the backbuffer
;    jp z, noWait
;    dec a
;    ;ld a, #0                        ;; |
;    ld (waitFrame), a               ;; |
;    ret                           ;; |
;    
;    noWait:
    call warrior_checkPlayerInput   ;; Normal state is fully controlled by the player
    ret


warrior_setStateAttack::


    call warrior_setAttackAnimation ;; |
    ld a, b
    cp #0                           ;; If animation cannot be set, 
    ret nz                          ;;    avoid set state attack

    call warrior_setSwordCollision

    ld a, #1                        ;; |
    ld (hero_attacking), a          ;; flagAttack = true
    
    ld a, #warrior_stAttack         ;; |
    ld (warrior_state), a           ;; warrior_state = warrior_stAttack
    
    ld a, #_g_sfx_swordAttackId     ;; Sound effect ID
    ld c, #1                        ;; Channel
    ld b, #0                        ;; Volume (inverted 0=max)
    call cpct_akp2_SFXPlay_asm
    ;call warrior_setAttackAnimation
    ret


warrior_updateStateAttack::
    ld a, animation_currentFrame + entity_animation (ix)
    cp #-2
    ret nz    ;; return if not reached the last frame yet
    
    call warrior_setStateNormal
    call warrior_restoreToNormalFrames

;    ld a, #1
;    ld (waitFrame), a

    ;call entity_erase
    call warrior_checkPlayerInput
    ret


hero_setStateHit::
warrior_setStateHit:
    push ix
    push af
    call panel_decrementLifeN
    pop af
    pop ix

    ld (displaceDir), a
    
    ld a, #12
    ld (displaceDist), a

    ld a, #warrior_stHit            ;; |
    ld (warrior_state), a           ;; warrior_state = warrior_stHit

    
    ;call entity_eraseEnd
    call warrior_restoreToNormalFrames
    ;call entity_draw
    ld a, #_g_sfx_warriorHitId
    ld c, #1
    ld b, #0
    call cpct_akp2_SFXPlay_asm
    ;ld a, #0                        ;; |
    ;ld (hero_attacking), a 

    ret

warrior_updateStateHit::
    ;call warrior_checkPlayerInput
    ld a, (displaceDir)
    call entity_displace

    ld a, (displaceDist)            ;; |
    dec a                           ;; decrement dist
    ld (displaceDist), a            ;; if dist != 0
    ret nz                          ;;     not finished displacing

    ld a, (hero_life)
    cp #0
    jp nz, warrior_setStateNormal

    call warrior_setStateDead

    ;ld a, orc_life (ix)             ;; |
;    dec a                           ;; decrement orc_life
;    ld orc_life (ix), a             ;; if orc_life != 0
;    jp nz, orc_setStateThink        ;;     set state think

    ret

warrior_setStateDead:
    ld a, #_g_sfx_overId     ;; Sound effect ID
    ld c, #1                        ;; Channel
    ld b, #0                        ;; Volume (inverted 0=max)
    call cpct_akp2_SFXPlay_asm
    call warrior_unsetStickedState

    ld a, #warrior_stDead            ;; |
    ld (warrior_state), a           ;; warrior_state = warrior_stDead

    ld a, #_g_aniDead
    call entity_setAnimation

    ld a, #gameOver_stShowDead
    ld (gameOver_stShow),a
    ret

warrior_updateStateDead:
    ld a, entity_animation + animation_currentFrame (ix)
    cp #-2
    ret nz

    ;call gameOver_show
    
    call cpct_scanKeyboard_asm
    ld    hl, #Key_C
    call  cpct_isKeyPressed_asm
    cp    #0
    jp nz, gameState_restartCheckpoint

    ld    hl, #Key_R
    call  cpct_isKeyPressed_asm
    cp    #0
    jp nz, gameState_restart

    ret

hero_setStateWin::
warrior_setStateWin:
    ld a, #_g_sfx_winId     ;; Sound effect ID
    ld c, #1                        ;; Channel
    ld b, #0                        ;; Volume (inverted 0=max)
    call cpct_akp2_SFXPlay_asm
    call warrior_unsetStickedState
    
    ld a, #warrior_stWin            ;; |
    ld (warrior_state), a           ;; warrior_state = warrior_stDead

    ld a, #_g_aniStand
    call entity_setAnimation

    ld a, #gameOver_stShowWin
    ld (gameOver_stShow),a
    ret

warrior_updateStateWin:
    ld a, entity_animation + animation_currentFrame (ix)
    cp #-2
    ret nz

    ;call gameOver_showWin
    
    call cpct_scanKeyboard_asm
    ld    hl, #Key_R
    call  cpct_isKeyPressed_asm
    cp    #0
    ret z
    
    call gameState_restart

    ret



hero_restoreToNormalFrames::
warrior_restoreToNormalFrames:
    ld a, (hero_attacking)
    cp #0
    ret z

    ld bc, #0x1004
    ld hl, #_g_warrior
    call entity_setTileset

    ld a, (preAttackX)
    call entity_setPosX

    ld a, (preAttackY)
    call entity_setPosY

    ld a, #0                        ;; |
    ld (hero_attacking), a          ;; flagAttack = false
    ret

warrior_checkPlayerInput:
    ;; Update keyboard state table
    call cpct_scanKeyboard_asm

    ld    hl, (keyFire)                ;;
    call  cpct_isKeyPressed_asm         ;;
    cp    #0                            ;;
    jr    z, attackKeyIsNotPressed       ;; if (keyFire == true)
        ld a, (flagAttackPressed)       ;;     |
        cp #0                           ;;     if (flagAttackPressed != false)
        jp nz, ignoreAttackPressed      ;;         return
        ld a, #1                        ;;     |
        ld (flagAttackPressed), a       ;;     flagAttackPressed = true
        call warrior_setStateAttack     ;;     change to state Attack
        ret
    attackKeyIsNotPressed:               ;; else
    ld a, #0                            ;;     |
    ld (flagAttackPressed), a           ;;     flagAttackPressed = false

    ignoreAttackPressed:

    ld a, (hero_attacking)
    cp #0
    ret nz

    ld a, #0
    ld (ph_inputMoving+1),a


    ld    hl, (keyUp)               ;; hl = key code to check
    call  cpct_isKeyPressed_asm     ;; l = a = key state
    cp    #0                        ;; | 
    jr    z, w_not_pressed          ;; if (a!=0) {
        ld a, #warrior_up
        ld (preDir), a

        ld  a, entity_pixY (ix)     ;;     |
        cp  #map_minY               ;;     |
        call z, warrior_scrollDown  ;;     Hacer scroll si ha llegado al final

        call  entity_moveUp                ;;     heroMoveUp()
        ld a, #1
        ld (ph_inputMoving+1),a

    w_not_pressed:                  ;; }

    ld    hl, (keyDown)             ;; hl = key code to check
    call  cpct_isKeyPressed_asm     ;; l = a = key state
    cp    #0                        ;; | 
    jr    z, s_not_pressed          ;; if (a!=0) {
        ld a, #warrior_down
        ld (preDir), a

        ld  a, entity_pixY (ix)     ;;     |
        cp  #map_maxY-8             ;;     |
        call z, warrior_scrollUp    ;;     Hacer scroll si ha llegado al final

        call  entity_moveDown              ;;     heroMoveDown()
        ld a, #1
        ld (ph_inputMoving+1),a

        

    s_not_pressed:                  ;; }

    ld    hl, (keyRight)                ;; hl = key code to check
    call  cpct_isKeyPressed_asm     ;; l = a = key state
    cp    #0                        ;; | 
    jr    z, d_not_pressed          ;; if (a!=0) {
        ld a, #warrior_right
        ld (preDir), a

        ld  a, entity_pixX (ix)     ;;     |
        cp  #map_maxX-8             ;;     |
        call z, warrior_scrollLeft  ;;     Hacer scroll si ha llegado al final

        call  entity_moveRight             ;;     heroMoveRight()
        ld a, #1
        ld (ph_inputMoving+1),a
        jr a_not_pressed   

        

    d_not_pressed:                  ;; }

    ld    hl, (keyLeft)                ;; hl = key code to check
    call  cpct_isKeyPressed_asm     ;; l = a = key state
    cp    #0                        ;; | 
    jr    z, a_not_pressed          ;; if (a!=0) {
        ld a, #warrior_left
        ld (preDir), a

        ld  a, entity_pixX (ix)     ;;     |
        cp  #map_minX               ;;     |
        call z, warrior_scrollRight ;;     Hacer scroll si ha llegado al final
        cp  #map_minX+1               ;;     |
        call z, warrior_scrollRight ;;     Hacer scroll si ha llegado al final

        call  entity_moveLeft       ;;     heroMoveLeft()
        ld a, #1
        ld (ph_inputMoving+1),a

        

    a_not_pressed:                  ;; }

    ph_inputMoving:
    ld a, #00                       ;;
    cp #1                           ;;
    jr z, inputMoving               ;; // Si no se está moviendo
        call warrior_setStand
    inputMoving:                    ;;

    ret

hero_setStand::
warrior_setStand:
    ld a, (preDir)
    cp #warrior_up
    jp nz, noPreDirUp
        ld a, #_g_aniStandUp
        call entity_setAnimation
        ret
    noPreDirUp:
    cp #warrior_down
    jp nz, noPreDirDown
        ld a, #_g_aniStandDown
        call entity_setAnimation
        ret
    noPreDirDown:
    call entity_stand
    ret


warrior_scrollLeft:
    call warrior_unsetStickedState
    push ix
    call world_moveE
    pop ix
    cp #0
    ret z

    ;push ix
    ;call world_getCurrentMap
    ;pop ix
    call map_setCurrentMap

    ;ld hl, #_g_map2             
    call scroll_left
    ld a, #map_minX
    call entity_setPosX

    push ix
    call scene_setScene
    pop ix

    ret

warrior_scrollRight:
    call warrior_unsetStickedState
    push ix
    call world_moveW
    pop ix
    cp #0
    ret z

    ;push ix
    ;call world_getCurrentMap
    ;pop ix
    call map_setCurrentMap

    ;ld hl, #_g_map2             
    call scroll_right
    ld a, #map_maxX-8
    call entity_setPosX

    push ix
    call scene_setScene
    pop ix

    ret

warrior_scrollDown:
    call warrior_unsetStickedState
    push ix
    call world_moveN
    pop ix
    cp #0
    ret z

    ;push ix
    ;call world_getCurrentMap
    ;pop ix
    call map_setCurrentMap

    call scroll_down
    ld a, #map_maxY-8
    call entity_setPosY

    push ix
    call scene_setScene
    pop ix

    ret

warrior_scrollUp::
    call warrior_unsetStickedState
    push ix
    call world_moveS
    pop ix
    cp #0
    ret z

    ;push ix
    ;call world_getCurrentMap
    ;pop ix
    call map_setCurrentMap

    call scroll_up
    ld a, #map_minY
    call entity_setPosY

    push ix
    call scene_setScene
    pop ix
    
    ret


    
;;==================================
;; Ataque del warrior
;; DESTROYS:
;;==================================
warrior_setAttackAnimation::
    ;;Activate flag attack
    ;ld a, #1
    ;ld (flagAttack), a


    ;;STEP 1: Detect current animation and set attack animation 
    ld a, animation_currentAni + entity_animation (ix)     ;;Load a current animation
    cp #_g_aniStand
    jp z, warrior_executeHorizontalAttack       ;;If current animation is Stand warrior_executeAnimationAttack()
    cp #_g_aniWalkLeft
    jp z, warrior_executeHorizontalAttack        ;;else if current animation is walk left horizontalAttack()
    cp #_g_aniWalkRight
    jp z, warrior_executeHorizontalAttack       ;;else if current animation is right walk horizontalAttack()
    
    cp #_g_aniStandDown
    jp z, warrior_executeDownAttack        ;;else if current animation is down walk downAttack()
    cp #_g_aniWalkDown
    jp z, warrior_executeDownAttack        ;;else if current animation is down walk downAttack()
    
    cp #_g_aniStandUp
    jp z, warrior_executeUpAttack          ;;else if current animation is up walk upAttack()
    cp #_g_aniWalkUp
    jp z, warrior_executeUpAttack          ;;else if current animation is up walk upAttack()
    ;Else continue current animation
    ret


warrior_checkSwordCollision:
    ld__d_ixh
    ld__e_ixl
    ex de, hl
    call map_checkCollision
    ret
;;==================================
;; Execute horizontal animation
;; DESTROYS:
;;==================================
warrior_executeHorizontalAttack:
    call entity_getTilePos
    
    ld a, entity_flipH (ix)
    cp #0
    jp z, lookingRight
        ld a, #0
        cp c
        ld b, #1
        ret p
        ;;CHEQUEAR SI A LA IZQUIERDA HAY SOLIDO
        ld a, entity_pixX (ix)
        sub #2
        call entity_setPosX
        call warrior_checkSwordCollision
        cp #1
        jp nz, attackLeft
        ld a, entity_pixX (ix)
        add #2
        call entity_setPosX
        ld b, #1
        ret
        attackLeft:
        ld a, entity_pixX (ix)
        add #2
        call entity_setPosX
        jp lookingLeft
    lookingRight:
        ld a, #map_width-3
        cp c
        ld b, #1
        ret m
        ;;CHEQUEAR COLISION CON LA DERECHA
        ld a, entity_pixX (ix)
        add #2
        call entity_setPosX
        call warrior_checkSwordCollision
        cp #1
        jp nz, attackRight
        ld a, entity_pixX (ix)
        sub #2
        call entity_setPosX
        ld b, #1
        ret
        attackRight:
        ld a, entity_pixX (ix)
        sub #2
        call entity_setPosX
    lookingLeft:

    ld bc, #0x1007
    ld hl, #_g_warriorAttakH
    call entity_setTileset      ;; Establecer nuevas dime de tileset
    ld a, #_g_aniAttackH
    call entity_setAnimation

    ld a, entity_pixY (ix)      ;; |
    ld (preAttackY), a          ;; Save previous Y position before attack

    ld a, entity_flipH (ix)     ;; |
    cp #0                       ;; ceroFlag = entity_flipH

    ld a, entity_pixX (ix)      ;; |
    ld (preAttackX), a          ;; Save previous X position before attack

    push af
    ld a, #warrior_right
    ld (attackDir), a
    pop af

    ld b, #0
    ret z                       ;; if flipH==0 return

    ld b, #-6                   ;; |
    add b                       ;; Special case: animation starts 6 pixels to the left
    call entity_setPosX         ;; Adjust x postion
    
    ld a, #warrior_left
    ld (attackDir), a

    ld b, #0

    ret


;;==================================
;; Execute down animation
;; DESTROYS:
;;==================================
warrior_executeDownAttack:
    call entity_getTilePos
    ld a, #map_height-4
    cp b
    ld b, #1                    ;; if tilePosY > map_height
    ret m ;; return fail
    ;;CHEQUEAR SI ABAJO HAY SOLIDO
    ld a, entity_pixY (ix)
    add #2
    call entity_setPosY
    call warrior_checkSwordCollision
    cp #1
    jp nz, attackDown
    ld a, entity_pixY (ix)
    sub #2
    call entity_setPosY
    ld b, #1
    ret
    attackDown:
    ld a, entity_pixY (ix)
    sub #2
    call entity_setPosY

    ld bc, #0x1A04
    ld hl, #_g_warriorAttakV
    call entity_setTileset      ;; Establecer nuevas dime de tileset
    ld a, #_g_aniAttackD
    call entity_setAnimation

    ld a, entity_pixX (ix)      ;; |
    ld (preAttackX), a          ;; Save previous X position before attack

    ld a, entity_pixY (ix)      ;; |
    ld (preAttackY), a          ;; Save previous Y position before attack

    ld a, #warrior_down
    ld (attackDir), a

    ld b, #0
    ret


;;==================================
;; Execute up attack animation
;; DESTROYS:
;;==================================
warrior_executeUpAttack:
    call entity_getTilePos
    ld a, #1
    cp b
    ld b, #1                    ;; if tilePosY < 1 
    ret p ;; return B = fail
    ;;CHEQUEAR SI ARRIBA HAY SOLIDO
    ld a, entity_pixY (ix)
    sub #5
    call entity_setPosY
    call warrior_checkSwordCollision
    cp #1
    jp nz, attackUp
    ld a, entity_pixY (ix)
    add #5
    call entity_setPosY
    ld b, #1
    ret
    attackUp:
    ld a, entity_pixY (ix)
    add #5
    call entity_setPosY
    
    ld bc, #0x1A04
    ld hl, #_g_warriorAttakV
    call entity_setTileset      ;; Establecer nuevas dime de tileset
    ld a, #_g_aniAttackU
    call entity_setAnimation

    ld a, entity_pixX (ix)      ;; |
    ld (preAttackX), a          ;; Save previous X position before attack

    ld a, entity_pixY (ix)      ;; |
    ld (preAttackY), a          ;; Save previous Y position before attack

    ld b, #-5                   ;; |
    add b                       ;; Special case: animation starts 6 pixels up
    call entity_setPosY         ;; Adjust y postion

    ld a, #warrior_up
    ld (attackDir), a

    ld b, #0                    ;; return B = ok
    ret


warrior_setSwordCollision:
    ld a, (attackDir)
    cp #warrior_left
    jp nz, noAttackDirLeft
        ld a, entity_posX (ix)      ;; |
        ld (hero_sword), a          ;; |
        ld a, entity_posY (ix)      ;; |
        ld (hero_sword+1), a        ;; Update sword collision
        jp finishSetSwordCollision
    noAttackDirLeft:
    cp #warrior_right
    jp nz, noAttackDirRight
        ld a, entity_posX (ix)      ;; |
        ld b, #4
        add a,b
        ld (hero_sword), a          ;; |
        ld a, entity_posY (ix)      ;; |
        ld (hero_sword+1), a        ;; Update sword collision
        jp finishSetSwordCollision
    noAttackDirRight:
    cp #warrior_up
    jp nz, noAttackDirUp
        ld a, entity_posX (ix)      ;; |
        ld (hero_sword), a          ;; |
        ld a, entity_posY (ix)      ;; |
        ld (hero_sword+1), a        ;; Update sword collision
        jp finishSetSwordCollision
    noAttackDirUp:
        ;; Attacking down
        ld a, entity_posX (ix)      ;; |
        ld (hero_sword), a          ;; |
        ld a, entity_posY (ix)      ;; |
        ld b, #8
        add a, b
        ld (hero_sword+1), a        ;; Update sword collision

    finishSetSwordCollision:
    ret

