;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################

.area _DATA
.area _CODE

;.area _main_file_ (ABS) 
;.org 0x0300

;.include "cpctelera.h.s"

.include "cpctSymbols.h.s"
;.include "entity.h.s"
.include "map.h.s"
.include "dbuffer.h.s"
.include "interrupt.h.s"
.include "scroll.h.s"
.include "panel.h.s"
.include "split.h.s"
.include "scene.h.s"
.include "menu.h.s"
.include "fps.h.s"

.include "worldDefinition.h.s"
.include "gameState.h.s"

.include "input.h.s"    
.equ input_mapSize, 10

.globl _g_sfx

.globl panel_test

.globl _g_palette
.globl _g_paletteMenu
.globl _g_tiles1
.globl _g_map1
.globl _g_map2

.equ kl_rom_walk, 0xBCCB
.equ scr_set_border, 0xBC38
.equ scr_set_ink, 0xBC32

menuFile: .asciz "assets.bin"

.include "menuSymbols.h.s"

.include "../menu/incl/loadMenu.incl"

gameStart:
    call map_setCurrentMap
    call panel_init
    call scene_init
    call dbuffer_swap
    call scene_setScene
    call fps_resetCyclesCount
    
mainLoop:
    call cpct_waitVSYNC_asm         ;; Wait for vertical screen retrace    ;

    call dbuffer_swap
    call scene_update
    call panel_test

    jp mainLoop                    ;; repeat main loop


BEGINMAIN::
g_currentMap::


cpct1_setPalette_asm:
    .include "../incl/cpct_setPalette.asm"

cpct1_setPALColour_asm:
    .include "../incl/cpct_setPALColour.asm"

cpct1_miniload_asm:
    .include "../incl/cpct_miniload.asm"
    ret    


;cpct1_setVideoMode_asm:
;    .include "../incl/cpct_setVideoMode.asm"

.include "../incl/cpct_loadFromDisc.s"


;;============================
;; Punto de entrada principal
;;============================
_main::
    ;; ONLY FOR DSK BUILD
    .if tgt_dsk
        push de
        push hl
        ld bc, #0x0000              ;; |
        call scr_set_border         ;; Set border to black (using firmware)

        ld a, #0                    ;; |
        setPaleteLoop:              ;; |
            ld bc, #0x0000          ;; |
            push af                 ;; |
            call scr_set_ink        ;; |
            pop af                  ;; |
            inc a                   ;; |
            cp #15                  ;; |
            jp nz, setPaleteLoop    ;; Set all inks to black (using firmware)
            
        pop hl
        pop de

        call kl_rom_walk            ;; Reenable all roms (including disc rom)

        ld hl, #0xC000              ;; Destination read location
        ld bc, #menuFile            ;; Name of the file to read
        ;ld de, #0x8000              ;; 2kb read buffer
        call cpct_loadFromDisc_asm  ;; Load file
    .endif


    ;; Experimental:
    ;; Esta llamada crasheaba si se hacía despues de setStackLocation
    ;; Pero:
    ;;   - También se soluciona poniendo dos nop justo aqui en lugar de esta llamada
    ;;   - También se soluciona poniendo la pila en 7F00
    call cpct_disableFirmware_asm       ;; Disable CPC Firmware

    ld hl, #0x0100                      ;; La pila bajará desde la posición 0x0100
    call cpct_setStackLocation_asm      ;; OJO: Sólo puede apilar hasta 0x0040, es decir, 0x1c00-0x40 = 192 bytes


    ;; ONLY FOR DSK BUILD
    .if tgt_dsk
        ld hl, #0xC000              ;; | 
        ld de, #0x8000              ;; |
        ld bc, #0x4000              ;; Copy 16Kb from 4000 to 8000 (menu assets and code);
        ldir   
    .endif


    ;; ONLY FOR CDT BUILD
    .if tgt_cdt
        ld ix, #0x8000
        .include "../menu/incl/menuSize.incl"
        ;ld de, #10569
        call cpct1_miniload_asm
    .endif
    
    ;; Init system
    ld c, #0                            ;; |
    call cpct_setVideoMode_asm          ;; Set video mode 0
    ld    hl, #0x5410                   ;; |
    call  cpct1_setPALColour_asm         ;; Set border color to black (hwcolor 0x54)
    ld hl, #_g_paletteMenu                  ;; |
    ld de, #16                          ;; |
    call cpct1_setPalette_asm            ;; Set the main color palette
    call dbuffer_clearFront              ;; Clear the front buffer
    call dbuffer_setHighFront

    ;; Init screen split
    ld hl, #highCRTC ;+(80*22)/2
    call split_setBottomPtr
    call split_init

    ;; Init Interrupts
    ld hl, #split_setSync
    call interrupt_setISR1
    ld hl, #split_setTopFrame
    call interrupt_setISR2
    ld hl, #split_setBottomFrame
    call interrupt_setISR3
    ld hl, #split_setEnd
    call interrupt_setISR6
    call interrupt_init

    ;; Start program going to the main menu
    .include "../menu/incl/startMenu.incl"

    ;; Init SFX
    ld hl, #_g_sfx
    call cpct_akp2_SFXInit_asm
    ld hl, #cpct_akp2_SFXPlayStream_asm
    call interrupt_setISR4

    ;; Init game
    call dbuffer_clearFront              ;; Clear the front buffer
    ld hl, #_g_palette                  ;; |
    ld de, #16                          ;; |
    call cpct1_setPalette_asm            ;; Set the main color palette
    call worldDefinition_init
    call input_selectKeys
    

    jp gameStart



input_selectKeys::
    ld a, (game_keybSelected)
    cp #0
    jp nz, keyboardSelected
        call input_selectJoystick
        ret
    keyboardSelected:
        ;; Do nothing (keyboard already defined)
    ret

;; ===================================
;; Selects joystick as player input
;; ===================================
input_selectJoystick:
    ld hl, #input_joyMap
    ld de, #input_map
    ld bc, #input_mapSize
    ldir                    ; Copy joystick map to input map
    ret


ENDMAIN::

.equ spaceLeft, 440-(ENDMAIN-BEGINMAIN)+1

extraSpace: define_space spaceLeft
ENDCURRENTMAPBLOCK::
    