;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################

.area _CODE


;; ========================================================
;; ========================================================
;; INCLUDED GLOBAL SYMBOLS
;; ========================================================
;; ========================================================

.include "cpctSymbols.h.s"
.include "dbuffer.h.s"
.include "map.h.s"
.include "world.h.s"
.include "math.h.s"

.globl _g_tiles1
.globl _g_tiles2
.globl _g_tiles3
.globl _g_map1
.globl _g_font

.globl loc1  ;; Start location

.globl loc12 ;; Stairs
.globl loc20 ;; Idol

.globl loc13 ;; Checkpoint 1
.globl loc18 ;; Checkpoint 2

.globl world_currentLocation

.equ map_size, 440

;; ========================================================
;; ========================================================
;; CONSTANTS
;; ========================================================
;; ========================================================


;; ========================================================
;; ========================================================
;; PUBLIC DATA
;; ========================================================
;; ========================================================

;_g_currentMap:: .dw #0x0000
;g_currentMap:: define_space map_size
.globl g_currentMap


map_tilesets: .dw _g_tiles1, _g_tiles2, _g_tiles3

map_solidThreshold: .db 48
map_clearTile: .db 55

;; ========================================================
;; ========================================================
;; PRIVATE DATA
;; ========================================================
;; ========================================================
;posX: .db 8
;posY: .db 0

map_currentTileset: .db 0 ; id number of tileset (position in map_tilesets array)

    ; .dw #0x0000 ;; Pointer to the current tileset


;; ========================================================
;; ========================================================
;; FUNCIONES PÚBLICAS
;; ========================================================
;; ========================================================

;; ==========================================================
;; Sets the current pointer to the tileset to be used
;; when calling map_draw
;;
;; Input:
;; HL = tileset = Pointer to the tileset
;; ==========================================================
;map_setTileset::
;    ld (tileset), hl
;    ret

;map_setPos::
;    ld (posX), bc
;    ret

map_getCurrentTileset::
    ld hl, #map_tilesets
    ld a, (map_currentTileset)
    add a, a
    ld e, a
    ld d, #0x00
    add hl, de
    ld e, (hl)
    inc hl
    ld d, (hl)
    ex de, hl
    ret


map_setCurrentTileset:
    call world_getCurrentTileset    ;; A = pointer to the current tileset
    ld (map_currentTileset), a      ;; set current tileset

    cp #0
    jp nz, noth0
        ld a, #48                   ;;      solidThreshold = 48
        ld b, #55                   ;;      clearTile = 55
        jr setTh
    noth0:
    cp #1
    jr nz, noth1                       ;;   
        ld a, #24                   ;;      solidThreshold = 24
        ld b, #27                   ;;      clearTile = 27
        jr setTh                    ;;
    noth1:
        ld a, #20                   ;;      solidThreshold = 21
        ld b, #23                   ;;      clearTile = 23
    setTh:                          ;;
    ld (map_solidThreshold),a       ;;
    ld a, b
    ld (map_clearTile), a
    ret


map_getSolidThreshold:
    push af
    ld a, (map_solidThreshold)
    ld b,a
    pop af
    ret


;; ==========================================================
;; Establece el mapa actual que obtiene de world
;; Nota: El mapa debe tener 20x22 tiles
;;
;; ==========================================================
map_setCurrentMap::
    ;push ix
;    call world_getCurrentMap
;    pop ix
;    ld de, #g_currentMap
;    ld bc, #map_size
;    ldir

    push ix                         ;; |
    
    call world_getCurrentMap        ;; |
    ex de, hl                       ;; DE = pointer to the current map 
    
    call map_setCurrentTileset

    call world_mapIsReduced         ;; A = true if map is reduced
    
    pop ix                          ;; |

    cp #0
    jr nz, mapIsReduced
        call map_deflate
        jr finishDeflate
    mapIsReduced:;
        ;call map_deflate
        push ix
        call world_getBaseMap           ;; HL = pointer to base map 
        ex de, hl
        pop ix       
        call map_deflate
        push ix
        call world_getCurrentMap        ;; |
        ex de, hl                       ;; DE = pointer to the current map
        pop ix
        call map_deflateReduced    

    finishDeflate:

    call map_saveCheckPoint
    
    call map_configWays

    call map_hasTreasure
    cp #1
    ret nz
    call map_setTreasure 

    

    ret



map_deflate:

    ld hl, #0                      
    
    loopSetMap:
        push hl                     ;; |
        call cpct_get6Bits_asm      ;; |
        ld a, l                     ;; |
        pop hl                      ;; a = element from copressed 6bit array
        
        push hl                     ;; |
        ld bc, #g_currentMap        ;; |
        add hl, bc                  ;; |
        ;ld (hl), #21
        ld (hl), a                  ;; |
        pop hl                      ;; set 8bit value to uncompressed map

        inc hl                      ;; increase index

        ld bc, #map_size            ;; |
        ld a, c                     ;; |
        cp l                        ;; |
        jp nz, loopSetMap           ;; |
        ld a, b                     ;; |
        cp h                        ;; |
        jp nz, loopSetMap           ;; Loop until we get hl = map_size

    ret


tmpDeflate_wPtr: .dw 0x0000
tmpDeflate_rPtr: .dw 0x0000
tmpDeflate_rIdx: .dw 0x0000
tmpDeflate_x:    .db 0x00
tmpDeflate_y:    .db 0x00
tmpDeflate_tile: .db 0x00

map_deflateReduced::
    ld hl, #g_currentMap
    ld (tmpDeflate_wPtr), hl
    
    ld (tmpDeflate_rPtr), de
    
    ld hl, #0
    ld (tmpDeflate_rIdx), hl

    

    ld a, #0                                ;;
    ld (tmpDeflate_y), a                    ;; for y=0 to 19
    dr_outerLoop:                           ;; {

        ld a, #1                            ;;
        ld (tmpDeflate_x), a                ;;   for x=1 to 18
        dr_innerLoop:                       ;;   {

            ld de, (tmpDeflate_rPtr)        ;;
            ld hl, (tmpDeflate_rIdx)        ;;
            call cpct_get6Bits_asm          ;;
            ld a, l                         ;;
            ld (tmpDeflate_tile), a         ;;     tile = get6bits(rPtr, rIdx)

            ld hl, (tmpDeflate_rIdx)        ;; 
            inc hl                          ;;
            ld (tmpDeflate_rIdx), hl        ;;     rPtr++ 
            
            ld hl, (tmpDeflate_wPtr)        ;;
            ld a, (tmpDeflate_x)            ;;
            ld e, a                         ;;
            ld d, #0x00                     ;;
            add hl, de                      ;;     

            ld a, (tmpDeflate_tile)         ;; 
            ld (hl), a                      ;;     wPtr[x] = tile
            
            ld a, (tmpDeflate_x)            ;;
            inc a                           ;; 
            ld (tmpDeflate_x), a            ;;     x++

            cp #19                          ;;
            jp nz, dr_innerLoop             ;;   }


        ld hl, (tmpDeflate_wPtr)            ;;
        ld bc, #map_width                   ;;
        add hl, bc                          ;;
        ld (tmpDeflate_wPtr), hl            ;;   wPtr += map_width

        ld a, (tmpDeflate_y)                ;;
        inc a                               ;;
        ld (tmpDeflate_y), a                ;;   y++

        cp #20                              ;;
        jp nz, dr_outerLoop                 ;; }
   
    ret

;; ==========================================================
;; Establece las salidas del mapa actual según la definicion
;; presente en worldDefinition
;;
;; ==========================================================
map_configWays::
    push ix
    call world_canMoveE
    cp #0
    jp z, cantMoveE;
        ld bc, #0x0A13
        call map_getTile
        call map_clearWayV
    cantMoveE:
    call world_canMoveW
    cp #0
    jp z, cantMoveW
        ld bc, #0x0A00
        call map_getTile
        call map_clearWayV
    cantMoveW:
    call world_canMoveN
    cp #0
    jp z, cantMoveN
        call map_hasStairs
        cp #1
        jp z, cantMoveN
        ld bc, #0x0009
        call map_getTile
        call map_clearWayH
        ld bc, #20
        add hl,bc
        call map_clearWayH

    cantMoveN:
    call world_canMoveS
    cp #0
    jp z, cantMoveS
        ld bc, #0x1409
        call map_getTile
        call map_clearWayH

    cantMoveS:
    pop ix

    ret

map_clearWayH:
    ld a, (map_clearTile)
    ld (hl), a
    inc hl
    ld (hl), a
    ld bc, #20
    add hl,bc
    ld (hl), a
    dec hl
    ld (hl), a
    ret

map_clearWayV:
    ld a, (map_clearTile)
    ld bc, #20
    ld (hl), #8
    add hl,bc
    ld (hl), a
    add hl,bc
    ld (hl), a
    ret


map_hasStairs:
    ld hl, (world_currentLocation)
    ld de, #loc12
    call cp_hl_de
    ret

map_hasTreasure:
    ld hl, (world_currentLocation)
    ld de, #loc20
    call cp_hl_de 
    ret

map_saveCheckPoint::
    ld hl, (world_currentLocation)
    ld de, #loc13
    call cp_hl_de
    jp z, map_setCheckPoint
    
    ld hl, (world_currentLocation)
    ld de, #loc18
    call cp_hl_de
    jp z, map_setCheckPoint
    ret

map_setCheckPoint:
    ld hl, (world_currentLocation)
    ld (world_restart+1), hl
    ret    

map_deleteCheckPoint::
    ld hl, #loc1
    ld (world_restart+1), hl
    ret


map_setTreasure:
    ld bc, #0x0808
    call map_getTile
    ld a, #19
    ld (hl), a
    
    ld bc, #0x0908
    call map_getTile
    ld a, #18
    ld (hl), a

    ld bc, #0x0A08
    call map_getTile
    ld a, #17
    ld (hl), a
    ret

cp_hl_de:
    ld a, h
    cp d
    ld a, #0
    ret nz
    ld a, l
    cp e
    ld a, #0
    ret nz
    ld a, #1
    ret

;; ==========================================================
;; Gets a pointer to the tile corresponding to the x and y
;; coordinates for the current tilemap
;;
;; Input registers:
;; B     = posY    : Y coordinate of the tile
;; C     = posX    : X coordinate of the tile
;;
;; Returns:
;; HL = pointer to the tile as requested
;;
;; Destroys:
;; HL, BC, DE
;; ==========================================================
map_getTile::
    ld d, #0x00                         ;; |
    ld e, #map_width                    ;; | DE = map_width
    ;ld hl, (_g_currentMap)              ;; | 
    ld hl, #g_currentMap                 ;; | 
    call cpct_etm_getTilemapPtr_asm     ;; |
    ret


;; ==========================================================
;; Dibuja el mapa completo en pantalla 
;;
;; Input:
;; ==========================================================
map_draw::
    call map_getCurrentTileset
    ld c, #map_width                        ;; |
    ld b, #map_height                       ;; |
    ld e, #map_width                        ;; |
    ld d, #0x00                             ;; Configure the drawTilemap function
    ;ld hl, #_g_tiles1                       ;; in order to draw the entire tilemap
    
    call cpct_etm_setDrawTilemap4x8_asm     ;; and set the current tileset

    ld de, (#_g_backBuffer)                 ;; |
    ld c, #map_posX                         ;; |
    ld b, #map_posY                         ;; Get video address for map position
    call cpct_getScreenPtr_asm              ;; from its screen coordinates
    ;ld de, (_g_currentMap)                  ;; |
    ld de, #g_currentMap
    call cpct_etm_drawTilemap4x8_asm        ;; Draw the tilemap
    ret


;; ==========================================================
;; Redibuja la zona de pantalla ocupada por los tiles
;; que están bajo un sprite con el fin de borrar éste
;;
;; Input:
;; D     = posY    : posición Y en pixels del sprite
;; E     = posX    : posixion X en pixels del sprite
;; B     = h       : Alto en tiles del sprite a borrar
;; C     = w       : Ancho en tiles del sprite a borrar
;; ==========================================================
map_eraseSprite::

    ;; Calcular el tamaño en tiles a redibujar
    ;; y guardar el resultado en BC = (Alto,ancho)
    ;ld e, #1                           ;; Ancho inicial: 1 tile
    ld a, e                             ;; |
    sub a, #map_posX                    ;; restar origen del mapa
    ld e, a
    and #0x03                           ;; |
    jr  z, map_eraseNoExtraX            ;; Si la posX no es múltiplo de 4 (ancho en bytes por tile)
        inc c                           ;; El ancho a redibujar tendrá 1 tile extra
    map_eraseNoExtraX:

    ;ld d, #2                           ;; Alto inicial: 2 tiles
    ld a, d                             ;; |
    sub a, #map_posY                    ;; restar origen del mapa
    ld d, a
    and #0x07                           ;; |
    jr  z, map_eraseNoExtraY            ;; Si la posX no es múltiplo de 8 (alto en pixels por tile)
        inc b                           ;; El alto a redibujar tendrá 1 tile extra
    map_eraseNoExtraY:

    push de                             ;;
    
    srl d                               ;; This part prevents erasing out of the map
    srl d                               ;; |
    srl d                               ;; |
    ld a, d                             ;; |
    add b                               ;; A = posY/3 + height
    cp #map_height                      ;; 
    jp m, noVAdjust                     ;; if (A>map_height)
        ld a, #map_height               ;;    |
        sub d                           ;;    |
        ld b, a                         ;;    height = map_height - posY/3
    noVAdjust:                          ;; endif

    call map_getCurrentTileset
    ld de, #map_width                   ;; |
    ;ld hl, #_g_tiles1                   ;; |

    call cpct_etm_setDrawTilemap4x8_asm ;; Sets internal drawTilemap configuration
    
    pop de                              ;; |
    push de                             ;; Hold DE as the screen position

    ld a, e
    and #0xFC                           ;; alinear posición con la cuadrícula de tiles
    add a, #map_posX
    ld c, a

    ld a, d
    and #0xF8                           ;; alinear posición con la cuadrícula de tiles
    add a, #map_posY
    ld b, a
    
    ph_eraseSpriteBuffer:
    ld de, (#_g_backBuffer)             ;; |
    call cpct_getScreenPtr_asm          ;; | HL = puntero video para el heroe

    pop bc                              ;; Recover BC = screen position (y,x)
    
    srl c                               ;; |
    srl c                               ;; x = x / 4

    srl b                               ;; |
    srl b                               ;; |
    srl b                               ;; y = y / 8
    

    push hl                             ;; |
    ld d, #0x00                         ;; |
    ld e, #map_width                    ;; | DE = map_width
    ;ld hl, (_g_currentMap)              ;; | 
    ld hl, #g_currentMap
    call cpct_etm_getTilemapPtr_asm     ;; |
    ex de, hl                           ;; DE = tilemapPtr at pos x,y

    pop hl                              ;; HL = screenPtr

                                        ;; Draw tiles at right position and size
    call cpct_etm_drawTilemap4x8_asm    ;; in order to erase the sprite

    ret


map_eraseSpriteEnd::
    push hl
    push de
    push bc
    ld hl, #_g_frontBuffer
    ld (ph_eraseSpriteBuffer+2), hl
    call map_eraseSprite
    pop bc
    pop de
    ld hl, #_g_backBuffer
    ld (ph_eraseSpriteBuffer+2), hl
    call map_eraseSprite
    pop hl
    ret

map_eraseSpriteFront::
    push hl
    push de
    push bc
    ld hl, #_g_frontBuffer
    ld (ph_eraseSpriteBuffer+2), hl
    call map_eraseSprite
    pop bc
    pop de
    ld hl, #_g_backBuffer
    ld (ph_eraseSpriteBuffer+2), hl
    ;call map_eraseSprite
    pop hl
    ret



;; ===============================================================
;; Comprueba si el sprite pasado por parámetro colisiona
;; con un tile solido.
;;
;; Un tile es sólido si su numero de id está por debajo
;; del umbral establecido para el mapa
;;
;; Limitaciones:
;;    Sólo es 100% efectivo si el bounding box a detectar 
;;    tiene como máximo el mismo ancho que los tiles del mapa
;;
;; Input:
;; HL    = boundingBox  : puntero al bounding box a comprobar
;; ===============================================================
map_checkCollision::
    
    ld a, (hl)              ;; | A = box.posX
    sub #map_posX               ;; | A -= map.posX    
    ld b, a                 ;; | Calcular la coordenada X del mapa en tiles 
    push bc
    srl b                   ;; | para el lado izquierdo del bounding box
    srl b                   ;; B = posTileX = (box.posX - map.posX) / 4

    inc hl                  ;; |
    ld a, (hl)              ;; | A = box.posY
    inc hl                  ;; |

    ex af,af'
    ld a, (hl)
    ld (ph_collisionWidth+1),a
    ex af,af'

    inc hl                  ;; |
    add (hl)                ;; | 
    dec a                   ;; | A += box.height - 1
    sub #map_posY               ;; | A -= map.posY
    ld c, a                 ;; |
    srl c                   ;; | Calcular la coordenada Y del mapa en tiles
    srl c                   ;; | para el lado inferior del bounding box
    srl c                   ;; C = posTileY = (box.posY + box.height - 1 - map.posY) / 8

    push bc

    ld h, #map_width             ;; |
    ld l, #0x00              ;; HL = map.width << 8
    ld d, l                  ;; |
    ld e, c                  ;; DE = posTileY

    ld       b, #0x08                  ;; |
repeatSum:                             ;; |
    add      hl, hl                    ;; |
    jr       NC, noCarry               ;; |
    add      hl, de                    ;; |
noCarry:                               ;; | Algoritmo para multiplicar dos bytes
    djnz   repeatSum                   ;; | HL = posTileY * map.width


    pop de                  ;; Recuperamos el anterior BC (posTileX, posTileY)
    push hl                 ;; Guardar posTileY*map.width


    ld b, #0x00             ;; |
    ld c, d                 ;; BC = posTileX

    add hl, bc              ;; HL = (posTileY*map.width + posTileX)

    ;ld de, (_g_currentMap)     ;; |
    ld de, #g_currentMap
    add hl, de              ;; |
    ld a, (hl)              ;; A = *(mapArray + HL)

    pop hl
    
    call map_getSolidThreshold
    cp b      ;; Comprobamos si hay colision en la esquina inferior izquierda
    ;cp #solidThreshold      ;; Comprobamos si hay colision en la esquina inferior izquierda    
    pop bc
    
    jp p, noCollisionLeft
        ld a, #0x01         ;; Hay colisión
        ret                 ;; Devolver A = 1

    noCollisionLeft:    
    

    ph_collisionWidth:
    ld  a, #00
    add a,b
    dec a
    ld  b, a
    srl b                   ;; | para el lado derecho del bounding box
    srl b                   ;; B = posTileX = (box.posX + box.width - map.posX) / 4;

    ld c, b                 ;; BC = posTileX
    ld b, #0x00             ;; |
    
    add hl, bc              ;; HL = (posTileY*map.width + posTileX);

    ;ld de, (_g_currentMap)     ;; |
    ld de, #g_currentMap
    add hl, de              ;; |
    ld a, (hl)              ;; A = *(mapArray + HL);
    call map_getSolidThreshold
    cp b      ;; Comprobamos si hay colision en la esquina inferior izquierda
    ;cp #solidThreshold      ;; Comprobamos si hay colision en la esquina inferior izquierda
    jp p, noCollision
        ld a, #0x01         ;; Hay colisión
        ret                 ;; Devolver A = 1


    noCollision:
    ld a, #0x00             ;; No hay colisión
    ret                     ;; Devolver A = 0