;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################

.area _CODE

;; ========================================================
;; ========================================================
;; INCLUDED GLOBAL SYMBOLS
;; ========================================================
;; ========================================================
.include "cpctSymbols.h.s"
.include "dbuffer.h.s"
.include "math.h.s"
.include "gameState.h.s"

.include "menuSymbols.h.s"

;.globl _g_panel_0
;.globl _g_panel_1
.globl _g_key_0
.globl _g_heart_0   ;;COmplete
.globl _g_heart_1   ;;Half
.globl _g_heart_2   ;;Empty



;; ========================================================
;; ========================================================
;; PRIVATE DATA
;; ========================================================
;; ========================================================
;; Constants
.equ pos, #80*22
.equ size, #0x1828
.equ key_posOrig, #80*23+47+2048
.equ life_posOrig, #80*23+17+6144
.equ money_Pos, #80*24+34
moneyQuant_marker_orig:  .asciz "000000"

;;Labels
scrPtr: .dw 0x8AE0
keyPos: .dw key_posOrig

panel_moneyQuantMarker:: .dw moneyQuant_marker_orig

lifePos: .dw life_posOrig


;; ========================================================
;; Initializes graphics for the game panel
;; i.e. the background image, initial life, etc.
;;
;; ========================================================
panel_init::
    call panel_draw
    call panel_drawMoney   
    call panel_drawLife_init
    ret


;; ========================================================
;; Draws game panel. 
;;
;; Input:
;; HL = scrPtr = Pointer to the start of the video memory
;;   this should be the start of the screen, not the desired
;;   position of the panel, as that is defined by the
;;   constant "pos"
;; ========================================================
panel_draw::
    ;push hl
    ld hl, #highBuffer ;(_g_backBuffer)
    ld bc, #pos
    add hl, bc
    ld de, #_g_panel_0
    ex de, hl
    ld bc, #size
    call cpct_drawSprite_asm

    ;pop hl
    ;push hl
    ld hl, #highBuffer ;(_g_backBuffer)
    ld bc, #pos+40
    add hl, bc
    ld de, #_g_panel_1
    ex de, hl
    ld bc, #size
    call cpct_drawSprite_asm
    ;pop hl
    ret


;; ========================================================
;; Draws game panel in both back and front buffers
;;
;; ========================================================
;panel_draw::
;    ld hl, (_g_backBuffer)
;    call panel_drawAt
;    ld hl, (_g_frontBuffer)
;    call panel_drawAt
;    ret

;; ========================================================
;; Copies the game panel one buffer to another
;; Input:
;; HL = src : Source buffer
;; DE = dst : Destination buffer
;; ========================================================
panel_copy::
    ld bc, #pos

    add hl, bc
    ex de, hl
    add hl, bc
    ex de, hl

    ld a, #0x08
    copyPanelLoop:
        ld bc, #80*3
        ldir
        ld bc, #0x710
        add hl, bc
        ex de, hl
        add hl, bc
        ex de, hl
        dec a
    jp nz,copyPanelLoop
    ret

;; ========================================================
;; Icrement key. 
;; ========================================================
panel_incrementKey::
    ld a,(keyQuant)
    cp #5
    jp m,less_than_five
    ret
    less_than_five:
    inc a
    ld (keyQuant), a
    ld a, (keyPos)
    inc a
    inc a
    ld (keyPos), a
    ;ld hl, (_g_backBuffer)
    call panel_drawKey
    ;ld hl, (_g_frontBuffer)
    ;call panel_drawKey
    ret

;; ========================================================
;; Decrement key.
;; ========================================================
panel_decrementKey::
    ld a,(keyQuant)
    cp #1
    jp p,more_than_zero
    ret

    more_than_zero:
    dec a
    ld (keyQuant), a
    ld a, (keyPos)
    dec a
    dec a
    ld (keyPos), a
    ;ld hl, (_g_backBuffer)
    call panel_eraseKey
    ;ld hl, (_g_frontBuffer)
    ;call panel_eraseKey
    ret

;; ========================================================
;; Draw key. 
;; ========================================================
panel_drawKey:
    ld hl, #highBuffer ;(_g_backBuffer)
    ;push hl
    ld bc, (keyPos)
    add hl, bc
    ld de, #_g_key_0
    ex de, hl
    ld bc, #0x0C02
    call cpct_drawSprite_asm
    ;pop hl
    ret

;; ========================================================
;; Erase key. 
;; ========================================================
panel_eraseKey:
    ld hl, #highBuffer ;(_g_backBuffer)
    ;push hl
    ld bc, (keyPos)
    inc bc
    inc bc
    add hl, bc
    ex de, hl
    ld a, #0xFF
    ld b, #0x0C
    ld c, #0x02
    call cpct_drawSolidBox_asm
    ;pop hl
    ret

    ;; ========================================================
;; Increment half heart.
;;  DESTROYED REGISTERS: AF,HL,IX 
;;  Cycles: 266
;; ========================================================
panel_incrementLife::
    ld a,(lifeQuant)
    cp #10                  ;; if dont have max life
    jp m,less_than_ten
    ret

    less_than_ten:
    ld ix, (lifePos)
    inc a                   ;;Inc one life (half heart)
    ld (lifeQuant), a       ;; Save new life quant
    bit 0, a                  ;; If new life is odd
    jp z, incPosition
    ld a, #1               ;; Load type heart (1 is half heart draw)
    jp  draw_Life

    incPosition:    ;; Inc position for draw life
    ;cp #10          ;;If max life dont inc pos
    ;jp z, load_heart;
    inc ix
    inc ix
    inc ix
    
    load_heart:
    ld a, #0        ;;Load type heart(0is complete heart draw )     
    
    draw_Life:
    push af       
    ;ld hl, (_g_backBuffer)
    call panel_drawLife
    pop af
    ;ld hl, (_g_frontBuffer)
    ;call panel_drawLife   
    ld (lifePos), ix
    ret 


;; ========================================================
;; Increment the five hearts.
;; DESTROYED REGISTERS: AF, IX, HL
;; Cycles: 50
;; ========================================================    
panel_drawLife_init::
    all_life:    
    call panel_incrementLife
    ld a, (lifeQuant)
    cp #10
    jp nz, all_life
    ret

;; ========================================================
;; Decrement half heart.
;; DESTROYED REGISTERS: AF, IX, HL
;; Cycles: 266
;; ========================================================
panel_decrementLife::
    ld a,(lifeQuant)
    cp #1
    jp p,not_empty
    ret

    not_empty:    
    ld ix, (lifePos)        ;;Pos draw    
    dec a                   ;;Dec one life (half heart)
    ld (lifeQuant), a
    bit 0, a                   ;; If new life is even
    jp nz, dec_pos
    ld a, #2                ;; Load type heart ( is empy heart draw)
    jp draw_lifee

    dec_pos:
    dec ix
    dec ix
    dec ix

    load_heart2:
    ld a, #1        ;;Load type heart(2 is half heart draw )     
    
    draw_lifee:    
    ld (lifePos), ix      ;;Load new position draw (it can be the same)
    push af           
    ;ld hl, (_g_backBuffer)
    call panel_drawLife
    pop af
    ;ld hl, (_g_frontBuffer)
    ;call panel_drawLife
    ret

panel_decrementLifeN::
    
    loopDecrementLife:
        push bc
        call panel_decrementLife
        pop bc
        djnz loopDecrementLife
    ret

panel_incrementLifeN::
    
    loopIncrementLife:
        push bc
        call panel_incrementLife
        pop bc
        djnz loopIncrementLife
    ret

;; ========================================================
;; Draw Life.
;;  DESTROYED REGISTERS: BC, AF, HL, DE 
;; INPUT: A-> Type sprite
;; ========================================================
panel_drawLife:
    ;push hl
    ld hl, #highBuffer ;(_g_backBuffer)
    ld bc, (lifePos)
    add hl, bc
    bit 0, a           ;;Comprobate type sprite draw
    jp nz, halfHeart
    bit 1, a
    jp nz, emptyHeart 
    ;Else completeheart
    ld de, #_g_heart_0      ;;Load sprite 0
    jp draw_Life_Sprite
    halfHeart: 
    ld de, #_g_heart_1      ;;Load sprite 1
    jp draw_Life_Sprite
    emptyHeart:
    ld de, #_g_heart_2      ;;Load sprite 2
    jp draw_Life_Sprite
    draw_Life_Sprite:
    ex de, hl
    ld bc, #0x0903
    call cpct_drawSprite_asm
    ;pop hl
    ret


;; ========================================================
;; Increment money in panel.
;;  DESTROYED REGISTERS: BC, IX, HL, DE 
;; INPUT: HL-> amount money (less than 65535)
;; ========================================================
panel_incrementMoney::
    ld ix, (panel_moneyQuantMarker)
    ld bc, #5
    add ix, bc
    ld b, #6 
    ld c, #10 
    ld e, #0    ;;Acarreo
    digito:
        push bc
        call wdivc
        pop bc
        add (ix)
        add e
        cp #58       ;;Comprobate if exist carry
        jp m, no_activar_acarreo
        ld e, #1
        sub #10
        jp guardar

        no_activar_acarreo:
        ld e, #0

        guardar:    
        ld (ix), a
        dec ix
        djnz digito

    call panel_drawMoney
    ret                

;; ========================================================
;; Reset money in panel.
;;  DESTROYED REGISTERS: BC, IX, AF 
;; ========================================================     
panel_resetMoney::
    ld ix, (panel_moneyQuantMarker)
    ld bc, #5
    add ix, bc
    ld b, #6
    resetDigito:
        ld a, #0
        add #48
        ld (ix), a
        dec ix
        djnz resetDigito
    call panel_drawMoney
    ret

panel_resetKeys::
    call panel_decrementKey
    call panel_decrementKey
    call panel_decrementKey
    call panel_decrementKey
    call panel_decrementKey
    ret


panel_resetLife::
    ld b, #10
    call panel_decrementLifeN
    ld b, #10
    call panel_incrementLifeN
    ret

;; ========================================================
;; Draw money in panel.
;;  DESTROYED REGISTERS: BC, HL, DE 
;; ========================================================
panel_drawMoney::
    ld hl, #highBuffer ;(_g_backBuffer)
    ld bc, #money_Pos
    add hl, bc
    ex de, hl
    ld bc, (panel_moneyQuantMarker)
    call cpctex_drawTileStringM0

;    ld hl, (_g_frontBuffer)
;    ld bc, #money_Pos
;    add hl, bc
;    ex de, hl
;    ld bc, #moneyQuant_orig
;    call cpctex_drawTileStringM0
    ret