;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################

.include "cpctSymbols.h.s"
.include "map.h.s"
.include "dbuffer.h.s"
.include "entityVector.h.s"
.include "world.h.s"
.include "fps.h.s"
.include "worldDefinition.h.s"

.include "gameOver.h.s"

.globl hero

;.globl entity_vector0
;.globl entity_vector1

;; =============================
;; Scene variables
;; =============================
scene_motionDivisor::       .db 0x00    ; Slow motion divisor state


;;=====================================================
;; Initializes the scene
;;
;;=====================================================
scene_init::
    call map_draw

    ld ix, #hero
    call entity_draw
    call dbuffer_swap

    call map_draw
    
    ld ix, #hero
    call entity_draw
    
    ret


;;============================================================
;; Initializes all entities for a given scene
;;
;;============================================================
scene_setScene::
    call entity_clearAll            ;; Clear all possible entities from the previous scene
    
    call world_getCurrentEntities   ;; HL = scene entities list


;    ld a, h                     ;; |
;    cp #0                       ;; |
;    jr nz, setEntitiesLoop      ;; |
;    ld a, l                     ;; |
;    cp #0                       ;; |
;    jr nz, setEntitiesLoop      ;; |
;    jr setEntitiesEnd           ;; Finish if scene has not entities definition

    
    setEntitiesLoop:
        ld a, (hl)
        cp #-1
        jp z, setEntitiesEnd
        cp #2
        jp nz, noSkip
            ld de, #entDefSize
            add hl, de
            jp setEntitiesLoop

        noSkip:
        ld (ph_worldPtrAsign+1), hl
        inc hl
        ld e, (hl)                  ;; |
        inc hl                      ;; |
        ld d, (hl)                  ;; |
        inc hl                      ;; |
        ex de, hl                   ;; HL = pointer to entity prototype
        
;        ld a, h                     ;; |
;        cp #0                       ;; |
;        jr nz, notLastEntity        ;; |
;        jp notLastEntity1
;        continueCheck:
;        ld a, l                     ;; |
;        cp #0                       ;; |
;        jr nz, notLastEntity        ;; |
;        jr setEntitiesEnd           ;; Finish if we don't have entities left;

;        notLastEntity1:
;        
;        ex de, hl
;        inc hl
;        inc hl
;        ld a, e                     ;; |
;        cp #1                       ;; | if (pointer==0001) 
;        jp z, setEntitiesLoop       ;; |     skip this entity (has been collected)
;        dec hl
;        dec hl        
;        ex de, hl
;        jp continueCheck
;        notLastEntity:

        push de                     ;; |
        call entity_new             ;; |
        ld__ixh_d                   ;; |
        ld__ixl_e                   ;; IX = new entity
        
        pop hl                      ;; Restore scene entities list;
        ld a, (hl)                  ;; A = entity pos X
        call entity_setPosX         ;;

        inc hl                      ;; |
        ld a, (hl)                  ;; A = entity pos y
        call entity_setPosY         ;;

        exx
        ph_worldPtrAsign:
        ld hl, #0x0000
        call entity_setWorldPtr
        exx

        inc hl
        jr setEntitiesLoop

    setEntitiesEnd:

;    ld de, #entity_vector0
;    call entity_delete

    ret


;;============================================================
;; Performs an update cycle for all entities: 
;;   1. erase, 2. update, 3. draw
;;
;;============================================================
scene_update::

    ld a, (scene_motionDivisor)     ;; Changes the slow motion divisor state
    xor #1                          ;; |
    ld (scene_motionDivisor), a     ;; |
    
    ;setBorder #0x5C
    ld hl, #entity_erase
    call entity_forEach
    ;setBorder #0x54

    ld ix, #hero
    call entity_erase
    
    ld a, #4
    push af

    updateLoop:

        call fps_decreaseCyclesCount

        ld ix, #hero
        call entity_update
        ;setBorder #0x4C
        ld hl, #entity_update
        call entity_forEach
        ;setBorder #0x54
        
        pop af
        dec a
        cp #0
        jp z, endLoop
        push af

        ld a, (fps_cyclesCount)
        cp #0
        jp nz, updateLoop
    
    pop af
    endLoop:
    call fps_resetCyclesCount
    
    ;setBorder #0x56
    ;ld hl, #entity_draw
    ;call entity_forEach

    ld hl, #entity_drawIfDead
    call entity_forEach

    ld ix, #hero
    call entity_draw;;

    ld hl, #entity_drawIfItem
    call entity_forEach

    ld hl, #entity_drawIfAlive
    call entity_forEach
    ;setBorder #0x54

    call gameOver_update

    
    ret
