;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################

.area _CODE

;; ========================================================
;; ========================================================
;; SÍMBOLOS GOLBALES INCLUIDOS
;; ========================================================
;; ========================================================

.include "cpctSymbols.h.s"
.include "map.h.s"
.include "dbuffer.h.s"
.include "panel.h.s"
.include "split.h.s"
.include "fps.h.s"

;.globl _g_tiles1

;; ========================================================
;; ========================================================
;; DATOS PRIVADOS
;; ========================================================
;; ========================================================
nextMap:     .dw 0x0000     ; Mapa que se mostrará al hacer scroll
videoOffset: .db 0x00       ; Offset de video (registro 12)
videoPage:   .db 0x00       ; Página de video (registro 13)
readPtr:     .dw 0x0000     ; Puntero de lectura del mapa

currentScrollState:
drawPtr:     .dw 0x0000     ; Puntero de dibujado 
incVideo:    .dw 0x0000     ; Incremento para los registros del CRTC
incRead:     .dw 0x0000     ; Incremento para el puntero de lectura del mapa
incDraw:     .dw 0x0000     ; Incremento para el puntero de dibujado
drawSize:    .dw 0x0000     ; Tamaño de dibujado (en tiles)
hMode:       .db 0x01       ; Modo horizontal (0/1)
evenStep:    .db 0x01       ; 1 = iteración par, 0 = iteración impar

; Parámetros de inicio para scroll hacia la izquierda
scrollStateLeft:
    .dw highBuffer+map_posX+map_width*4+80*(map_posY/8)-2, 1, 1, 2
    .db 1, map_height, 0, 1

; Parámetros de inicio para scroll hacia la derecha
scrollStateRight:
    .dw lowBuffer+map_posX+80*(map_posY/8), -1, -1, -2
    .db 1, map_height, 0, 1

; Parámetros de inicio para scroll hacia la arriba
scrollStateUp:
    .dw highBuffer+map_posX+80*(map_posY/8)+80*map_height-80, 40, map_width, 80
    .db map_width, 1, 1, 1

; Parámetros de inicio para scroll hacia la abajo
scrollStateDown:
    .dw lowBuffer+0x800+map_posX+80*(map_posY/8), -40, -map_width, -80
    .db map_width, 1, 1, 1

.equ SIZEOF_scrollState, 12


;; ========================================================
;; ========================================================
;; FUNCIONES PÚBLICAS
;; ========================================================
;; ========================================================


;; ======================================================================
;; Cambia de tilemap haciendo scroll a la izquierda
;;
;; Input:
;; ======================================================================
scroll_left::

    ld hl, #g_currentMap
    ld (nextMap), hl
    
    ;ld a, #0x00                         ;; |
    ;ld (videoOffset), a                 ;; videoOffset = 0

    ld (readPtr), hl

    ld hl, #highBuffer                 ;; |
    ld de, #lowBuffer                  ;; Restaurar el panel copiandolo desde el front
    call panel_copy                         ;; buffer al back buffer

    ld hl, #lowCRTC
    call split_setBottomPtr

    ;ld de, (_g_frontCRTC)
    ;ld (videoOffset), de
    call dbuffer_setHighFront
    ld hl, #highCRTC
    ld (videoOffset), hl


    ld hl, #scrollStateLeft             ;; |
    ld de, #currentScrollState          ;; |
    ld bc, #SIZEOF_scrollState          ;; Establecer el estado inicial
    ldir                                ;; para realizar el scroll


    ld hl, #cpctex_drawHalf1TileAligned4x8_asm
    ld (ph_firstHalfFn+1), hl           ;; Establecer función para dibujar primera mitad de tile

    ld hl, #cpctex_drawHalf2TileAligned4x8_asm
    ld (ph_secondHalfFn+1), hl          ;; Establecer función para dibujar segunda mitad de tile    

    call scroll_init

    ld a, #39
    call scroll_loop                    ;; Iniciar el bucle para hacer scroll
    ;loop:
    ;jp loop

    call scroll_finish                  ;; Finaliza el scroll

    ret

;; ======================================================================
;; Cambia de tilemap haciendo scroll a la derecha
;;
;; Input:
;; ======================================================================
scroll_right::
    ld hl, #g_currentMap
    ld (nextMap), hl

    ld bc, #map_width-1
    add hl, bc
    ld (readPtr), hl


    call cpct_waitVSYNC_asm
                                        ;; Mostrar buffer bajo 
    call dbuffer_setLowFront            ;; (alineado convenientemente para este scroll)  

    call cpct_waitVSYNC_asm
    ld hl, #highCRTC
    call split_setBottomPtr


    ld hl, #lowCRTC+0x400
    ld (videoOffset), hl
    call dbuffer_setCRTCValue


    ld hl, #scrollStateRight            ;; |
    ld de, #currentScrollState          ;; |
    ld bc, #SIZEOF_scrollState          ;; Establecer el estado inicial
    ldir                                ;; para realizar el scroll


    ld hl, #cpctex_drawHalf2TileAligned4x8_asm
    ld (ph_firstHalfFn+1), hl           ;; Establecer función para dibujar primera mitad de tile

    ld hl, #cpctex_drawHalf1TileAligned4x8_asm
    ld (ph_secondHalfFn+1), hl          ;; Establecer función para dibujar segunda mitad de tile    

    call scroll_init

    ld a, #39
    call scroll_loop                    ;; Iniciar el bucle para hacer scroll

    call scroll_finish                  ;; Finaliza el scroll

    ret


;; ======================================================================
;; Cambia de tilemap haciendo scroll hacia arriba
;;
;; Input:
;; ======================================================================
scroll_up::
    ld hl, #g_currentMap
    ld (nextMap), hl
    
    ld (readPtr), hl                    ;; Establecer puntero de lecutra del mapa

                                         ;; Mostrar buffer alto
    call dbuffer_setHighFront            ;; (alineado convenientemente para este scroll)

    ld hl, #highBuffer                 ;; |
    ld de, #lowBuffer                  ;; Restaurar el panel copiandolo desde el front
    call panel_copy                         ;; buffer al back buffer

    ld hl, #lowCRTC
    call split_setBottomPtr

    ld hl, #highCRTC
    ld (videoOffset), hl                 ;; |
    call dbuffer_setCRTCValue            ;; Establecer los registros 12 y 13 del CRTC

    ld hl, #scrollStateUp               ;; |
    ld de, #currentScrollState          ;; |
    ld bc, #SIZEOF_scrollState          ;; Establecer el estado inicial
    ldir                                ;; para realizar el scroll
    
    ld hl, #cpct_drawTileAligned4x8_f_asm
    ld (ph_firstHalfFn+1), hl           ;; Establecer función para dibujar primera mitad de tile

    call scroll_init

    ld a, #22                           ;; |
    call scroll_loop                    ;; Comenzar bucle del scroll

    call scroll_finish                  ;; Restaurar doble buffer

    ret


;; ======================================================================
;; Cambia de tilemap haciendo scroll hacia abajo
;;
;; Input:
;; ======================================================================
scroll_down::
    ld hl, #g_currentMap
    ld (nextMap), hl

    ld bc, #map_width*#map_height-#map_width
    add hl, bc
    ld (readPtr), hl


    call cpct_waitVSYNC_asm
                                        ;; Mostrar buffer bajo 
    call dbuffer_setLowFront            ;; (alineado convenientemente para este scroll)  

    

    call cpct_waitVSYNC_asm
    ld hl, #highCRTC
    call split_setBottomPtr


    ld hl, #lowCRTC+0x400
    ld (videoOffset), hl                ;; |
    call dbuffer_setCRTCValue           ;; Establecer los registros 12 y 13 del CRTC
    
    ld hl, #scrollStateDown             ;; |
    ld de, #currentScrollState          ;; |
    ld bc, #SIZEOF_scrollState          ;; Establecer el estado inicial
    ldir                                ;; para realizar el scroll

    ld hl, #cpct_drawTileAligned4x8_f_asm
    ld (ph_firstHalfFn+1), hl           ;; Establecer función para dibujar primera mitad de tile

    call scroll_init

    ld a, #22
    call scroll_loop

    call scroll_finish

    ret




;; ========================================================
;; ========================================================
;; FUNCIONES PRIVADAS
;; ========================================================
;; ========================================================

;; ===================================================================
;; Inicializa los parámetros de configuración para dibujar el mapa
;;
;; ===================================================================
scroll_init:
    ld bc, (drawSize)
    ;ld c, #map_width                       ;; |
    ;ld b, #map_height                      ;; |
    call map_getCurrentTileset
    ld e, #map_width                        ;; |
    ld d, #0x00                             ;; Configure the drawTilemap function
    ;ld hl, #_g_tiles1                       ;; in order to draw the entire tilemap
    call cpct_etm_setDrawTilemap4x8_asm     ;; and set the current tileset
    ret

;; ===================================================================
;; Realiza un scroll completo hacia el otro tilemap
;;
;; ===================================================================
scroll_loop:
    ;ld a, #39                           ;; |
    push af                             ;; |
    beginScroll:                        ;; |
    pop af                              ;; |
    dec a                               ;; |
    push af                             ;; |
    jr z, #endScroll                    ;; for (a=39; a!=0; a--) {

        call cpct_waitVSYNC_asm         ;;     Wait for vertical screen retrace
        
        ;setBorder #0x55                 
        call scroll_step                ;;     Actualizar estado del scroll
        call scroll_draw                ;;     Dibujar/borrar las zonas necesarias
        ;setBorder #0x54

        jp beginScroll                  ;; }
    endScroll:                          ;; |
    pop af                              ;; |

    ret

;; ===================================================================
;; Mueve la pantalla un paso hacia la dirección configurada
;;
;; ===================================================================
scroll_step::
    push af
    ;; Mover a la siguiente posición del offset de video
    ;;
    ld hl, (videoOffset)
    ld de, (incVideo)
    add hl, de
    ld (videoOffset), hl

    call dbuffer_setCRTCValue

    ;; Determinar la siguiente columna de tiles a dibujar
    ;;
    ld a, (hMode)           
    cp #1
    jr z, stepHMode

    ld a, (evenStep)                    ;; |
    xor #1                              ;; |
    ld (evenStep), a                    ;; Actualizar el flag par
    
    jp nz, stepNoAdd                ;; Si el flag es impar
        stepHMode:
        ld hl, (readPtr)                    ;; Cargar el tilemap
        ld de, (incRead)
        add hl, de
        ld (readPtr), hl                ;;     Incrementar el puntero al tilemap
        ph_firstHalfFn:                 ;;     |
        ld bc, #0x0000                  ;;     |
        call cpctex_setDrawTileFunction ;;     Dibujaremos la primera mitad de los tiles
        jp stepAdd                  ;;  
    stepNoAdd:                      ;; Si el flag es par
        ph_secondHalfFn:                ;;     |
        ld bc, #0x0000                  ;;     |
        call cpctex_setDrawTileFunction ;;     Dibujaremos la segunda mitad de los tiles
    stepAdd:
    

    ld de, (incDraw)
    ld hl, (drawPtr)                    ;; |
    add hl, de                          ;; |
    ld (drawPtr), hl                    ;; drawPtr += 2

 
    ;; Parchea fallo en scroll_up que se produce en el salto de pagina de memoria de video
    ;; Cuando se sale del rango de linea de carácter [0..24] y llega a la linea 25
    ;; los graficos aparecen un pixel más abajo.
    ;; Para solucionarlo, en ese momento restamos 800h para subir un pixel
    ;; 
    ;; Si el puntero a memoria de video acaba en 00h, (por ejemplo 8000h)
    ;; esto es más complicado de hacer porque el salto de pagina se produce en 
    ;; en mitad de fila de carácteres (concretamente en el carácter 12)
    ;; Para solucionarlo, establecemos el inicio de memoria de video en una dirección
    ;; terminada en 12*4 = 30h, (por ejemplo 8030h). De esta forma hacemos coincidir
    ;; el fallo con el comienzo de linea cuando llegamos a la dirección 8800h
    ld hl, (drawPtr)
    ld a, h
    cp #0xC8 ;
    ;cp #0xC7 
    jr nz, noScreenJump
    ld a, l
    cp #0x00 
    ;cp #0xD0
    jr nz, noScreenJump
        ld bc, #-0x0800
        add hl, bc
        ld (drawPtr), hl
    noScreenJump:
    
    pop af

    ret


;; ===================================================================
;; Dibuja/borra los tiles necesarios tras mover la pantalla con el scroll
;;
;; ===================================================================
scroll_draw::

    ;; Dibujar nueva columna de tiles a la derecha
    ;;
;    ld hl, (readPtr)                    ;; Pasar tilemap por parámetro
;    push hl                             ;; a través de la pila;
;    ld hl, (drawPtr)                    ;; Pasar puntero de dibujado por parámetro
;    push hl                             ;; a través de la pila;
;                                        ;; Las coordenadas del primer tile de la columna
;    ld bc, #0x0000                      ;; son 0,0 porque lo que hemos movido es el puntero al tilemap
;    ld hl, (drawSize)                   ;; Cargar tamaño de columna / fila a dibujar;
;    ex de, hl                           ;; | Map Height, Width to draw in tiles
;    ld a, #map_width                    ;; | Total map width in tiles
;    call cpctex_drawTileMap4x8          ;; Dibujar una nueva columna de tiles

    ld de, (readPtr)
    ld hl, (drawPtr)
    call cpct_etm_drawTilemap4x8_asm


    ;; Parchea fallo en scroll_right que se produce en el salto de pagina de memoria de video
    ;; Al mover a la izquierda, la primera fila corresponde a la zona de la siguiente página
    ;; con lo cual que da fuera del rango de linea de carácter [0..24] y llega a la linea 25
    ;; los graficos aparecen un pixel más arriba al pintar el primer tile de la columna.
    ;; Para solucionarlo, una vez pintada toda la nueva columna, 
    ;; pintamos de nuevo el primer tile en una posición 800h bytes adelantado (baja una linea)
    ld a, (incDraw)
    cp #-2
    jp nz, noScreenJumpRight

        ld bc, #0x0101                      ;; Cargar tamaño 1x1 para 1 tile        
        call cpct_etm_setDrawTilemapSize4x8_asm

        ld de, (readPtr)
        ld hl, (drawPtr)
        ld bc, #0x800
        add hl, bc
        call cpct_etm_drawTilemap4x8_asm

        ld bc, (drawSize)
        call cpct_etm_setDrawTilemapSize4x8_asm        

        ;ld hl, (readPtr)                    
;        push hl                             
;        ld hl, (drawPtr)
;        ld bc, #0x800
;        add hl, bc
;        push hl   
;        ld bc, #0x0000                      ;; son 0,0 porque lo que hemos movido es el puntero al tilemap
;        ld de, #0x0101                      ;; Cargar tamaño 1x1 para 1 tile
;        ld a, #map_width                    ;; | Total map width in tiles
;        call cpctex_drawTileMap4x8          ;; Dibujar una nueva columna de tiles
    noScreenJumpRight:

    ret


;; ===================================================================
;; Al terminar, esta función restaura el videoMemoryOffset a 0
;; y repinta el nuevo tilemap en los dos buffers (frontbuffer y backbuffer)
;; 
;; ===================================================================
scroll_finish:
    ld bc, #cpct_drawTileAligned4x8_f_asm   ;; |
    call cpctex_setDrawTileFunction         ;; Restaurar función de dibujado de tilemap

    ;ld hl, (nextMap)                        ;; |
    ;ld (_g_currentMap), hl                  ;; Establecer el nuevo tilemap

    call map_draw                           ;; Dibujar toda la pantalla en el backbuffer

    call cpct_waitVSYNC_asm                 ;; Esperar a retrazado vertical
    
    ld hl, (_g_frontCRTC)                   ;; |
    call dbuffer_setCRTCValue               ;; Restaurar pagina y offset de video previos
    
    ld hl, #highCRTC
    call split_setBottomPtr

    call dbuffer_swap                       ;; Intercambiar buffers

    ld hl, (_g_frontBuffer)                 ;; |
    ld de, (_g_backBuffer)                  ;; Restaurar el panel copiandolo desde el front
    call panel_copy                         ;; buffer al back buffer

    call map_draw                           ;; Redibujar la pantalla en el offset 0

    call fps_resetCyclesCount
    ret