;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################

.area _CODE

.include "cpctSymbols.h.s"
.include "dbuffer.h.s"


split_setBottomPtr::
    ld (ph_bottomPtr+1), hl
    ret

;; =======================================================
;; Initializes the split. Prepares the CRTC 
;; and interruptions to work with an interrupt handler
;;
;; A call to interrupt_init should follow just after 
;; split_init
;; =======================================================
split_init::
    call cpct_waitVSYNC_asm
    halt
    halt
    
    ld bc,#0xbc07               ;; Set CRTC07 (VSYNC) = 0x00
    out (c),c                   ;; |
    ld bc,#0xbd00               ;; |
    out (c),c                   ;; |

    call cpct_waitVSYNC_asm     ;; Wait for vertical retrace

    ld bc,#0xbc07               ;; Set CRTC07 (VSYNC) = 0xFF
    out (c),c                   ;; |
    ld bc,#0xbdff               ;; |
    out (c),c                   ;; |

    ;; Wait 5 halts so the next interrupt will fire the
    ;; first function for split (setSync) at the right time slot
    ;; Please note that a call to interrupt_init should follow after split_init
    halt
    halt
    halt
    halt
    halt

    ret


;; =======================================================
;; Makes the split using a loop. Useful if we only need
;; to show a still image and dont want to setup an
;; interrupt handler
;;
;; =======================================================
split_loop::
    halt
    call split_setSync

    ;-----------------------------------------------------------------------------
    halt
    call split_setTopFrame

    ;-----------------------------------------------------------------------------
    halt
    call split_setBottomFrame

    ;------------------------------------------------------------------------------
    ;blk3
    halt
    ;setBorder #0x58

    ;------------------------------------------------------------------------------
    halt
    ;setBorder #0x5d

    ;------------------------------------------------------------------------------
    halt
    call split_setEnd

    ;------------------------------------------------------------------------------

    jp split_loop



;; =======================================================
;; Callback for ISR1
;; End drawing bottom border and sync for the next frame
;; =======================================================
split_setSync::
    ld bc,#0xbc04           ;; Set CRTC04 (VTotal) = 3 - 1
    out (c),c               ;; Border size at the bottom is 3 characters height
    ld bc,#0xbd00+3-1       ;; This section will only show a border as we
    out (c),c               ;; stated in the previous interruption

    ld bc,#0xbc07           ;; Set CRTC07 (VSYNC) = 0x00
    out (c),c               ;; So we force to start as we are very close at this point
    ld bc,#0xbd00           ;; (default is 30)
    out (c),c               ;; |

    call cpct_waitVSYNC_asm ;; Wait for vertical retrace
    ;setBorder #0x44
    
    ld bc,#0xbc07           ;; Set CRTC07 (VSYNC) = 0xFF
    out (c),c               ;; We will never reach position 0xFF. This is just
    ld bc,#0xbdff           ;; to ensure that we never reach a VSYNC when
    out (c),c               ;; starting a new screen section

    ld bc,#0xbc04           ;; Set CRTC04 (VTotal) = 9 - 1
    out (c),c               ;; Still showing border. At top it is 9 characters height 
    ld bc,#0xbd00+9-1       ;; (although we have only 5 visible)
    out (c),c               ;; |

    ret


;; =======================================================
;; Callback for ISR2
;; Setup the top section of the screen
;; =======================================================
split_setTopFrame::
    ;setBorder #0x55
   
    ld bc,#0xbc06           ;; Set CRTC06 (VDisplay) = 25
    out (c),c               ;; Stop showing border
    ld bc,#0xbd00+25        ;; Now drawing main section (25 characters height)
    out (c),c               ;; |

    call dbuffer_getCRTCValue
    ;ld hl,#0x2000           ;; Set CRTC12 (DspAddrH) = 0x20
    ld bc,#0xbc0c           ;; |
    out (c),c               ;; |
    inc b                   ;; |
    out (c),h               ;; |

    ld bc,#0xbc0d           ;; Set CRTC13 (DspAddrL) = 0x00
    out (c),c               ;; The CRTC is now reading memory at 0x8000
    inc b                   ;; |
    out (c),l               ;; |

    ret

;; =======================================================
;; Callback for ISR3
;; Setup the bottom section of the screen
;; =======================================================
split_setBottomFrame::
    ;setBorder #0x5c

    ld bc,#0xbc04           ;; Set CRTC04 (VTotal) = 22 - 1
    out (c),c               ;; This shows 22 character lines for the first section
    ld bc,#0xbd00+22-1      ;; |    
    out (c),c;              ;; |

    ph_bottomPtr:
    ld hl, #lowCRTC
    ;ld hl, (_g_backCRTC)
    ;call dbuffer_getCRTCValue
    ld bc, #40*22
    add hl, bc
    ;ld hl,#0x3018           ;; Set CRTC12 (DspAddrH) = 0x30
    ld bc,#0xbc0c           ;; |
    out (c),c               ;; |
    inc b                   ;; |
    out (c),h               ;; |
 
    ld bc,#0xbc0d           ;; Set CRTC13 (DspAddrL) = 0x18
    out (c),c               ;; The CRTC is now reading memory at 0x3018
    inc b                   ;; |
    out (c),l               ;; |

    ret

;; =======================================================
;; Callback for ISR6
;; Setup end of the drawable area
;; =======================================================
split_setEnd::
    ;setBorder #0x4c

    ld bc,#0xbc04           ;; Set CRTC04 (VTotal) = 22 - 1
    out (c),c               ;; This shows 3 character lines for the last section
    ld bc,#0xbd00+3-1       ;; |
    out (c),c;              ;; |

;    ld hl,#0x1000           ;; Set CRTC12 (DspAddrH) = 0x10
;    ld bc,#0xbc0c           ;; |
;    out (c),c               ;; |
;    inc b                   ;; |
;    out (c),h               ;; |;

;    ld bc,#0xbc0d           ;; Set CRTC13 (DspAddrL) = 0x00
;    out (c),c               ;; This won't be shown so we can let point to any
;    inc b                   ;; address we want
;    out (c),l               ;; |

    ld bc,#0xbc06           ;; Force show border
    out (c),c               ;; |
    ld bc,#0xbd00           ;; |
    out (c),c               ;; |
    ret


