;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of legend of steel.
;;
;;    Legend of steel is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Legend of steel is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Legend of steel.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2018 edition.
;;
;;     Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;     Castillo Zaragoza, Juan Miguel       jmczaragoza13@gmail.com
;;     Pérez Pérez, Luis Fernando           luisfernandodos2@hotmail.com
;;     Ponsoda Montiel, Daniel              dponsoda@gmail.com
;;
;;############################ END GPL LICENSE BLOCK #############################

;.include "entityOrc.h.s"

.include "worldDefinition.h.s"

.globl loc1

;; Current world location
world_currentLocation:: .dw loc1


;;==================================================
;; Returns current entities
;;
;; Returns:
;; HL = Pointer to current location's entities list
;;==================================================
world_getCurrentEntities::
    ld ix, (world_currentLocation)
    ld h, location_entities + 1 (ix)
    ld l, location_entities (ix)
    ret

;;============================
;; Returns current map
;;
;;============================
world_getCurrentMap::
    ld ix, (world_currentLocation)
    ld h, location_current + 1 (ix)
    ld l, location_current (ix)
    ret

world_getCurrentTileset::
    ld ix, (world_currentLocation)
    ld a, location_tileset (ix)
    ret    

world_getBaseMap::
    ld ix, (world_currentLocation)
    ld h, location_baseMap + 1 (ix)
    ld l, location_baseMap (ix)
    ret    

world_mapIsReduced::
    call world_getBaseMap
    ld a, h
    cp #0
    ld a,#1
    ret nz
    ld a, l
    cp #0
    ld a,#1
    ret nz

    ld a, #0
    ret

;;========================================================
;; Tries to move to the EAST from the current location
;;
;; Output:
;; 1 = location changed, 0 = can't move to that location
;;========================================================
world_moveE::
    call world_canMoveE
    jp world_move

;;========================================================
;; Tries to move to the WEST from the current location
;;
;; Output:
;; 1 = location changed, 0 = can't move to that location
;;========================================================
world_moveW::
    call world_canMoveW
    jp world_move

;;========================================================
;; Tries to move to the NORTH from the current location
;;
;; Output:
;; 1 = location changed, 0 = can't move to that location
;;========================================================
world_moveN::
    call world_canMoveN
    jp world_move

;;========================================================
;; Tries to move to the SOUTH from the current location
;;
;; Output:
;; 1 = location changed, 0 = can't move to that location
;;========================================================
world_moveS::
    call world_canMoveS
    jp world_move


 world_move:: 
    cp #0
    ret z  
    ld (world_currentLocation), bc  ;; If can move, set the new location
    ld a, #1                        ;; and return true
    ret

;world_move:
;    ld a, b                         ;; Checks if the character can move to the
;    cp #0                           ;; requested direction.
;    jr nz, canMove                  ;; 
;    ld a, c                         ;;
;    cp #0                           ;; 
;    jr nz, canMove                  ;;
;    ld a, #0                        ;; If can't move (ie no next map there), return false
;    ret    
;

world_canMoveE::
    call world_getLocationE
    jp world_canMove

world_canMoveW::
    call world_getLocationW
    jp world_canMove

world_canMoveN::
    call world_getLocationN
    jp world_canMove

world_canMoveS::
    call world_getLocationS
    jp world_canMove

world_canMove:
    ld a, b                         ;; Checks if the character can move to the
    cp #0                           ;; requested direction.
    jr nz, canMove                  ;; 
    ld a, c                         ;;
    cp #0                           ;; 
    jr nz, canMove                  ;;
    ld a, #0                        ;; If can't move (ie no next map there), return false
    ret                             ;;
    canMove:
    ld a, #1
    ret

world_getLocationE::
    ld ix, (world_currentLocation)
    ld b, location_e + 1 (ix)
    ld c, location_e (ix)
    ret

world_getLocationW::
    ld ix, (world_currentLocation)
    ld b, location_w + 1 (ix)
    ld c, location_w (ix)
    ret

world_getLocationN::
    ld ix, (world_currentLocation)
    ld b, location_n + 1 (ix)
    ld c, location_n (ix)
    ret

world_getLocationS::
    ld ix, (world_currentLocation)
    ld b, location_s + 1 (ix)
    ld c, location_s (ix)
    ret

world_restart::
    ld bc, #loc1
    ld (world_currentLocation), bc
    
    call worldDefinition_init
    
    ret